//------------------------------------------------------------------------------
// Birdseye.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: keithkin
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

//VEOrientation
VEOrientation = new function()
{
	this.North = Msn.VE.Orientation.North;
	this.East = Msn.VE.Orientation.East;
	this.West = Msn.VE.Orientation.West;
	this.South = Msn.VE.Orientation.South;
	//For neighbors
	this.SouthWest = "Southwest";
	this.NorthEast = "Northeast";
	this.SouthEast = "Southeast";
	this.NorthWest = "Northwest";	
};

//VEBirdseyeScene
//   Wraps around an ObliqueScene from MapControl and does some conversions.
function VEBirdseyeScene(obliqueScene)
{
	VEValidator.ValidateNonNull(obliqueScene, "obliqueScene");
	
	var m_obliqueScene = obliqueScene;
	//Since this is oblique, use the AlwaysEncodeSpec which will force the VELatLongs from the factory to 
	//be encoded.
	var m_veLatLongFactory = new VELatLongFactory(new VELatLongFactoryAlwaysEncodeSpec());
	var m_veLatLongDecoder = new VELatLongDecoder();

	this.PixelToLatLong = function(x, y, zoomLevel)
	{
		VE_Analytics.LogAPI('VE | APIs','VEBirdseyeScene - PixelToLatLong');
		var pixel = new Msn.VE.Pixel(x, y);
		var latLong = m_obliqueScene.PixelToLatLong(pixel, zoomLevel);
		return m_veLatLongFactory.CreateVELatLong(latLong.latitude, latLong.longitude);
	}
	
	this.LatLongToPixel = function(veLatLong, zoomLevel)
	{
		VE_Analytics.LogAPI('VE | APIs','VEBirdseyeScene - LatLongToPixel');
		VEValidator.ValidateObject(veLatLong, "veLatLong", VELatLong, "VELatLong");
		var veLatLongDecoded = m_veLatLongDecoder.Decode(veLatLong);
		var latLong = new Msn.VE.LatLong(veLatLongDecoded.Latitude, veLatLongDecoded.Longitude);
		return m_obliqueScene.LatLongToPixel(latLong, zoomLevel);
	}
	
	this.IsValidTile = function(tileX, tileY, zoomLevel)
	{
		VE_Analytics.LogAPI('VE | APIs','VEBirdseyeScene - IsValidTile');
		return m_obliqueScene.IsValidTile(tileX, tileY, zoomLevel);
	}
	
	this.GetID = function()
	{
		VE_Analytics.LogAPI('VE | APIs','VEBirdseyeScene - GetID');
		return m_obliqueScene.GetID();
	}
	
	this.GetTileFilename = function(tileX, tileY, zoomLevel)
	{
		VE_Analytics.LogAPI('VE | APIs','VEBirdseyeScene - GetTileFilename');
		return m_obliqueScene.GetTileFilename();
	}
	
	this.GetThumbnailFilename = function()
	{
		VE_Analytics.LogAPI('VE | APIs','VEBirdseyeScene - GetThumbnailFilename');
		return m_obliqueScene.GetThumbnailFilename();
	}
	
	this.GetNeighbor = function(i)
	{
		VE_Analytics.LogAPI('VE | APIs','VEBirdseyeScene - GetNeighbor');
		var neighbor = null;
		var scene=m_obliqueScene.neighborScenes[i];
		if (scene != null)
		{
			neighbor = new Msn.VE.ObliqueScene(scene);
			return new VEBirdseyeScene(neighbor);
		}
		return null;
	}
	
	this.GetRotation = function(i)
	{
		VE_Analytics.LogAPI('VE | APIs','VEBirdseyeScene - GetRotation');
		var rotation = null;
		var scene=m_obliqueScene.GetRotation(i);
		if (scene != null)
		{
			rotation  = new VEBirdseyeScene(scene);
		}
		return rotation;
	}
	
	this.GetOrientation = function()
	{
		VE_Analytics.LogAPI('VE | APIs','VEBirdseyeScene - GetOrientation');
		return m_obliqueScene.GetOrientation();
	}
	
	this.GetBounds = function()
	{
		return m_obliqueScene.GetBounds();
	}
	
	this.GetWidth = function()
	{
		VE_Analytics.LogAPI('VE | APIs','VEBirdseyeScene - GetWidth');
		return m_obliqueScene.GetWidth();
	}
	
	this.GetHeight = function()
	{
		VE_Analytics.LogAPI('VE | APIs','VEBirdseyeScene - GetHeight');
		return m_obliqueScene.GetHeight();
	}
	
	this.ContainsLatLong = function(veLatLong)
	{
		VE_Analytics.LogAPI('VE | APIs','VEBirdseyeScene - ContainsLatLong');
		VEValidator.ValidateObject(veLatLong, "veLatLong", VELatLong, "VELatLong");
		var veLatLongDecoded = m_veLatLongDecoder.Decode(veLatLong);
		var latLong = new Msn.VE.LatLong(veLatLongDecoded.Latitude, veLatLongDecoded.Longitude);
		return m_obliqueScene.ContainsLatLong(latLong);
	}
	
	this.ContainsPixel = function(x, y, zoomLevel)
	{
		VE_Analytics.LogAPI('VE | APIs','VEBirdseyeScene - ContainsPixel');
		var pixel = new Msn.VE.Pixel(x, y);
		return m_obliqueScene.ContainsPixel(pixel, zoomLevel);
	}
	
}

