//------------------------------------------------------------------------------
// API/DirectionsManager.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: Cthota, keithkin
//
// <header>	$DirectionsManager$</header>
//------------------------------------------------------------------------------
var L_cannotrenderroute_text = "The route could not be drawn because this web browser does not support SVG and VML.";

VEDistanceUnit = new function()
{
	this.Miles = "m";
	this.Kilometers = "k";
};

VERouteType = new function()
{
	this.Shortest = "s";
	this.Quickest = "q";
};

//VERoute
//   StartLocation
//   EndLocation
//   Itinerary
function VERoute(startLocation, endLocation, veRouteItinerary)
{
	VEValidator.ValidateObject(startLocation, "startLocation", VERouteLocation, "VERouteLocation");
	VEValidator.ValidateObject(endLocation, "endLocation", VERouteLocation, "VERouteLocation");
	VEValidator.ValidateObject(veRouteItinerary, "veRouteItinerary", VERouteItinerary, "VERouteItinerary");
	this.StartLocation = startLocation;
	this.EndLocation = endLocation;
	this.Itinerary = veRouteItinerary;
}

//VERouteLocation
//   Address
//   LatLong
function VERouteLocation(address, veLatLong)
{
	VEValidator.ValidateNonNull(address, "address");
	VEValidator.ValidateObject(veLatLong, "veLatLong", VELatLong, "VELatLong");
	this.Address = address;
	this.LatLong = veLatLong;
}

//VERouteItinerary
//   Distance
//   DistanceUnit
//   Time
//   RouteType
//   Segments
function VERouteItinerary(distance, distanceUnit, time, veRouteType)
{
	//Note: do not validate input, params can sometimes be null from server
	this.Distance = distance;
	this.DistanceUnit = distanceUnit;
	this.Time = time;
	this.RouteType = veRouteType;
	this.Segments = new Array();
	
	this.AddSegment = function(segment)
	{
		VEValidator.ValidateObject(segment, "segment", VERouteSegment, "VERouteSegment");
		this.Segments.push(segment);
	}
}

//VERouteSegment
//   Instruction
//   Distance
//   LatLong
function VERouteSegment(instruction, distance, veLatLong)
{
	//Note: do not validate input, params can sometimes be null from server
	VEValidator.ValidateObject(veLatLong, "veLatLong", VELatLong, "VELatLong");
	this.Instruction = instruction;
	this.Distance = distance;
	this.LatLong = veLatLong;
}

function VEDirectionsManager(vemap)
{	
	VEValidator.ValidateObject(vemap, "vemap", VEMap, "VEMap");
	this.m_vemap = vemap;
	var self = this;
	this.lastStartString = "";
	this.lastEndString = "";
	this.laststart = "";
	this.lastend  = "";
	this.lastcallback = null;
	this.disambigwhat = "";
	this.veroutecache = new Array();
	var m_veLatLongFactory = new VELatLongFactory(new VELatLongFactorySpecFromMap(this.m_vemap));
	var m_veLatLongDecoder = new VELatLongDecoder();
	
	//Initialize
	this.Initialize = function ()
	{
		//Add for polylines
		//xmlns:v="urn:schemas-microsoft-com:vml"
		try
		{
			document.namespaces.add("v", "urn:schemas-microsoft-com:vml");
		}
		catch(e)
		{
			//no reason to throw an error. VML is used for IE. SVG is used for FF.
			//The Graphics manager handles this for us.
		}
		
		if(this.veroutecache == null || this.veroutecache == "undefined")
		{
			this.veroutecache = new Array();
		}
		else
		{
			this.RemoveRoutePins();
		}
	}

	//GetDrivingDirections
	//   parameters:
	//      start:         start location. Either a string or VELatLong
	//      end:           end location. Either a string or VELatLong
	//      distanceUnits:
	//      routeType:
	//      fnCallback:
	//
	this.GetDrivingDirections = function (start, end, distanceUnits, routeType, fnCallback)
	{
		VEMap.ValidateState();
		this.ClearRoute();
		var a = "";
		var b = "";	
		var c = "m";
		var d = "q";	
		
		try
		{
			if(start == null || start == "undefined" || start.length <= 0)
			{
				throw new VEException("VEDirectionsManager:GetDrivingDirections", "err_invaliddirections", L_invaliddirections_text)
			}
			else	
			{
				a = start;
			}	
			
			if(end == null || end == "undefined" || end.length <= 0)
			{
				throw new VEException("VEDirectionsManager:GetDrivingDirections", "err_invaliddirections", L_invaliddirections_text)
			}
			else
			{
				b = end;
			}
			
			
			if(distanceUnits != null && distanceUnits != "undefined")
			{
				this.lastdist = distanceUnits;		
				if(distanceUnits == VEDistanceUnit.Kilometers)
				{
					c = "k";
				}
			}
			else
			{
				this.lastdist = VEDistanceUnit.Miles;		
			}

			if(routeType != null && routeType != "undefined")
			{
				this.lasttype = routeType;		
				if(routeType == VERouteType.Shortest)
				{
					d = "s";
				}
			}
			else
			{
				this.lastdist = VERouteType.Quickest;					
			}

			var parms = new Array();
			
			this.laststart = start;
			if (typeof(start) == "object")
			
			{
				//This is a start VELatLong
				var veLatLongDecoded = m_veLatLongDecoder.Decode(start);
				parms.push(new VEParameter("startlat", veLatLongDecoded.Latitude));
				parms.push(new VEParameter("startlon", veLatLongDecoded.Longitude));
			}
			else
			{
				//This is a start location string
				parms.push(new VEParameter("start", a));
			}
			
			this.lastend = end;
			if (typeof(end) == "object")
			{
				//This is an end VELatLong
				var veLatLongDecoded = m_veLatLongDecoder.Decode(end);
				parms.push(new VEParameter("endlat", veLatLongDecoded.Latitude));
				parms.push(new VEParameter("endlon", veLatLongDecoded.Longitude));
			}
			else
			{
				//This is an end location string
				parms.push(new VEParameter("end", b));
			}
			
			parms.push(new VEParameter("units", c));
			parms.push(new VEParameter("type", d));
			
			//Add map GUID to the request URL so the server generated calls can map to the proper map instance
			parms.push(new VEParameter(Msn.VE.API.Constants.mapguid, this.m_vemap.GUID));

			//Assign the callback
			this.vedirectionscallback = fnCallback;
			//This has to be a string since this will be dynamically merged into a link
			this.vedddisambiguatecallback	= "VEMap._GetMapFromGUID(" + this.m_vemap.GUID + ")._dm.DisambiguateDirections";
			
			var network = new VENetwork();
			network.ServiceUrl = Msn.VE.API.Constants.directionsservice;	
			network.BeginInvoke(null, parms, null);
		}
		catch(ex)
		{
			//log?
			throw ex;
		}
	}

	//Populate
	this.Populate = function(startAddress, 
		startLatLong, 
		endAddress, 
		endLatLong, 
		steps, 
		lats, 
		lons, 
		distance, 
		distanceUnit, 
		routeType,
		time, 
		bestViewRectangle, 
		routeRegionsByZoomLevel,
		zoomLevelsForGeneralizedLines)
	{
		try
		{
			if (startLatLong == null || endLatLong == null || steps == null || lats == null || 
			lons == null || distance == null || distanceUnit == null || time == null || 
			bestViewRectangle == null || routeRegionsByZoomLevel == null || zoomLevelsForGeneralizedLines == null)
			{
				throw new VEException("VEDirectionsManager:GetDrivingDirections", "err_noroute", L_noroute_text)
			}

			//if the start address from the server is not set, use the last set start address string
			if ((startAddress == null) || (startAddress == "undefined") || (startAddress.length == 0))
			{
				startAddress = this.lastStartString;
			}
			
			//if the end address from the server is not set, use the last set end address string
			if ((endAddress == null) || (endAddress == "undefined") || (endAddress.length == 0))
			{
				endAddress = this.lastEndString;
			}
			
			//Create start/end VERouteLocation objects
			var StartLocation = new VERouteLocation(startAddress, m_veLatLongFactory.CreateVELatLong(startLatLong[0], startLatLong[1]));
			var EndLocation = new VERouteLocation(endAddress, m_veLatLongFactory.CreateVELatLong(endLatLong[0], endLatLong[1]));

			var routeDecoder = new Msn.VE.DirectionsDecoder();

			lats = routeDecoder.DecodeCoordinatesString(lats);
			lons = routeDecoder.DecodeCoordinatesString(lons);

			//Create VERouteItinerary object
			var itineraryDistanceUnit = distanceUnit;
			if(distanceUnit == null || distanceUnit.length == 0)
				itineraryDistanceUnit = "mi";
			var routeItinerary = new VERouteItinerary(distance, itineraryDistanceUnit, time, routeType);
			
			//Create start segment	
			var StartSegment = new VERouteSegment(L_startat_text + " " + startAddress, 0, m_veLatLongFactory.CreateVELatLong(startLatLong[0], startLatLong[1]));
			routeItinerary.AddSegment(StartSegment);
					
			var firstone = true;
			
			var vepp = null;
			
			//Create steps segments
			if(steps != null)
			{
				var i = steps.length;
				for(var j=0;j<i;j++)
				{
					var ri = steps[j];
					var Segment = new VERouteSegment(ri.name, ri.distance, m_veLatLongFactory.CreateVELatLong(ri.latitude, ri.longitude));
					routeItinerary.AddSegment(Segment);
					
					//Add a pushpin
					if(firstone)
					{
						vepp = new VEPushpin(VENetwork.GetExecutionID(), m_veLatLongFactory.CreateVELatLong(ri.latitude, ri.longitude), Msn.VE.API.Constants.vedirectionsstarticon, L_Start_Text, ri.name);
						this.m_vemap.AddPushpin(vepp);
						this.veroutecache.push(vepp);
						firstone = false;
					}
					else
					{
						vepp = new VEPushpin(VENetwork.GetExecutionID(), m_veLatLongFactory.CreateVELatLong(ri.latitude, ri.longitude), Msn.VE.API.Constants.vedirectionsstepicon.replace("%1", j).replace("%2", i), L_DirectionsStep_Text + ' ' + j, ri.name);						
						this.m_vemap.AddPushpin(vepp);
						this.veroutecache.push(vepp);
					}
				}
				
				//Create End segment	
				var EndSegment = new VERouteSegment(L_arriveat_text + " " + endAddress, 0, m_veLatLongFactory.CreateVELatLong(endLatLong[0], endLatLong[1]));
				routeItinerary.AddSegment(EndSegment);
				
				//Add pushpins for start and end
				vepp = new VEPushpin(VENetwork.GetExecutionID() , m_veLatLongFactory.CreateVELatLong(endLatLong[0], endLatLong[1]), Msn.VE.API.Constants.vedirectionsendicon, L_End_Text, L_arriveat_text + " " + endAddress);
				this.m_vemap.AddPushpin(vepp);
				this.veroutecache.push(vepp);
				
				//Set View port		
				this.m_vemap.vemapcontrol.SetViewport(startLatLong[0], startLatLong[1], endLatLong[0], endLatLong[1]);		
				
				//Create the VERoute
				var route = new VERoute(StartLocation, EndLocation, routeItinerary);
			}
			
			//Render Route
			this.CreateRouteHighLight("veDDHighlight", lats, lons, routeRegionsByZoomLevel, zoomLevelsForGeneralizedLines);

			//Callback
			if(this.vedirectionscallback != null && this.vedirectionscallback != "undefined")
			{
				this.vedirectionscallback(route);
			}
		}
		catch(ex)
		{
			//log?
			throw ex;
		}
	}

	this.CreateRouteHighLight = function(id, lats, lons, regionsbyZoomLevel, zoomLevelsforGeneralizedLines)
	{
		VEMap.ValidateState();		
		
		if(this.verenderingdisabled)
		{
			this.ShowMessage(L_cannotrenderroute_text);
			return;
		}
		
		var routeHighlightWeight = 6;
		var routeHighlightColor = "#11DD11";
		var routeHighlightZIndex = 4;

		//Create highlight
		if (this.verouteHighlight != null)
		{
			this.RemoveRouteHighLight();
		}
		
		this.verouteHighlight = this.m_vemap.vemapcontrol.AddLine(id,  
											lats, 
											lons, 
											routeHighlightWeight, 
											routeHighlightColor, 
											routeHighlightZIndex, 
											regionsbyZoomLevel, 
											zoomLevelsforGeneralizedLines);		
	}

	//ClearRoute
	this.ClearRoute = function()
	{
		this.RemoveRoutePins();
		this.RemoveRouteHighLight();
		VEPushpin.Hide();
		this.m_vemap.m_veambiguouslist.Hide();
	}

	this.RemoveRoutePins = function()
	{
		try
		{
			VEMap.ValidateState();
			var pp = null;
			if(this.veroutecache != null)
			{
				var len = this.veroutecache.length;
				for(var i=0;i<len;i++)
				{
					pp = this.veroutecache[i];
					if(pp == null || pp == "undefined")
					{
						continue;
					}
					this.m_vemap.DeletePushpin(pp.ID);
				}

				for(var i=0;i<len;i++)
				{
					this.veroutecache.pop();
				}				
			}
		}
		catch(err)
		{
			//Swallow
		}
	}

	this.RemoveRouteHighLight = function()
	{
		VEMap.ValidateState();
			
		if (this.verouteHighlight == null)
		{
			return;
		}		
			
		this.m_vemap.vemapcontrol.RemoveLine(this.verouteHighlight.id);
	}

	this.ShowMessage = function(a, b)
	{
		this.m_vemap.ShowMessage(b);
	}

	this.ShowNotFoundMessage = function(a, b) 
	{
		//empty - ignore 
	}
	
	this.HandleFailedGetDirections = function(message)
	{
		if ((message != null) && (message != "undefined"))
		{
			//Clear the route (which closes open ambiguous lists). It is necessary to do a setTimeout
			//because if the start address has a problem, but end address is disambiguated -- this
			//is all done in one server response and we want to clear it after disambiguation appears
			//and just show a message.
			window.setTimeout("VEMap._GetMapFromGUID('" + this.m_vemap.GUID + "')._dm.ClearRoute()", 10);
			this.m_vemap.ShowMessage(message.replace(/%1/g, MapControl.Features.PlatformName));
		}
	}

	this.Disambiguate = function (a, b)
	{
		//For one route call to server, both start and end Disambiguate can
		//be called. If the ambiguous list is already visible, do not
		//show it. This allows the user to disambiguate the start of the route and
		//then have the route call go to the server again. The next server call
		//will pick up the end disambiguation.  If you try to allow both disambiguations
		//to occur simultaneously, you can have infinite disambiguation loops.
		if (!this.m_vemap.m_veambiguouslist.IsVisible())
		{
			this.disambigwhat=a;
			this.m_vemap.m_veambiguouslist.Show(b, this.vedddisambiguatecallback);
		}
	}

	this.DisambiguateDirections = function (a, b, c, d, e)
	{
		var start = "";
		var end = "";
		//Take the first one and recalculate
		if(this.disambigwhat == "end")
		{
			start = this.laststart;
			this.lastEndString = a;
			
			//If the lat/lon parameters are present from the server after disambiguation,
			//use them to identify the end point so there is no more disambiguation.
			if ((b != null) && (b != "undefined") && (c != null) && (c != "undefined"))
			{
				end = m_veLatLongFactory.CreateVELatLong(b, c);
			}
			else
			{
				end = a;
			}
			
			
		}
		else
		{
			end = this.lastend;
			this.lastStartString = a;

			//If the lat/lon parameters are present from the server after disambiguation,
			//use them to identify the start point so there is no more disambiguation.
			if ((b != null) && (b != "undefined") && (c != null) && (c != "undefined"))
			{
				start = new VELatLong(b, c);
			}
			else
			{
				start = a;
			}			
		}	

		this.GetDrivingDirections(start, end, this.lastdist, this.lasttype, this.vedirectionscallback);
	}

}

function VE_Directions()
{
}

function VE_RouteInstruction(name, latitude, longitude, distance)
{
	this.name = name;
	this.distance = distance;
	this.latitude = latitude;
	this.longitude = longitude;
	this.permanentPin = false;
}

function VE_Location(name, latitude, longitude)
{
	this.name = name;
	this.latitude = latitude;
	this.longitude = longitude;
}
