//------------------------------------------------------------------------------
// API/LayerManager.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: Cthota, keithkin
//
// <header>	$LayerManager$</header>
//------------------------------------------------------------------------------

//VELayerType enumeration
//  VELayerType.VECollection   VE User Collection layer type
//  VELayerType.GeoRSS         Geo RSS layer type
VELayerType = new function()
{
	this.VECollection = "c";
	this.GeoRSS = "g";
};

//VELayerSpecification
//   Defines the specification for a layer
//   the constructor is optional. Users can pass parameters to it
//   or set properties separately.
function VELayerSpecification(type, layerId, layerSource, method, fcallback, iconUrl)
{
	this.Type = null;        //VELayerType of layer
	this.ID = null;          //layer id
	this.LayerSource = null; //source of layer
	this.Method = null;      //HTTP method
	this.FnCallback = null;  //function callback
	this.IconUrl = null;

	if ((type != "undefined") && (type != null)) 
	{
		VEValidator.ValidateLayerType(type, "veLayerType");
		this.Type = type;
	}

	if ((layerId != "undefined") && (layerId != null)) 
	{
		this.ID = layerId;
	}

	if ((layerSource != "undefined") && (layerSource != null)) 
	{
		this.LayerSource = layerSource;
	}

	if ((method != "undefined") && (method != null)) 
	{
		this.Method = method;
	}
	
	if ((fcallback != "undefined") && (fcallback != null)) 
	{
		this.FnCallback = fcallback;
	}
	
	if ((iconUrl != "undefined") && (iconUrl != null))
	{
		this.IconUrl = iconUrl;
	}
};

//class VELayerManager
//public interface:
//   VELayerManager(vemap)
//      constructor
//      parameters:
//         vemap: VEMap instance this layer manager is associated with
//
//   AddLayer
//   AddGeoRSSLayer
//   AddGeoRSSLayerPushpins
//   AddCollectionLayer
//   Show
//   Hide
//   DeleteLayer
//   DeleteLayerById
//   Dispose
//
//
function VELayerManager(vemap)
{
	var self = this;   //so event callbacks can access "this"
	VEValidator.ValidateNonNull(vemap, "vemap");
	this.m_vemap = vemap;
	this.m_vemapcontrol = this.m_vemap.vemapcontrol;
	this.m_velayers = new Array();
	this.m_veLatLongDecoder = new VELatLongDecoder();
	this.m_spec = new VELatLongFactorySpecFromMap(this.m_vemap);
	var m_veLatLongFactory = new VELatLongFactory(this.m_spec);	

	//AddLayer
	this.AddLayer = function(veLayerSpec)
	{
		VEValidator.ValidateObject(veLayerSpec, "veLayerSpec", VELayerSpecification, "VELayerSpecification");
		VEValidator.ValidateLayerType(veLayerSpec.Type, "veLayerSpec.Type");
		VEValidator.ValidateNonNull(veLayerSpec.ID, "veLayerSpec.ID");
		VEValidator.ValidateNonNull(veLayerSpec.LayerSource, "veLayerSpec.LayerSource");
		
		if (veLayerSpec.Type == VELayerType.GeoRSS)
		{
			VEValidator.ValidateNonNull(veLayerSpec.Method, "veLayerSpec.Method");
			this.AddGeoRSSLayer(veLayerSpec);
		}
		else if(veLayerSpec.Type == VELayerType.VECollection)
		{
			this.AddVECollectionLayer(veLayerSpec);
		}
		else
		{
			throw new VEException("VEMap:AddLayer", "err_invalidlayertype", L_invalidlayertype_text);
		}
	}
	
	//AddGeoRSSLayer
	//      veLayerSpec   the VELayerSpecification for the layer
	this.AddGeoRSSLayer = function(veLayerSpec)
	{
		VEMap.ValidateState();
		
		try
		{
			var layer = this.GetLayerById(veLayerSpec.ID);
			//Throw exception if layer already exists
			if ((layer != null) && (layer != "undefined"))
			{
				throw new VEException("VELayerManager:AddGeoRSSLayer", "err_invalidlayerid", L_invalidlayerid_text);
			}

			//Create the layer
			var veLayer = new VELayer(veLayerSpec, VENetwork.GetExecutionID(), this);
			this.m_velayers.push(veLayer);

			//Make the call with the callback to the specific layer.
			//The callback has to go to the specific layer instance so multiple asynchronous calls to AddLayer will
			//work simultaneously
			VENetwork.DownloadXml(veLayerSpec.LayerSource, veLayerSpec.Method, veLayer.AddGeoRSSLayerCallback);
		}
		catch(err)
		{
			throw err;
		}
	}
	
	//AddVECollectionLayer
	//   Adds a pushpin layer to the map from a VE user collection
	//      veLayerSpec   the VELayerSpecification for the layer
	this.AddVECollectionLayer = function(veLayerSpec)
	{
		VEMap.ValidateState();
		
		try
		{
			var layer = this.GetLayerById(veLayerSpec.ID);
			//Throw exception if layer already exists
			if ((layer != null) && (layer != "undefined"))
			{
				throw new VEException("VELayerManager:AddCollectionLayer", "err_invalidlayerid", L_invalidlayerid_text);
			}
			
			//Create the layer
			var veLayer = new VELayer(veLayerSpec, VENetwork.GetExecutionID(), this);
			this.m_velayers.push(veLayer);

			//Create request to Retrieve all annotations from the server
			//  The "mapguid" parameter allows multiple maps to work with collections asynchronously
			//  The "contextid" parameter is used here so multiple layers will work within the same map asynchronously
			var parms = new Array();
			parms.push(new VEParameter("action", VE_AnnotationActions.RetrieveAll));
			parms.push(new VEParameter(VE_CollectionsManagerConstants.C_Id, veLayer.Spec.LayerSource));
			parms.push(new VEParameter(VE_CollectionsManagerConstants.Market, Msn.VE.API.Globals.language));
			parms.push(new VEParameter(Msn.VE.API.Constants.mapguid, this.m_vemap.GUID));
			parms.push(new VEParameter(Msn.VE.API.Constants.contextid, veLayer.GUID));

			//Execute the request
			var network = new VENetwork();
			network.ServiceUrl = Msn.VE.API.Constants.collectionservice;
			network.BeginInvoke(null, parms, null);
		}
		catch(err)
		{
			throw err;
		}
	}
	
	//GetLayerById
	//   returns VELayer for the passed in id
	//   If it does not exist, returns null
	this.GetLayerById = function(layerID)
	{
		var layerlength = this.m_velayers.length;
		for (var i=0; i < layerlength; ++i)
		{
			if (this.m_velayers[i].Spec.ID == layerID)
			{
				return this.m_velayers[i];
			}
		}
		return null;
	}

	//GetLayerByGUID
	//   returns VELayer for the passed in layer GUID
	//   If it does not exist, returns null
	this.GetLayerByGUID = function(layerGUID)
	{
		var layerlength = this.m_velayers.length;
		for (var i=0; i < layerlength; ++i)
		{
			if (this.m_velayers[i].GUID == layerGUID)
			{
				return this.m_velayers[i];
			}
		}
		return null;
	}

	//AddGeoRSSLayerPushpins
	//   Adds the GeoRSS pushpins from the passed in DOM document to the passed in layer
	this.AddGeoRSSLayerPushpins = function(domDocument, layer)
	{
		//Set for the view port
		var pinArray = new Array();
			
		var title= null;
		var description= null;
		var link= null;
		var icon= null;
		var lat= null;
		var lon= null;	
		var x = domDocument.getElementsByTagName("item");	
		for (i=0;i<x.length;i++)
		{
			count = i;
			for (j=0;j<x[i].childNodes.length;j++)
			{
				if (x[i].childNodes[j].nodeType != 1) 
				{
					continue;			
				}					
				if(x[i].childNodes[j].nodeName == "title")
				{
					if (x[i].childNodes[j].firstChild)
						title = x[i].childNodes[j].firstChild.nodeValue;
					else 
						title = "";
				}
				else if(x[i].childNodes[j].nodeName == "description")
				{
					if (x[i].childNodes[j].firstChild)
						description = x[i].childNodes[j].firstChild.nodeValue;
					else
						description = "";
				}
				else if(x[i].childNodes[j].nodeName == "link")
				{
					link = x[i].childNodes[j].firstChild.nodeValue;
				}
				else if(x[i].childNodes[j].nodeName == "icon")
				{
					icon = x[i].childNodes[j].firstChild.nodeValue;
				}
				else if(x[i].childNodes[j].nodeName == "virtualearth:icon")
				{
					icon = x[i].childNodes[j].firstChild.nodeValue;
				}			
				else if(x[i].childNodes[j].nodeName == "geo:lat")
				{
					lat = x[i].childNodes[j].firstChild.nodeValue;
				}
				else if(x[i].childNodes[j].nodeName == "geo:lon")
				{
					lon = x[i].childNodes[j].firstChild.nodeValue;
				}
				else if(x[i].childNodes[j].nodeName == "geo:long")
				{
					lon = x[i].childNodes[j].firstChild.nodeValue;
				}
				//GML (currently supporting GML 3.1)
				//See http://opengis.net/gml/
				else if(x[i].childNodes[j].nodeName == "gml:name")
				{
					if (x[i].childNodes[j].firstChild)
						title = x[i].childNodes[j].firstChild.nodeValue;
					else 
						title = "";
				}
				else if(x[i].childNodes[j].nodeName == "gml:description")
				{
					if (x[i].childNodes[j].firstChild)
						description = x[i].childNodes[j].firstChild.nodeValue;
					else
						description = "";
				}
				else if ((x[i].childNodes[j].nodeName == "gml:Point") ||
						 (x[i].childNodes[j].nodeName == "gml:point"))   //gml:Point is the standard - accept gml:point to be forgiving
				{
					if ((x[i].childNodes[j].firstChild) &&
						(x[i].childNodes[j].firstChild.nodeName == "gml:pos")) //support GML:pos inside gml:Point
					{
						if (x[i].childNodes[j].firstChild.firstChild)
						{
							var nodeValue = x[i].childNodes[j].firstChild.firstChild.nodeValue;
							latlon = ExtractLatLonPairFromXMLList(nodeValue);
							if (latlon.length > 1)
							{
								lat = latlon[0];
								lon = latlon[1];
							}
						}
					}
				}
				else if (x[i].childNodes[j].nodeName == "gml:pos") //support gml:pos outside of gml:Point to be forgiving
				{
					if (x[i].childNodes[j].firstChild)
						{
							var nodeValue = x[i].childNodes[j].firstChild.nodeValue;
							latlon = ExtractLatLonPairFromXMLList(nodeValue);
							if (latlon.length > 1)
							{
								lat = latlon[0];
								lon = latlon[1];
							}
						}
				}
			}
			
			if(lat == null || lat == "undefined" || lat.length <= 0 ||
			lon == null || lon == "undefined" || lon.length <= 0)
			{
				continue;
			}
			
			//If the icon is specified in the spec, use it -- override the one from the XML
			if ((layer.Spec.IconUrl != null) && (layer.Spec.IconUrl != "undefined"))
			{
				icon = layer.Spec.IconUrl;
			}
			
			//If no icon is specified, use the default icon
			if(icon == null || icon == "undefined" || icon.length <= 0)
			{
				icon = Msn.VE.API.Constants.iconurl;
			}
			
			var details = description;
			if(link != null && link != "undefined")			
				details += "<p><a href='" + link + "' class=\"VE_Pushpin_Popup_Link\" target=_blank>. . .</a></p>";
			 		
			var pinid = "Layer" + layer.Spec.ID + "_" + i;
			var pin = new VEPushpin(pinid, m_veLatLongFactory.CreateVELatLong(lat, lon), icon, title, details);
			pin.IsInLayer = true;
			pinArray.push(pin);
		}

		//Add pins to the layer
		layer.Pins = pinArray;

		//Show the layer
		this.Show(layer.Spec.ID);

		//Make the user-defined layer callback if specified
		if((layer.Spec.FnCallback != null) && (layer.Spec.FnCallback != "undefined"))
		{
			layer.Spec.FnCallback(pinArray);
		}
	}
		
	//ExtractLatLonPairFromXMLList
	function ExtractLatLonPairFromXMLList(value)
	{
		//Remove whitespace at the beginning of the value
		value = value.replace(/^\s+/g, "");

		//Remove whitespace at the end of the value
		value = value.replace(/\s+$/g, "");

		var latlon = new Array();
		if (value.indexOf(",") > 0)
		{
			latlon = value.split(",");
		}
		else
		{
			//Replace whitespace in the middle of the coordinates with 1 space
			value = value.replace(/\s+/g, " ");
			
			//split the lat/long coordinates from the space
			latlon =value.split(" ");
		}
		return latlon;
	}

	//RetrieveAllAnnotationsCallback
	//   callback from the server responding to "RetrieveAllAnnotations" request
	//   to add pins to the map
	this.RetrieveAllAnnotationsCallback =function(annotations, contextid)
	{
		if ((contextid != "undefined") && (contextid != null))
		{
			//The passed in contextid from the server is the GUID for the layer.
			//This is necessary so multiple layers on the same map can work 
			//asynchronously
			var layer = self.GetLayerByGUID(contextid)
			if ((layer != null) && (layer != "undefined"))
			{
				var icon= Msn.VE.API.Constants.iconurl;  //set icon to the default
				//If the icon is specified in the spec, use it -- override the default
				if ((layer.Spec.IconUrl != null) && (layer.Spec.IconUrl != "undefined"))
				{
					icon = layer.Spec.IconUrl;
				}

				var pinArray = new Array();
				for (var i=0; i < annotations.length; ++i)
				{
					var title= IOSec.EncodeHtml(annotations[i].Title);
					var description=IOSec.EncodeHtml(annotations[i].Notes);
					var link= annotations[i].Url;															
					var lat= annotations[i].Latitude;
					var lon= annotations[i].Longitude;
					var photoUrl = annotations[i].PhotoUrl;
					var details = description;
					var pinid = "Layer" + layer.Spec.ID + "_" + i;
					
					//Only show the photoUrl if it exists and has a length > 1
					if((photoUrl != null) && (photoUrl != "undefined") && (photoUrl.length > 1))
					{
						title = "<a href=\"\" onclick=\"window.open('" + photoUrl + "', '_blank' , 'menubar=0,resizable=1,scrollbars=0,status=0,titlebar=0,toolbar=0,width=800,height=600,screenX=200,left=200,screenY=200,top=200');return false;\"><img border=0 id=eroImg_" + pinid + " style=\"position: relative; display: block; padding-right: 10px; padding-left: 0px; padding-bottom: 5px; padding-top: 3px; width: 80px; height: 80px; float: left;\" src=\"" + photoUrl + "\"/></a>" + title;
					}

					//Only show the link if it exists 
					if((link != null) && (link != "undefined") && (link.length > 0))
						details += "<p><a href='" + link + "' class=\"VE_Pushpin_Popup_Link\" target=_blank>. . .</a></p>";

					var pin = new VEPushpin(pinid, m_veLatLongFactory.CreateVELatLong(lat, lon), icon, title, details);
					pin.IsInLayer = true;
					pinArray.push(pin);
				}
				
				//Add pins to the layer
				layer.Pins = pinArray;

				//Show the layer
				self.Show(layer.Spec.ID);
				
				//Make the user-defined layer callback if specified
				if(layer.Spec.FnCallback != null && layer.Spec.FnCallback != "undefined")
				{
					layer.Spec.FnCallback(pinArray);
				} 
			}
		}	
	}
}

//Dispose
VELayerManager.prototype.Dispose = function()
{
	//Delete the layers
	this.DeleteAllLayers();
	this.m_velayers = null;
	this.m_vemapcontrol = null;
	this.m_vemap = null;
	this.m_veLatLongDecoder = null;
	this.m_spec = null;
}

//Show
VELayerManager.prototype.Show = function(layerId)
{
	VEMap.ValidateState();
	try
	{
		VEValidator.ValidateNonNull(layerId, "layerId");

		var layer = this.GetLayerById(layerId);
		if ((layer != null) && (layer != "undefined"))
		{
			var llArray = new Array();
			var pins = layer.Pins;
			var length = pins.length;
			for(var j=0;j<length;j++)
			{
				var ll = new Object();
				var latLong = pins[j].LatLong;
				if (this.m_spec.IsEncode())
				{
					latLong = this.m_veLatLongDecoder.Decode(latLong);
				}
				ll.latitude = latLong.Latitude;
				ll.longitude = latLong.Longitude;
				llArray.push(ll);
	
				try
				{
					this.m_vemap.AddPushpin(pins[j]);
				}
				catch(ex)
				{
					//ignore the error here so if the layer is already visible and the pushpins are already on the map, 
					//adding the pushpins will not throw an error. We want Show on a visible layer to work ok
				}
			}

			//Set viewport		
			if(llArray != null && llArray.length > 0)
				this.m_vemapcontrol.SetBestMapView(llArray);
		}
		else
		{
			throw new VEException("VEMap:RemoveLayerbyId", "err_invalidlayerid", L_invalidlayerid_text);
		}
	}
	catch(err)
	{
		throw err;
	}
}

//Hide
VELayerManager.prototype.Hide = function(layerId)
{
	VEMap.ValidateState();
	try
	{
		VEValidator.ValidateNonNull(layerId, "layerId");
		
		var layer = this.GetLayerById(layerId);
		if ((layer != null) && (layer != "undefined"))
		{
			try
			{
				this.DeletePushpinsForLayer(layer);
			}
			catch(ex)
			{
				//ignore the error here so if the layer is already hidden 
				//hiding the pushpins will not throw an error. We want Hide on a hidden layer to work ok
			}
		}
		else
		{
			throw new VEException("VEMap:RemoveLayerbyId", "err_invalidlayerid", L_invalidlayerid_text);
		}
	}
	catch(err)
	{
		throw err;
	}
}

//DeleteAllLayers
//   Deletes all layers
VELayerManager.prototype.DeleteAllLayers = function()
{
	var len = this.m_velayers.length;
	for (var i=0; i < len; ++i)
	{
		//Note we cannot just call DeleteLayer here because it modifies the array we
		//are iterating over -- must call methods that do not modify the array
		//this.DeletePushpinsForLayer(this.m_velayers[i]);
		this.m_velayers[i].Dispose();
	}
	this.m_velayers = new Array();
}

//DeleteLayer
//   Deletes a layer
//   parameters:
//      layer:        layer object to delete
VELayerManager.prototype.DeleteLayer = function(layer)
{
	this.DeleteLayerById(layer.Spec.ID);
}

//DeleteLayerById
//   Deletes a layer
//   parameters:
//      id:        id of layer to delete
VELayerManager.prototype.DeleteLayerById = function(layerId)
{
	try
	{
		if(this.m_velayers == null || this.m_velayers == "undefined")
		{
			throw new VEException("VEMap:RemoveLayerbyId", "err_GraphicsInitError", L_GraphicsInitError_Text);
			return;
		}

		var layerlength = this.m_velayers.length;
		var i=0;
		while (i < layerlength && layerId != this.m_velayers[i].Spec.ID)
		{
			i++;
		}

		if (i < layerlength)
		{
			try
			{
				this.DeletePushpinsForLayer(this.m_velayers[i]);
			}
			catch(ex)
			{
				//ignore the error here so if the layer is already hidden 
				//hiding the pushpins will not throw an error. We want Hide on a hidden layer to work ok
			}

			this.m_velayers[i].Dispose();
			this.m_velayers[i] = null;
			this.m_velayers.splice(i,1);
		}
		else
		{
			throw new VEException("VEMap:RemoveLayerbyId", "err_invalidlayerid", L_invalidlayerid_text);
			return;
		}
	}
	catch(ex)
	{
		throw ex;
		return;
	}
}

//DeletePushpinsForLayer
//   Deletes all pushpins for the passed in layer
//   parameters:
//      layer:        layer to delete pushpins from
VELayerManager.prototype.DeletePushpinsForLayer = function(layer)
{
	VEMap.ValidateState();
	VEValidator.ValidateNonNull(layer, "layer");
	
	try
	{
		var pins = layer.Pins;
		var length = pins.length;
		for(var i=0;i<length;i++)
		{
			this.m_vemap.DeletePushpin(pins[i].ID);
		}
	}
	catch(ex)
	{
		throw ex;
		return;
	}
}

//VELayer
//function VELayerCache(layerId, source, pins, fnCallback)
function VELayer(veLayerSpec, layerGUID, veLayerManager)
{	
	VEValidator.ValidateObject(veLayerSpec, "veLayerSpec", VELayerSpecification, "VELayerSpecification");
	this.Spec = veLayerSpec;
	this.GUID = layerGUID;
	this.Pins = new Array();
	var self = this;
	this.m_veLayerManager = veLayerManager;
	
	//AddGeoRSSLayerCallback
	//   required event callback so the particular layer instance can be identified by self
	//   when multiple collections are being added asynchronously
	this.AddGeoRSSLayerCallback = function(domDocument)
	{
		self.m_veLayerManager.AddGeoRSSLayerPushpins(domDocument, self);
	}
	
	//Dispose
	//   cleans up VELayer
	this.Dispose = function()
	{
		for (var i=0; i < this.Pins.length; ++i)
		{
			if ((this.Pins[i] != null) && (this.Pins[i] != "undefined"))
			{
				this.Pins[i].Dispose();
				this.Pins[i] = null;
			}
		}
		this.Pins = null;
		this.GUID = null;
		this.Spec = null;
		this.m_veLayerManager = null;
	}
}
