//------------------------------------------------------------------------------
// API/Map.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: Cthota, keithkin
//
// <header>	$Map$</header>
//------------------------------------------------------------------------------
//Includes
#include ..\..\js\window.js
#include globals.js

#include ..\..\js\MapControl\Namespaces.js
#include ..\..\js\Select.js
#include ..\..\js\IOSec.js
#include ..\..\js\panel\panel.js
#include ..\..\js\panel\events.js
#include ..\..\js\Search\SearchResult.js
#include ..\..\js\Ads.js
#include ..\..\js\directions\Decoding.js
#include ..\..\js\graphics\graphics.js	
#include ..\..\js\help.js
#include ..\..\js\CrossBrowser.js

#include network.js
#include exception.js
#include validator.js
#include pushpin.js
#include scratchpad.js
#include searchmanager.js
#include message.js
#include DirectionsManager.js
#include Ambiguouslist.js
#include GraphicsManager.js
#include LayerManager.js
#include LatLon.js
#include LatLonDecoder.js
#include LatLonFactory.js
#include Polyline.js
#include Color.js
#include ..\..\js\UserCollections\CollectionsConsts.js
#include ..\..\js\UserCollections\Annotation.js
#include ..\..\js\MapControl\Encoding\NumberEncoding.js
#include ..\..\js\MapControl\Encoding\LatLonEncoding.js
#include MapStyle.js
#include Birdseye.js
#include ..\..\js\Analytics.js

//The following JS files are for ERO support
#include ..\..\js\util\microsoft.web.css.js
#include ..\..\js\Animation\animation.js
#include ..\..\js\util\microsoft.web.geometry.js
#include ..\..\js\util\microsoft.web.ui.ieglyph.js
#include ..\..\js\ERO\ero.js

//Download atlas compatibility script if necessary
if(!(navigator.userAgent.indexOf("IE")>=0))
	VENetwork.DownloadScript(Msn.VE.API.Constants.atlascompatjs, DownloadMapStyles);
else
	DownloadMapStyles();

//Download s_code.js
//Currently turn off Omniture logging - See Bug 4794.
//VENetwork.DownloadScript( Msn.VE.API.Constants.omnitureurl , null );

function DownloadMapStyles()
{
	//Download Styles
	//Attach the ERO stylesheet before the API stylesheet so we can choose to override styles
	VENetwork.AttachStyleSheet(Msn.VE.API.Constants.erostylesheet);
	VENetwork.AttachStyleSheet(Msn.VE.API.Constants.stylesheet);
}

/*
function SetupAnalytics()
{	
	if ( window.s == 'undefined' || window.s == null )
		return;
		
	if ( !Msn.VE.API.Globals.analyticsInitialized ) 
		Msn.VE.API.Globals.analyticsInitialized = true;
	else
		return;	
	
	// add logging element	
	var loggingElement = document.createElement("div");
	loggingElement.id = "logging";
	loggingElement.style.display = "none";	
	document.body.appendChild(loggingElement);

		
	// Disable the omniture onclick event handler stuck to the body of the element.
	// This will get around the bug in their element type detection code.
	document.body.detachEvent("onclick", s.bc);

	// Set the flag on whether or not analytics is enabled.
	VE_Analytics.analyticsEnabled =  Msn.VE.API.Globals.veanalyticsenabled.toLowerCase() == 'true' ? true : false ; //read from web config

	s.linkInternalFilters="javascript:,.msn.";	
	s.server   = location.hostname;
	s.pageName = "VE | APIs";
	s.channel  = "VirtualEarth";
	// Product Name
	s.prop1 = "VirtualEarth";
	// Culture and Language 
	s.prop2 = "en-us";
	// MapTheme 
	s.prop3 = "";
	// ZoomLevel 
	s.prop4 = "";
	// Region Code 
	s.prop5 = "";
	// Navigation Action 
	s.prop6 = "";
	// Navigation Control
	s.prop7 = "";
	// Search Keyword 
	s.prop8 = "";
	// Search Location 
	s.prop9 = "";	
}
*/
			
//class VEMap
//public interface:
//   VELayerManager(elementID)
//      constructor
//      parameters:
//         elementID: DOM ID to place map at
//
//   LoadMap
//   InitializeMap
//   SetCenter
//   SetCenterAndZoom
//   ...
//
// static methods:
//   VEMap.GetVersion
//
//
function VEMap(elementID)
{
/*
	// call SetupAnalytics
	try
	{
		SetupAnalytics();
	}	
	catch(ex)
	{
	}
*/	
	var self = this;
	this.ID = elementID;
	this.GUID = VENetwork.GetExecutionID(); //GUID for this map instance
	
	//Store map instances statically by GUID for lookup if we ever need them
	if (Msn.VE.API.Globals.vemapinstances == null || Msn.VE.API.Globals.vemapinstances == "undefined")
	{
		Msn.VE.API.Globals.vemapinstances = new Array();
	}
	Msn.VE.API.Globals.vemapinstances[this.GUID] = this;

	this.network = new VENetwork();
	this.mapelement = document.getElementById(elementID);
	this.pushpins = new Array();
	
	if(this.mapelement == null)
		throw new VEException("VEMap:cstr", "err_invalidelement", L_invalidelement_text)		

	//Initialize Driving directions manager
	this.m_vedirectionsmanager = new VEDirectionsManager(this);
	this.m_vedirectionsmanager.Initialize();
	this._dm = this.m_vedirectionsmanager; //shorthand internal alias for internal server-side callbacks
	
	//Initialize Search manager
	this.m_vesearchmanager = new VE_SearchManager(this);
	this.m_vesearchmanager.Initialize();
	this._sm = this.m_vesearchmanager; //shorthand internal alias for internal server-side callbacks
	
	//Initialize Message
	this.m_vemessage = new VEMessage(this);
	
	//Initialize Ambiguous List
	this.m_veambiguouslist = new VEAmbiguouslist(this);
	
	//Initialize VELatLong Factory
	//NOTE: Any VELatLongs created and returned by the API need to be created by this factory.
	//This will ensure that the VELatLong is encoded properly when in birdseye mode.
	var m_veLatLongFactory = new VELatLongFactory(new VELatLongFactorySpecFromMap(this));
	
	//Initialize VELatLong Decoder
	//NOTE: Any API method that accepts a VELatLong needs to run it through this decoder
	//to make sure any encoded VELatLongs are decoded so we can get Latitude and Longitude parameters.
	//When used this decoder will return a copy of the VELatLong. Do not assign it to the original reference
	//or you will expose decoded VELatLongs to the customer.
	var m_veLatLongDecoder = new VELatLongDecoder();
	
	//Get Window Defaults
	windowWidth = GetWindowWidth();
	windowHeight = GetWindowHeight();
	scrollbarWidth = GetScrollbarWidth ();

	//LoadMap
	this.LoadMap=function(veLatLong, zoomLevel, mapStyle, fixedMap)
	{
		if ((veLatLong != null) && (veLatLong != "undefined"))
		{
			VEValidator.ValidateObject(veLatLong, "veLatLong", VELatLong, "VELatLong");
			var veLatLongDecoded = m_veLatLongDecoder.Decode(veLatLong);
			this.initialLatitude = veLatLongDecoded.Latitude;
			this.initialLongitude = veLatLongDecoded.Longitude;
		}

		this.fixedMap = fixedMap;	
		this.initialZoomLevel = zoomLevel;
		this.initialMapStyle = mapStyle;
		
		this.veonmaploadevent = this.onLoadMap;
		
		//Clear any children
		this.mapelement.innerHTML = "";
		//Then assign loading text
		this.mapelement.innerHTML = "<table width=100% height=100%><tr valign=middle><td align=center valign=middle><h3>" + L_loading_text + "</h3></td></tr></table>";
		
		//Init Map
		this.InitializeMap();
		
		VE_Analytics.LogAPI('VE | APIs','VEMap - LoadMap');		
	}

	//_ReArrangeControls
	//Reorganizes controls on resize
	this._ReArrangeControls = function()
	{
		if(self.controlzIndexes != null &&
			self.controls != null &&
			(self.controlzIndexes.length == 
			self.controls.length))
		{
			var len = self.controls.length;
			for(var i=0;i<len;i++)
			{
				document.body.removeChild(self.controls[i]);
				var control = self.controls[i];
				control.style.top = self.controltops[i];
				control.style.left = self.controllefts[i];
				self._AddControlInner(control, 
							self.controlzIndexes[i]);
			}
		}
	}
	
	//_ClearView
	//Clears view for any popups
	this._ClearView = function()
	{
		VEMap.ValidateState();
		VEPushpin.Hide();
		self.m_vemessage.Hide();
	}

	//Internal method
	//SetViewport 
	this.SetViewport = function(lat1, lon1, lat2, lon2)
	{
		VEMap.ValidateState();
		try
		{	
			return this.vemapcontrol.SetViewport(lat1, lon1,lat2, lon2);
		}
		catch(ex)
		{
			throw ex;
			return;
		}
	}
	
	//GetCenter
	//  Returns center VELatLong
	this.GetCenter = function()
	{
		VEMap.ValidateState();
		var lat = this.vemapcontrol.GetCenterLatitude();
		var lon = this.vemapcontrol.GetCenterLongitude();				
		var oVELatLong = m_veLatLongFactory.CreateVELatLong(lat, lon);
		
		VE_Analytics.LogAPI('VE | APIs','VEMap - GetCenter');		
		return oVELatLong;
	}

	//GetMapView
	//   Returns a VELatLongRectangle containing the northwest and southeast VELatLongs of the bouding rectangle
	//   of the current view.
	this.GetMapView = function()
	{
		VEMap.ValidateState();
		try
		{
			var topLeftLatLong= this.vemapcontrol.PixelToLatLong(new Msn.VE.Pixel(0, 0));
			var bottomRightLatLong= this.vemapcontrol.PixelToLatLong(new Msn.VE.Pixel(this.GetWidth(), this.GetHeight()));
			var veLatLongRect = new VELatLongRectangle(
										m_veLatLongFactory.CreateVELatLong(topLeftLatLong.latitude, topLeftLatLong.longitude),
										m_veLatLongFactory.CreateVELatLong(bottomRightLatLong.latitude, bottomRightLatLong.longitude) );
										
			VE_Analytics.LogAPI('VE | APIs','VEMap - GetMapView');									
			return veLatLongRect;
		}
		catch(ex)
		{
			throw ex;
			return;
		}
	}

	//PixelToLatLong 
	//   converts an x,y coordinate and Zoom level to a VELatLong
	this.PixelToLatLong = function(x, y, zoomLevel)
	{
		VEMap.ValidateState();
		try
		{
			VEValidator.ValidateInt(x, "x");
			VEValidator.ValidateInt(y, "y");
			// call parseInt to avoid passing in string with integer values
			var vepixel = new Msn.VE.Pixel(parseInt(x), parseInt(y));
			var latlon = this.vemapcontrol.PixelToLatLong(vepixel, zoomLevel);			
			var oVELatLong = m_veLatLongFactory.CreateVELatLong(latlon.latitude, latlon.longitude);
			
			VE_Analytics.LogAPI('VE | APIs','VEMap - PixelToLatLong');			
			return oVELatLong;
		}
		catch(ex)
		{
			throw ex;
			return;		
		}			
	}

	//SetCenter
	//   sets the center of the map to the passed in VELatLong
	this.SetCenter=function(veLatLong)
	{
		VEMap.ValidateState();
		try
		{
			VEValidator.ValidateObject(veLatLong, "veLatLong", VELatLong, "VELatLong");
			var veLatLongDecoded = m_veLatLongDecoder.Decode(veLatLong);
			
			VE_Analytics.LogAPI('VE | APIs','VEMap - SetCenter');		
			return this.vemapcontrol.SetCenter(veLatLongDecoded.Latitude, veLatLongDecoded.Longitude);
		}
		catch(ex)
		{
			throw ex;
			return;
		}
	}

	//SetCenter
	//   sets the center of the map to the passed in VELatLong and zooms to the zoomLevel
	this.SetCenterAndZoom=function(veLatLong, zoomLevel)
	{
		VEMap.ValidateState();
		try
		{
			VEValidator.ValidateObject(veLatLong, "veLatLong", VELatLong, "VELatLong");
			VEValidator.ValidateInt(zoomLevel, "zoomLevel");
			var veLatLongDecoded = m_veLatLongDecoder.Decode(veLatLong);
			
			VE_Analytics.LogAPI('VE | APIs','VEMap - SetCenterAndZoom');		
			return this.vemapcontrol.SetCenterAndZoom(veLatLongDecoded.Latitude, veLatLongDecoded.Longitude, zoomLevel);
		}
		catch(ex)
		{
			throw ex;
			return;
		}
		
	}

	//IncludePointInView
	this.IncludePointInView=function(veLatLong)
	{
		VEMap.ValidateState();
		try
		{
			VEValidator.ValidateObject(veLatLong, "veLatLong", VELatLong, "VELatLong");
			var veLatLongDecoded = m_veLatLongDecoder.Decode(veLatLong);
			
			VE_Analytics.LogAPI('VE | APIs','VEMap - IncludePointInView');		
			return this.vemapcontrol.IncludePointInViewport(veLatLongDecoded.Latitude, veLatLongDecoded.Longitude);
		}
		catch(ex)
		{
			throw ex;
			return;		
		}
	}

	//LatLongToPixel
	this.LatLongToPixel=function(veLatLong, zoomLevel)
	{
		VEMap.ValidateState();
		try
		{
			VEValidator.ValidateObject(veLatLong, "veLatLong", VELatLong, "VELatLong");
			var veLatLongDecoded = m_veLatLongDecoder.Decode(veLatLong);
			var latlong = new Msn.VE.LatLong(veLatLongDecoded.Latitude, veLatLongDecoded.Longitude);
			
			VE_Analytics.LogAPI('VE | APIs','VEMap - LatLongToPixel');			
			return this.vemapcontrol.LatLongToPixel(latlong, zoomLevel);		
		}
		catch(ex)
		{
			throw ex;
			return;		
		}	
	}

	//PanToLatLong
	this.PanToLatLong=function(veLatLong)
	{
		VEMap.ValidateState();
		try
		{
			VEValidator.ValidateObject(veLatLong, "veLatLong", VELatLong, "VELatLong");
			var veLatLongDecoded = m_veLatLongDecoder.Decode(veLatLong);
			
			VE_Analytics.LogAPI('VE | APIs','VEMap - PanToLatLong');		
			return this.vemapcontrol.PanToLatLong(veLatLongDecoded.Latitude, veLatLongDecoded.Longitude);
		}
		catch(ex)
		{
			throw ex;
			return;		
		}		
	}

	//SetMapView
	//   accepts VELatLongRectangle
	//   accepts array of VELatLongs
	//   accepts array of VEPolylines
	this.SetMapView=function(arrObject)
	{
		VEMap.ValidateState();
		try
		{		
			VEValidator.ValidateNonNull(arrObject, "arrObject");
			
			var llArray = new Array();
			//Check to see if the passed in object is a VELatLongRectangle
			if (VELatLongRectangle.prototype.isPrototypeOf(arrObject))
			{
				var ll = new Object();
				ll.latitude = arrObject.TopLeftLatLong.Latitude;
				ll.longitude = arrObject.TopLeftLatLong.Longitude;
				llArray.push(ll);
				
				var ll = new Object();
				ll.latitude = arrObject.BottomRightLatLong.Latitude;
				ll.longitude = arrObject.BottomRightLatLong.Longitude;
				llArray.push(ll);
			}
			else
			{
				//Check to see if the passed in object is an array of VELatLongs
				VEValidator.ValidateNonNull(arrObject[0], "arrObject[0]");
				if (VELatLong.prototype.isPrototypeOf(arrObject[0]))
				{	
					for(var i=0;i<arrObject.length;i++)
					{
						var veLatLongDecoded = m_veLatLongDecoder.Decode(arrObject[i]);
						var ll = new Object();
						ll.latitude = veLatLongDecoded.Latitude;
						ll.longitude = veLatLongDecoded.Longitude;
						llArray.push(ll);
					}			
				}
				//Check to see if the passed in object is an array of VEPolylines
				else if (VEPolyline.prototype.isPrototypeOf(arrObject[0]))
				{
					for(var i=0;i<arrObject.length;i++)
					{
						var vePolyline = arrObject[i];
						var polyLatLongs = vePolyline.GetLatLongs();
						for (var j=0; j < polyLatLongs.length; j++)
						{
							var ll = new Object();
							ll.latitude = polyLatLongs[j].Latitude;
							ll.longitude = polyLatLongs[j].Longitude;
							llArray.push(ll);
						}			
					}
				}
			}

			VE_Analytics.LogAPI('VE | APIs','VEMap - SetMapView');		
			return this.vemapcontrol.SetBestMapView(llArray);
		}
		catch(ex)
		{
			throw ex;
			return;		
		}					
	}

	//AddPushpin
	this.AddPushpin=function(vePushpin)
	{
		VEMap.ValidateState();
		VEValidator.ValidateObject(vePushpin, "vePushpin", VEPushpin, "VEPushpin");
		//See if there are conflicts
		var len = this.pushpins.length;
		for (var i=0; i<len; i++)
		{
			var p = this.pushpins[i];
			if(p.ID == vePushpin.ID)
			{
				throw new VEException("VEMap:AddPushpin", "err_invalidpushpinid", L_invalidpushpinid_text);
			}
		}
		
		//Add to the collection
		this.pushpins.push(vePushpin);
		
		//Tell the pushpin which map instance it is being added to
		vePushpin._SetMapInstance(this);
		var veLatLongDecoded = m_veLatLongDecoder.Decode(vePushpin.LatLong);
		this.vemapcontrol.AddPushpin(vePushpin.ID,
									veLatLongDecoded.Latitude,
									veLatLongDecoded.Longitude,
									25,
									25,
									"VEAPI_Pushpin",
									vePushpin.GetContent(),
									Msn.VE.API.Globals.vepushpinpanelzIndex -1);
									
		VE_Analytics.LogAPI('VE | APIs','VEMap - AddPushpin');			
	}

	//Disambiguate Callback 
	this._DisambiguateCallback = "VEMap._GetMapFromGUID(" + this.GUID + ")._Disambiguate";
	
	//_Disambiguate
	this._Disambiguate = function(a, lat1, lon1, lat2, lon2)
	{
		VEMap.ValidateState();
		var el = document.getElementById(this.ID + "_vewhereinput");
		if(el)
		{
			el.value = unescape(a);
		}
		this.vemapcontrol.SetViewport(lat1, lon1, lat2, lon2);
		
		if(this.lastwhatsearch != null && this.lastwhatsearch.length > 0)
			this.Find(this.lastwhatsearch, null, 1, this.m_vesearchmanager.vesearchcallback);
	}
	
	//This is an internal method
	this._DoFind = function()
	{
		VEMap.ValidateState();
		try
		{		
			this.lastwhatsearch = document.getElementById(this.ID + "_vewhatinput").value;
			this.lastwheresearch = document.getElementById(this.ID + "_vewhereinput").value;
			this.m_vesearchmanager.Find(this.lastwhatsearch, this.lastwheresearch, null, this._DisambiguateCallback, 1);
		}
		catch(e)
		{
			this.ShowMessage(e.message);
		}
	}
	
	//This is an internal method
	this.UpdateAmbiguousList = function(a)
	{
		this.m_vesearchmanager.UpdateAmbiguousList(a);
	}
	
}

//_GetMapFromGUID
//   static method returns a map instance from its GUID
VEMap._GetMapFromGUID = function(mapGUID)
{
	if ((Msn.VE.API.Globals.vemapinstances[mapGUID] == null) ||
		(Msn.VE.API.Globals.vemapinstances[mapGUID] == "undefined"))
	{
		throw new VEException("VEMap:_GetMapFromGUID", "err_notinitialized", L_notinitialized_text);
		return;
	}
	return Msn.VE.API.Globals.vemapinstances[mapGUID];
}

VEMap.prototype.InitializeMap = function()
{
	RegisterNamespaces("Msn.VE");
	//Clear any children
	this.mapelement.innerHTML = "";
	
	//Now initialize the map control
	this.mapelement.style.overflow = "hidden";
	
	
	//Only set the height and width if there is not a class style defined.
	//This lets developers choose to use CSS to load map styles.
	//The class style may set the height and width.
	//If the class style sets the height and width, style.height and style.width
	//properties still may not be set. Also, we cannot just check for currentStyle.height and
	//currentStyle.width because Firefox sets those even when a DIV has no style.
	//Therefore, if there is a class style defined, don't change the width/height.
	if ((this.mapelement.className == null) ||
		(this.mapelement.className == "undefined") ||
		(this.mapelement.className == ""))
	{
		//If height and width are not defined, set defaults		
		if(this.mapelement.style == null ||
			this.mapelement.style.height == null ||
			this.mapelement.style.height == "undefined" ||
			this.mapelement.style.height == "")
		{
			this.mapelement.style.height = Msn.VE.API.Globals.vemapheight + "px";
		}
		
		if(this.mapelement.style == null ||
			this.mapelement.style.width == null ||
			this.mapelement.style.width == "undefined" ||
			this.mapelement.style.width == "")
		{
			this.mapelement.style.width = Msn.VE.API.Globals.vemapwidth + "px";
		}
	}

	//Now assign parameters	
	var params = new Object();	
	
	//Set latitude
	if(this.initialLatitude != null && 
		this.initialLatitude != "undefined")
		params.latitude = this.initialLatitude;
	else
		params.latitude = Msn.VE.API.Globals.vemaplatitude;

	//Set longitude
	if(this.initialLongitude != null && 
		this.initialLongitude != "undefined")
		params.longitude = this.initialLongitude;
	else
		params.longitude = Msn.VE.API.Globals.vemaplongitude;

	//Set zoom level
	if(this.initialZoomLevel != null &&
		this.initialZoomLevel != "undefined")
		params.zoomlevel = this.initialZoomLevel;
	else
		params.zoomlevel = Msn.VE.API.Globals.vemapzoom;		

	//Set mapstyle
	if(this.initialMapStyle != null &&
		this.initialMapStyle != "undefined")
		params.mapstyle  = this.initialMapStyle;
	else
		params.mapstyle  = Msn.VE.API.Globals.vemapstyle;	

	this.m_dashboardId = this.ID + "_dashboard";

	if(this.fixedMap != true)
	{
		params.showDashboard = true;
		params.dashboardSize = Msn.VE.DashboardSize.Normal;
		params.dashboardX = 5;
		params.dashboardY = 5;
		params.dashboardId = this.m_dashboardId;
		params.showScaleBar = true;
	}
	
	params.obliqueEnabled = true;
	params.obliqueUrl = Msn.VE.API.Constants.imageryurl;
		
	if(this.fixedMap == true)
	{
		params.fixedView = true;
	}

	params.disableLogo = false;

	// Create the map control
	this.vemapcontrol = new Msn.VE.MapControl(this.mapelement, params);
	this.vemapcontrol.Init();
	
	if(!this.fixedMap == true)
	{
		this.vemapcontrol.AttachEvent("onchangeview",this._ClearView);
	}
	
	//Initialize Graphics
	RegisterNamespaces("Msn.Drawing");	

	//Initialize Graphics Manager
	this.m_vegraphicsmanager = new VEGraphicsManager(this);
	this.m_vegraphicsmanager.Initialize();
	
	//Initialize Layer Manager
	this.m_velayermanager = new VELayerManager(this);
	this._lm = this.m_velayermanager;  //shorthand internal alias for internal server-side callbacks
		
	//Initialize Help panel
	if ((document.getElementById('help') == null) ||
		(document.getElementById('help') == "undefined"))
	{
		VE_Help.CreateHelpPanel();
	}

	//Fire on map load event
	if(this.veonmaploadevent)
	{
		this.veonmaploadevent(this);
	}
	
	//Add Onresize
	this.controlzIndexes = new Array();
	this.controls = new Array();	
	this.controltops = new Array();
	this.controllefts = new Array();	
	this.vemapcontrol.AttachEvent("onresize",this._ReArrangeControls);	
	
	//Wireup Cleanup 
	window.onunload=this.Dispose;
}

VEMap.prototype.Dispose = function()
{
	try
	{
		VEMap.ValidateState();
		if(this.vemapcontrol != null)
		{
			this.vemapcontrol.DetachEvent("onchangeview", this._ClearView);
			this.vemapcontrol.DetachEvent("onresize", this._ReArrangeControls);
		}
		
		this.vemapcontrol.DetachEvent("onclick", VEPushpin.Hide);

		//Kill all controls for this map
		var len = this.controlzIndexes.length;
		for(var i=0;i<len;i++)
		{
			this.controlzIndexes.pop();
		}
		var len = this.controls.length;
		for(var i=0;i<len;i++)
		{
			try
			{
				document.body.removeChild(this.controls[i]);
			}
			catch(error)
			{
			}
			this.controls[i] = null;
			this.controltops[i] = null;
			this.controllefts[i] = null;
		}
		this.controls = null;
		this.controlzIndexes = null;
		
		this.m_vegraphicsmanager.Dispose();
		this.m_velayermanager.Dispose();
		this.m_vemessage.Dispose();
		this.m_veambiguouslist.Dispose();

		//Remove the map from the static instance table
		Msn.VE.API.Globals.vemapinstances[this.GUID]=null;

		this.veonmaploadevent = null;
		this.veloadingdiv = null;
		
		for (var i=0; i < this.pushpins.length; ++i)
		{
			if ((this.pushpins[i] != null) && (this.pushpins[i] != "undefined"))
			{
				this.pushpins[i].Dispose();
				this.pushpins[i] = null;
			}
		}
		this.pushpins = null;

		this.ID = null;
		this.vemapcontrol.Destroy();

		//This should be the last one to dispose		
		this.vemapcontrol = null;	

		//If all map instances are gone, dispose the globals/statics
		var iMapInstanceCount = 0;
		for (var v in Msn.VE.API.Globals.vemapinstances)
		{
		   if (Msn.VE.API.Globals.vemapinstances[v] != null)
		   {
		      ++iMapInstanceCount;
		   }
		}
		if (iMapInstanceCount == 0)
		{
			//Destroy Help panel
			var e=document.getElementById('help');
			if ((e != null) && (e != "undefined"))
			{
				document.body.removeChild(e);
			}

			//Dispose the single VEPushpin ERO
			//Don't do this for now - it can cause JS Errors when trying to recreate the ERO
			//after destroying and re-creating a map and adding pins
			//VEPushpin.DisposeERO();
			
			Msn.VE.API.Globals.Dispose();
			
		}
	}
	catch(err)
	{
	}
}

VEMap.ValidateState = function()
{
	//Since map instances are no longer static, just return true for now.
	//In future, can validate individual map instances
	return true;
}

VEMap.prototype.DeleteAllPushpins=function()
{
	VEMap.ValidateState();
	VEPushpin.Hide();

	if(this.pushpins != null &&
	   this.pushpins != "undefined")
	{
		var len = this.pushpins.length;
		for(var i=0;i<len;i++)
		{
			var p=this.pushpins.pop();
			
			//Only dispose the pin if it is not in a layer. If it is in a layer,
			//the LayerManager will handle the cleanup.
			if (!p.IsInLayer)
			{
				p.Dispose();
			}
		}
	}
	
	VE_Analytics.LogAPI('VE | APIs','VEMap - DeleteAllPushpins');		
	return this.vemapcontrol.ClearPushpins();
}

VEMap.prototype.GetMapStyle=function()
{
	VEMap.ValidateState();
	VE_Analytics.LogAPI('VE | APIs','VEMap - GetMapStyle');	
	return this.vemapcontrol.GetMapStyle();
}

//GetBirdseyeScene
//   If in Birdseye mode,returns a VEBirdseyeScene object
VEMap.prototype.GetBirdseyeScene=function()
{
	VEMap.ValidateState();
	var veBirdseyeScene = null;
	var obliqueScene = this.vemapcontrol.GetObliqueScene();
	if ((obliqueScene != null) && (obliqueScene != "undefined"))
	{
		veBirdseyeScene = new VEBirdseyeScene(obliqueScene);
	}
	
	VE_Analytics.LogAPI('VE | APIs','VEMap - GetBirdseyeScene');	
	return veBirdseyeScene;
}

//IsBirdseyeAvailable
//   Returns true if Birdseye mode is possible o=in the current view. False otherwise.
VEMap.prototype.IsBirdseyeAvailable=function()
{
	VEMap.ValidateState();
	
	VE_Analytics.LogAPI('VE | APIs','VEMap - IsBirdseyeAvailable');
	return this.vemapcontrol.IsObliqueAvailable();
}

//SetBirdseyeOrientation
//   Sets the Birdseye orientation from the passed in VEOrientation object
VEMap.prototype.SetBirdseyeOrientation=function(orientation)
{
	VEMap.ValidateState();
	
	VE_Analytics.LogAPI('VE | APIs','VEMap - SetBirdseyeOrientation');	
	return this.vemapcontrol.SetObliqueOrientation(orientation);
}

//SetBirdseyeScene
//   Sets the Birdseye orientation from the passed in VEOrientation object
VEMap.prototype.SetBirdseyeScene=function(id)
{
	VEMap.ValidateState();
	
	VE_Analytics.LogAPI('VE | APIs','VEMap - SetBirdseyeScene');	
	return this.vemapcontrol.SetObliqueScene(id);
}

VEMap.prototype.GetZoomLevel=function()
{
	VEMap.ValidateState();
	
	VE_Analytics.LogAPI('VE | APIs','VEMap - GetZoomLevel');	
	return this.vemapcontrol.GetZoomLevel();
}

VEMap.prototype.Pan=function(deltaX, deltaY)
{
	VEMap.ValidateState();
	try
	{
		VEValidator.ValidateInt(deltaX, "deltaX");
		VEValidator.ValidateInt(deltaY, "deltaY");
		
		VE_Analytics.LogAPI('VE | APIs','VEMap - Pan');	
		// call parseInt to avoid passing in string with integer values
		return this.vemapcontrol.PanMap(parseInt(deltaX), parseInt(deltaY));
	}
	catch(ex)
	{
		throw ex;
		return;		
	}		
}

VEMap.prototype.DeletePushpin=function(id)
{
	VEMap.ValidateState();
	try
	{
		var len = this.pushpins.length;
		for (var i=0; i<len; i++)
		{
			var p = this.pushpins[i];
			if ((p != null) && (p.ID == id))
			{
				//Only dispose the pin if it is not in a layer. If it is in a layer,
				//the LayerManager will handle the cleanup.
				if (!p.IsInLayer)
				{
					p.Dispose();
				}
				//Found the pushpin. Remove it.
				this.pushpins.splice(i, 1);
				
				VE_Analytics.LogAPI('VE | APIs','VEMap - DeletePushpin');	
				return this.vemapcontrol.RemovePushpin(id);
			}
		}
		//Pushpin not found, throw an exception.
		throw new VEException("VEMap:DeletePushpin", "err_invalidpushpinid", L_invalidpushpinid_text);
	}
	catch(ex)
	{	
		throw ex;
		return;
	}
}

VEMap.prototype.Resize=function(width, height)
{
	VEMap.ValidateState();
	try
	{
		VEValidator.ValidateInt(width, "width");
		VEValidator.ValidateInt(height, "height");
		return this.vemapcontrol.Resize(width, height);	
	}
	catch(ex)
	{
		throw ex;
		return;		
	}				
}

VEMap.prototype.SetMapStyle=function(mapStyle)
{
	VEMap.ValidateState();
	try
	{
		VEValidator.ValidateMapStyle(mapStyle, "mapStyle");
		this.vemapcontrol.SetMapStyle(mapStyle);
		
		VE_Analytics.LogAPI('VE | APIs','VEMap - SetMapStyle');	
	}
	catch(ex)
	{
		throw ex;
		return;
	}
	
}

VEMap.prototype.SetZoomLevel=function(zoomLevel)
{
	VEMap.ValidateState();
	try
	{
		VEValidator.ValidateInt(zoomLevel, "zoomLevel");
		
		VE_Analytics.LogAPI('VE | APIs','VEMap - SetZoomLevel');	
		return this.vemapcontrol.SetZoom(zoomLevel);	
	}
	catch(ex)
	{
		throw ex;
		return;		
	}			
}

VEMap.prototype.ZoomIn=function()
{
	VEMap.ValidateState();
	
	VE_Analytics.LogAPI('VE | APIs','VEMap - ZoomIn');	
	this.vemapcontrol.ZoomIn();
}

VEMap.prototype.ZoomOut=function()
{
	VEMap.ValidateState();
	
	VE_Analytics.LogAPI('VE | APIs','VEMap - ZoomOut');
	this.vemapcontrol.ZoomOut();
}

//AttachEvent
VEMap.prototype.AttachEvent=function(eventname, eventhandler)
{
	VEMap.ValidateState();
	try
	{
		VEValidator.ValidateNonNull(eventname, "eventname");				
		VEValidator.ValidateNonNull(eventhandler, "eventhandler");				
		this.vemapcontrol.AttachEvent(eventname, eventhandler);
		
		VE_Analytics.LogAPI('VE | APIs','VEMap - AttachEvent');	
	}
	catch(ex)
	{
		throw ex;
		return;
	}
}

//DetachEvent
VEMap.prototype.DetachEvent=function(eventname, eventhandler)
{
	VEMap.ValidateState();
	try
	{
		VEValidator.ValidateNonNull(eventname, "eventname");
		VEValidator.ValidateNonNull(eventhandler, "eventhandler");
		this.vemapcontrol.DetachEvent(eventname, eventhandler);
		
		VE_Analytics.LogAPI('VE | APIs','VEMap - DetachEvent');	
	}
	catch(ex)
	{
		throw ex;
		return;
	}
}

VEMap.prototype.FindLocation=function(where, findCallback)
{
	VEMap.ValidateState();
	this.lastwhatsearch = null;
	this.lastwheresearch = where;
	this.m_vesearchmanager.Find(null, where, findCallback, this._DisambiguateCallback, 1);
	
	VE_Analytics.LogAPI('VE | APIs','VEMap - FindLocation');		
}

VEMap.prototype.FindNearby=function(what, startPageIndex, findCallback)
{
	VEMap.ValidateState();	
	this.lastwhatsearch = what;
	this.lastwheresearch = null;
	this.m_vesearchmanager.Find(what, null, findCallback, this._DisambiguateCallback, startPageIndex);
	
	VE_Analytics.LogAPI('VE | APIs','VEMap - FindNearby');	
}

VEMap.prototype.Find=function(what, where, startPageIndex, findCallback)
{
	VEMap.ValidateState();	
	this.lastwhatsearch = what;
	this.lastwheresearch = where;
	this.m_vesearchmanager.Find(what, where, findCallback, this._DisambiguateCallback, startPageIndex);
	
	VE_Analytics.LogAPI('VE | APIs','VEMap - Find');	
}

VEMap.prototype.GetRoute=function(start, end, distanceUnit, routeType, routeCallback)
{
	VEMap.ValidateState();
	this.m_vedirectionsmanager.GetDrivingDirections(start, end, distanceUnit, routeType, routeCallback);
	
	VE_Analytics.LogAPI('VE | APIs','VEMap - GetRoute');	
}

VEMap.prototype.DeleteRoute=function()
{
	VEMap.ValidateState();
	this.m_vedirectionsmanager.ClearRoute();
	
	VE_Analytics.LogAPI('VE | APIs','VEMap - DeleteRoute');
}

VEMap.prototype.ShowMessage=function(message)
{
	VEMap.ValidateState();
	this.m_vemessage.Show(message);
	
	VE_Analytics.LogAPI('VE | APIs','VEMap - ShowMessage');
}

VEMap.prototype.GetHeight = function()
{
	VEMap.ValidateState();
	//Get the height from the style height.
	var height=parseInt(this.mapelement.style.height.replace("px", ""));
	//If the style height is not available, get the height from offsetHeight
	if (isNaN(height))
	{
		height = this.mapelement.offsetHeight;
	}
	return height;
}

VEMap.prototype.GetWidth = function()
{
	VEMap.ValidateState();
	//Get the width from the style width.
	var width=parseInt(this.mapelement.style.width.replace("px", ""));
	//If the style width is not available, get the width from offsetWidth
	if (isNaN(width))
	{
		width = this.mapelement.offsetWidth;
	}
	return width;
}

//GetLeft
VEMap.prototype.GetLeft = function()
{
	var curleft = 0;
	if(this.mapelement.offsetLeft)
	{
		curleft = this.mapelement.offsetLeft;
	}
	else if (this.mapelement.offsetParent)
	{
		while (this.mapelement.offsetParent)
		{
			window.status = curleft;
			curleft += this.mapelement.offsetLeft
			this.mapelement = this.mapelement.offsetParent;
		}
	}
	else if (this.mapelement.x)
	{
		curleft += this.mapelement.x;
	}
	
	VE_Analytics.LogAPI('VE | APIs','VEMap - GetLeft');	
	return curleft;
}

//GetTop
VEMap.prototype.GetTop = function()
{
	var curtop = 0;
	if(this.mapelement.offsetTop)
	{
		curtop = this.mapelement.offsetTop;
	}
	else if (this.mapelement.offsetParent)
	{
		while (this.mapelement.offsetParent)
		{
			this.mapelement.offsetTop
			this.mapelement = this.mapelement.offsetParent;
		}
	}
	else if (this.mapelement.y)
	{
		curtop += this.mapelement.y;
	}
	
	VE_Analytics.LogAPI('VE | APIs','VEMap - GetTop');	
	return curtop;
}

VEMap.prototype.SetFindResultsPanel=function(elementID)
{
	VEMap.ValidateState();
	if(elementID == null || elementID == "undefined")
		throw new VEException("VEMap:SetFindResultsPanel", "err_invalidelement", L_invalidelement_text);
	var el = document.getElementById(elementID);
	if(el == null || el == "undefined")
		throw new VEException("VEMap:SetFindResultsPanel", "err_invalidelement", L_invalidelement_text);
	this.searchelement = elementID;		
}

VEMap.prototype._AddControlInner=function(control, zIndex)
{
	//Set absolute positioning
	control.style.position = "absolute";
	//200s are user level
	control.style.zIndex =201;
	//Adjust top and left according to the map
	var top = this.GetTop();
	var left = this.GetLeft();
	if(!control.style.top)
	{
		control.style.top = "0px";
	}
	if(!control.style.left)
	{
		control.style.left = "0px";
	}
	var tempTop = control.style.top;
	var tempLeft = control.style.left;
	if(isNaN(tempTop))
	{
		tempTop = tempTop.toString().toLowerCase();
	}
	if(isNaN(tempLeft))
	{
		tempLeft = tempLeft.toString().toLowerCase();
	}	
	top += parseInt(tempTop.replace("px"));
	left += parseInt(tempLeft.replace("px"));	
	control.style.top = top + "px";
	control.style.left = left + "px";			
	//Append to map
	document.body.appendChild(control);
}

VEMap.prototype.AddControl=function(control, zIndex)
{
	VEMap.ValidateState();
	if(control == null || control == "undefined")
	{
		throw new VEException("VEMap:AddControl", "err_invalidelement", L_invalidelement_text);		
	}
	
	//If the controls container has not been initialized, then throw an exception.
	if ((this.controls == null) || (this.controls == "undefined"))
	{
		throw new VEException("VEMap:AddControl", "err_notinitialized", L_notinitialized_text);
	}
	
	//Cache it so that we can reuse them on resize
	this.controlzIndexes.push(zIndex);
	this.controls.push(control);
	this.controltops.push(control.style.top);
	this.controllefts.push(control.style.left);
	this._AddControlInner(control, zIndex);
	
	VE_Analytics.LogAPI('VE | APIs','VEMap - AddControl');	
}

VEMap.prototype.HideFindControl=function()
{
	VEMap.ValidateState();	
	var el = document.getElementById(this.ID + "_vefindcontrolinput");
	if(el != null && el != "undefined")
	{
		el.style.display = "none";
	}	
	VE_Analytics.LogAPI('VE | APIs','VEMap - HideFindControl');	
}

VEMap.prototype.ShowDashboard=function()
{
	this._ToggleDashboard(true);
	VE_Analytics.LogAPI('VE | APIs','VEMap - ShowDashboard');	
}

VEMap.prototype.HideDashboard=function()
{
	this._ToggleDashboard(false);
	VE_Analytics.LogAPI('VE | APIs','VEMap - HideDashboard');	
}

VEMap.prototype._ToggleDashboard=function(flag)
{
	VEMap.ValidateState();	
	
	var el = document.getElementById(this.m_dashboardId);
	if(el != null)
	{
		if(flag)			
			el.style.display = "block";
		else
			el.style.display = "none";			

		return;		
	}
	
	//look for dashboard child and hide it
	if(this.mapelement.childNodes != null &&
	   this.mapelement.childNodes != "undefined")
	{
		var len = this.mapelement.childNodes.length;
		var el = null;
		for(var i=0;i<len;i++)
		{
			el = this.mapelement.childNodes[i];
			if(el == null)
				continue;
			if(el.className != null && el.className == "Dashboard Dashboard_normal")
			{
				if(flag)			
					el.style.display = "block";
				else
					el.style.display = "none";			
				break;
			}
		}
	}	
}

VEMap.prototype.ShowFindControl=function(disableWhat)
{
	VEMap.ValidateState();
	//see if it is already there
	var findControlId = this.ID + "_vefindcontrolinput";
	var el = document.getElementById(findControlId);
	if(el != null && el != "undefined")
	{
		el.style.display = "block";
	}
	else
	{
		//Create what & where box
		el = document.createElement("div");
		el.className = "VE_FindControl";
		el.id = findControlId;
		el.style.zIndex = 199;
		el.style.position = "absolute";
		el.style.top = "5px";
		el.style.left = "195px";		
		el.style.padding = "5px";

		var whatInputContainerId = this.ID + "_vewhatinputcontainer";
		var whatInputId = this.ID + "_vewhatinput";
		var whereInputId = this.ID + "_vewhereinput";
		
		el.innerHTML = "<div class=VE_WhatControl id='" + whatInputContainerId + "'>" + L_what_text + " " + "<input id=\"" +  whatInputId + "\" type=\"text\" name=\"vewhatinput\" size=\"25\" onfocus=\"this.select()\" onblur=\"this.value = this.value\" />" + "</div>" + 
		"<div class=VE_WhereControl>" + L_where_text + " " + "<input id=\"" + whereInputId + "\" type=\"text\" name=\"vewhereinput\" size=\"25\" style=\"color: #333333\"" + "ondrop=\"this.value='';this.style.color='black';SelectText(this,0,0);\" />" + "</div>" + "<div class=VE_FindButton>" + "<button id=\"searchbttn\" onclick=\"VEMap._GetMapFromGUID(" + this.GUID + ")._DoFind();\" type=\"submit\" name=\"submit\" value=\"Local Search\">" + L_find_text + "</button>" + 
		"</div>";

		this.AddControl(el);

		if(disableWhat)
		{
			var whatelement = document.getElementById(whatInputContainerId);
			if(whatelement)
			{
				whatelement.style.display = "none";
			}
		}	
	}
	VE_Analytics.LogAPI('VE | APIs','VEMap - ShowFindControl');
}

VEMap.prototype.Clear=function()
{
	VEMap.ValidateState();
	this._ClearView();
	
	//Delete pushpins
	this.DeleteAllPushpins();
	
	//Clear Routefs
	this.m_vedirectionsmanager.RemoveRouteHighLight();
	
	//Remove polylines
	this.m_vegraphicsmanager.Clear();
	
	//Clear Disambig popup too
	this.m_veambiguouslist.Hide();
	
	this.m_vemessage.Hide();
	
	VE_Analytics.LogAPI('VE | APIs','VEMap - Clear');
}

VEMap.prototype._ShowLoading=function()
{
	VEMap.ValidateState();

	if(!this.veloadingdiv)
	{
		this.veloadingdiv = document.createElement("div");
		this.veloadingdiv.className = "VE_Network_Loading";
		this.veloadingdiv.style.top = "75px";
		this.veloadingdiv.style.left = "80px";
		this.veloadingdiv.innerHTML = L_loading_text;
		this.AddControl(this.veloadingdiv, 202);
	}

	if(this.veloadingdiv.style.display != "block")
		this.veloadingdiv.style.display = "block";
	else
		this.veloadingdiv.style.display = "none";				
}

//Draw line method
//AddPolyline
//   params:
//      vePolyLine      VEPolyline
VEMap.prototype.AddPolyline=function(vePolyLine)
{
	VEMap.ValidateState();
	this.m_vegraphicsmanager.DrawLine(vePolyLine);
	
	VE_Analytics.LogAPI('VE | APIs','VEMap - AddPolyline');
}

VEMap.prototype.DeletePolyline=function(id)
{
	VEMap.ValidateState();
	this.m_vegraphicsmanager.RemoveLinebyId(id);
	
	VE_Analytics.LogAPI('VE | APIs','VEMap - DeletePolyline');
}

VEMap.prototype.DeleteAllPolylines=function()
{
	VEMap.ValidateState();
	this.m_vegraphicsmanager.RemoveAllLines();
	
	VE_Analytics.LogAPI('VE | APIs','VEMap - DeleteAllPolylines');	
}

VEMap.prototype.AddLayer=function(veLayerSpec)
{
	VEMap.ValidateState();
	VEValidator.ValidateObject(veLayerSpec, "veLayerSpec", VELayerSpecification, "VELayerSpecification");
	this.m_velayermanager.AddLayer(veLayerSpec);
	
	VE_Analytics.LogAPI('VE | APIs','VEMap - AddLayer');	
}

VEMap.prototype.ShowLayer=function(id)
{
	VEMap.ValidateState();
	this.m_velayermanager.Show(id);
	
	VE_Analytics.LogAPI('VE | APIs','VEMap - ShowLayer');	
}

VEMap.prototype.HideLayer=function(id)
{
	VEMap.ValidateState();
	VEPushpin.Hide();
	this.m_velayermanager.Hide(id);
	
	VE_Analytics.LogAPI('VE | APIs','VEMap - HideLayer');	
}

VEMap.prototype.DeleteLayer=function(id)
{
	VEMap.ValidateState();
	this.m_velayermanager.DeleteLayerById(id);
	this.m_vegraphicsmanager.Clear();
	this.m_vegraphicsmanager.Draw();
	
	VE_Analytics.LogAPI('VE | APIs','VEMap - DeleteLayer');
}

VEMap.GetVersion = function()
{
	VE_Analytics.LogAPI('VE | APIs','VEMap - GetVersion');	
	return Msn.VE.API.Globals.vecurrentversion;
}
