//------------------------------------------------------------------------------
// API/Pushpin.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: Cthota, keithkin
//
// <header>	$Pushpin$</header>
//------------------------------------------------------------------------------

function VEPushpin(pinId, veLatLong, iconUrl, title, details, iconStyle, titleStyle, detailsStyle)
{
	VEValidator.ValidateNonNull(pinId, "pinId");	
	VEValidator.ValidateNonNull(veLatLong, "veLatLong");	
	var self=this;	
	this.IsInLayer = false;
	this.ID=pinId; 
	this.LatLong=veLatLong;
	this.Title = title;
	if(iconUrl == null || iconUrl == "undefined" || iconUrl.length == 0)
		this.Iconurl=Msn.VE.API.Constants.iconurl;
	else
		this.Iconurl=iconUrl;	
	this.Details=details;	
	
	if(iconStyle == null || iconStyle == "undefined" || iconStyle.length == 0)
	{
		this.IconStyle = "";
	}
	else
	{
		this.IconStyle = iconStyle;
	}
	
	
	if(titleStyle == null || titleStyle == "undefined" || titleStyle.length == 0)
	{
		this.TitleStyle = "VE_Pushpin_Popup_Title";
	}
	else
	{
		this.TitleStyle = titleStyle;
	}

	if(detailsStyle == null || detailsStyle == "undefined" || detailsStyle.length == 0)
	{
		this.DetailsStyle = "VE_Pushpin_Popup_Body";
	}
	else
	{
		this.DetailsStyle = detailsStyle;
	}

	//Set the window ERO instance for pushpin popups
	if ((window.ero == null) || (window.ero == "undefined"))
	{
		window.ero = ERO.getInstance();
	}
}

VEPushpin.ShowDetailOnMouseOver = true;
VEPushpin.OnMouseOverCallback = null;

//Dispose
VEPushpin.prototype.Dispose = function()
{
	this.DetailsStyle == null;
	this.TitleStyle = null;
	this.IconStyle = null;
	this.Details = null;
	this.IconUrl = null;
	this.Title = null;
	this.LatLong = null;
	this.ID = null;
	this.m_vemapcontrol = null;
	this.m_vemap = null;
}

//VEPushpin.Hide
//isImmediate is an optional parameter. When set to true, the ERO hides immediately.
//When set to false or not specified, the ERO hides on a delay.
VEPushpin.Hide = function(isImmediate)
{
	VEMap.ValidateState();

	//Hide the ERO
	if (window.ero != null)
	{
		if ((isImmediate == "undefined") || (isImmediate == null))
		{
			isImmediate = false;
		}
		window.ero.hide(isImmediate);
	}
}

VEPushpin.Show = function(mapguid, pinId, latitude, longitude, title, content, titleStyle, contentStyle)
{
	var mapInstance = VEMap._GetMapFromGUID(mapguid);
	if ((mapInstance == null) || (mapInstance == "undefined"))	
		return;  //TODO throw exception here

	VEMap.ValidateState();
	var currentX = mapInstance.vemapcontrol.GetX(longitude) + mapInstance.GetLeft();
	var currentY = mapInstance.vemapcontrol.GetY(latitude) + mapInstance.GetTop();
	
	if(VEPushpin.ShowDetailOnMouseOver)
	{
		//Get the pushpin DOM element
		var e=document.getElementById(pinId+ "_" + mapInstance.GUID);
		if ((e != null) &&(e != "undefined"))
		{
			//Display the ERO pop-up
			window.ero.setBoundingArea(new Microsoft.Web.Geometry.Point(0, 0), new Microsoft.Web.Geometry.Point(document.body.clientWidth, document.body.clientHeight));

			var eroContent = '<p>';
			if (title.length > 0)
			{
				eroContent += '<div class="' + titleStyle + '">' + unescape(title) + '</div>';
			}
			
			if (content.length >0)
			{
				eroContent += '<div class="' + contentStyle + '">' + unescape(content) + '</div>';
			}
			
			//For Firefox/Mozilla add a few extra space so the ERO beak is positioned properly
			//if the title or content is empty. Ideally we should fix this with CSS, but for 
			//now this is the fix.
			if (!document.all && ((title.length == 0) || (content.length == 0)))
			{
				eroContent += '<br/><br/>';
			}
			eroContent += '</p>';
			window.ero.setContent(eroContent);
			window.ero.display(e);
		}
	}
	
	if(VEPushpin.OnMouseOverCallback != null)
	{
		VEPushpin.OnMouseOverCallback(currentX, currentY, title, unescape(content));
	}
}

function GetContent()
{
	//Generate a unique pin DOM id based on the pin Id and the map GUID
	var pinId = this.ID + "_" + this.m_vemap.GUID;
	var content = "<img class='" + this.IconStyle + "' src='" + this.Iconurl + "' id='" + pinId + "' ";
	var isTitleValid = (this.Title != null && this.Title != "undefined" && this.Title.length > 0);
	var isDetailsValid = (this.Details != null && this.Details != "undefined" && this.Details.length > 0);

	//If Title or Details is valid, set up the Title/Details content and the mouseover events
	if (isTitleValid || isDetailsValid)
	{
	   //onmouseout, hide the pop-up. Also, hide the pop-up immediately when the user clicks down on the mouse 
	   //so the pop-up will disappear if the user starts dragging/clicking on the pushpin. Intercepting a different
	   //event such as onclick or the map events and then trying to hide the pop-up is too late.
		content += " onmouseout='VEPushpin.Hide();' onmousedown='VEPushpin.Hide(true);' onmouseover='VEPushpin.Show(\"" + this.m_vemap.GUID + "\",\"" + this.ID + "\"," + this.LatLong.Latitude + "," + this.LatLong.Longitude;

		if (isTitleValid)
		{
			content += ", \"" + escape(this.Title) + "\"";
		}
		else
		{
			content += ",\"\"";
		}
		
		if (isDetailsValid)
		{
			content += ", \"" + escape(this.Details) + "\"";
		}
		else
		{
			content += ",\"\"";
		}
		
		content += ",\"" +  this.TitleStyle + "\"";
		content += ",\"" +  this.DetailsStyle + "\"";
		
		content += ");' ";
	}
	content += "/>";
	return content;
}

VEPushpin.DisposeERO=function()
{
	//Dispose the ERO 
	if ((window.ero != null) && (window.ero != "undefined"))
	{
		window.ero.destroy();
		window.ero = null;
	}
}

//_SetMapInstance
VEPushpin.prototype._SetMapInstance = function(vemap)
{
	this.m_vemap = vemap;
	this.m_vemapcontrol = vemap.vemapcontrol;
}

//Prototype assignment
VEPushpin.prototype.GetContent=GetContent;