//------------------------------------------------------------------------------
// API/SearchManager.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: Cthota, keithkin
//
// <header>	$SearchManager$</header>
//------------------------------------------------------------------------------

//VEFindResult
function VEFindResult(id, name, description, veLatLong, isSponsored)
{
	//Note the id passed in can be null if there is not an id associated with the VEFindResult
	//Therefore we are not validating it here.
	VEValidator.ValidateNonNull(name, "name");
	VEValidator.ValidateNonNull(description, "description");
	VEValidator.ValidateNonNull(isSponsored, "isSponsored");
	//Note the LatLong passed in can be null if there is not a location associated with the VEFindResult
	//Therefore we are not validating it here.
	this.ID = id;
	this.Name = name;
	this.Description = description;
	this.LatLong = veLatLong;
	this.IsSponsored = isSponsored;
}

//VE_SearchManager
function VE_SearchManager(vemap)
{
	VEValidator.ValidateObject(vemap, "vemap", VEMap, "VEMap");
	this.m_vemap = vemap;
	var m_veLatLongFactory = new VELatLongFactory(new VELatLongFactorySpecFromMap(this.m_vemap));
	
	this.Initialize = function()
	{
	}

	this.Find = function(what, where, findCallback, disAmbiguateCallback, startPageIndex)
	{
		VEMap.ValidateState();
		
		var a = "";
		var b = "";	
		
		if(what == null || what == "undefined" || what.length <= 0)
		{
			a = "";
		}
		else
		{
			a = what;
		}	
		
		if(where == null || where == "undefined" || where.length <= 0)
		{
			b = "";
		}
		else
		{
			b = where;
		}
		
		if(a == "" && b == "")
			throw new VEException("VEMap:Find", "err_invalidwhatwhere", L_invalidwhatwhere_text)

		var parms = new Array();
		var what = new VEParameter("a", a);
		parms.push(what);
		var where = new VEParameter("b", b);
		parms.push(where);				
		
		var c = 0.0;
		var d = 0.0;
		var e = 0.0;
		var f = 0.0;
		var g = 1;
		var i = 1;
		var r = "false";

		if(a != null && a.length > 0)
		{
			var topLeft = this.m_vemap.vemapcontrol.PixelToLatLong(new Msn.VE.Pixel(0, 0));
			var bottomRight = this.m_vemap.vemapcontrol.PixelToLatLong(new Msn.VE.Pixel(this.m_vemap.GetWidth(), this.m_vemap.GetHeight()));

			if (topLeft == null || bottomRight == null)
				return;	
			
			c = topLeft.latitude;
			d = bottomRight.longitude;
			e = bottomRight.latitude;
			f = topLeft.longitude;
			
			// Swap latitudes if they are in the wrong order.
			if (c < e)
			{
				var t = c;
				c = e;
				e = t;
			}
			
			// Swap longitudes if they are in the wrong order.
			if (d < f)
			{
				var t = d;
				d = f;
				f = t;
			}

			if(startPageIndex != null && startPageIndex != "undefined")
			{
				VEValidator.ValidateInt(startPageIndex, "startPageIndex");
				g = parseInt(startPageIndex);
			}
			
			//Create params
			var param = new VEParameter("c", c);
			parms.push(param);		
			param = new VEParameter("d", d);
			parms.push(param);		
			param = new VEParameter("e", e);
			parms.push(param);		
			param = new VEParameter("f", f);
			parms.push(param);		
			param = new VEParameter("g", g);
			parms.push(param);		
			param = new VEParameter("i", i);
			parms.push(param);		
			param = new VEParameter("r", r);
			parms.push(param);		
		}

		//Add map GUID to the request URL so the server generated calls can map to the proper map instance
		parms.push(new VEParameter(Msn.VE.API.Constants.mapguid, this.m_vemap.GUID));

		//Assign the callback
		this.vesearchcallback = findCallback;
		this.vesearchdisambiguatecallback = disAmbiguateCallback;
		
		var network = new VENetwork();
		network.ServiceUrl = Msn.VE.API.Constants.searchservice;	
		network.BeginInvoke(null, parms, null);	
	}

	this._HandleFailedWhereSearch = function(message)
	{
		VEMap.ValidateState();
		if ((message != null) && (message != "undefined"))
		{
			this.m_vemap.ShowMessage(message.replace(/%1/g, MapControl.Features.PlatformName));
		}
	}
	
	this._ApplyResults = function (a, b, c, d, e)
	{
		VEMap.ValidateState();

		//return findresults
		var findResults = new Array();
		var searchContainer = null;	
		
		if(!Msn.VE.API.Globals.veiscommercialcontrol)
		{	
			if(this.m_vemap.searchelement != null)
			{
				searchContainer = document.getElementById(this.m_vemap.searchelement);
				searchContainer.innerHTML = "";
			}
			else
			{
				/*
				Msn.VE.API.Globals.vefindresultsnpanel = new VE_Panel("vefindresultspanel", 
								VEMap.GetLeft() + 5, 
								VEMap.GetTop() + 110,
								Msn.VE.API.Globals.vefindresultsnpanelwidth,
								VEMap.GetHeight()-160, 
								Msn.VE.API.Globals.vefindresultsnpanelcolor, 
								Msn.VE.API.Globals.vefindresultsnpanelzIndex,
								Msn.VE.API.Globals.vemapinstance.lastwhatsearch, 
								"", "", 				
								null, false);
				Msn.VE.API.Globals.vefindresultsnpanel.onCloseClick = function(){if(Msn.VE.API.Globals.vefindresultsnpanel)Msn.VE.API.Globals.vefindresultsnpanel.Hide();};
				*/
			}
				
			//ignore "a" which relates to an index	
			
			//b - Array of Search Results
			if(b != null)
			{			
				for(var i=0;i<b.length;i++)
				{
					var findResult = new VEFindResult(b[i].id, b[i].name, b[i].description, m_veLatLongFactory.CreateVELatLong(b[i].latitude, b[i].longitude), false);
					findResults.push(findResult);

					if(searchContainer != null)
					{
						searchContainer.appendChild(VE_SearchManager.CreateSearchResultElement(b[i]));
					}
					try
					{
						var pushpin = new VEPushpin(b[i].id, m_veLatLongFactory.CreateVELatLong(b[i].latitude, b[i].longitude), null, b[i].name, b[i].description);
						this.m_vemap.AddPushpin(pushpin);
					}
					catch(err)
					{
					}
				}
			}	
				
			//c - Array of Ads
			if(c != null)
			{
				for(var i=0;i<c.length;i++)
				{
					//The ads may not have latitude and longitude, so only set LatLong when it is set
					var findResultLatLong = null;
					if ( (c[i].latitude != null) && (c[i].latitude != "undefined") &&
					     (c[i].longitude != null)&& (c[i].longitude != "undefined") &&
					     (c[i].latitude >= -85.0) && (c[i].latitude <= 85.0) &&
					     (c[i].longitude >= -180.0) && (c[i].longitude <= 180.0)
					     )				     
					{
						findResultLatLong = m_veLatLongFactory.CreateVELatLong(c[i].latitude, c[i].longitude);
					}
					var findResult = new VEFindResult(c[i].id, c[i].title, c[i].description, findResultLatLong, true);
					findResults.push(findResult);

					if(searchContainer != null)
					{			
						searchContainer.appendChild(VE_SearchManager.CreateAdElement(c[i]));
					}
					try
					{
						//Only add the pushpin if the LatLong is specified
						if (findResultLatLong != null)
						{
							var pushpin = new VEPushpin(c[i].id, findResultLatLong, null, c[i].title, c[i].description);
							this.m_vemap.AddPushpin(pushpin);
						}
					}
					catch(err)
					{
					}
				}
			}	
			
			//d = hasmore
			
			//e = message
			if(e != null && e != "undefined" && e.length != 0)
			{	
				this.m_vemap.ShowMessage(e);
			}		
		}
		
		findResults.HasMore = d;
		findResults.Message = e;		
		
		//Assign latest results
		Msn.VE.API.Globals.vesearchpoiresults = findResults;
		
		//Only for commercial control you return the results
		//for non commercial though, you also render the results
		if(this.vesearchcallback != null && this.vesearchcallback != "undefined")
		{
			this.vesearchcallback(findResults);
		}

	}

	// Changes the spelling of a search
	this.ApplySuggestion=function(index, suggestion)
	{
		VEMap.ValidateState();
		var el = document.getElementById("vewhatinput");
		if(el)
		{
			el.value = suggestion;
		}
		this.Find(suggestion, this.m_vemap.lastwheresearch, null, "VEMap._GetMapFromGUID(" + this.m_vemap.GUID + ")._Disambiguate", 1);
	}

	this.CreateAdElement = function(ar)
	{
		VEMap.ValidateState();
		if(ar == null || ar == "undefined")
			return null;
			
		var el = document.createElement("div");
		el.id = "veadresults_" + ar.id;
		el.className = "VE_AdResult"	
		
		var title = document.createElement("a");
		title.className = "VE_AdResult_Title";
		title.innerHTML = ar.title;
		title.href = ar.url;
		el.appendChild(title);
		
		var desc = document.createElement("div");
		desc.className = "VE_AdResult_Description";
		desc.innerHTML = ar.description;
		el.appendChild(desc);
		
		return el;	
	}

	this.CreateSearchResultElement = function(sr)
	{
		VEMap.ValidateState();
		if(sr == null || sr == "undefined")
			return null;
			
		var el = document.createElement("div");
		el.id = "vesearchresults_" + sr.id;
		el.className = "VE_SearchResult"	
		
		var title = document.createElement("a");
		title.className = "VE_SearchResult_Title";
		title.innerHTML = sr.name;
		el.appendChild(title);
		
		var desc = document.createElement("div");
		desc.className = "VE_SearchResult_Description";
		desc.innerHTML = sr.description;
		el.appendChild(desc);
		
		var phone = document.createElement("div");
		phone.className = "VE_SearchResult_Phone";
		phone.innerHTML = sr.phone;
		el.appendChild(phone);

		var type = document.createElement("div");
		type.className = "VE_SearchResult_Type";
		type.innerHTML = sr.phone;
		el.appendChild(type);

		var rating = document.createElement("div");
		rating.className = "VE_SearchResult_Rating";
		rating.innerHTML = sr.phone;
		el.appendChild(rating);
		
		return el;
	}

	this.UpdateAmbiguousList = function(a)
	{	
		VEMap.ValidateState();
		if(a == null || a== "undefined" || a.length == 0)
			return;
		Msn.VE.API.Globals.vesearchlocresults = a;
		this.m_vemap.m_veambiguouslist.Show(a, this.vesearchdisambiguatecallback);
	}
}

//Define the dummy placeholder
Msn.VE.Search = new Object();
Msn.VE.Search.strLastWhere="";