//------------------------------------------------------------------------------
// Annotation/AnnotationActions.js - Things that are done through an Annotation
//                                   object
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: BayardB, shankak
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

//
// annotates a location
// finds context_pin if no pin is passed in
// opens annotation panel
VE_Annotations.AnnotateLocation = function (pin)
{
	if ( !pin )
	{
		pin = document.getElementById ( "context_pin" );
	}
	VE_Annotations.OpenPanel (pin, VE_Annotations.PanelColor, VE_Annotations.CreateTitle);
}

//
// specific case for the tools menu option
VE_Annotations.AnnotateMapCenter = function ()
{
	var lat = map.GetCenterLatitude ();
	var lon = map.GetCenterLongitude ();
	VE_ContextMenu.PlaceContextPin (lat, lon);
	VE_Annotations.AnnotateLocation ();
	VE_Analytics.Log("VE | User PushPin Pane","AddPushPin");
}

//
// this is a shorthand for the scratchpad method
// to delete a pin
VE_Annotations.DeleteAnnotation = function (pin)
{
	VE_Scratchpad.RemoveEntity ( null, pin.id );
}

//
// creates callback link for annotation popup
VE_Annotations._CreateAnnotationPopupLink = function (action, pin)
{
	var link = '';
	if (action == 'email')
	{
		var pinAnnotationData = VE_AnnotationData.GetAnnotation ( pin.id );
		var title = pinAnnotationData.title;
		var notes = pinAnnotationData.notes;
		var infoUrl = pinAnnotationData.infoUrl;
		var photoUrl = pinAnnotationData.photoUrl;
		
		// use IOSec ()
		var body = IOSec.EncodeUrl(title + "\n" + notes + "\n"  + GetUrlPrefix() + "?v=" + state.GetCurrentUrlApiVersion() + "&sp=" + IOSec.EncodeUrl ( VE_AnnotationData.SerializeAnnotation ( pin.id ) ) + "\n\n");
		link += 'mailto:?subject=' + IOSec.EncodeUrl( L_AtWindowsLiveLocal_Text.replace(/%1/g, title).replace(/%2/g,Msn.VE.Features.ProductName) ) + '&body=' + body;
	}
	else
	{
		link += 'javascript:VE_Annotations._FollowAnnotationPopupLink(\'' + action + '\',\'' + pin.id + '\')';
	}
	
	link = IOSec.TruncateUrlSafe ( link, 2020 );	
	return link;
}

//
// callback for annotation popup
VE_Annotations._FollowAnnotationPopupLink = function (action, pinId)
{
	VE_Annotations.ClosePopup ();
	// TODO: remove use of 'pin' and 'vePushpin' from annotations
	var pin = document.getElementById (pinId);
	var lat = pin.vePushpin.GetLatitude ();
	var lon = pin.vePushpin.GetLongitude ();
	var pinAnnotationData = VE_AnnotationData.GetAnnotation ( pinId );
	var title = pinAnnotationData.title;
	
	if (action == 'move')
	{
		VE_Annotations.EnterMovePushpinMode(pin);
		VE_Analytics.Log("VE | ERO","Move Pushpin");		
	}	
	else if ("edit" == action)
	{
		//Fix: Bug 3909
		//Remove the context pin if it's present. This is necessary in case the user
		//has the Annotations panel already open as a result of trying to add a pushpin
		//and then clicking on Edit here which will remove the panel and open a new one.
		VE_ContextMenu.RemoveContextPin ();
		VE_Annotations.OpenPanel (pin, VE_Annotations.PanelColor, VE_Annotations.EditTitle);
		VE_Analytics.Log("VE | ERO","Edit Pushpin");				
	}
	else if ("delete" == action)
	{
		VE_Annotations.DeleteAnnotation ( pin );
		var spItem = scratchPad.getItems().retrieveByKey(pinId);
		scratchPad.removeItem(spItem);		
		VE_Analytics.Log("VE | ERO","Delete Pushpin");
	}	
	else if ("driveFrom" == action)
	{
		VE_Directions.SetStart (title, lat, lon);
		VE_Analytics.Log("VE | User PushPin Pane","Get Directions");
	}	
	else if ("driveTo" == action)
	{
		VE_Directions.SetEnd (title, lat, lon);
		VE_Analytics.Log("VE | User PushPin Pane","Get Directions");
	}
	else
	{
		return false;
	}
}
