//------------------------------------------------------------------------------
// Annotation/AnnotationPanel.js - Creation of annotation panel
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: BayardB, sstchur, shankak
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

//
// keeps track of the panel's open state
VE_Annotations.PanelOpen = false;
VE_Annotations.PinBeingMoved = null;
//
// holds the annotations panel
VE_Annotations.Panel = null;

VE_Annotations.MaxAnnotationTitle = 100;
VE_Annotations.MaxAnnotationNotes = 1000;
VE_Annotations.MaxAnnotationPhotoUrl = 500;
VE_Annotations.MaxAnnotationMoreInfoUrl = 500;


// opens the annotation panel
// given a pin, it will fill in lat/lon
// default color is black
VE_Annotations.OpenPanel = function ( pin, color, title, latitude, longitude )
{	
	if(VE_CollectionsManager.collectionIndex == VE_CollectionsManager.unsavedCollectionIndex || VE_CollectionsManager.collectionIndex == null)
	{
		//BUG FIX : 3248 : Cannot add more than 15 User pushpins..
		if ( VE_Scratchpad.entities.length >= VE_Scratchpad.maxScratchpadItems
				&&	 ( !pin					 // when 'pin' is non-null, we're replacing an element
				|| pin.id == 'context_pin' ) // when 'pin.id' is 'context_pin', not replacing
			)
		{
			VE_ContextMenu.RemoveContextPin ();
			ShowMessage ( VE_Annotations.ScratchPadFullMsg );
			return false;
		}
	}
	
 	// close the search popup if it's open
	VE_SearchManager.CloseSearchPopup ();
	// close the context menu if it's open
	VE_ContextMenu.CloseMenu ();
	// close the annotations panel if it's open
	VE_Annotations.ClosePanel ();
	// close the annotations popup if it's open
	VE_Annotations.ClosePopup ();
	
	// TODO: remove use of 'pin' and 'vePushpin' from annotations
	if ( !latitude )
	{
	  latitude = pin.vePushpin.GetLatitude();
	}
	if ( !longitude )
	{
	  longitude = pin.vePushpin.GetLongitude();
	}
	if ( !color )
	{
	  color = VE_Annotations.PanelColor;
	}
	if ( !title )
	{
		title = VE_Annotations.EditTitle;
	}
	
	// If the menu has not been created, create it
	if(!VE_Annotations.Panel)
	{
		VE_Annotations.Panel = new VE_Panel(
			"annotationPanel",					// id
			5,							          	// x
			80,								          // y
			VE_Annotations.PanelWidth,			// width
			VE_Annotations.PanelHeight,		// height
			color,							        // color
			VE_SearchManager.popupZIndex,	// z index
			title,				// title
			"",								// body
			"",							// footer
			null,						// mouseover title
			true);					// opaque?
			
		var p = VE_Annotations.Panel;
		p.SetFooterSize(0);
		p.SetToolbarSize(0);		
		p.onCloseClick = VE_Annotations.CancelAnnotation;
	}
	
	
	// Adjust the effective width of the screen for the scratchpad
	var w = windowWidth;
	if(VE_Scratchpad.panel && VE_Scratchpad.panel.visible ||
	   VE_Directions.panel && VE_Directions.panel.visible)
	{
		w -= 200;
	}
	
	// Convert the latlong to pixel
	var pixel = map.LatLongToPixel(new Msn.VE.LatLong(latitude, longitude));
	
	var annoPanel = VE_Annotations.Panel;
  annoPanel.SetColor(color);
  annoPanel.SetTitle(title);
  annoPanel.DisableClosebox();
  annoPanel.SetBody(VE_Annotations.CreatePanelContent(pin));
  annoPanel.el.style.display = 'block';
  annoPanel.el.style.visibility = 'hidden';
  var desiredPoint = new Microsoft.Web.Geometry.Point(pixel.x, pixel.y);
  
  var pt = Microsoft.Web.Geometry.Functions.getBestBoundingPoint(annoPanel.el, desiredPoint, getVisibleMapRect());
	annoPanel.el.style.top = pt.y + 'px';
	annoPanel.el.style.left = pt.x + 'px';
	annoPanel.Show();
	annoPanel.el.style.visibility = 'visible';
	
	var formTitle = document.getElementById('annotationFormTitle');
	var formNotes = document.getElementById('annotationFormNotes');
	var formPhotoUrl = document.getElementById ('annotationFormPhotoUrl');
	var formMoreInfoUrl = document.getElementById('annotationFormMoreInfoUrl');
		
	annoData = VE_AnnotationData.GetAnnotation(pin.id);
	if (annoData && title == VE_Annotations.EditTitle)							// existing item, therefore populate the anno panel w/ the existing data
	{
		formTitle.value = annoData.title;
		formNotes.value = annoData.notes;
		formPhotoUrl.value = annoData.photoUrl;
		formMoreInfoUrl.value = annoData.infoUrl;
		
		// when modifying an existing annotation, put the cursor at the end of the title
		SelectText(formTitle, formTitle.value.length, formTitle.value.length);
	}	
	else
	{	
		formTitle.value = L_AnnotationConfDefaultTitle_Text;
		formTitle.select();
	}	
}


VE_Annotations.EnterMovePushpinMode = function(pin)
{
	//Entering move mode...Close all the popups in the page...
	closeAllPopups();
	
	VE_Annotations.PinBeingMoved = pin;
	VE_Annotations.PinParent = pin.parentNode;
	var pinParent = pin.parentNode;
	
	setElementCursor(pinParent, Microsoft.Web.Css.Cursors.Crosshair);
	ShowMessage(L_AnnotationMovePushpinInstructions_Text);
	
	pinParent.attachEvent('onmousedown', VE_Annotations.MovePushpin);	
	document.body.attachEvent('onkeypress', VE_Annotations.KeyPress);
};

VE_Annotations.MovePushpin = function(e)
{
	var pin = VE_Annotations.PinBeingMoved;
	if (!pin)
		return;
	
	var evt = GetEvent(e);
	
	// Honor only the primary mouse button to leave MovePushpinMode
	var button = evt.which || evt.button;
	if (button != 1)
		return;
		
	try
	{	
		var pixel = new Msn.VE.Pixel(evt.clientX, evt.clientY);
		pin.vePushpin.Move(pixel, Msn.VE.PushPinTypes.Annotation);
		
		var latitude = pin.vePushpin.GetLatitude();
		var longitude = pin.vePushpin.GetLongitude();		
		
		VE_AnnotationData.StoreAnnotation(pin.id, latitude, longitude);			/* Update it, actually -- not "store" it */
		
		var spItem = scratchPad.getItems().retrieveByKey(pin.id);
		if(spItem)
			scratchPad.updateItem(spItem, spItem.getTitle(), spItem.getDescription(), latitude, longitude, spItem.getInfoUrl(), spItem.getPhotoUrl(), spItem.getKeywords());
		
		VE_Scratchpad.UpdateEntityCoordinates(pin.id, latitude, longitude);
		
		state.UpdateFromScratchpad();				
	}
	catch(ex)	{ /*Swalow*/	}
	finally
	{
		pixel = null;
		VE_Annotations.LeaveMovePushpinMode();
	}
	
};

VE_Annotations.LeaveMovePushpinMode = function(e)
{
	//For deleted pins, there is no vePushpin property	
	var pin = VE_Annotations.PinBeingMoved;
	if(pin != null && pin.vePushpin != null && pin.parentNode != null)
	{		
		var pinParent = pin.parentNode;
		pinParent.detachEvent('onmousedown', VE_Annotations.LeaveMovePushpinMode);	
		setElementCursor(pinParent, Microsoft.Web.Css.Cursors.Pointer);		
	}
	else
	{
		if(VE_Annotations.PinParent)
		{
			VE_Annotations.PinParent.detachEvent('onmousedown', VE_Annotations.LeaveMovePushpinMode);						
			setElementCursor(VE_Annotations.PinParent, Microsoft.Web.Css.Cursors.Pointer);
		}
	}
	document.body.detachEvent('onkeypress', VE_Annotations.KeyPress);
	VE_Annotations.PinBeingMoved = null;	
	VE_Annotations.PinParent = null;	
	VE_MessageControl.Hide();
};

VE_Annotations.KeyPress = function(e)
{	
	var evt = GetEvent(e);
	if (evt.keyCode == 27)
		VE_Annotations.LeaveMovePushpinMode();
}


// opens the annotation panel
// given a pin, it will fill in lat/lon
// default color is black
VE_Annotations.OpenPanelEX = function ( spItemId, color, title, latitude, longitude )
{	
	var spItem = scratchPad.getItems().retrieveByKey(spItemId);
	
	if(VE_CollectionsManager.collectionIndex == VE_CollectionsManager.unsavedCollectionIndex || VE_CollectionsManager.collectionIndex == null)
	{
		//BUG FIX : 3248 : Cannot add more than 15 User pushpins..
		if ( VE_Scratchpad.entities.length >= VE_Scratchpad.maxScratchpadItems
				&&	 ( !spItem					 // when 'pin' is non-null, we're replacing an element
				|| spItem.getGuid() == 'context_pin' ) // when 'pin.id' is 'context_pin', not replacing
			)
		{
			VE_ContextMenu.RemoveContextPin ();
			ShowMessage ( VE_Annotations.ScratchPadFullMsg );
			return false;
		}
	}
	
 	// close the search popup if it's open
	VE_SearchManager.CloseSearchPopup ();
	// close the context menu if it's open
	VE_ContextMenu.CloseMenu ();
	// close the annotations panel if it's open
	VE_Annotations.ClosePanel ();
	// close the annotations popup if it's open
	VE_Annotations.ClosePopup ();
	
	// TODO: remove use of 'pin' and 'vePushpin' from annotations
	if ( !latitude )
	{
	  latitude = spItem.getLatitude();
	}
	if ( !longitude )
	{
	  longitude = spItem.getLongitude();
	}
	if ( !color )
	{
	  color = VE_Annotations.PanelColor;
	}
	if ( !title )
	{
		title = VE_Annotations.EditTitle;
	}
	
	// If the menu has not been created, create it
	if(!VE_Annotations.Panel)
	{
		VE_Annotations.Panel = new VE_Panel(
			"annotationPanel",					// id
			5,							          	// x
			80,								          // y
			VE_Annotations.PanelWidth,			// width
			VE_Annotations.PanelHeight,		// height
			color,							        // color
			VE_SearchManager.popupZIndex,	// z index
			title,				// title
			"",								// body
			"",							// footer
			null,						// mouseover title
			true);					// opaque?
			
		var p = VE_Annotations.Panel;
		p.SetFooterSize(0);
		p.SetToolbarSize(0);
		p.SetToolbar('Click & drag to move pushpin');
		p.onCloseClick = VE_Annotations.CancelAnnotation;
	}
	
	
	// Adjust the effective width of the screen for the scratchpad
	var w = windowWidth;
	if(VE_Scratchpad.panel && VE_Scratchpad.panel.visible ||
	   VE_Directions.panel && VE_Directions.panel.visible)
	{
		w -= 200;
	}
	
	// Convert the latlong to pixel
	var pixel = map.LatLongToPixel(new Msn.VE.LatLong(latitude, longitude));
	
	var annoPanel = VE_Annotations.Panel;
  annoPanel.SetColor(color);
  annoPanel.SetTitle(title);
  annoPanel.DisableClosebox();
  annoPanel.SetBody(VE_Annotations.CreatePanelContentEX(spItemId,latitude,longitude));
  annoPanel.el.style.display = 'block';
  annoPanel.el.style.visibility = 'hidden';
  var desiredPoint = new Microsoft.Web.Geometry.Point(pixel.x, pixel.y);
  
  var pt = Microsoft.Web.Geometry.Functions.getBestBoundingPoint(annoPanel.el, desiredPoint, getVisibleMapRect());
	annoPanel.el.style.top = pt.y + 'px';
	annoPanel.el.style.left = pt.x + 'px';
	annoPanel.Show();
	annoPanel.el.style.visibility = 'visible';
	
	
	var formTitle = document.getElementById('annotationFormTitle');
	var formNotes = document.getElementById('annotationFormNotes');
	var formPhotoUrl = document.getElementById ('annotationFormPhotoUrl');
	var formMoreInfoUrl = document.getElementById('annotationFormMoreInfoUrl');
		
	if (spItem && title == VE_Annotations.EditTitle)							// existing item, therefore populate the anno panel w/ the existing data
	{
		formTitle.value = spItem.getTitle();
		formNotes.value = spItem.getDescription();
		formPhotoUrl.value = spItem.getPhotoUrl() == null ? ' ' : unescape(spItem.getPhotoUrl());
		formMoreInfoUrl.value = spItem.getInfoUrl() == null ? ' ' : unescape(spItem.getInfoUrl());
		
		// when modifying an existing annotation, put the cursor at the end of the title
		SelectText(formTitle, formTitle.value.length, formTitle.value.length);
	}	
	else
	{	
		formTitle.value = L_AnnotationConfDefaultTitle_Text;
		formTitle.select();
	}
	
	spItem = null;
}

//
// close the panel
VE_Annotations.ClosePanel = function ()
{
  var p = VE_Annotations.Panel;  
  if (p)
  {
    p.Hide ();
  }
  
  VE_Annotations.PanelOpen = false; 
}

//
// builds the content of the panel from a pin
VE_Annotations.CreatePanelContentEX = function (spItemId, latitude, longitude)
{
	if(spItemId == null || latitude == null || longitude == null)
	{
		throw ("Scratchpad Item ID argument cannot be null");
		return;
	}
	
	var html = 
	' <form id = "wl_ve_annotationForm" onkeydown = "return VE_Annotations.PanelKeyDownHandler(event, \'' + spItemId + '\',\'' + latitude + '\',\'' + longitude + '\');" onsubmit = "return false;">' +
			'<fieldset>' +
				'<legend></legend>' +
				'<label>' + L_Title_Text + '</label>' +
				'<input id = "annotationFormTitle" type = "text" tabindex = "10" maxlength="'+VE_Annotations.MaxAnnotationTitle+'"/>' +
			'</fieldset>' +
			
			'<fieldset>' +
				'<legend></legend>' +
				'<label>' + L_Notes_Text + '</label>' +
				'<textarea id = "annotationFormNotes" tabindex = "11" onkeypress = "return VE_Annotations.checkMaxLength(this, '+VE_Annotations.MaxAnnotationNotes+');" ' +
				'onpaste = "return VE_Annotations.doPaste(this, ' + VE_Annotations.MaxAnnotationNotes + ');" ' + '></textarea>' +
			'</fieldset>' +
			
			/* Keywords for next release -- not Discovery
			'<fieldset>' +
				'<legend></legend>' +
				'<label>' + L_Keywords_Text + '</label>' +
				'<input id = "annotationKeywords" type = "text" />' +
			'</fieldset>' + */
			
			'<fieldset>' +
				'<legend></legend>' +
				'<label>' + L_PhotoUrl_Text + '</label>' +
				'<input id = "annotationFormPhotoUrl" type = "text" tabindex = "12" maxlength="'+VE_Annotations.MaxAnnotationPhotoUrl+'"/>' +
			'</fieldset>' +
			
			'<fieldset>' +
				'<legend></legend>' +
				'<label>' + L_MoreInfoUrl_Text + '</label>' +
				'<input id = "annotationFormMoreInfoUrl" type = "text" tabindex = "13" maxlength="'+VE_Annotations.MaxAnnotationMoreInfoUrl+'"/>' +
			'</fieldset>' +
			
			'<fieldset id = "annotationForm_fsOptions">' +
				'<legend></legend>' +
				'<input id = "wl_ve_annotations_cancel" class = "higButton" tabindex = "15" type = "button" onclick = "VE_Annotations.CancelAnnotation(\'' + spItemId + '\');" value = "' + L_Cancel_Text + '" />' +
				'<input id = "wl_ve_annotations_save" class = "higButton" tabindex = "14" type = "button" onclick = "VE_Annotations.SaveAnnotation(\'' + spItemId + '\',\'' + latitude + '\',\'' + longitude + '\');" value = "' + L_AnnotationPanelSave_Text + '" />' +			
			'</fieldset>' +
	'</form>';
	return html;  
}

//
// builds the content of the panel from a pin
VE_Annotations.CreatePanelContent = function (pin)
{
  var html = 
 ' <form id = "wl_ve_annotationForm" onkeydown = "return VE_Annotations.PanelKeyDownHandler(event, \'' + pin.id + '\');" onsubmit = "return false;">' +
		'<fieldset>' +
			'<legend></legend>' +
			'<label>' + L_Title_Text + '</label>' +
			'<input id = "annotationFormTitle" type = "text" tabindex = "10" maxlength="'+VE_Annotations.MaxAnnotationTitle+'"/>' +
		'</fieldset>' +
		
		'<fieldset>' +
			'<legend></legend>' +
			'<label>' + L_Notes_Text + '</label>' +
			'<textarea id = "annotationFormNotes" tabindex = "11" onkeypress = "return VE_Annotations.checkMaxLength(this, '+VE_Annotations.MaxAnnotationNotes+');" ' +
			'onpaste = "return VE_Annotations.doPaste(this, ' + VE_Annotations.MaxAnnotationNotes + ');" ' + '></textarea>' +
		'</fieldset>' +
		
		/* Keywords for next release -- not Discovery
		'<fieldset>' +
			'<legend></legend>' +
			'<label>' + L_Keywords_Text + '</label>' +
			'<input id = "annotationKeywords" type = "text" />' +
		'</fieldset>' + */
		
		'<fieldset>' +
			'<legend></legend>' +
			'<label>' + L_PhotoUrl_Text + '</label>' +
			'<input id = "annotationFormPhotoUrl" type = "text" tabindex = "12" maxlength="'+VE_Annotations.MaxAnnotationPhotoUrl+'"/>' +
		'</fieldset>' +
		
		'<fieldset>' +
			'<legend></legend>' +
			'<label>' + L_MoreInfoUrl_Text + '</label>' +
			'<input id = "annotationFormMoreInfoUrl" type = "text" tabindex = "13" maxlength="'+VE_Annotations.MaxAnnotationMoreInfoUrl+'"/>' +
		'</fieldset>' +
		
		'<fieldset id = "annotationForm_fsOptions">' +
			'<legend></legend>' +
			'<input id = "wl_ve_annotations_cancel" class = "higButton" tabindex = "15" type = "button" onclick = "VE_Annotations.CancelAnnotation(\'' + pin.id + '\');" value = "' + L_Cancel_Text + '" />' +
			'<input id = "wl_ve_annotations_save" class = "higButton" tabindex = "14" type = "button" onclick = "VE_Annotations.SaveAnnotation(\'' + pin.id + '\');" value = "' + L_AnnotationPanelSave_Text + '" />' +			
		'</fieldset>' +
  '</form>';
  return html;  
}

//
// this method maps:
// esc => cancel annotation
// enter => save annotation
//
// pass in an event and pinId
VE_Annotations.PanelKeyDownHandler = function (e, pinId, latitude, longitude)
{
	if (!e)
	{
		e = window.event;
	}
	
	switch (e.keyCode)
	{
		case 27: // esc
			VE_Annotations.CancelAnnotation ();
			return false;
			break;
		case 13: // enter
			if (e.ctrlKey || e.shiftKey || e.altKey)
			{
				break;
			}
			if ( e.srcElement.id == "wl_ve_annotations_cancel" )
				VE_Annotations.CancelAnnotation ();
			else	
				VE_Annotations.SaveAnnotation (pinId, latitude, longitude);
			return false;
			break;
		default:
			break;
	}
	
	return true;
}

//
// cancels an annotation and deletes the associated pin if it is new
VE_Annotations.CancelAnnotation = function ()
{
	VE_ContextMenu.RemoveContextPin ();
  VE_Annotations.ClosePanel ();
  return false;
}

//
// saves an annotation (removes the old one and adds a new one for update)
VE_Annotations.SaveAnnotation = function ( pinId , latitude, longitude)
{
  VE_Annotations.checkTextArea(VE_Annotations.MaxAnnotationNotes);
  
  var pin = document.getElementById ( pinId );
  
  if(pin)
  {
	// TODO: remove use of 'pin' and 'vePushpin' from annotations
	var lat = pin.vePushpin.GetLatitude ();
	var lon = pin.vePushpin.GetLongitude ();
  }
  else
  {
	var lat = latitude;
	var lon = longitude;
  }


  var annoTitle = document.getElementById('annotationFormTitle').value;
  var annoNotes = document.getElementById('annotationFormNotes').value;
  var annoPhotoUrl = document.getElementById('annotationFormPhotoUrl').value;
  var annoInfoUrl = document.getElementById('annotationFormMoreInfoUrl').value;
  /* Keywords for next release -- not Discovery
  var annoKeywords = document.getElementById('annotationKeywords').value; */
  var annoKeywords = '';  
  
  if ( annoTitle.replace(/^\s+/g, "").replace(/\s+$/g, "").length < VE_Annotations.MinTitleChars ) // this will break if MinTitleChars > 1 - but it will be easy to find
  {
 	document.getElementById ( "annotationFormTitle" ).value = L_AnnotationPanelTitleMissingText_Text;
	document.getElementById ( "annotationFormTitle" ).select ();
	return false;
  }
  
  if ( annoNotes.replace(/^\s+/g, "").replace(/\s+$/g, "").length < VE_Annotations.MinNotesChars ) // this will break if MinNotesChars > 1 - but it will be easy to find
  {
	document.getElementById ( "annotationFormNotes" ).value = L_AnnotationPanelNotesMissingText_Text;
	document.getElementById ( "annotationFormNotes" ).select ();
	return false;
  }  

  annoTitle = escape(annoTitle.substring (0, VE_Annotations.MaxAnnotationTitle));
  annoNotes = escape(annoNotes.substring ( 0, VE_Annotations.MaxAnnotationNotes  ));/*MaxNotesChars*/
  annoInfoUrl = escape(annoInfoUrl.substring (0, VE_Annotations.MaxAnnotationMoreInfoUrl ));
  annoPhotoUrl = escape(annoPhotoUrl.substring (0, VE_Annotations.MaxAnnotationPhotoUrl ));  
  
  if(annoTitle  && annoTitle != 'undefined' && annoTitle.length > 0)
	annoTitle = annoTitle.replace(/\|/g, ' ').replace(/\^/g, ' ').replace(/^\s+/g, "").replace(/\s+$/g, "");
  
  if(annoNotes  && annoNotes != 'undefined' && annoNotes.length > 0)
	annoNotes = annoNotes.replace(/\|/g, ' ').replace(/\^/g, ' ').replace(/^\s+/g, "").replace(/\s+$/g, "");

  if(annoInfoUrl  && annoInfoUrl != 'undefined' && annoInfoUrl.length > 0)
	annoInfoUrl = annoInfoUrl.replace(/\|/g, ' ').replace(/\^/g, ' ').replace(/^\s+/g, "").replace(/\s+$/g, "");
	
  if(annoPhotoUrl  && annoPhotoUrl != 'undefined' && annoPhotoUrl.length > 0)
	annoPhotoUrl = annoPhotoUrl.replace(/\|/g, ' ').replace(/\^/g, ' ').replace(/^\s+/g, "").replace(/\s+$/g, "");	
    
  if(latitude == null  && longitude == null)
  {	  
		if (pinId != 'context_pin' && VE_AnnotationData.GetAnnotation(pinId))
		{

			VE_Scratchpad.UpdateLocation(pinId, annoTitle, annoNotes, annoInfoUrl, annoPhotoUrl);  //update existing one				
			
			var spItem = scratchPad.getItems().retrieveByKey(pinId);	
					
			//Updates the 
			//UI and the Server Storage
			//Fires the on-update event
			//to store in the storage
			scratchPad.updateItem(spItem,	
									annoTitle, 
									annoNotes, 
									lat, 
									lon, 
									annoInfoUrl, 
									annoPhotoUrl, 
									annoKeywords);

		}
		else
		{		
			// bayardb: create a location to use to generate a pushpin id
			var id = VE_Scratchpad.locationCounter;  
			var location = new VE_SearchResult(id, annoTitle, annoNotes, "", "", "aN", lat, lon, annoInfoUrl, annoPhotoUrl, annoKeywords);
			pinId = VE_Scratchpad.GeneratePinId (location);
			delete location;
			
  			VE_Scratchpad.AddLocation(annoTitle, lat, lon, annoNotes, "aN", annoInfoUrl, annoPhotoUrl, annoKeywords); //add new one
		}	  
		
		VE_AnnotationData.StoreAnnotation( pinId, lat, lon, annoTitle, annoNotes, annoInfoUrl, annoPhotoUrl );
  }
  else
  {			
		VE_Scratchpad.UpdateLocation(pinId, annoTitle, annoNotes, annoInfoUrl, annoPhotoUrl);  //update existing one				
	
		//this is a special modification so that it fits the existing deal
		//and also incorporates the new scratchPad's paradigm.
		var spItem = scratchPad.getItems().retrieveByKey(pinId);

		if(spItem && spItem != '' && spItem != 'undefined')
		{
			scratchPad.updateItem(spItem,	annoTitle, 
										annoNotes, 
										lat, 
										lon, 
										annoInfoUrl, 
										annoPhotoUrl, 
										annoKeywords
								);

		}
		
		VE_AnnotationData.StoreAnnotation( pinId, lat, lon, annoTitle, annoNotes, annoInfoUrl, annoPhotoUrl );

  }

  state.UpdateFromScratchpad(); // force the permalink to update
  VE_ContextMenu.RemoveContextPin ();
  VE_Annotations.ClosePanel ();  
  return false;
}

VE_Annotations.GetTextValue = function(value)
{

	var tmp = null;
	//Only create the span if the value is defined, not null, and has some non-whitespace chars
	if( value != "undefined" && value != null && value.length > 0 && 
		(value.match(/\S/) != null) )
	{
		tmp = document.createElement('span');
		if(tmp == null)
			return "";
		tmp.innerHTML = value;
		value = tmp.firstChild.nodeValue;
		return value;		
	}
	
	tmp = null;	
	
	return "";
}

VE_Annotations.checkMaxLength = function (Obj, MaxLen)
{
	if (document.all)
		return (Obj.value.length < MaxLen);
	else
		return true;
}

VE_Annotations.doPaste = function (Obj, MaxLen)
{
	intCurrLength = Obj.value.length;
	selection=document.selection.createRange();	
	strClip = window.clipboardData.getData("Text");
	intClipLength = strClip.length ;
    intAvailLength = MaxLen - intCurrLength + selection.text.length;
    if (intAvailLength > 0)
	{			
		selection.text = strClip.substr(0, intAvailLength);			
	}
	return false;
}

VE_Annotations.checkTextArea = function (MaxLen)
{    
    var textarea = document.getElementById('annotationFormNotes');
    
	if (textarea.value.length > MaxLen)
	{
		textarea.value = textarea.value.substr(0,MaxLen);
	}	
}
