//------------------------------------------------------------------------------
// Directions/Decoding.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: EShinn
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

// Decodes compressed directions information sent from server.
Msn.VE.DirectionsDecoder = function()
{
	var encodedCoordinateLength = 4;
	var encodedCoordinateBase = 1000000;

	// Decodes a string of numbers stored in byte notation.
	// Numbers are not stored in any complement notation and use the highest bit as the sign bit if 
	// the caller marks the encoded numbers as signed.
	
	// encodedNumbers - the string that contains the encoded numbers
	// numberLength - This is a the length of the string representing a single encoded number
	// base - This is the base that the number is stored in.  All decoded numbers will be divided by the base.
	// signed - True if the encoded numbers are signed.
	function DecodeEncodedNumbers(encodedNumbers, numberLength, base, signed)
	{
		// If we don't have a valid encoded coordinate string, return an empty array
		if (!encodedNumbers || encodedNumbers.length < numberLength)
		{
			return [];
		}
		
		// We only want to decode full coordinate strings.
		var length = encodedNumbers.length - (encodedNumbers.length % numberLength);
		
		var numbers = new Array();
		
		var negative = false;
		var currentNumber = 0;
		var lastByteIndex = numberLength - 1;
		
		// Go through the encoded coordinate string character by character
		for (var i = 0; i < length; i++)
		{
			// Get the integer equivalent of the current char.
			var currentCharCode = encodedNumbers.charCodeAt(i);
			
			// Calculate which byte we are on in this number.
			var byteIndex = i % numberLength;
			
			// If this is the start of a new coordinate, we need to do some processing.
			if (signed && byteIndex == 0)
			{
				negative = currentCharCode & 0x80;
				currentCharCode &= 0x7F;
			}
			
			// Append the current byte.
			currentNumber |= currentCharCode;

			if (byteIndex == lastByteIndex)
			{
				// If we are at the end of the current number, store it.
				var translatedNumber = currentNumber / base;
				numbers.push(negative ? -translatedNumber : translatedNumber);

				// Reset the current number.
				currentNumber = 0;
				negative = false;
			}
			else
			{
				// Otherwise, shift it over to make room for the next byte.
				currentNumber <<= 8;			
			}			
		}
		
		// Return the decoded numbers.
		return numbers;
	}	
	
	// Decodes an encoded coordinates string.
	// An encoded coordinate is 4 8-bit character string.
	// It is a fixed point representation of a floating point number with the following format (+/-)123.456789
	function DecodeCoordinatesString(encodedCoordinates)
	{
		return DecodeEncodedNumbers(encodedCoordinates, encodedCoordinateLength, encodedCoordinateBase, true);
	}

	this.DecodeCoordinatesString = DecodeCoordinatesString;
}