//------------------------------------------------------------------------------
// Directions/Popup.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: EShinn, sstchur
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

// Constants for popups
VE_Directions.popupWidth = 230;

// Disables animation, recenters map, then enables animation (if originally enabled)
VE_Directions.RecenterAndShowPopup=function(instNum)
{
	// Don't display if the map is being updated or we are in oblique mode.
	if (map.panning || map.zooming)
	{
		return;
	}
	
	var route = VE_Directions.displayedRoute;

	// If we have nothing to display, don't show it.
	if (!route || instNum < 0 || instNum > route.instructions.length)
	{
		return;
	}

	var instruction = route.instructions[instNum];
	
	// Jump map to instruction point.
	map.SetAnimationEnabled(false);
	map.SetCenterAndZoom(instruction.latitude, instruction.longitude, 15);
	map.SetAnimationEnabled(state.GetAnimatedMovementsEnabled());

	// Show the pop-up.
	VE_Directions._ShowDirectionsPopup(instNum);
}

VE_Directions._OnDirectionsPushpinMouseout = function(e)
{
	ero.hide();
}

// Attempts to find the associated instruction for the pushpin that spawned the event
// Then it will try to display the directions popup for that instruction
VE_Directions._OnDirectionsPushpinMouseover=function(e)
{
	var returnArray = VE_SearchManager._GetEntityReference(e);
	if (!returnArray)
	{
		return false;
	}		
	var instNum = parseInt(returnArray[0]);
	VE_Directions._ShowDirectionsPopup(instNum);
}

// Displays a pop-up related to a particular direction step.
VE_Directions._ShowDirectionsPopup=function(instNum)
{
	if (VE_ContextMenu.MenuOpen) 
	{
		VE_ContextMenu.RemoveContextPin();
		VE_ContextMenu.CloseMenu();
	}
	
	//Fix: Bug 3909
	//Do not close the Annotations Panel because it is annoying if the 
	//user clicks on Add Pushpin and then hovers over a pin on the way to the panel
	//VE_ContextMenu.RemoveContextPin ();
	//VE_Annotations.ClosePanel ();
	
	// Don't display if the map is being updated.
	if (map.panning || map.zooming)	
		return;
	
	var pinId = 'veDDstep_' + instNum + '_pin_pinDiv'; 
	var pinDirectionStep = document.getElementById(pinId);
	
	var route = VE_Directions.displayedRoute;
	
	// If we have nothing to display, don't show it.
	if (!route || instNum < 0 || instNum > route.instructions.length)
	{
		return;
	}
	
	// Remove any existing popup & temporary pushpins.
	VE_Directions.ClosePopup();

	var instruction = route.instructions[instNum];	
	var stepPushpin = null;
	
	if (!instruction.permanentPin && VE_Directions.routeDisplayManager != null)
	{
		// This isn't a permanent pushpin - have the display manager add it.
		stepPushpin = VE_Directions.routeDisplayManager.AddPinForInstructionStep(instNum);
	}
	else
	{
		// Otherwise get the permanent pushpin from the DOM
		stepPushpin = document.getElementById(route.GenerateInstructionPinId(instNum));	
	}

	// If we have no pushpin, we don't display a pop-up.
	if (!stepPushpin)
	{
		return;
	}
	
	// If we have no pushpin, we don't display a pop-up.
	if (!stepPushpin)
	{
		return;
	}
	
	// Push the pin of the previously open pop up back to its place.
	VE_SearchManager._RestorePushpinZIndex();

	// Keep track of the pushpin & its original z-index.
	VE_SearchManager.popupPushpin = stepPushpin;
	VE_SearchManager.popupPushpinOrigZIndex = VE_Scratchpad.pinZIndex;
	
	
	ero.clearActions();
	var content = VE_Directions.routeDisplayManager.GeneratePopupContent(instNum);
	ero.setContent(content);	
	ero.dockToElement(pinDirectionStep);
}

// This function is a small wrapper around the existing VE_SearchManager.CloseSearchPopup() call
// It also removes the pushpin for a step if it is not supposed to be permanent.
VE_Directions.ClosePopup=function()
{
	// THis method will only close directions pop-ups
	if (VE_SearchManager.popupEntity == null || VE_SearchManager.popupEntity.constructor != VE_RouteInstruction)
	{
		return;
	}
	
	// Remove the pushpin if this is not a permanent pushpin
	if (!VE_SearchManager.popupEntity.permanentPin && VE_Directions.routeDisplayManager != null)
	{
		// Determine the instruction # from the pin id.
		var pinId = VE_SearchManager.popupPushpin.id;		
		var a = pinId.indexOf('_');
		var b = pinId.lastIndexOf('_');		
		var instructionNumber  = parseInt(pinId.substr(a+1, b-a-1));
		
		// Remove the pin for instruction.
		VE_Directions.routeDisplayManager.RemovePinForInstructionStep(instructionNumber);
	}

	VE_SearchManager.CloseSearchPopup();
}

