//------------------------------------------------------------------------------
// Directions/Serialization.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: EShinn, keithkin
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------


// Returns true if this the string is a valid serialization of a route point
VE_Directions.ValidSerializedRoutePoint=function(routePoint, apiVersion)
{
	// If the string could not possibly be a valid serialized route point, return false.
	if (routePoint == null || routePoint.constructor != String || routePoint.length < 5 || routePoint.charAt(3) != ".")
	{
		return false;
	}
	
	// Set it to the current URL API Version if it hasn't been decided yet...
	// Normally, we would make a call to state at this point, but we don't know when this is getting
	// called so state might not exist at this point in time.
	if (apiVersion == null)
	{
		apiVersion = currentUrlApiVersion;
	}

	// This is the 3 character identifier the says how the remainder of the string after the breaking character
	// should be interpreted.
	var pointType = routePoint.substring(0,3);
	// This is the rest of the route point that needs to be translated.
	var pointInformation = routePoint.substring(4);

	switch (pointType)
	{
		case "adr":
			// We don't actually care about the contents because it will get geocoded.
			return true;
			break;
		case "pos":
			var routePosArray = VE_Directions.ParsePosRoutePoint(pointInformation, apiVersion);
			return ((routePosArray != null) && (routePosArray != "undefined"));
			break;

		default:
			// Any unrecognized type is considered invalid.
			return false;
	}
}

// Parses a serialized position route point ("pos")
// and returns an array [title, latitude, longitude] for the position.
// If the serialized position route point is invalid, returns null.
VE_Directions.ParsePosRoutePoint=function(serializedPoint, apiVersion)
{
	var titleLatLonArray = null;
	var secondarySeparator = VE_SerializationConstants.secondarySeparator[apiVersion];
	var latLonParts = serializedPoint.split(secondarySeparator);

	var lat = "undefined";
	var lon = "undefined";
	var i=0;

	var isValidEncoding = false;
	if (latLonParts.length > i)
	{
		try
		{
			//try to decode the latlong from an encoded string first
			var encoder = new VELatLongEncoding();
			var decodedLatLong = encoder.Decode(latLonParts[i]);
			//If the decoder throws an exception, the following lines will not get executed
			lat=decodedLatLong[0];
			lon=decodedLatLong[1];
			isValidEncoding = true;
			++i;
		}
		catch(ex)
		{
			isValidEncoding = false;
		}
	}

	//If the decoding failed, it may be an unencoded lat/lon. Try to read unencoded lat/lon coordinates from the serialization.
	if (!isValidEncoding)
	{
		if (latLonParts.length > i)
		{
			lat = parseFloat(latLonParts[i++]);
		}
		
		if (latLonParts.length > i)
		{
			lon = parseFloat(latLonParts[i++]);
		}
	} 

	// if either of these aren't numbers, discard
	if (isNaN (lat) || isNaN (lon))
	{
		return null;
	}
			
	// See if we have a valid lat/lon
	var latValidator = new VE_RangeValidator(-85.0, 85.0);
	var lonValidator = new VE_RangeValidator(-180.0, 180.0);

	//If lat/lon is valid, form the array with title, lat, lon
	if (latValidator.ValidValue(lat) && lonValidator.ValidValue(lon))
	{
		var title = unescape(latLonParts.length > i ? latLonParts.slice(i).join(secondarySeparator) : "");
		return [title,lat,lon];
	}

	return null;
}

// Transforms a valid serialized route point into an array that represents it.
VE_Directions.ParseSerializedRoutePoint=function(routePoint, apiVersion)
{
	// Set it to the current URL API Version if it hasn't been decided yet...
	// Normally, we would make a call to state at this point, but we don't know when this is getting
	// called so state might not exist at this point in time.
	if (apiVersion == null)
	{
		apiVersion = currentUrlApiVersion;
	}

	// If it's not valid, we don't want it.
	if (!VE_Directions.ValidSerializedRoutePoint(routePoint, apiVersion))
	{
		return null;
	}
	
	// This is the 3 character identifier the says how the remainder of the string after the breaking character
	// should be interpreted.
	var pointType = routePoint.substring(0,3);
	// This is the rest of the route point that needs to be translated.
	var pointInformation = routePoint.substring(4);

	switch (pointType)
	{
		case "adr":
			return [unescape(pointInformation)];
			break;
		case "pos":
			return VE_Directions.ParsePosRoutePoint(pointInformation, apiVersion);
			break;

		default:
			// Any unrecognized type is considered invalid.
			return null;
	}
}


