//------------------------------------------------------------------------------
// MapControl/Dashboard/Compass.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: Codye
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

// Constructor.  Creates a new compass that is added to the specified parent element.
function Compass(parent)
{
	// The dom element for the control
	var element = document.createElement("div");
	
	// The position of the compass control.  The position is recalculated
	// at every mouse down event.
	var x=0, y=0;

	// Represents whether or not the control is currently panning
	var panning = false;

	// The maximum speed that the pan control will allow
	var maxScrollSpeed = 15;

	// Creates the control and sets up the event listeners
	this.Init = function()
	{
		element.className = "Compass";
		element.attachEvent("onmousedown", MouseDown);
		element.attachEvent("onmouseup", MouseUp);
		element.attachEvent("onmousemove", MouseMove);
		element.attachEvent("ondblclick", IgnoreEvent);
		parent.appendChild(element);
	};
	
	// Unhooks events and nulls out elements to release memory.
	this.Destroy = function()
	{
		element.detachEvent("onmousedown", MouseDown);
		element.detachEvent("onmouseup", MouseUp);
		element.detachEvent("onmousemove", MouseMove);
		element.detachEvent("ondblclick", IgnoreEvent);
		
		element = null;
	};

	// Handles the mouse down event, starting the continuous pan.
	function MouseDown(e)
	{
		// Get and cancel the event.  Don't want it to bubble up to the parent element.
		e = GetEvent(e);
		CancelEvent(e);
		
		// Compute the position at every mouse down.  This is necessary if the 
		// position of the control changes.
		x = GetLeftPosition(element);
		y = GetTopPosition(element);
	
		if(element.setCapture)
		{
			element.setCapture();
		}
		
		var dx = GetMouseX(e) - x - (element.offsetWidth/2);
		var dy = GetMouseY(e) - y - (element.offsetHeight/2);
		dx = Math.min(Math.max(dx, -maxScrollSpeed), maxScrollSpeed);
		dy = Math.min(Math.max(dy, -maxScrollSpeed), maxScrollSpeed);
		map.ContinuousPan(dx, dy);
		panning = true;
		
		return false;
	}
	
	// Handles the mouse move event, adjusting the speed and direction of the pan.
	function MouseMove(e)
	{
		// Get and cancel the event.  Don't want it to bubble up to the parent element.
		e = GetEvent(e);
		CancelEvent(e);
		
		// Mouse move only matters if the mouse is down and we're actually panning.
		if(panning)
		{
			var dx = GetMouseX(e) - x - (element.offsetWidth/2);
			var dy = GetMouseY(e) - y - (element.offsetHeight/2);
			dx = Math.min(Math.max(dx, -maxScrollSpeed), maxScrollSpeed);
			dy = Math.min(Math.max(dy, -maxScrollSpeed), maxScrollSpeed);
			map.ContinuousPan(dx, dy);
		}
		
		return false;
	}

	// Handles the mouse up event, ending the continuous pan.
	function MouseUp(e)
	{
		// Get and cancel the event.  Don't want it to bubble up to the parent element.
		e = GetEvent(e);
		CancelEvent(e);
		
		if(element.releaseCapture)
		{
			element.releaseCapture();
		}
		
		map.StopContinuousPan();
		panning = false;
		
		return false;
	}
	
	function Hide()
	{
		element.style.display = "none";
	}
	
	function Show()
	{
		element.style.display = "block";
	}
	
	this.Hide = Hide;
	this.Show = Show;
}
