//------------------------------------------------------------------------------
// MapControl/Dashboard/Dashboard.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: Codye, sstchur
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

// The dashboard is a widget that can control the map.  It includes links for adjusting the map
// style, a compass for panning, a zoom slider, notification for oblique, and oblique controls
// when in that mode.
Msn.VE.Dashboard = function(element, map, size)
{
	// Header
	var header = document.createElement('div');
	header.className = 'header';
	element.appendChild(header);
	var toggleGlyph = new Microsoft.Web.UI.IEGlyph(null, 'toggleGlyph');
	var toggleGlyphElement = toggleGlyph.getElement();
	header.appendChild( toggleGlyphElement );
	
	// Dashboard starts out expanded
	element.className += ' expanded';

	// The tab objects are just the links for the corresponding map styles.
	var roadTab = document.createElement("div");
	var aerialTab = document.createElement("div");
	var obliqueTab = document.createElement("div");
	var selectedTab = null;
	
	var separator = document.createElement("hr");
	
	// The labels container wraps the toggle labels checkbox and the corresponding tooltip.
	var labelsContainer = document.createElement("div");
	var labelsCheckbox = document.createElement("input");
	
	// IE won't let you change the value of a checkbox before it has been displayed.  We want
	// to default to checked (labels on, hybrid mode), so we have to keep track of the first
	// aerial view.  If it's the first view, we override the value of the checkbox and force 
	// hybrid mode.
	var firstAerial = true;
	
	// The compass control
	var compass = null;
	
	var obliqueCompass = null;
	
	// The ortho zoom widget.  This is the slider control, only used in ortho modes.
	var orthoZoom = null;
	
	// The oblique zoom widget.  This is just 2 boxes representing the 2 zoom levels, only used in oblique.
	var obliqueZoom = null;
		
	// The oblique notification shows a image thumbnail and a brief message which subtly alerts
	// the user that oblique imagery is now available.  It is only displayed in ortho modes.
	var obliqueNotification = null;
	
	// The oblique selector is the 3x3 set of images representing neighboring images and the current image.
	// It also includes up to 3 links below for rotated images in roughly the same spot.
	var obliqueSelector = null;
	
	var containerTable = document.createElement("table");
	containerTable.className = "dashboardContainerTable";
	var containerTBody = document.createElement("tbody");
	var compassZoomRow = document.createElement("tr");
	containerTable.appendChild(containerTBody);
	containerTBody.appendChild(compassZoomRow);
	
	var compassCell  = document.createElement("td");
	
	var zoomCell = document.createElement("td");
	zoomCell.vAlign = "top";
	
	var notificationCell = document.createElement("td");
	notificationCell.colSpan = 2;
	notificationCell.setAttribute("colspan", 2);
	notificationCell.className = "obliqueSelectorTable";
	
	compassZoomRow.appendChild(compassCell);
	compassZoomRow.appendChild(zoomCell);
	
	var notificationRow = document.createElement("tr");
	notificationRow.appendChild(notificationCell);
	
	containerTBody.appendChild(notificationRow);
		
	// Creates the control and sets up the event listeners
	this.Init = function()
	{
		var mapStyle = map.GetMapStyle();
		
		if ( Msn.VE.Marketization.IsEnabled( MapControl.Features.MapStyle.Road ) )
			CreateTab(roadTab, L_DashboardRoad_Text , OnRoad);
		if ( Msn.VE.Marketization.IsEnabled( MapControl.Features.MapStyle.Aerial ) )
			CreateTab(aerialTab, L_DashboardAerial_Text, OnAerial);
		if ( Msn.VE.Marketization.IsEnabled( MapControl.Features.MapStyle.BirdsEye ) )
			CreateTab(obliqueTab, L_DashboardBirdsEye_Text , OnOblique); 

		separator.className = "Dashboard_separator";
		element.appendChild(separator);
		
		element.appendChild(containerTable);
		if (!size || size == Msn.VE.DashboardSize.Normal)
		{
			compass = new Compass(compassCell, 5, 30);
			compass.Init();
			compass.Show();
		}	    
	    
		obliqueCompass = new ObliqueCompass(compassCell);
		obliqueCompass.Hide();
		
		orthoZoom = new OrthoZoomBar(zoomCell, 100, 30);
		orthoZoom.Init();
		
		obliqueZoom = new ObliqueZoomBar(zoomCell);
		
		//Hide the ToggleGlyph by default
		this.HideToggleGlyph();
		
		// Create show labels option only if supported
		if ( Msn.VE.Marketization.IsEnabled( MapControl.Features.MapStyle.Hybrid ) )
		{
			CreateLabelsToggle();
		}
		// create Oblique selector only if supported
		if ( Msn.VE.Marketization.IsEnabled( MapControl.Features.MapStyle.BirdsEye ) )
		{
			CreateObliqueNotification();		
			obliqueSelector = new ObliqueSelector(notificationCell);
			obliqueSelector.Init();
		}	
		
		map.AttachEvent("onendzoom", OnZoom);
		map.AttachEvent("onchangemapstyle", OnChangeMapStyle);
		
		// attach event to the Oblique mode only if supported
		if ( Msn.VE.Marketization.IsEnabled( MapControl.Features.MapStyle.BirdsEye ) )
		{
			map.AttachEvent("onobliqueenter", OnObliqueEnter);
			map.AttachEvent("onobliqueleave", OnObliqueLeave);
			map.AttachEvent("onobliquechange", OnObliqueChange);
		}	
		// Enable/Disable Oblique only if supported
		if ( Msn.VE.Marketization.IsEnabled( MapControl.Features.MapStyle.BirdsEye ) )
		{
			if (map.IsObliqueAvailable())
			{
				EnableOblique();
			}
			else
			{
				DisableOblique();
			}
		}
		
		switch (mapStyle)
		{
			case 'r':
				SetRoadTab();
				break;
			
			case 'a':
			case 'h':
				SetAerialTab();
				break;
				
			case 'o':
				SetObliqueTab();
				break;
		}
	};
	
	// Destroys the dashboard and attempts to free up as much memory as possible.
	// Unhooks all events and dereferences all objects.
	this.Destroy = function()
	{
		// Clean up the tabs
		if ( Msn.VE.Marketization.IsEnabled( MapControl.Features.MapStyle.Road ) )
			DestroyTab(roadTab, OnRoad);
		if ( Msn.VE.Marketization.IsEnabled( MapControl.Features.MapStyle.Aerial ) )
			DestroyTab(aerialTab, OnAerial);
		if ( Msn.VE.Marketization.IsEnabled( MapControl.Features.MapStyle.BirdsEye ) )
			DestroyTab(obliqueTab, OnOblique);
		
		selectedTab = roadTab = aerialTab = obliqueTab = null;
		
		header.removeChild( toggleGlyphElement );
		element.removeChild( header );		
		
		header = toggleGlyphElement = toggleGlyph = null;
		
		// detach events and elements for show labels option only if supported
		if ( Msn.VE.Marketization.IsEnabled( MapControl.Features.MapStyle.Hybrid ) )
		{
			// The toggle labels UI
			labelsContainer.removeChild(labelsCheckbox);
			zoomCell.removeChild(labelsContainer);
			labelsContainer.detachEvent("onclick", OnToggleLabels);
			labelsContainer = labelsCheckbox = null;
		}
		
		// Destroy the compass
		if (compass)
		{
			compass.Destroy();
			compass = null;
		}
		
		if ( obliqueCompass )
		{		
			obliqueCompass.Destroy();
			obliqueCompass = null;
		}
		if ( orthoZoom )
		{
			orthoZoom.Destroy();
			orthoZoom = null;
		}
		if(obliqueZoom)
		{
			obliqueZoom.Destroy();
			obliqueZoom = null;
		}
		
		if ( obliqueNotification )
		{
			obliqueNotification.detachEvent("onclick", OnOblique);
			obliqueNotification = null;
		}	
		// Destroy the oblique imagery selector
		if (obliqueSelector)
		{
			obliqueSelector.Destroy();
			obliqueSelector = null;
		}
		
		// Unhook from map events and null out the reference to the map
		if (map)
		{
			map.DetachEvent("onendzoom", OnZoom);
			map.DetachEvent("onchangemapstyle", OnChangeMapStyle);
			
			// detachevent to the Oblique mode only if supported
			if ( Msn.VE.Marketization.IsEnabled( MapControl.Features.MapStyle.BirdsEye ) )
			{
				map.DetachEvent("onobliqueenter", OnObliqueEnter);
				map.DetachEvent("onobliqueleave", OnObliqueLeave);
				map.DetachEvent("onobliquechange", OnObliqueChange);
			}	
			
			map = null;
		}
		
	};
	
	
	// Added by sstchur:
	// Sets the X position of the dashboard	
	this.SetX = function(xPos)
	{
		element.style.left = xPos + 'px';
	};
	
	// Added by sstchur:
	// Get a reference to the Dashboard element
	this.GetElement = function()
	{
		return element;
	};
	
	this.GetHeader = function()
	{
		return header;
	};
	
	// Returns the top of the dashboard element
	this.GetY = function()
	{
		return GetTopPosition(element);
	};
	
	// Returns the height of the dashboard element
	this.GetHeight = function()
	{
		return element.offsetHeight;
	};
	
	// Show the Toggle Glyph on Dashboard
	this.ShowToggleGlyph = function()
	{
		if ((toggleGlyphElement != null) && (toggleGlyphElement != "undefined"))
		{
			toggleGlyphElement.style.display = "block";
		}
	};

	// Hide the Toggle Glyph on Dashboard
	this.HideToggleGlyph = function()
	{
		if ((toggleGlyphElement != null) && (toggleGlyphElement != "undefined"))
		{
			toggleGlyphElement.style.display = "none";
		}
	};

	// Creates a new tab element
	function CreateTab(tab, name, callback, selected)
	{
		tab.className = "Dashboard_tab Dashboard_unselected";
		tab.href = "javascript:void(0)";
		tab.innerText = name ; 		
		tab.attachEvent("onclick", callback);

		var lastChild = header.lastChild;
		if (lastChild && typeof lastChild != 'undefined')
			{ header.insertBefore(tab, lastChild); }
		else
			{ header.appendChild(tab); }
	}
	
	// Destroys a tab
	function DestroyTab(tab, callback)
	{	
		tab.detachEvent("onclick", callback);
		//Two levels of checks
		//null check for header
		//removeChild exceptions 
		if(header != null)
		{	try
			{		
				header.removeChild(tab);
			}
			catch(exp)
			{
				//swallow
			}
		}
	}
	
	// Selects a tab.  If a different tab was previously selected, it
	// reverts it to unselected state.
	function SelectTab(tab)
	{
		if (selectedTab)
		{
			selectedTab.className = "Dashboard_tab Dashboard_unselected";
		}
		
		tab.className = "Dashboard_tab Dashboard_selected";
		selectedTab = tab;
	}
	
	// Creates the labels toggler, which is only used in aerial mode.  The toggle
	// is a way for users to more easily understand the difference between
	// aerial and hybrid map styles.
	function CreateLabelsToggle()
	{
		labelsCheckbox.type = "checkbox";
		labelsContainer.className = "Dashboard_toggler";
		labelsContainer.style.display = "none";
		labelsContainer.attachEvent("onclick", OnToggleLabels);
		labelsContainer.appendChild(labelsCheckbox);
		labelsContainer.appendChild(document.createTextNode(L_DashboardShowLabels_Text));
		zoomCell.appendChild(labelsContainer);
	}

	// Creates the oblique notification area.  The notification shows a thumbnail
	// of the best image for the current location and a short message.  If the user
	// clicks on the notification, the map should change to oblique mode, just as if
	// the user had clicked on the oblique link.
	function CreateObliqueNotification()
	{
		obliqueNotification = document.createElement("div");
		obliqueNotification.className = "Dashboard_notification";
		obliqueNotification.innerHTML = "<div id=\"obliqueNotifyIcon\">&nbsp;</div> " + L_DashboardBirdsEyeText_Text;
		obliqueNotification.style.display = "none";
		obliqueNotification.attachEvent("onclick", OnOblique);
		notificationCell.appendChild(obliqueNotification);
	}
	
	// Sets the road tab.  
	// * Turns off the label toggler if it's visible.  
	// * Turns off the oblique selector if it's visible.
	// * Turns on the oblique notification if it's applicable.
	function SetRoadTab()
	{	
		SelectTab(roadTab);		
		
		// hide show labels option only if supported
		if ( Msn.VE.Marketization.IsEnabled( MapControl.Features.MapStyle.Hybrid ) )
		{
			labelsContainer.style.display = "none";
		}	
		
		if ( obliqueSelector )
			obliqueSelector.Hide();
		obliqueZoom.Hide();
		orthoZoom.Show();
		
		// display Birds eye view only if supported
		if (map.IsObliqueAvailable() && Msn.VE.Marketization.IsEnabled( MapControl.Features.MapStyle.BirdsEye ) )
		{
			obliqueNotification.style.display = "block";
			
		}
		if (compass)
		{
			compass.Show();
		}
		obliqueCompass.Hide();
	}
	
	// Sets the aerial tab
	// * Turns on the label toggler if it's visible.  
	// * Turns off the oblique selector if it's visible.
	// * Sets the checkbox to the appropriate value
	// * Turns on the oblique notification if it's applicable.
	function SetAerialTab()
	{	
		SelectTab(aerialTab);
		
		// display show labels option only if supported
		if ( Msn.VE.Marketization.IsEnabled( MapControl.Features.MapStyle.Hybrid ) )
		{
			labelsContainer.style.display = "block";
		}	
		if ( obliqueSelector ) 
			obliqueSelector.Hide();
		obliqueZoom.Hide();
		orthoZoom.Show();
		
		// Check/uncheck "show labels" option only if supported
		if ( Msn.VE.Marketization.IsEnabled( MapControl.Features.MapStyle.Hybrid ) )
		{
			if (map.GetMapStyle() == 'h'  )
			{
				labelsCheckbox.checked = true;
			}
			else
			{
				labelsCheckbox.checked = false;
			}
		}	
		
		// display Birds eye view only if supported
		if (map.IsObliqueAvailable() && Msn.VE.Marketization.IsEnabled( MapControl.Features.MapStyle.BirdsEye ) )
		{
			obliqueNotification.style.display = "block";
		}
		if (compass)
		{
			compass.Show();
		}
		obliqueCompass.Hide();
	}
	
	// Sets the oblique tab
	// * Turns off the label toggler if it's visible. 
	// * Turns off the oblique notification if it's applicable. 
	// * Turns on the oblique selector if it's visible.
	function SetObliqueTab()
	{
		SelectTab(obliqueTab);
		labelsContainer.style.display = "none";
		obliqueNotification.style.display = "none";
		obliqueSelector.Show();
		orthoZoom.Hide();
		obliqueZoom.Show();
		if (compass)
		{
			compass.Hide();
		}
		obliqueCompass.Show();
	}
	
	// Enables the oblique link and shows the oblique notification.
	function EnableOblique()
	{
		obliqueTab.className = "Dashboard_tab Dashboard_unselected";
		obliqueNotification.style.display = "block";
	}
	
	// Disables the oblique link and hides the oblique notification
	function DisableOblique()
	{
		obliqueTab.className = "Dashboard_tab Dashboard_disabled";
		obliqueNotification.style.display = "none";
	}
	
	// Handler for the road link.  Sends the message to the map control and
	// updates the dashboard UI.
	function OnRoad()
	{	
		map.SetMapStyle('r');
	}
	
	// Handler for the aerial link.  Sends the message to the map control and
	// updates the dashboard UI.
	function OnAerial()
	{
		// set Mapstyle to hybrid only if supported
		if ( (firstAerial || labelsCheckbox.checked) && Msn.VE.Marketization.IsEnabled( MapControl.Features.MapStyle.Hybrid ) )
		{
			map.SetMapStyle('h');
		}
		else
		{
			map.SetMapStyle('a');
		}
		firstAerial = false;
	}
	
	// Handler for the oblique link.  Sends the message to the map control and
	// updates the dashboard UI.
	function OnOblique()
	{
		map.SetMapStyle('o');
	}
	
	// Handler for the labels checkbox when in aerial mode.  The labels checkbox
	// is a simple toggle between aerial and hybrid map styles.
	function OnToggleLabels(e)
	{	
		var mapStyle = map.GetMapStyle();
		if (mapStyle == 'a')
		{
			map.SetMapStyle('h');
			labelsCheckbox.checked = true;
		}
		else if (mapStyle == 'h')
		{
			map.SetMapStyle('a');
			labelsCheckbox.checked = false;
		}
	}
	
	// Handler for when the map control zooms.  Updates the appropriate zoom widgets.
	function OnZoom(e)
	{
		orthoZoom.UpdateFromMap();
		obliqueZoom.UpdateFromMap();
	}

	// Handler for when oblique imagery becomes available.  Enables links, shows the notification.
	function OnObliqueEnter(e)
	{
		if (map.GetMapStyle() == "o")
		{
			return;
		}
	
		EnableOblique();
		obliqueCompass.UpdateFromMap();
	}
	
	// Handler for when oblique is no longer available.  Disables links, hides the notification.
	function OnObliqueLeave(e)
	{
		DisableOblique();
	}
	
	// Handler for when the map control changes map styles by another means (perhaps keyboard shortcuts 
	// or another UI control).  Updates the UI accordingly
	function OnChangeMapStyle(e)
	{
		switch (e.view.mapStyle)
		{
			case 'r':
				SetRoadTab();
				break;
			
			case 'a':
			case 'h':
				SetAerialTab();
				break;
				
			case 'o':
				SetObliqueTab();
				break;
		}
	}
	
	// Handler for when oblique changes images.  Update the selector's neighbor and rotation links
	function OnObliqueChange(e)
	{
		obliqueSelector.Update();
		obliqueCompass.UpdateFromMap();
	}
	
#include Compass.js
#include ObliqueSelector.js
#include OrthoZoomBar.js
#include ObliqueZoomBar.js
#include ObliqueCompass.js
};
