//------------------------------------------------------------------------------
// MapControl/Dashboard/ObliqueSelector.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: a-bwrigh
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------


/** Class represents a rotatable compass point 
  *
  */
function ObliqueCompassPoint(id,dir,pos)
{
	var element = document.createElement("div");
	element.setAttribute("id",id);
	element.innerHTML = dir;
	var previousClass = "obliqueCompassPointOff";
	
	var currentPositionIndex = pos;
		
	element.attachEvent("onmouseover", OnMouseOver);
	element.attachEvent("onmouseout", OnMouseOut);
	element.attachEvent("onclick", OnClick);	
	
	this.onclick     = null;
	this.onmouseover = null;
	this.onmouseout  = null;	
	
	var me = this;
	
	var disabled = false;
	
	var rotator = new Rotator(element,17,17);
	
	/* handle mouse over events. Ignores mouse over events
	 * if compass point is disabled 
	 */
	function OnMouseOver(e)
	{
		if(disabled)
		{
			return ;
		}
		previousClass = element.className;
		element.className = "obliqueCompassPointHover";
		if(me.onmouseover)
		{
			me.onmouseover(e);
		}
	}
	/* handle mouse onclick events. Ignores mouse click events
	 * if compass point is disabled 
	 */
	function OnClick(e)
	{		
		if(disabled)
		{
			return ;
		}
		OnMouseOver(e);
				
		previousClass =  "obliqueCompassPointOn";
		if(me.onclick)
		{
			me.onclick(e);
		}
	}
	/* handle mouse out events. Ignores mouse out events
	 * if compass point is disabled 
	 */
	function OnMouseOut(e)
	{
		if(disabled)
		{
			return ;
		}
		element.className = previousClass;
		if(me.onmouseout)
		{
			me.onmouseout(e);
		}
	}
	
	/**   
	  * set the point to on state
      */
	function On()
	{		
		previousClass =  "obliqueCompassPointOn";
		element.className = "obliqueCompassPointOn";
	}
	/**   
	  * set the point to off state
      */
	function Off()
	{		
		previousClass =  "obliqueCompassPointOff";
		element.className = "obliqueCompassPointOff";
	}
	
	/**
	  * Disable the point to ignore mouse events
	  */
	function Disable()
	{
		disabled = true;
		Off();
	}
	/**
	  * enable the point to accept mouse events
	  */
	function Enable()
	{
		disabled = false;
		On();
	}
	
	function GetElement()
	{
		return element;
	}
	
	function GetCurrentPositionIndex()
	{
		return currentPositionIndex;
	}
	
	function SetCurrentPositionIndex(pos)
	{
		currentPositionIndex = pos;
	}
	/**
	  * Set the point to it's initial state and enable it
	  */
	function Reset()
	{
		rotator.Reset();
		Enable();
	}
	
	function SetCurrentPosition(position)
	{
		element.style.left = position.left;
		element.style.top  = position.top;
		
	}
	
	
	/**
	  * Class used to animation compass point in a circle
	  * param fulcrumX and fulcrumY is the center point
	  * around which the compass point will be rotated.
	  */
	function Rotator(elm,fulcrumX,fulcrumY)
	{
		var element = elm;
		var currentAngle = 0;
		var deltaAngle =  0.3;
		var targetAngle = 0;
		var deltaTime = 40;
		var radius = 25;
		var animating = false;
		var quartCircle = Math.PI / 2; //radians
		var isCCW = true;
		var limit = deltaAngle+0.1;
		
		/**
		  * Utility function to test if val1 is within 
	      * range of val2 using limit
	      */
	  
		function IsInRange(limit,val1,val2)
		{
			if(val1 >= val2-limit && val1 <= val2+limit)
			{
				return true;
			}
			return false;
		}
		
		/**
		  * Run loop that handles circular animation
		  */
		function Run()
		{			
			currentAngle+=deltaAngle;	
			if(currentAngle > Math.PI*2)
					currentAngle -= Math.PI*2;
			else if(currentAngle < 0)
			{
				currentAngle += Math.PI*2
			}
			if(IsInRange(limit,currentAngle,targetAngle))
			{		
				deltaAngle = 0.3;
				currentAngle = targetAngle;
				
				MoveToAngle(currentAngle);
				animating = false;
				return;
			}
			MoveToAngle(currentAngle);
			window.setTimeout(Run,deltaTime);
		}
		
		/**
		  * Move the element to the specified angle
		  * angle is in radians.
		  */
		function MoveToAngle(angle)
		{
			var x =  fulcrumX + (radius*Math.sin(angle));
			var y = fulcrumY + (radius*Math.cos(angle));
			element.style.left = x+"px";
			element.style.top = y+"px";
			
		}
		
		/**
		  * Kick off an animation to rotate the 
		  * element to the param angle
		  */
		function RotateTo(angle)
		{
			if(animating)
			return;
			animating = true;
			targetAngle = angle;
			Run();
		}
		
		/**
		  * rotate element to specified index 
		  * CCW is a bool value to define CCW or CW rotation
		  * index values are as follows:
		  * 0 - bottom
		  * 1 - right
		  * 3 - top
		  * 4 - left
		  */
		function RotateToIndex(index,CCW)
		{		
			isCCW = CCW;
			if(!isCCW)
			{   //negate the angle
				deltaAngle*=-1;
			}
			RotateTo(quartCircle*index);
		}
		
		function Reset()
		{
			animating = false;
			currentAngle = 0;
		}
		this.RotateTo = RotateTo;
		this.RotateToIndex = RotateToIndex;
		this.Reset = Reset;
	}	
	
	function Destroy()
	{
		element.parentNode.removeChild(element);
		element.detachEvent("onmouseover", OnMouseOver);
		element.detachEvent("onmouseout", OnMouseOut);
		element.detachEvent("onclick", OnClick);			
		element = null;
	}
	
	this.GetElement = GetElement;
	this.GetCurrentPositionIndex = GetCurrentPositionIndex;
	this.SetCurrentPositionIndex = SetCurrentPositionIndex;
	this.SetCurrentPosition = SetCurrentPosition;
	this.On = On;
	this.Off = Off;
	this.RotateToIndex = rotator.RotateToIndex;
	this.Reset = Reset;
	this.Disable = Disable;
	this.Enable = Enable;	
	this.Destroy = Destroy;
}

/**
  * Class: ObliqueCompass
  * UI Widget that manages the Oblique compass view 
  * and controls the rotation of the Oblique map
  */
function ObliqueCompass(container)
{
	var pointPositions = new Array();
	pointPositions.push({"top":"-8px","left":"17px"});
	pointPositions.push({"top":"17px","left":"42px"});
	pointPositions.push({"top":"42px","left":"17px"});
	pointPositions.push({"top":"17px","left":"-8px"});	
	
	var compassContainer = document.createElement("div");
	compassContainer.setAttribute("id","obliqueCompassContainer");
	compassContainer.title = L_ObliqueCompassSelectDirection_Text;	
	
	var northElm = new ObliqueCompassPoint("obliqueCompassPointN","N",0);
	var northElmRef = northElm.GetElement();
	northElmRef.attachEvent("onclick", OnNorthClick);
	northElmRef.attachEvent("onmouseover", OnNorthOver);
	northElmRef.attachEvent("onmouseout", SetArrowUp);
	
	var eastElm = new ObliqueCompassPoint("obliqueCompassPointE","E",1);
	var eastElmRef = eastElm.GetElement();
	eastElmRef.attachEvent("onclick", OnEastClick);
	eastElmRef.attachEvent("onmouseover", OnEastOver);
	eastElmRef.attachEvent("onmouseout", SetArrowUp);
			
	var southElm = new ObliqueCompassPoint("obliqueCompassPointS","S",2);
	var southElmRef = southElm.GetElement();
	southElmRef.attachEvent("onclick", OnSouthClick);
	southElmRef.attachEvent("onmouseover", OnSouthOver);
	southElmRef.attachEvent("onmouseout", SetArrowUp);
	
	var westElm = new ObliqueCompassPoint("obliqueCompassPointW","W",3);
	var westElmRef = westElm.GetElement();
	westElmRef.attachEvent("onclick", OnWestClick);
	westElmRef.attachEvent("onmouseover", OnWestOver);
	westElmRef.attachEvent("onmouseout", SetArrowUp);
		
	var arrowElm = document.createElement("div");
	
	compassContainer.appendChild(northElmRef);
	compassContainer.appendChild(eastElmRef);
	compassContainer.appendChild(southElmRef);
	compassContainer.appendChild(westElmRef);
	compassContainer.appendChild(arrowElm);
	
	container.appendChild(compassContainer);
	
	// Update the position from the map
	UpdateFromMap();

	/**
	  * Make sure the position of the points is always within range
	  */
	function WrapPosition(pos)
	{
		if(pos < 0)
		{
			pos = 4 - Math.abs(pos);
		}
		return pos;
	}
	
	/**
	  * Rotate all 4 points based on the current position of "compassPoint"
	  * If the point that was clicked is at position 3 then all points will
	  * be rotate CCW otherwise they'll be rotated CW.
	  */
	function RotatePointACWToTop(compassPoint)
	{
		var pos = compassPoint.GetCurrentPositionIndex();
		
		eastElm.SetCurrentPositionIndex(WrapPosition(eastElm.GetCurrentPositionIndex()-pos));
		westElm.SetCurrentPositionIndex(WrapPosition(westElm.GetCurrentPositionIndex()-pos));
		northElm.SetCurrentPositionIndex(WrapPosition(northElm.GetCurrentPositionIndex()-pos));
		southElm.SetCurrentPositionIndex(WrapPosition(southElm.GetCurrentPositionIndex()-pos));		
		var ccw = true;
		if(pos == 3)
		{
			ccw = false;
		}

		northElm.RotateToIndex(2-northElm.GetCurrentPositionIndex() < 0 ? northElm.GetCurrentPositionIndex(): 2-northElm.GetCurrentPositionIndex(),ccw);
		eastElm.RotateToIndex(2-eastElm.GetCurrentPositionIndex() < 0? eastElm.GetCurrentPositionIndex() : 2-eastElm.GetCurrentPositionIndex(),ccw);
		southElm.RotateToIndex(2-southElm.GetCurrentPositionIndex() < 0? southElm.GetCurrentPositionIndex(): 2-southElm.GetCurrentPositionIndex(),ccw );
		westElm.RotateToIndex(2-westElm.GetCurrentPositionIndex() < 0 ? westElm.GetCurrentPositionIndex(): 2-westElm.GetCurrentPositionIndex(),ccw);
	}
	
	/**
	  * Set the compass arrow direction based on the index value
	  * 0: up
	  * 1: right
	  * 2: down
	  * 3: left
	  */
	function SetArrowDirectionForIndex(index)
	{
		switch(index)
		{
			case 0 : SetArrowUp();break;
			case 1 : SetArrowRight();break;
			case 2 : SetArrowDown();break;
			case 3 : SetArrowLeft();break;
		}
	}
	
	function SetArrowUp()
	{
		arrowElm.className = "obliqueCompassArrowU";
	}
	
	function SetArrowDown()
	{
		arrowElm.className = "obliqueCompassArrowD";
	}
	
	function SetArrowRight()
	{
		arrowElm.className = "obliqueCompassArrowR";
	}
	
	function SetArrowLeft()
	{
		arrowElm.className = "obliqueCompassArrowL";
	}
	
	/**
	  * Handle south point clicked. This kicks off compass rotation
	  */
	function OnSouthClick(e)
	{
		RotatePointACWToTop(southElm);
		SetArrowDirectionForIndex(0);
		var scene = map.GetObliqueScene();
		if(scene)
		{
			var aScene = scene.GetRotation("South");
			if(aScene)
			{
				//map.SetObliqueScene(aScene.GetID());
				map.SetObliqueOrientation("South");
			}
		}
	}
	
	/**
	  * Handle north point clicked. This kicks off compass rotation
	  */
	function OnNorthClick(e)
	{
		RotatePointACWToTop(northElm);	
		SetArrowDirectionForIndex(0);
		var scene = map.GetObliqueScene();
		if(scene)
		{
			var aScene = scene.GetRotation("North");
			if(aScene)
			{
				//map.SetObliqueScene(aScene.GetID());
				map.SetObliqueOrientation("North");
			}
		}
	}
	
	/**
	  * Handle east point clicked. This kicks off compass rotation
	  */
	function OnEastClick(e)
	{
		RotatePointACWToTop(eastElm);
		SetArrowDirectionForIndex(0);
		var scene = map.GetObliqueScene();
		if(scene)
		{
			var aScene = scene.GetRotation("East");
			if(aScene)
			{
				//map.SetObliqueScene(aScene.GetID());
				map.SetObliqueOrientation("East");
			}
		}
	}
	/**
	  * Handle west point clicked. This kicks off compass rotation
	  */
	function OnWestClick(e)
	{
		RotatePointACWToTop(westElm);
		SetArrowDirectionForIndex(0);
		var scene = map.GetObliqueScene();
		if(scene)
		{
			var aScene = scene.GetRotation("West");
			if(aScene)
			{
				//map.SetObliqueScene(aScene.GetID());
				map.SetObliqueOrientation("West");
			}
		}
	}
	
	function OnSouthOver(e)
	{
		SetArrowDirectionForIndex(southElm.GetCurrentPositionIndex());
	}
	
	function OnEastOver(e)
	{
		SetArrowDirectionForIndex(eastElm.GetCurrentPositionIndex());
	}
	
	function OnWestOver(e)
	{
		SetArrowDirectionForIndex(westElm.GetCurrentPositionIndex());
	}
	
	function OnNorthOver(e)
	{
		SetArrowDirectionForIndex(northElm.GetCurrentPositionIndex());
	}
	
	function Hide()
	{
		compassContainer.style.display = "none";
	}
	
	function Show()
	{
		compassContainer.style.display = "block";
	}
	
	/**
	  * When the oblique view changes we need to make sure
	  * the compass is in the correct state. Enable all points
	  * that have valid scenes
	  */
	function UpdateFromMap()
	{
		var scene = map.GetObliqueScene();
		if (!scene)
		{
			return;
		}
		
		switch (scene.GetOrientation())
		{
			case "North":
				RotatePointACWToTop(northElm);
				break;
				
			case "South":
				RotatePointACWToTop(southElm);
				break;
				
			case "East":
				RotatePointACWToTop(eastElm);
				break;
				
			case "West":
				RotatePointACWToTop(westElm);
				break;
		}
		
		// Set the arrow up
		SetArrowDirectionForIndex(0);
		
		if (scene.GetRotation("North"))
		{
			northElm.Enable();
		}
		else
		{
			northElm.Disable();
		}
		
		if (scene.GetRotation("South"))
		{
			southElm.Enable();
		}
		else
		{
			southElm.Disable();
		}		
		
		if (scene.GetRotation("East"))
		{
			eastElm.Enable();
		}
		else
		{
			eastElm.Disable();
		}
		
		if (scene.GetRotation("West"))
		{
			westElm.Enable();
		}
		else
		{
			westElm.Disable();
		}
	}
	
	function Destroy()
	{
		northElm.Destroy();
		eastElm.Destroy();
		southElm.Destroy();
		westElm.Destroy();
		
		northElmRef.detachEvent("onclick", OnNorthClick);
		northElmRef.detachEvent("onmouseover", OnNorthOver);
		northElmRef.detachEvent("onmouseout", SetArrowUp);
		
		eastElmRef.detachEvent("onclick", OnEastClick);
		eastElmRef.detachEvent("onmouseover", OnEastOver);
		eastElmRef.detachEvent("onmouseout", SetArrowUp);
			
		southElmRef.detachEvent("onclick", OnSouthClick);
		southElmRef.detachEvent("onmouseover", OnSouthOver);
		southElmRef.detachEvent("onmouseout", SetArrowUp);
	
		westElmRef.detachEvent("onclick", OnWestClick);
		westElmRef.detachEvent("onmouseover", OnWestOver);
		westElmRef.detachEvent("onmouseout", SetArrowUp);
	
		compassContainer.parentNode.removeChild(compassContainer);
		compassContainer = null;
	}

	this.Hide = Hide;
	this.Show = Show;	
	this.UpdateFromMap = UpdateFromMap;
	this.Destroy = Destroy;	
}
