//------------------------------------------------------------------------------
// MapControl/Events.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: CodyE, keithkin
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

// Creates a new map control event
function MapEvent(view, oblique, error)
{
	this.view = view;
	this.oblique = oblique;
	this.error = error;
}

// Creates a new map control event based on the current map settings.
function CreateEvent(latlong, zoomLevel, error)
{
	var view = currentView.MakeCopy();
	
	if (latlong != undefined)
	{
		//If running in API mode, use a VELatLong in the event
		if (Msn.VE.API != null)
		{
			//Instantiate a VELatLongFactory from a specification created from the MapView.
			//Generate a VELatLong that may be encoded if the current map mode is oblique.
			//Use the capital LatLong property notation to be consistent with API
			var veLatLongFactory = new VELatLongFactory(new VELatLongFactorySpecFromMapView(view));
			view.LatLong = veLatLongFactory.CreateVELatLong(latlong.latitude, latlong.longitude);
		}
		else
		{
			view.latlong = latlong;
		}
	}
	
	if (zoomLevel != undefined)
	{
		view.zoomLevel = zoomLevel;
	}
	
	if (error == undefined)
	{
		error = "";
	}
	
	var oblique = null;
	if (obliqueMode)
	{
		oblique = obliqueMode.GetEventInfo();
	}
	
	return new MapEvent(view, oblique, error);
}

// Attaches an event handler to an event
function AttachEvent(name, callback)
{
	var eventList = eventTable[name];
	
	if (!eventList)
	{
		eventList = new Array();
		eventTable[name] = eventList;
	}
	
	for (var i=0; i<eventList.length; i++)
	{
		if (eventList[i] == callback)
		{
			return true;
		}
	}
	
	eventList.push(callback);
}

// Detaches an event handler to an event
function DetachEvent(name, callback)
{
	var eventList = eventTable[name];
	
	if (!eventList)
	{
		return;
	}
	
	for (var i=0; i<eventList.length; i++)
	{
		if (eventList[i] == callback)
		{
			eventList.splice(i, 1);
		}
	}
}

// Fires an event
function Fire(name, args)
{
	var eventList = eventTable[name];
	
	if (!eventList)
	{
		return;
	}
	
	if (!args)
	{
		args = CreateEvent();
	}
	
	for (var i=0; i<eventList.length; i++)
	{
		eventList[i](args);
	}
}

// Destroys the event handler table.  This is key for releasing a lot of memory.
function DestroyEventTable()
{
	while (eventTable.length)
	{
		var eventList = eventTable.pop();
		while (eventList.length)
		{
			eventList.pop();
		}
		eventList = null;
	}
	
	eventTable = null;
}
