//------------------------------------------------------------------------------
// MapControl/Mouse.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: CodyE, sstchur
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

// Handles the mouse down event.  This begins a drag-pan session.
function MouseDown(e)
{
	e = GetEvent(e);	
	CancelEvent(e);
			
	// If the control is zooming, then return and ignore
	if (zooming)
	{
		return false;
	}
	
	// If the control is panning, then stop panning
	if (panning)
	{
		StopContinuousPan();
	}
	
	// If the control has oblique capabilities, cancel any availability requests
	// This is to prevent flooding the server with oblique update checks.
	if (obliqueMode)
	{
		obliqueMode.CancelRequest();
	}
	
	// If this was the middle button, use the box tool instead
	// firefox	
	if (e.which && e.which == 2)
	{
		currentTool = boxTool;
	}
	// ie
	else if (!e.which && e.button && e.button == 4)
	{
		currentTool = boxTool;		
	}
	// general
	else if (e.altKey)
	{
		currentTool = boxTool;
	}
	
	dragging = true;
	
	currentTool.OnMouseDown(e);
	
	return false;
}

// Handles mouse move.  If drag-panning, this will pan the map.
function MouseMove(e)
{
	e = GetEvent(e);	
	CancelEvent(e);
	
	if (dragging)
	{
		currentTool.OnMouseMove(e);
	}

	return false;
}

// Handles the mouse up event.  This ends a drag-pan session.
function MouseUp(e)
{
	e = GetEvent(e);
	CancelEvent(e);
	
	dragging = false;	
	currentTool.OnMouseUp(e);
	currentTool = panTool;
	
	// Get focus for keyboard control
	try
	{
		var y = window.pageYOffset;
		keyboard.focus();
		if (typeof y == 'number')
			{ window.scrollTo(0, y); }
	}
	catch(err)
	{
	}
	return false;
}

// Handles mouse double click.  This initiates an angled animated zoom.
function MouseDoubleClick(e)
{
	e = GetEvent(e);
	CancelEvent(e);
	
	UpdateFromParent();
	
	// If the control is panning or zooming, then return and ignore
	if (panning || zooming)
	{
		return false;
	}

	var view = preferredView.MakeCopy();
	view.SetCenter(new Msn.VE.Pixel(originX + offsetX + GetMouseX(e) - x, originY + offsetY + GetMouseY(e) - y));

	// If the alt key is pressed, zoom out.  Otherwise, zoom in.
	if (e.altKey)
	{
		view.SetZoomLevel(currentView.zoomLevel - 1);
	}
	else
	{
		view.SetZoomLevel(currentView.zoomLevel + 1);
	}
	
	SetView(view);
	return false;
}

// Handles the mouse wheel.  This will zoom in or out.
function MouseWheel(e)
{
	e = GetEvent(e);
	CancelEvent(e);
	
	// If the control is panning or zooming, then return and ignore
	if (panning || zooming)
	{
		return false;
	}
	
	var delta = GetMouseScrollDelta(e);
	if (delta > 0) 
	{
		ZoomIn();
	}
	else if (delta < 0)
	{
		ZoomOut();
	}
	
	return false;
}

// Handles the context menu event.  
function ContextMenu(e)
{
	e = GetEvent(e);
	CancelEvent(e);
	
	var clickX = originX + offsetX + GetMouseX(e) - x;
	var clickY = originY + offsetY + GetMouseY(e) - y;
	var clickEvent = CreateEvent(currentMode.PixelToLatLong(new Msn.VE.Pixel(clickX, clickY), currentView.zoomLevel));
	Fire("oncontextmenu", clickEvent);
		
	return false;
}
