//------------------------------------------------------------------------------
// MapControl/Pushpin.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: CodyE
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

function PushPinOffset(x,y)
{
	this.x = x;
	this.y = y;
}

function getPushPinOffset(poiType)
{
	// Shortcut the Msn.VE.PushPinTypes Namespace Chaind
	var pinType = Msn.VE.PushPinTypes;
	
	var offset;
	switch (poiType)
	{	
		case pinType.Annotation:			
			offset = new PushPinOffset(-2, -29/2 - 5);
			break;
			
		case pinType.SearchResult:
			offset = new PushPinOffset(0, -29/2 + 3);
			break;
			
		case pinType.Direction:
			offset = new PushPinOffset(-3, -26/2 - 3);
			break;
			
		case pinType.DirectionTemp:			
			offset =new PushPinOffset(2, -26/2 + 3);
			break;
		
		case pinType.TrafficLight:
			offset =new PushPinOffset(0, -26/2);
			break;
		
		case pinType.TrafficOthers:
			offset =new PushPinOffset(0, -29/2);
			break;		
		case pinType.YouAreHere:
			offset =new PushPinOffset(0, -26/2);
			break;
		case pinType.Default:
		default:			
			offset = new PushPinOffset(0, 0);
			break;
	}
	
	return offset;
}

// Get the pushpins
function GetPushpins()
{
	return pushpins;
}

// Adds a pushpin to the map
function AddPushpin(id, lat, lon, width, height, className, innerHtml, zIndex, pinType)
{
	var pin = new Pushpin();	
	pin.Init(id, lat, lon, width, height, className, innerHtml, zIndex, pinType);	
	pushpins.push(pin);
	
	// TODO:  Determine if this should return the Pushpin object or if it should return the DOM element
	return pin.pin;
}

// Removes a pushpin from the map
function RemovePushpin(id)
{
	for (var i=0; i<pushpins.length; i++)
	{
		var p = pushpins[i];
		if(p.id == id)
		{
			pushpins.splice(i, 1);
			p.Destroy();
			return;
		}
	}
}

// Clears all pushpins from the map
function ClearPushpins()
{
	while(pushpins.length > 0)
		pushpins.pop().Destroy();
}

// Repositions all of the pushpins according to the origin position.
function RepositionPushpins()
{
	for(var i=0; i<pushpins.length; i++)
	{
		pushpins[i].Reposition();
	}
}

// The tile class is a container for a map tile.  It maintains
// a tiles x,y, width, height, and opacity.  A tile has two states:
// current and next.  The tile can be drawn at any factor between 
// those 2 states.
function Pushpin()
{
	var pin = document.createElement("a");
	pin.href = 'javascript://pushin hover';
	pin.onclick = function(){ return ParseShiftKeyForLinks(event); };
	pin.vePushpin = this;
	this.pin = pin;
	
	var x1=0, y1=0, x2=0, y2=0;
	var center = null;
	var w = 0;
	var h = 0;
	var n = zoomTotalSteps + 1;
	var xs = new Array(n);
	var ys = new Array(n);
	var Offset;
	
	// Initializes the pushpin
	this.Init = function(id, lat, lon, width, height, className, innerHtml, zIndex, pinType)
	{
		this.id = id;
		pin.id = id;
		pin.className = className;
		pin.style.position = "absolute";
		//pin.style.zIndex = zIndex;
		pin.innerHTML = innerHtml;
		pin.pinType = pinType || Msn.VE.PushPinTypes.Default;
		
		Offset = getPushPinOffset(pin.pinType);
				
		if(!p_htParams.fixedView)
		{
			pin.attachEvent("ondblclick", DoubleClick);
			pin.attachEvent("onmousewheel", MouseWheel);
		}
		pin.unselectable = "on";

		center = new Msn.VE.LatLong(lat, lon);
		
		w = width;
		h = height;
		
		var pixel = currentMode.LatLongToPixel(center, currentView.zoomLevel);
		if (pixel)
		{
			x1 = MathRound(pixel.x - originX);
			y1 = MathRound(pixel.y - originY);
			x2 = x1;
			y2 = y1;
			PrecomputeSteps();
			SetFactor(0);
			pin.style.display = "block";
		}
		else
		{
			pin.style.display = "none";
		}
		
		map.appendChild(pin);
	};
	
	// Destroys the pushpin and frees all the associated memory
	this.Destroy = function()
	{
		pin.detachEvent("ondblclick", DoubleClick);
		pin.detachEvent("onmousewheel", MouseWheel);
		
		RemoveFromMap();
		pin.vePushpin = null;
		pin = null;
		this.pin = null;
		
		while (xs.length > 0)
		{
			xs.pop();
		}
		
		while (ys.length > 0)
		{
			ys.pop();
		}
	};
	
	// Gets the latitude of the pin.
	this.GetLatitude = function()
	{
		return center.latitude;
	};
	
	// Gets the longitude of the pin.
	this.GetLongitude = function()
	{
		return center.longitude;
	};

	// Set all steps in the array to the "current" state.
	function ClearSteps()
	{
		var n = zoomTotalSteps;
		for (var i=0; i<=n; i++)
		{
			xs[i] = (x1 - w/2 + Offset.x)+"px";
			ys[i] = (y1 - h/2 + Offset.y)+"px";
		}
	}

	// Precomputes the location for each intermediate step
	function PrecomputeSteps()
	{
		var n = zoomTotalSteps;
		for (var i=0; i<=n; i++)
		{
			var a = i / n;
			var b = 1.0 - a;
			xs[i] = MathFloor(b*x1 + a*x2 - w/2 + Offset.x) + "px";
			ys[i] = MathFloor(b*y1 + a*y2 - h/2 + Offset.y) + "px";
		}
	}

	// Updates the tile, somewhere between the current and the next state.
	function SetFactor(i)
	{		
		pin.style.left = xs[i];
		pin.style.top = ys[i];
	}

	// Swaps the current and next states.
	function SwapStates()
	{
		var t=0;
		t = x1; x1 = x2; x2 = t;
		t = y1; y1 = y2; y2 = t;
	}
	
	function Reposition()
	{
		var pixel = currentMode.LatLongToPixel(center, currentView.zoomLevel);
		if (pixel)
		{
			x1 = MathRound(pixel.x - originX);
			y1 = MathRound(pixel.y - originY);
			ClearSteps();
			SetFactor(0);
			pin.style.display = "block";
		}
		else
		{
			pin.style.display = "none";
		}
	}
	
	function PrepareForZoom(newOriginX, newOriginY, newZoom)
	{
		x1 -= offsetX;
		y1 -= offsetY;
		
		var pixel = currentMode.LatLongToPixel(center, newZoom);
		if (pixel)
		{
			x2 = MathRound(pixel.x - newOriginX);
			y2 = MathRound(pixel.y - newOriginY);
			
			PrecomputeSteps();
			pin.style.display = "block";
		}
		else
		{
			pin.style.display = "none";
		}
	}

	// Removes this tile from the control's map element.
	function RemoveFromMap()
	{
		if (pin.parentNode == map)
		{
			map.removeChild(pin);
		}
	}

	// Handles double click
	function DoubleClick(e)
	{
		e = GetEvent(e);
		CancelEvent(e);
		
		// If the control does not exist
		// or the control is panning or zooming
		// return and ignore
		if(panning || zooming)
		{
			return false;
		}

		var view = preferredView.MakeCopy();
		view.SetCenterLatLong(center);

		// If the alt key is pressed, zoom out.  Otherwise, zoom in.
		if (e.altKey)
		{
			view.SetZoomLevel(currentView.zoomLevel - 1);
		}
		else
		{
			view.SetZoomLevel(currentView.zoomLevel + 1);
		}
		
		SetView(view);
		return false;
	}

	// Handles the mouse wheel.  This will zoom in or out.
	function MouseWheel(e)
	{
		e = GetEvent(e);
		CancelEvent(e);
		
		// If the control does not exist
		// or the control is panning or zooming
		// return and ignore
		if(panning || zooming)
		{
			return false;
		}
		
		var delta = GetMouseScrollDelta(e);
		if (delta > 0) 
		{
			ZoomIn();
		}
		else if (delta < 0)
		{
			ZoomOut();
		}
		
		return false;
	}
	
	function Move(pixel, pinType)
	{
		center = PixelToLatLong(pixel);
		Reposition();
	}

	this.ClearSteps = ClearSteps;
	this.PrecomputeSteps = PrecomputeSteps;
	this.SetFactor = SetFactor;
	this.SwapStates = SwapStates;
	this.Reposition = Reposition;
	this.PrepareForZoom = PrepareForZoom;
	this.RemoveFromMap = RemoveFromMap
	this.Move = Move;
}