//------------------------------------------------------------------------------
// MapControl/Traffic.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: a-jayab, cthota
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------
//Sets Traffic 
//Returning whether traffic is already visible
function GetTrafficAvailability()
{	
	return trafficAvailable;
}

function GetSlidingExpirationForAutoRefresh()
{
	return __trafficSlidingExpirationTime__;
}

function GetAutoRefreshRate()
{
	return __trafficAutoRefreshRate__;
}

function GetMarketsFile()
{	
	return "__trafficMarkets__";
}

function GetIncidentsFile()
{
	return "__trafficIncidents__";
}	

var setTrafficViewflag = true;

function SetTrafficView()
{
	if(setTrafficViewflag)
	{
		setTrafficViewflag = false;
		trafficAvailable = true;	   			
		oldTrafficList = currentTrafficTilesList;
		// Clear the existing tiles
		ClearTiles(currentTrafficTilesList);
		currentTrafficTilesList = new Array();
		
		//then call traffic tiling	
		window.setTimeout(StartTrafficView, 1);
	}
	else
		window.setTimeout(SetTrafficView, 1);
}

//This function gets traffic tiles from traffic server based on current viewport
//It should only be called from SetTrafficView.
function StartTrafficView()
{	
	if(setTrafficViewflag) //just in case
		return;
		
	for (var y=tileViewportY1; y<=tileViewportY2; y++)
	{
		for (var x=tileViewportX1; x<=tileViewportX2; x++)
		{
			//1 is reserved for traffic
			var tile = RequestTile(x, y, currentView.zoomLevel, currentView.mapStyle, 1);
			currentTrafficTilesList.push(tile);			
		}
	}
	
	setTrafficViewflag = true;
}

//This function removes the traffic layer using the Traffic legend
function RemoveTrafficLayer()
{
	trafficAvailable = false; 
	if(currentTrafficTilesList != null)
		ClearTiles(currentTrafficTilesList);	
	ClearTrafficLayer();
}

//This function removes the traffic layer
function ClearTrafficLayer()
{	
	// Remove the old traffic tiles
	if(oldTrafficList!= null)
	{		
		ClearTiles(oldTrafficList);
		oldTrafficList = null;
	}
}

//This function lays traffic tiles over road tiles
function SetTrafficLayer()
{
	for(var i=0; i<currentTrafficTilesList.length; i++)
	{
		currentTrafficTilesList[i].SwapStates();
		currentTrafficTilesList[i].ClearSteps();			
		currentTrafficTilesList[i].SetFactor(0);
	}
}

//This function swaps tiles for reconstructing traffic views on zoom change
function SwapTrafficLayer()
{
	// map container can be set at left=0px, top=0px
	var oldOriginX = originX + offsetX;
	var oldOriginY = originY + offsetY;
	var oldZoom = currentView.zoomLevel;
	var newZoom = layerRequestedView.zoomLevel;
	
	// New origin
	var newOriginX = MathRound(layerRequestedView.center.x - width/2);
	var newOriginY = MathRound(layerRequestedView.center.y - height/2);

	for(var i=0; i<currentTrafficTilesList.length; i++)
	{
		currentTrafficTilesList[i].PrepareSwapTile(oldOriginX, oldOriginY, oldZoom, newOriginX, newOriginY, newZoom);
	}
}

function UpdateTrafficLayer()
{
	// No updates during zoom.  UpdateView is really only applicable during panning.
	if (zooming)
	{
		return;
	}
	
	var ox = originX + offsetX;
	var oy = originY + offsetY;

	var x1 = MathFloor((ox - buffer) / tileSize);
	var y1 = MathFloor((oy - buffer) / tileSize);
	var x2 = MathFloor((ox + width + buffer) / tileSize);
	var y2 = MathFloor((oy + height + buffer) / tileSize);
	
	var X1 = tileViewportX1;
    var X2 = tileViewportX2;
    var Y1 = tileViewportY1;
    var Y2 = tileViewportY2;
    var W1 = tileViewportWidth;
    var H1 = tileViewportHeight;
    
    	// While the map is too far to the left
	while (tileViewportX1 < x1)
	{
		// Remove each of the tiles in the left column
		// Start from the bottom and work backwards so the tiles on top retain their correct indexes
		for (var y=tileViewportHeight-1; y>=0; y--)
		{			
			var ttile = currentTrafficTilesList.splice(y * tileViewportWidth, 1)[0];
			try
			{
				ttile.RemoveFromMap();
			}
			catch(ex)
			{
				//swallow
			}
			
		}
		
		tileViewportX1++;
		tileViewportWidth--;
	}
	
	// While the map isn't far enough to the left
	while (tileViewportX1 > x1)
	{
		// Add tiles to a new left column
		tileViewportX1--;
		tileViewportWidth++;
		
		for (var y=0; y<tileViewportHeight; y++)
		{			
			var ttile = RequestTile(tileViewportX1, tileViewportY1 + y, currentView.zoomLevel, currentView.mapStyle, 1);
			currentTrafficTilesList.splice(y * tileViewportWidth, 0, ttile);
		}
	}
	
	// While the map is too far to the top
	while (tileViewportY1 < y1)
	{
		// Remove the top row
		for (var x=0; x<tileViewportWidth; x++)
		{			
			var ttile = currentTrafficTilesList.shift();
			try
			{
				ttile.RemoveFromMap();
			}
			catch(ex)
			{
				//swallow
			}
		}		
		
		tileViewportY1++;
		tileViewportHeight--;
	}
	
	// While the map isn't far enough to the top
	while (tileViewportY1 > y1)
	{
		// Add a row to the top
		tileViewportY1--;
		tileViewportHeight++;
		
		// Start from the right side of the row so the tiles can be added to the front each time
		for (var x=tileViewportWidth-1; x>=0; x--)
		{
			var ttile = RequestTile(tileViewportX1 + x, tileViewportY1, currentView.zoomLevel, currentView.mapStyle, 1);
			currentTrafficTilesList.unshift(ttile);
		}
	}
	
	// While the map is too far to the right
	while (tileViewportX2 > x2)
	{
		// Remove each of the tiles in the right column
		// Start from the bottom and work backwards so the tiles on top retain their correct indexes
		for (var y=tileViewportHeight-1; y>=0; y--)
		{			
			var ttile = currentTrafficTilesList.splice(y * tileViewportWidth + tileViewportWidth - 1, 1)[0];
			try
			{
				ttile.RemoveFromMap();
			}
			catch(ex)
			{
				//swallow
			}
		}
		
		tileViewportX2--;
		tileViewportWidth--;
	}
	
	// While the map isn't far enough to the right
	while (tileViewportX2 < x2)
	{
		// Add tiles to a new right column
		tileViewportX2++;
		tileViewportWidth++;
		
		for (var y=0; y<tileViewportHeight; y++)
		{			
			var ttile = RequestTile(tileViewportX2, tileViewportY1 + y, currentView.zoomLevel, currentView.mapStyle, 1);		
			currentTrafficTilesList.splice(y * tileViewportWidth + tileViewportWidth - 1, 0, ttile);
		}
	}
	
	// While the map is too far down
	while (tileViewportY2 > y2)
	{
		// Remove the bottom row
		for (var x=0; x<tileViewportWidth; x++)
		{
			var ttile = currentTrafficTilesList.pop();
			try
			{
				ttile.RemoveFromMap();
			}
			catch(ex)
			{
				//swallow
			}
		}
		
		tileViewportY2--;
		tileViewportHeight--;
	}
	
	// While the map isn't far down enough
	while (tileViewportY2 < y2)
	{
		// Add a row to the bottom
		tileViewportY2++;
		tileViewportHeight++;
		
		// Start from the right side of the row so the tiles can be added to the front each time
		for (var x=0; x<tileViewportWidth; x++)
		{
			var ttile = RequestTile(tileViewportX1 + x, tileViewportY2, currentView.zoomLevel, currentView.mapStyle, 1);
			currentTrafficTilesList.push(ttile);
		}
	}		
}
