//------------------------------------------------------------------------------
// Panel/Panel.js 
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: CodyE, sstchur
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

// Constructor, creates a new panel
/*	sstchur:
	Added the argument, parent, which allows you to create a VE Panel as a child of some other element.  If this argument
	is missing or null, parent will default to document.body */
		
function VE_Panel(id, x, y, width, height, color, zIndex, title, body, footer, mouseovertitle, bOpaque, parent)
{
	this.index = 0;
	this.x = x;
	this.y = y;
	this.width = width;
	this.height = height;
	this.dynamicHeightMax = 250;
	this.color = color;
	
	this.toolbarHeight = 20;
	this.footerHeight = 20;
	this.min = false;
	this.visible = true;
	
	this.onTitleClick = null;
	this.onCloseClick = null;
	this.onMaximize = null;
	
	// Create the parent element
	var el = VE_Panel._CreateElement("div", id, "VE_Panel_el", zIndex);
	this.el = el;	
	
	this.titleDisabled = false;	
	this.title = document.createElement('div');
	this.title.id = id + '_title';
	this.title.className = 'VE_Panel_title';
	this.title.appendChild(document.createElement('h1'));
	this.title.onclick = VE_Panel._OnTitleClick;
	el.appendChild(this.title);
	this.SetTitle(title);
	
	// Create the close button
	this.closeboxDisabled = false;
	this.cb = VE_Panel._CreateElement("a", id+"_cb", "VE_Panel_cb VE_Panel_cb_" + color, zIndex+1);	
	this.cb.onclick = VE_Panel._OnCloseClick;
	this.cb.unselectable = "on";
	el.appendChild(this.cb);
	
	// Create the toolbar
	this.tb = VE_Panel._CreateElement("div", id+"_tb", "VE_Panel_tb VE_Panel_tb_" + color, zIndex+1);
	this.tb.unselectable = "on";
	el.appendChild(this.tb);
	
	// Create the body
	this.body = VE_Panel._CreateElement("div", id+"_body", "VE_Panel_body", zIndex+1);
	this.body.innerHTML = body;
	el.appendChild(this.body);
	
	// Create the footer
	this.foot = VE_Panel._CreateElement("div", id+"_foot", "VE_Panel_foot VE_Panel_foot_" + color, zIndex+1);
	this.foot.innerHTML = footer;
	this.foot.unselectable = "on";
	el.appendChild(this.foot);
	
	this.Resize();
	if ( !bOpaque )
	{
		this.SetOpacity(90);
	}
	
	// Add this panel to the global list
	VE_Panel.panels.push(this);
	
	// Add the panel to the document
	
	/*	sstchur:
	Added this conditional to ensure the the element is appended to a valid parent */
	if (!parent)
		parent = document.body;

	parent.appendChild(el);
}


// The global list of panels.
VE_Panel.panels = new Array();

// The thickness of the shadow
VE_Panel.shadowThickness = 3;


// Creates a new dom element.
VE_Panel._CreateElement=function(type, id, className, zIndex)
{
	var el = document.createElement(type);
	el.id = id;
	el.className = className;	
	//sstchur:  HACK -- this should be overridable via CSS --      el.style.position = "absolute";
	el.style.zIndex = zIndex;
	return el;
}


// Positions a dom element.
VE_Panel._PositionElement=function(el, x, y, w, h)
{
	el.style.top = y + "px";
	el.style.left = x + "px";
	el.style.width = w + "px";
	el.style.height = h + "px";	
}


// Sets the position and size expressions of a panel.
VE_Panel.prototype.SetPosition=function(x, y, w, h)
{	
	this.x = x;
	this.y = y;
	this.width = w;
	this.height = h;
}


// Sets the height of the footer section
VE_Panel.prototype.SetToolbarSize=function(toolbarHeight)
{
	this.toolbarHeight = toolbarHeight;
	var d = eval(toolbarHeight) > 0 ? "block" : "none";
	this.tb.style.display = d;
	this.Resize();
}


// Sets the height of the footer section
VE_Panel.prototype.SetFooterSize=function(footerHeight)
{
	this.footerHeight = footerHeight;
	var d = eval(footerHeight) > 0 ? "block" : "none";
	this.foot.style.display = d;
	this.Resize();
}


// Resizes a panel.  This evaluates the position and size expressions.
VE_Panel.prototype.Resize=function()
{

	//HACK: cthota - make it error free for APIs
	if(header == null || footer == null)
	{
		return;
	}

	// The TaskArea should always be resized based on the height of the browser window
	var taskAreaHeight = document.body.clientHeight - header.offsetHeight - footer.offsetHeight;
	
	/*	Account for IE rounding error in CSS.
			Note:	We're making the assumption here that .bottom was defined in pixels.  If it wasn't, all bets are off.
	*/
	if (document.all)				// this is really not a very good way to check for IE
	{
		var css = footer.currentStyle;
		if (css && typeof css != 'undefined')
		{
			var strBottom = css['bottom'];
			if (strBottom && typeof strBottom != 'undefined')
			{
				var nBottom = Math.abs(parseInt(strBottom));
				taskAreaHeight += nBottom;
			}
		}
	}
	
	if (taskAreaHeight >= 0 && typeof taskAreaHeight == 'number')
		taskArea_data.style.height = taskAreaHeight + 'px';

	
	if (this.el.id == 'contextMenu' || this.el.id == 'scratchpad' || this.el.id == 'annotationPanel' || this.el.id == 'annotationPopup' || this.el.id == 'searchPopup' || this.el.id == 'help')
	{	
		if (this.height != 'auto' && typeof this.height == 'number')
			this.el.style.height = eval(this.height) + 'px';
			
		if (this.width != 'auto' && typeof this.width == 'number')
			this.el.style.width = eval(this.width) + 'px';
		
		if (this.x != 'auto' && typeof this.x == 'number')
			this.el.style.left = eval(this.x) + 'px';
		
		if (this.y != 'auto' && typeof this.y == 'number')
			this.el.style.top = eval(this.y) + 'px';			
	}
	else
	{
		var count = taskArea.getCount();
		
		var titleMargin = 3;		// 3px is the total margin property for the VE_Panel_el_minimized class
		var titleHeight = this.title.offsetHeight + titleMargin;
		
		//shankak : bodyheight now also includes the footer which now has the Clear button.
		var bodyHeight = (taskArea_data.offsetHeight - (titleHeight * count) - this.tb.offsetHeight - this.foot.offsetHeight);
		if (bodyHeight >= 0 && typeof bodyHeight == 'number')
			this.body.style.height = bodyHeight + 'px';
	}
}


//
// use this after SetDynamicBody () to 
// change panel height to fit dynamic content
//
// note: overwites contents of this.height with a numeric value
VE_Panel.prototype.SetHeightToFit = function ()
{
	var contentid = this.id + '_body_table';
  var content = document.getElementById ( contentid );
  if (!content)
  {
		return false;
  }
  this.height = 0;
  var width = Math.max(eval(this.width), 100);
  if ( content.offsetWidth > width - 14 ) // 14 for table padding
  {
		this.height += scrollbarWidth; // room for scroll bar
  }
  
  this.height += (this.titleDisabled) ? 14 : 21 + 14; // title bar + table padding
  this.height += this.toolbarHeight;
  this.height += content.offsetHeight;
  this.height += this.footerHeight;
	this.height = Math.min (this.dynamicHeightMax, this.height);
}

//
// disables/removes the closebox element
// alters layout ( need to call Resize () )
VE_Panel.prototype.DisableClosebox = function ()
{
	if ( this.closeboxDisabled )
	{
		return;
	}
	this.closeboxDisabled = true;
	this.el.removeChild (this.cb);
}

//
// enables the closebox element
// alters layout ( need to call Resize () )
VE_Panel.prototype.EnableClosebox = function ()
{
	if ( !this.closeboxDisabled )
	{
		return;
	}
	this.closeboxDisabled = false;
	this.el.insertBefore (this.cb, this.tb);
}

//
// disables/removes the title element
// alters layout ( need to call Resize () )
// does not actually kill title text, just prevents it from appearing
VE_Panel.prototype.DisableTitle = function ()
{
	if ( this.titleDisabled )
	{
		return;
	}
	this.titleDisabled = true;
	this.el.removeChild (this.cb);
	this.el.removeChild (this.title);
}

//
// enables the title element
// alters layout ( need to call Resize () )
// old title text is restored
VE_Panel.prototype.EnableTitle = function ()
{
	if ( !this.titleDisabled )
	{
		return;
	}
	this.titleDisabled = false;	
	this.el.insertBefore (this.cb, this.tb);
	this.el.insertBefore (this.title, this.cb);	
}


// Sets the content of the title bar.
VE_Panel.prototype.SetTitle=function(c)
{
	var txtNode = document.createTextNode(c);
	
	var h1 = this.title.firstChild
	if (h1)
	{
		if (h1.firstChild)	
			h1.replaceChild(txtNode, h1.firstChild);
		else
			h1.appendChild(txtNode);
	}
}


// Sets the content of the toolbar.
VE_Panel.prototype.SetToolbar=function(c)
{
	this.tb.innerHTML=c;
}


// Sets the contents of the body.
VE_Panel.prototype.SetBody=function(c)
{
	this.body.innerHTML=c;
}

// Sets the contents of the body for use with SetHeightToFit ().
VE_Panel.prototype.SetDynamicBody=function(c)
{
	this.body.innerHTML = '<table id="' + this.id + '_body_table"><tr><td>' + c + '</td></tr></table>';
}


// Sets the contents of the footer.
VE_Panel.prototype.SetFooter=function(c)
{
	this.foot.innerHTML=c;
}


// Sets the opacity of the entire panel
VE_Panel.prototype.SetOpacity=function(o)
{
	if(o >= 100)
		o = 99.99;
		
	with(this.el.style)
	{
		filter = "alpha(opacity:"+o+")";  // IE
		
		o *= 0.01;
		KHTMLOpacity = o;  // Safari < 1.2, Konqueror
		MozOpacity = o;  // Older Mozilla and Firefox
		opacity = o;  // Safari 1.2, newer Firefox and Mozilla, CSS3
	}		
}


// Sets the color of the panel.
VE_Panel.prototype.SetColor=function(c)
{	
	this.color = c;
	this.title.className = "VE_Panel_title VE_Panel_title_" + c;
	this.foot.className = "VE_Panel_foot VE_Panel_foot_" + c;
	this.cb.className = "VE_Panel_cb VE_Panel_cb_" + c;
}


// Minimizes a panel, hiding the contents of the toolbar, body, and footer.
VE_Panel.prototype.Minimize=function()
{
	this.el.className = ' VE_Panel_el_minimized';
}


// Maximizes a panel, displaying the contents of the toolbar, body, and footer.
VE_Panel.prototype.Maximize=function()
{
	this.el.className = 'VE_Panel_el';	
	if (this.onMaximize)
		this.onMaximize(this._CreateEvent());
	this.Resize();	
}


// Shows a panel.
VE_Panel.prototype.Show=function()
{
	this.el.style.display = "block";
	this.visible = true;
}


// Hides the panel entirely.
VE_Panel.prototype.Hide=function()
{
	this.el.style.display = "none";
	this.visible = false;
}


// Destroys a panel, removes it from the document
VE_Panel.prototype.Destroy=function()
{
	// Remove it from the dom
	/*	sstchur:
	We can't guarantee that the element is a direct child of the body.  It is more appropriate
	to get a reference to the elements parent, and then remove the element from that parent.	
	document.body.removeChild(this.el);	*/
	this.el.parentNode.removeChild(this.el);
		
	// Remove it from the array
	var p = VE_Panel.panels;
	for(var i=0; i<p.length; i++)
	{
		if(p[i] == this)
		{
			p.splice(i, 1);
			return;
		}
	}
	
	this.onTitleClick = null;
	this.onCloseClick = null;
	this.onMaximize = null;	
}
