//------------------------------------------------------------------------------
// Search/Search.js 
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: CodyE, sstchur, keithkin
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

// Search panel z-index.
VE_Search.zIndex = 31;

// Creates a new search based on a search phrase
function VE_Search(phrase, type, category)
{
	// Account for a bug in IE whereby the 'this' keyword is not properly maintained
	var self = this;

	if (VE_SearchManager.searches.length >= 1)
		VE_SearchManager.searches[0].panel.Minimize();
	
	this.index = VE_SearchManager.searchIndex++;
	this.type = type;
	this.phrase = this.IsWhereAdsSearch() ? "^" : phrase;
	this.startIndex = 0;
	this.displayIndex = 0;
	this.color = VE_SearchManager.colors.pop();
	this.pinZIndex = VE_MapPushpin.baseZIndex;
	
	this.results = null;
	this.ads = null;
	this.adProvider = "";
	this.requestedAds = false;	
	this.hasMore = false;
	this.message = "";
	this.categoryId = -1;
	
	if(category != null && category != "undefined" && category > 0)
	{
		this.categoryId = category;
	}
	
	this.panel = new VE_Panel("search_" + this.index,     // id
														8,                          // x
														0,	                        // y
														'auto',                     // width
														'auto',                     // height
														'searchResults',            // type
														VE_Search.zIndex,						// z index
														phrase,											// title
														'<p class = "loading">' + L_SearchLoading_Text + '</p>'  ,      // body
														"",													// footer
														true,												// show mouseover for title bar 			
														true,												// bOpaque
														taskArea_data);											// parent
		
	/*	sstchur:
	We need to make the Task Area visible as well, since the search panel is being added as a child of the Task Area */	
	toggleTaskArea(true);
		
	this.panel.title.className +=  ' VE_Panel_title_' + this.color;
			
	this.panel.index = this.index;
	this.panel.foot.style.overflow = "auto";	
		
	// Make IE obey the pseudo class :hover for any search's TitleBar
	pseudoHover(this.panel.title);
		
	if (this.IsWhereAdsSearch())
	{
		this.panel.SetFooterSize("eval(this.height) - 20");
	}
	else
	{
		this.panel.SetFooterSize(VE_Search.FooterSize);
	}

	this.panel.onTitleClick = function(e)
	{
		var p = e.srcPanel;
		taskArea.setCurrent(p);
	;}
		
	this.panel.onCloseClick = function(e)
	{	
		var p = e.srcPanel;		
		var currentPanel = taskArea.getCurrent();		
		taskArea.removeItem(p);	
		VE_SearchManager._RemoveSearch(p.index);		
		if(VE_ScopingBar)
		{
			VE_ScopingBar.Show(false);
			dragController.setBoundingArea(getVisibleMapRect());			
		}
	};
	
	this.panel.onMaximize = function(e)
	{
		var p = e.srcPanel;
		var txtWhat = document.getElementById('what');
		txtWhat.value = self.phrase;
		setActive(txtWhat);		
		if(VE_ScopingBar)
		{
			VE_ScopingBar.Show(false);
			dragController.setBoundingArea(getVisibleMapRect());
		}
		
		if (typeof p.title != 'undefined')
			setActivePanel(p.title.className);
	};
	
	// sstchur:  Add this panel to the TaskArea; bring it to the front
	taskArea.addItem(this.panel);	
}

// Constants.

// footer size.
VE_Search.FooterSize = 120;


// Sets the results of a search
VE_Search.prototype.SetResults=function(results)
{
	this._ClearPins();
	this.results = results;	
	this._UpdatePins();
}

// Sets the z-index of the pushpins.
VE_Search.prototype._SetPinZIndex=function(pinZIndex)
{
	if (this.pinZIndex == pinZIndex)
	{
		return;
	}

	// set the pin z-index
	this.pinZIndex = pinZIndex;

	// If we don't have any results, exit.
	var r = this.results;
	if(!r || r.length == 0)
	{
		return;
	}
			
	// set each of the pins z-index to the new z-index.
	for(var i=0; i<r.length; i++)
	{
		if (r[i] == null || r[i].pushPin == null)
		{
			continue;
		}

		r[i].pushPin.style.zIndex = this.pinZIndex;
	}
}

// Clears all the active pushpins off of the map
VE_Search.prototype._ClearPins=function()
{
	var r = this.results;
	if(!r || r.length == 0)
	{
		return;
	}
		
	for(var i=0; i<r.length; i++)
	{
		if(r[i].pushPin)
		{
			map.RemovePushpin(r[i].pushPin.id);
			r[i].pushPin = null;
		}
	}
}

// Creates pushpins for the displayed results
VE_Search.prototype._UpdatePins=function()
{
	/*	sstchur:
			It seems odd to hide the ero here, but actually, it's quite possible for an ero to be currently docked to a pin
			that has just shifted positions do to the user panning the map.  If we allowed the ero to remain visible, it would,
			at best, point to nothing at all, and at worst, point to the wrong search result. */
			ero.hide();
	
	var r = this.results;
	if(!r || r.length == 0)
	{
		return;
	}
		
	var a = this.displayIndex;
	var b = Math.min(r.length, a + VE_SearchManager.displayCount);
	
	for(var i=a; i<b; i++)
	{
		r[i].pushPin = map.AddPushpin("pin_" + this.index + "_" + r[i].id,      // pushpinID
																	r[i].latitude,                            // latitude
																	r[i].longitude,                           // longitude
																	25,                                       // width
																	29,                                       // height
																	"VE_Pushpin VE_Pushpin_" + this.color,    // className
																	(i + 1 + this.startIndex) + '',           // innerHTML 
																	this.pinZIndex,								// z-index
																	Msn.VE.PushPinTypes.SearchResult);		   	// pin type
		
		r[i].pushPin.onmouseover = VE_SearchManager._ShowSearchPopup;
		r[i].pushPin.onmouseout = VE_SearchManager.CloseSearchPopup;
	}
}

// Updates the contents of the panel
VE_Search.prototype._UpdatePanel=function()
{
	var r = this.results;	
	var n = r.length;		
	
	if(!r || n == 0)
	{
		this.panel.SetBody('<p class = "message">' + this.message + '</p>');
		this.panel.SetToolbar("");
	}
	else
	{
		// Start and end index within the current batch
		var a = this.displayIndex;
		var b = Math.min(n, a + VE_SearchManager.displayCount);
		var body = "";
		
		if(this.startIndex == 0 && a == 0 && this.message)		
			body = '<p class = "message">' + this.message + '</p>';
		
		var srid = "ve_search_results" + this.index;
		body += "<div id='" + srid + "'>";
		
		//categories
		if(Msn.VE.Marketization.IsEnabled(Msn.VE.Features.Search.Categories))
		{
			if(this.categories != null && this.categories != "undefined" && this.categories.length > 0)
			{
				body += "<div class='ve_searchResultCategoryContainer'>" + L_RelatedCategories_Text + ": ";
				var len = this.categories.length;
				for(var j=0;j<len;j++)
				{
					body += "<a href='#' onclick=\"SearchWithCategory(" + this.categories[j].id + ", '" + this.phrase + "', " + this.index + ");return false;\">" + this.categories[j].name + "</a>";
					if(j != len-1)
						body += ", ";
				}
				body += "</div><br>";
			}
		}
		
		var Click2Call = null;
		if (click2callenabled && Msn.VE.Marketization.IsEnabled(Msn.VE.Features.Click2Call.Enabled))
 		{ 	
 			VE_Click2Call.setSearchResultPaneWidgetsSize(VE_SearchManager.displayCount);
			switch (this.index % 3)
 			{
 				case 0:
 					Click2Call = VE_Click2Call.getClick2CallManager(Click2CallEntryPoint.SearchResultPane0);
 					break;
 				case 1:
 					Click2Call = VE_Click2Call.getClick2CallManager(Click2CallEntryPoint.SearchResultPane1);
 					break;
 				case 2:
 					Click2Call = VE_Click2Call.getClick2CallManager(Click2CallEntryPoint.SearchResultPane2);
 					break;
 			} 			
 		}
		
		var searchResults = '<ul class = "list_searchResults">';
		
		for (var i = a; i < b; i++)
		{
			var pinId = 'pin_' + this.index + '_' + r[i].id;
			var liMouseOverEvent = 'pinToTop(\'' + pinId + '\')';
			var liMouseOutEvent = 'ero.hide(); revertPin(\'' + pinId + '\');';
			var h3DivMouseOverEvent = 'VE_SearchManager._ShowSearchPopup(event, ' + this.index + ', \'' + r[i].id + '\', this);';						
			var clickEvent = 'VE_SearchManager.ShowSearchResultDetails(\'' + r[i].id + '\')';
			
			var li =
			'<li onmouseover = "##liMouseOverEvent##" onmouseout = "##liMouseOutEvent##" class = "VE_Pushpin_searchResult VE_Pushpin_' + this.color + '">' +
				'<div class = "poiNumber" onmouseover = "##h3DivMouseOverEvent##">' + (this.startIndex + i + 1) + '</div>' +
				'<h3 onmouseover = "##h3DivMouseOverEvent##" onclick = "##clickEvent##">' + r[i].name + '</h3>' +
				'<p>' + r[i].description + '<br />' +
				r[i].phone + '<span id="c2c' + r[i].id + '" class = "c2c"></span></p>' +
			'</li>';
			
			li = li.replace(/##liMouseOverEvent##/g, liMouseOverEvent);
			li = li.replace(/##liMouseOutEvent##/g, liMouseOutEvent);
			li = li.replace(/##h3DivMouseOverEvent##/g, h3DivMouseOverEvent);
			li = li.replace(/##clickEvent##/g, clickEvent);
			
 			if (Click2Call && VE_Click2Call.isAllowedCountry(r[i].country)) Click2Call.CreateWidget('mpc.' + r[i].id + '@search.msn.com', 'c2c' + r[i].id);
 			
			searchResults += li;
		}
		
		searchResults += '</ul>';
		
		body += searchResults + "</div>";		
		
		this.panel.SetBody(body);
		
		pseudoHoverForChildren(this.panel.body, 'h3');
		pseudoHoverForChildren(this.panel.body, 'li');
		
		if (click2callenabled && Msn.VE.Marketization.IsEnabled(Msn.VE.Features.Click2Call.Enabled))
 		{ 			
 			if (Click2Call) Click2Call.RenderWidgets();
 		}

		var toolbar = 
		'<div>' +
			'<ul class = "nav">' +
				'<li class = "##prevPageLinkStatus##" title = "' + L_SearchPrevPage_Text + '" ##prevOnClick##></li>' +
				'<li class = "##nextPageLinkStatus##" title = "' + L_SearchNextPage_Text + '" ##nextOnClick##></li>' +
			'</ul>' +
		'</div>';
		
		// Set the correct CSS class (enabled or disabled) for the Prev and Next links
		var hasPrev = this.startIndex + a > 0;
		var hasNext = b < n || this.hasMore;
		toolbar = toolbar.replace(/##prevPageLinkStatus##/, hasPrev ? 'prevPageLink' : 'prevPageLink_disabled');
		toolbar = toolbar.replace(/##nextPageLinkStatus##/, hasNext ? 'nextPageLink' : 'nextPageLink_disabled');
		toolbar = toolbar.replace(/##prevOnClick##/, hasPrev ? ' onclick = "VE_SearchManager.PageBackward(' + this.index + ')"' : '');
		toolbar = toolbar.replace(/##nextOnClick##/, hasNext ? ' onclick = "VE_SearchManager.PageForward(' + this.index + ')"' : '');
		this.panel.SetToolbar(toolbar);
	}
	
	this._UpdateAds();
}

VE_Search.prototype.logSponsoredLinkClick = function(_phrase, _adIndex, _provider)
{
	return function()
	{
		VE_Analytics.LogAds(_phrase, 'Event6', 'Ad Clicked', 0, _adIndex, _provider);
	}
}

// Updates the ads footer on the search panel.
VE_Search.prototype._UpdateAds = function()
{
	this.panel.SetFooter("");
	this.panel.SetFooterSize(0);

	if(!this.IsWhereAdsSearch() && (!this.ads || this.ads.length == 0))
	{
		return "";
	}
	else
	{
		//Add ads before the search results
		//For Feb QFE
		
		var telcoAdsId = 've_telco_ads' + this.index;	
		
		//Remove the existing ads if any
		try
		{
			var old = document.getElementById(telcoAdsId);
			if(old != null)
			{
				document.body.removeChild(this.el);
			}
		}
		catch(ex)
		{
		}
		
		var divTelcoAds = document.createElement('div');
		divTelcoAds.id = telcoAdsId;
		divTelcoAds.className = 'VE_Panel_sponsoredLinks';
		
		if(this.ads != null && this.ads != "undefined" && this.ads.length > 0)
		{	
			var h3SponsoredLinks = document.createElement('h3');
			//Get attribution
			var attribution = "";						
			try
			{
				attribution = eval("Msn.VE.Features.Ads.AttributionText" + this.adProvider);				
			}
			catch(ex)
			{
			}			
			if(attribution != null && attribution.length > 0)
			{
				h3SponsoredLinks.appendChild(document.createTextNode(attribution));
			}
			else
			{
				h3SponsoredLinks.appendChild(document.createTextNode(L_SearchSponsoredLinks_Text));
			}

			divTelcoAds.appendChild(h3SponsoredLinks);			
			
			//Log number of ads received
			if(this.requestedAds)
			{			
				VE_Analytics.LogAds(this.phrase, "Event5", "Ads Received", this.ads.length, "0", this.adProvider);
				this.requestedAds = false;
			}
			
			var ulAds = document.createElement('ul');
			ulAds.className = 'list_sponsoredLinks';
			divTelcoAds.appendChild(ulAds);
			
			var adslength = this.ads.length >= 3 ? 3 : this.ads.length;
			for(var i = 0; i < adslength; i++)
			{
				var title = this.ads[i].title;
				var phrase = this.phrase;
				var description = this.ads[i].description;
				var url = this.ads[i].url;
				var adIndex = i + 1;
				var provider = this.adProvider;

				var li = document.createElement('li');				
				li.onclick = this.logSponsoredLinkClick(phrase, adIndex, provider);
				
				ulAds.appendChild(li);
				
				var h4Link = document.createElement('h4');
				li.appendChild(h4Link);
				
				var a = document.createElement('a');
				a.href = url;
				a.setAttribute('target', '_blank');
				a.appendChild(document.createTextNode(title));
				h4Link.appendChild(a);

				var pDesc = document.createElement('p');
				li.appendChild(pDesc);
				pDesc.appendChild(document.createTextNode(description));
				
			}
			
			var searchResultsId = 've_search_results' + this.index;
			var elem = document.getElementById(searchResultsId);

			if(elem != null)
			{
				this.panel.body.insertBefore(divTelcoAds, elem);
				
				/* A sad sad hack necessary to thawrt the IE peakaboo bug */
				var filter = this.panel.body.style.filter;
				if (typeof filter != 'undefined')
					this.panel.body.style.filter = filter;
			}
		}	
		else
		{
			if(this.requestedAds)
			{
				VE_Analytics.LogAds(this.phrase, "Event4", "Ads Requested", "0", "0", this.adProvider);			
				this.requestedAds = false;
			}		
		}
	}
}


// Destroys the search
VE_Search.prototype.Destroy=function()
{
	this._ClearPins();	
	this.panel.Destroy();	
}


// Gets an entity by entity ID
VE_Search.prototype.GetEntity=function(id)
{
	var r = this.results;
	if(!r || r.length == 0)
	{
		return null;
	}
	
	for(var i=0; i<r.length; i++)
	{
		if(r[i].id == id)
		{
			return r[i];
		}
	}
	
	return null;
}

// returns true if this is a where ads search.
VE_Search.prototype.IsWhereAdsSearch=function()
{
	return this.type == "WHERE";
}