//------------------------------------------------------------------------------
// Configuration.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: EShinn
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
// Dependencies
//
// ConfigurationItem.js
// ConfigurationCategory.js
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Referred to by
//
// State.js
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// VE_Config
//
// Object which manages a group of VE_ConfigItems
//------------------------------------------------------------------------------
function VE_Config()
{
	this.serializationVersion = currentUrlApiVersion;
	this._items = new Array();
	this._categories = new Array();
}

// Adds the provided VE_ConfigCat to this VE_Config
VE_Config.prototype.AddCategory = function(category)
{
	if (!category || category.constructor != VE_ConfigCat)
	{
		return;
	}
	
	for (var i = 0; i < this._categories.length; i++)
	{
		if (this._categories[i].name == category.name)
		{
			this._categories[i] = category;
			return;
		}
	}
	
	this._categories.push(category);
}

// Remove the named VE_Category and any associated VE_ConfigItems from this VE_Config.
VE_Config.prototype.RemoveCategory = function(categoryName)
{
	if (!categoryName)
	{
		return;
	}
	
	// Remove the category.
	for (var i = 0; i < this._categories.length; i++)
	{
		if (this._categories[i].name == categoryName)
		{
			this._categories.splice(i, 1);
			break;
		}
	}
	
	// Remove any configuration items for this category.
	for (var i = 0; i < this._items.length; )
	{
		if (this._items[i].category == categoryName)
		{
			this._items.splice(i, 1);
		}
		else
		{
			i++;
		}
	}
}

// Adds the provided VE_ConfigItem to this VE_Config
VE_Config.prototype.AddItem = function(item)
{
	if (!item || item.constructor != VE_ConfigItem)
	{
		return;
	}

	// Make sure the specified category exists.
	var categoryExists = false;	
	for (var i = 0; i < this._categories.length; i++)
	{
		if (this._categories[i].name == item.category)
		{
			categoryExists = true;
			break;
		}
	}	
	
	if (!categoryExists)
	{
		return;
	}
	
	for (var i = 0; i < this._items.length; i++)
	{
		// replace if exists
		if (this._items[i].name == item.name)
		{
			this._items[i] = item;
			return;
		}
	}
	
	this._items.push(item);
}

// Removes the specified VE_ConfigItem from this VE_Config
VE_Config.prototype.RemoveItem = function(name)
{
	for (var i = 0; i < this._items.length; i++)
	{
		if (this._items[i].name == name)
		{
			this._items.splice(i, 1);
			return;
		}
	}
}

// Returns the VE_ConfigItems for the specified category
VE_Config.prototype.GetItemsForCategory = function(category)
{
	var categoryItems = new Array();
	for (var i = 0; i < this._items.length; i++)
	{
		if (this._items[i].category == category)
		{
			categoryItems.push(this._items[i]);
		}
	}

	return categoryItems;
}

// Sets the value of the specified VE_ConfigItem - does nothing if VE_ConfigItem can be found.
VE_Config.prototype.SetValue = function(name, value)
{
	if (!name || value == null)
	{
		return;
	}
	
	for (var i = 0; i < this._items.length; i++)
	{
		if (this._items[i].name == name)
		{
			this._items[i].SetValue(value, this.serializationVersion);
			return;
		}
	}
}

// Gets the value of the specified VE_ConfigItem - returns null otherwise.
VE_Config.prototype.GetValue = function(name)
{
	if (!name)
	{
		return;
	}
	
	for (var i = 0; i < this._items.length; i++)
	{
		if (this._items[i].name == name)
		{
			return this._items[i].value;			
		}
	}
	
	return null;	
}

// Returns an all the VE_ConfigItem's ToString() representation for those that have a value
VE_Config.prototype.ToString = function(category, persistentOnly)
{
	var returnedString = "";
	var isFirst = true;

	for (var i = 0; i < this._items.length; i++)
	{
		// skip if a category was passed in & they don't match
		if (category && this._items[i].category != category) 
		{
			continue;
		}
		
		// skip if we don't have a value for this item or this is a non-persistent item & we asked for persistent only
		if (!this._items[i].HasValue() || (persistentOnly && !this._items[i].persist))
		{
			continue;
		}

		if (!isFirst) 
		{
			returnedString += "&";
		}
		returnedString += this._items[i].ToString();
		isFirst = false;

	}

	return returnedString;
}

// Loads the values contained within the passed in string (if it is in the VE_Config.ToString() format)
VE_Config.prototype.LoadValuesFromString = function(configurationToString)
{
	if (!configurationToString)
	{
		return;
	}

	var configItemStrings = configurationToString.split("&");
	for (var i = 0; i < configItemStrings.length; i++)
	{
		var nameValueStrings = configItemStrings[i].split("=");
		if (nameValueStrings.length != 2)
		{
			continue;
		}
		
		this.SetValue(nameValueStrings[0], nameValueStrings[1]);
	}
}