//------------------------------------------------------------------------------
// ConfigurationItem.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: EShinn
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Dependencies
//
// Validators.js
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Referred to by
//
// Configuration.js
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// VE_ConfigItem
//
// Represents a global setting
//------------------------------------------------------------------------------

// Constructor.
function VE_ConfigItem(name, description, category, validator, value, persist, userCanEdit)
{
	this.name = name;
	this.description = description;
	this.category = category;
	this.persist = persist;
	this.userCanEdit = userCanEdit;
	this._validator = validator;
	
	this.SetValue(value);
	
	// If a valid value wasn't passed in, use default value
	if (this.value == null)
	{
		this.value = this._validator.DefaultValue();
	}
}

// Returns the type of the validator.
VE_ConfigItem.prototype.GetType = function()
{
	return this._validator.type;
}

// Sets the value of this VE_ConfigItem
VE_ConfigItem.prototype.SetValue = function(value, apiVersion)
{
	var parsedValue = this._validator.ParseValue(value, apiVersion);
	if (this._validator.ValidValue(parsedValue, apiVersion)) 
	{
		this.value = parsedValue;
	}
}

// Returns true if we have a value.  For arrays, this is true if there are any items in the array.
VE_ConfigItem.prototype.HasValue = function()
{
	if (this._validator.validatorType == "VE_ArrayValidator")
	{
		return this.value.length > 0;
	}
	else
	{
		return this.value != null;
	}
}

// Returns a string representation of the VE_ConfigItem 
// Override if you wish to have a custom output.
VE_ConfigItem.prototype.ToString = function()
{
	var returnString = this.name + "=";
	if (this.value.constructor == Array)
	{
		var primarySeparator = VE_SerializationConstants.primarySeparator[currentUrlApiVersion];
		var isFirst = true;
		for (var i = 0; i < this.value.length; i++)
		{
			if (!isFirst)
			{
				returnString += primarySeparator;
			}
			
			returnString += VE_ConfigItem.EscapeValue(this.value[i]);
			isFirst = false;
		}
	}
	else
	{
		returnString += VE_ConfigItem.EscapeValue(this.value);
	}
	
	return returnString;
}

// Preps a value for serialization.
VE_ConfigItem.EscapeValue = function(value)
{
	if (!value || value.constructor != String)
	{
		return escape(value);
	}
	
	var primarySeparator = VE_SerializationConstants.primarySeparator[currentUrlApiVersion];

	// Replace the primary separator with spaces
	return value.replace(new RegExp("(" + primarySeparator + ")|(" + IOSec.EncodeUrl(primarySeparator) + ")", "g"), " ");
}