//------------------------------------------------------------------------------
// State.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: EShinn, sstchur, keithkin
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Dependencies
//
// Cookie.js
// Configuration.js
// ConfigurationItem.js
// Validators.js
// ConfigValues.js
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Referred to by
//
// Main.js
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// VE_State
//
// An object which enscapsulates the exact state of this VE instance at 
// any given time.
//------------------------------------------------------------------------------


// Make this a number so we don't have to constantly eval() it.
currentUrlApiVersion = eval(currentUrlApiVersion);

// Convenience class to pass around what actions should be taken when state is reloaded.
function VE_LoadActions()
{
	this.initialLoad = true;
	this.displayIntroPanel = true;
	this.doBestFit = true;
}

// Constructor - Restores state from a variety of sources.
function VE_State()
{
	this.loadActions = new VE_LoadActions();
	this._LoadStateDefault();
	//
	// subroutines that populate the scratchpad need access to the state
	// object before it has finished constructing (deserializing from an url/cookie).
	//
	// annotations, yp, and adr pushpins all need this BUT, since there
	// is a huge amount of latency between client and server, this was
	// never a problem.
	//
	// annotations is strictly client side, though, so the problem manifested.
	// 
	state = this;

	// If refreshing or entering VE for the first time...
	if ( GetUrlParameterString().indexOf(passportQuerystring) == -1 &&	
		 this.IsPermalink(GetUrlParameterString())
	   )
	{
		this._LoadStateFromCookie(true);
		var urlParameterString  = GetUrlParameterString();
		// The "q" parameter strings need to be handled differently & 
		// we need to load state from the session cookie
		if (urlParameterString && urlParameterString.indexOf("q=") > -1)
		{
			this._LoadStateFromCookie(false);
			this.loadActions.doBestFit = false;
			this.loadActions.initialLoad = false;
		}
		this.loadActions.doBestFit = this._LoadStateFromUrl();
	}
	// Otherwise we are returning to VE...
	else	
	{
		this._LoadStateFromCookie(true);
		this._LoadStateFromCookie(false);
		this.loadActions.doBestFit = false;
		this.loadActions.initialLoad = false;
	}


	if (this.GetSearchKeywords().length > 0)
	{
		this.loadActions.displayIntroPanel = false;
	}

	this._RestoreFromState();
}



// Sets up the configuration & default values.
VE_State.prototype.IsPermalink = function(str)
{
	return ((str.indexOf("cp=") > -1) ||
			(str.indexOf("lvl=") > -1) ||
			(str.indexOf("style=") > -1) ||
			(str.indexOf("cid=") > -1) ||
			(str.indexOf("sp=") > -1) ||
			(str.indexOf("trfc=") > -1) ||
			(str.indexOf("rtp=") > -1) ||
			(str.indexOf("ss=") > -1) ||
			(str.indexOf("autolocate=") > -1) ||
			(str.indexOf("where1=") > -1) ||
			(str.indexOf("q=") > -1) ||
			(str.indexOf("scene=") > -1) ||
			(str.indexOf("dt=") > -1) ||
			(str.indexOf("eaid=") > -1) ||
			(str.indexOf("eid=") > -1)  ||
			(str.indexOf("dsid=") > -1) ||
			(str.indexOf("zooms=") > -1) ||
			(str.indexOf("strt1=") > -1) ||
			(str.indexOf("city1=") > -1) ||
			(str.indexOf("pcode1=") > -1) ||
			(str.indexOf("cntrgn1=") > -1) 
			);
}

// Sets up the configuration & default values.
VE_State.prototype._LoadStateDefault = function()
{
	this._config = new VE_Config();
	
	var stringTypeValidator = new VE_TypeValidator("string");
	var booleanTypeValidator = new VE_TypeValidator("boolean");
	var numberTypeValidator = new VE_TypeValidator("number");
	
	var urlApiVersionValidator = new VE_RangeValidator(1, 2, true);
	
	//------------------------------------------------------------------------------
	// System
	//------------------------------------------------------------------------------
	this._config.AddCategory(new VE_ConfigCat(VE_State.CategorySystem, ""));

	// Current URL API version
	var currentUrlApiVersionItem = new VE_ConfigItem(VE_State.ItemCurrentUrlApiVersion, "", VE_State.CategorySystem, urlApiVersionValidator, currentUrlApiVersion, false, false);
	this._config.AddItem(currentUrlApiVersionItem);

	// Wi-Fi Service URL
	var wifiServiceUrlItem = new VE_ConfigItem(VE_State.ItemWifiServiceUrl, "", VE_State.CategorySystem, stringTypeValidator, wifiServiceUrl, false, false);
	this._config.AddItem(wifiServiceUrlItem);
	
	// Wi-Fi Download URL
	var wifiDownloadUrlItem = new VE_ConfigItem(VE_State.ItemWifiDownloadUrl, "", VE_State.CategorySystem, stringTypeValidator, wifiDownloadUrl, false, false);
	this._config.AddItem(wifiDownloadUrlItem);

	// Max Input Count	
	var maxInputCountValidator = new VE_RangeValidator(50, 100, true);
	var maxInputCountItem = new VE_ConfigItem(VE_State.ItemMaxInputCount, "", VE_State.CategorySystem, maxInputCountValidator, maxInputCount, false, false);
	this._config.AddItem(maxInputCountItem);
	
	// Analytics Enabled
	var analyticsEnabledItem = new VE_ConfigItem(VE_State.ItemAnalyticsEnabled, "", VE_State.CategorySystem, booleanTypeValidator, analyticsEnabled, false, false);
	this._config.AddItem(analyticsEnabledItem);

	// Since we are going to use this in multiple places,
	// might as well declare it early.	
	var maxInputStringLengthValidator = new VE_StringLengthValidator(this.GetMaxInputCount());
	var arrayStringLengthValueValidator = function(value)
	{
		if (value == null || value.constructor != Array)
		{
			return false;
		}
		
		// Make sure each string is the proper length
		for (var i = 0; i < value.length; i++)
		{
			if (!maxInputStringLengthValidator.ValidValue(value[i]))
			{
				return false;
			}
		}
		return true;
	};
	var arrayStringLengthValueParser = function(value)
	{
		return maxInputStringLengthValidator.ParseValue(value);
	};
	
	

	//------------------------------------------------------------------------------
	// Viewport
	//------------------------------------------------------------------------------
	this._config.AddCategory(new VE_ConfigCat(VE_State.CategoryViewport, ""));

	// Center point (lat + lon)
	var cpConfigItem = null;
	if ( window.revIPLat != "" && window.revIPLong != "" )
	{
		// set the default as revIP's lat long
		cpConfigItem = new VE_ConfigItem(VE_State.ItemCenterPoint, "", VE_State.CategoryViewport, new VELatLongValidator(), [window.revIPLat, window.revIPLong], true, false);
	}
	else
	{
		// set the default as marketdetection's lat long	
		cpConfigItem = new VE_ConfigItem(VE_State.ItemCenterPoint, "", VE_State.CategoryViewport, new VELatLongValidator(), [eval(defaultLatitude), eval(defaultLongitude)], true, false);
	}	
	this._config.AddItem(cpConfigItem);
	
	// Map Style
	var mapStyleValidator = new VE_ValidValuesValidator("string", ["a", "h", "r", "o"]);
	var mapStyleConfigItem = new VE_ConfigItem(VE_State.ItemMapStyle, "", VE_State.CategoryViewport, mapStyleValidator, defaultMapStyle, true, false);
	this._config.AddItem(mapStyleConfigItem);

	// Zoom
	var zoomValidator = new VE_RangeValidator(1, 19, true);
	var zoomConfigItem = new VE_ConfigItem(VE_State.ItemZoom, "", VE_State.CategoryViewport, zoomValidator, defaultZoom, true, false);
	this._config.AddItem(zoomConfigItem);

	// Oblique scene id
	var sceneConfigItem = new VE_ConfigItem(VE_State.ItemScene, "", VE_State.CategoryViewport, numberTypeValidator, null, true, false);
	this._config.AddItem(sceneConfigItem);


	//------------------------------------------------------------------------------
	// Start-up 
	//------------------------------------------------------------------------------
	this._config.AddCategory(new VE_ConfigCat(VE_State.CategoryStartup, ""));

	// Specified URL API Version
	var specifiedUrlApiVersionItem = new VE_ConfigItem(VE_State.ItemSpecifiedUrlApiVersion, "", VE_State.CategoryStartup, urlApiVersionValidator, currentUrlApiVersion, false, false);
	this._config.AddItem(specifiedUrlApiVersionItem);

	// Autolocate
	var autolocateItem = new VE_ConfigItem(VE_State.ItemAutolocate, "", VE_State.CategoryStartup, booleanTypeValidator, false, false, false);
	this._config.AddItem(autolocateItem);

	// Start Where
	var startWhereItem = new VE_ConfigItem(VE_State.ItemStartWhere, "", VE_State.CategoryStartup, maxInputStringLengthValidator, "", false, false);
	this._config.AddItem(startWhereItem);


	//------------------------------------------------------------------------------
	// Driving Directions 
	//------------------------------------------------------------------------------
	this._config.AddCategory(new VE_ConfigCat(VE_State.CategoryDirections, ""));
	
	// Route Points
	var routePointsArrayValidator = function(value, apiVersion)
	{
		// Set it to the current URL API Version if it hasn't been decided yet...
		// Normally, we would make a call to state at this point, but we don't know when this is getting
		// called so state might not exist at this point in time.
		if (apiVersion == null)
		{
			apiVersion = currentUrlApiVersion;
		}

		if (value == null || value.constructor != Array)
		{
			return false;
		}
		
		// These should all be in a valid route point serialized format
		for (var i = 0; i < value.length; i++)
		{
			if (value[i] && !VE_Directions.ValidSerializedRoutePoint(value[i], apiVersion))
			{
				return false;
			}
		}
		return true;
	};
	
	// We don't want to do any real parsing on this...just determine if it's a valid point.
	var routePointParsingFunction = function(value, apiVersion)
	{
		// Only accept strings.
		if (value == null || typeof(value) != "string")
		{
			return null;
		}

		// Return the route point with white space stripped out if this is a valid route point.
		if (VE_Directions.ValidSerializedRoutePoint(value, apiVersion))
		{
			return value.replace(/^\s*|\s*$/g,"");
		}

		// Otherwise just return null		
		return null;
	};
	
	var routePointsValidator = new VE_ArrayValidator("string", 0, null, true, routePointsArrayValidator, routePointParsingFunction);
	var routePointsItem = new VE_ConfigItem(VE_State.ItemRoutePoints, "", VE_State.CategoryDirections, routePointsValidator, [], true, false);
	this._config.AddItem(routePointsItem);		
		
	//------------------------------------------------------------------------------
	// Searches
	//------------------------------------------------------------------------------
	this._config.AddCategory(new VE_ConfigCat(VE_State.CategorySearches, ""));

	// Search Keywords
	var searchesValidator = new VE_ArrayValidator("string", 0, parseInt(maximumSearches), false, arrayStringLengthValueValidator, arrayStringLengthValueParser);
	var searchesConfigItem = new VE_ConfigItem(VE_State.ItemSearches, "", VE_State.CategorySearches, searchesValidator, [], true, false);
	this._config.AddItem(searchesConfigItem);
	
	// Combined Query (what & where)
	var combinedQueryConfigItem = new VE_ConfigItem(VE_State.ItemCombinedQuery, "", VE_State.CategorySearches, maxInputStringLengthValidator, "", false, false);
	this._config.AddItem(combinedQueryConfigItem);	

	// Server Page Size
	var serverPageSizeValidator = new VE_RangeValidator(1, 50, true);
	var serverPageSizeItem = new VE_ConfigItem(VE_State.ItemServerPageSize, "", VE_State.CategorySearches, serverPageSizeValidator, serverPageSize, false, false);
	this._config.AddItem(serverPageSizeItem);

	// Maximum Number of Open Searches
	var maxSearchesValidator = new VE_RangeValidator(1, 10, true);
	var maxSearchesItem = new VE_ConfigItem(VE_State.ItemMaxSearches, "", VE_State.CategorySearches, maxSearchesValidator, maximumSearches, false, false);
	this._config.AddItem(maxSearchesItem);
	
	
	//------------------------------------------------------------------------------
	// Scratchpad
	//------------------------------------------------------------------------------
	this._config.AddCategory(new VE_ConfigCat(VE_State.CategoryScratchpad, ""));

	// Scratchpad Entity Ids
	// scratchpad items can be 320 characters (annotations), so we need a longer
	// validator than the one used for search
	var maxScratchpadStringLengthValidator = new VE_StringLengthValidator(320);
	var arrayScratchpadStringLengthValueValidator = function(value)
	{
		if (value == null || value.constructor != Array)
		{
			return false;
		}
		
		// Make sure each string is the proper length
		for (var i = 0; i < value.length; i++)
		{
			if (!maxScratchpadStringLengthValidator.ValidValue(value[i]))
			{
				return false;
			}
		}
		return true;
	};
	var arrayScratchpadStringLengthValueParser = function(value)
	{
		return maxScratchpadStringLengthValidator.ParseValue(value);
	};
	var scratchpadValidator = new VE_ArrayValidator("string", 0, null, false, arrayScratchpadStringLengthValueValidator, arrayScratchpadStringLengthValueParser);
	var scratchpadConfigItem = new VE_ConfigItem(VE_State.ItemScratchpadIds, "", VE_State.CategoryScratchpad, scratchpadValidator, [], true, false);
	scratchpadConfigItem.ToString = VE_Scratchpad.ScratchpadSerializer;
	this._config.AddItem(scratchpadConfigItem);
	
	//------------------------------------------------------------------------------
	// Traffic
	//------------------------------------------------------------------------------
	this._config.AddCategory(new VE_ConfigCat(VE_State.CategoryTraffic, L_Traffic_Text));

	var trafficRangeValidator = new VE_RangeValidator(0, 1, true);
	var trafficConfigItem = new VE_ConfigItem(VE_State.ItemTraffic, "", VE_State.CategoryTraffic, trafficRangeValidator, 0, true, false);
	this._config.AddItem(trafficConfigItem);
	
	
	//------------------------------------------------------------------------------
	// User Options
	//------------------------------------------------------------------------------
	
	//------------------------------------------------------------------------------
	// On Exit
	//------------------------------------------------------------------------------
	this._config.AddCategory(new VE_ConfigCat(VE_State.CategoryUserOptionsExit, L_StateOnExit_Text ));

	// Save Last Location
	var saveLocationConfigItem = new VE_ConfigItem(VE_State.ItemSaveLocation, L_StateSaveLastLocation_Text, VE_State.CategoryUserOptionsExit, booleanTypeValidator, Msn.VE.Marketization.IsEnabled( Msn.VE.Features.Settings.SaveLastLocation ), true, true);
	this._config.AddItem(saveLocationConfigItem);

	// Save Searches
	var saveSearchesConfigItem = new VE_ConfigItem(VE_State.ItemSaveSearches, L_StateSaveSearches_Text, VE_State.CategoryUserOptionsExit, booleanTypeValidator, Msn.VE.Marketization.IsEnabled( Msn.VE.Features.Settings.SaveSearches ), true, true);
	this._config.AddItem(saveSearchesConfigItem);

	// Save Scratchpad
	var saveScratchpadConfigItem = new VE_ConfigItem(VE_State.ItemSaveScratchpad, L_StateSaveScratchPad_Text, VE_State.CategoryUserOptionsExit, booleanTypeValidator, Msn.VE.Marketization.IsEnabled( Msn.VE.Features.Settings.SaveScratchpad ), true, true);
	this._config.AddItem(saveScratchpadConfigItem);
	
	// Save Last Used Directions Unit
	var directionsUnitValidator = new VE_ValidValuesValidator("string", ["mi", "km"]);
	var saveDirectionsUnitConfigItem = new VE_ConfigItem(VE_State.ItemSaveDirectionsUnit, "", VE_State.CategoryUserOptionsExit, directionsUnitValidator, "mi", true, false);
	this._config.AddItem(saveDirectionsUnitConfigItem);
	
	//------------------------------------------------------------------------------
	// Map Navigation
	//------------------------------------------------------------------------------
	this._config.AddCategory(new VE_ConfigCat(VE_State.CategoryUserOptionsMap, L_StateMapNavigation_Text));
	
	// Animated Movements
	var enableAnimatedMovementConfigItem = new VE_ConfigItem(VE_State.ItemAnimatedMovementsEnabled, L_StateUseAnimatedPanZoom_Text , VE_State.CategoryUserOptionsMap, booleanTypeValidator, Msn.VE.Marketization.IsEnabled( Msn.VE.Features.Settings.UseAnimatedPanZoom ), true, true);
	this._config.AddItem(enableAnimatedMovementConfigItem);
	
	//------------------------------------------------------------------------------
	// Searching
	//------------------------------------------------------------------------------
	this._config.AddCategory(new VE_ConfigCat(VE_State.CategoryUserOptionsSearch, L_StateSearching_Text));

	// Client Page Size
	var clientPageSizeValidator = new VE_ValidValuesValidator("number", [5, 10]);
	var clientPageSizeConfigItem = new VE_ConfigItem(VE_State.ItemClientPageSize, L_StateSelectSearchResults_Text + ":", VE_State.CategoryUserOptionsSearch, clientPageSizeValidator, clientPageSize, true, true, true);
	this._config.AddItem(clientPageSizeConfigItem);


	//------------------------------------------------------------------------------
	// Locating Me
	//------------------------------------------------------------------------------
	this._config.AddCategory(new VE_ConfigCat(VE_State.CategoryUserOptionsLocate, L_StateLocatingMe_Text ));

	// Do not bug about Wi-Fi Auto-Locator
	var dontAskWifiConfigItem = new VE_ConfigItem(VE_State.ItemDontAskWiFiLocator, L_StateInstallLocationFinderNotification_Text , VE_State.CategoryUserOptionsLocate, booleanTypeValidator, Msn.VE.Marketization.IsEnabled( Msn.VE.Features.Settings.InstallLocationFinderNotification ), true, true);
	this._config.AddItem(dontAskWifiConfigItem);
	
}

// Loads the cookie stored state
VE_State.prototype._LoadStateFromCookie = function(persistent)
{
	var stateCookie = GetCookie(VE_State._GetCookieName(VE_State.CookieNameState, persistent));
	if (!stateCookie)
	{
		return;
	}
	
	// Load the serialization version from the separate cookie.
	var serializationVersion = currentUrlApiVersion;
	var serializationVersionCookie = GetCookie(VE_State.CookieNameSerializationVersion);

	// Only the first version will not have a stored version.
	if (!serializationVersionCookie)
	{
		serializationVersion = 1;
	}
	// Otherwise, try to decipher it.
	else
	{
		var urlApiVersionValidator = new VE_RangeValidator(1, 2, true);
		serializationVersion = urlApiVersionValidator.ParseValue(serializationVersionCookie);

		// If we can't decipher the stored version, this is an error condition & exit out prematurely.
		if (serializationVersion == null)
		{
			return;
		}
	}
	
	this._config.serializationVersion = serializationVersion;
	
	// Load the cookie.
	this._config.LoadValuesFromString(stateCookie);
	
	// start the collection viewer
	if(!persistent)
	{
		this._LoadCollectionFromCookie();
	}
}

VE_State.prototype._LoadCollectionFromCookie = function()
{
	var cid = GetCookie(VE_State.CookieNameCollectionId);
	
	var parameterString = GetUrlParameterString();
	//Loading from the cookie more than one time but document.location
	//has the cid paramter Yet.
	if(parameterString.indexOf(VE_State.ItemCollectionId) > -1)
	{
		parameterString = parameterString.split(VE_State.ItemCollectionId + "=");
		if(parameterString.length > 1)
		{
			cid = parameterString[1];
		}
	}
			
	//Here we load
	if(cid)
	{
		VE_State.prototype._LoadCollection(cid, true);
	}
}

// Loads state passed in with the url.
VE_State.prototype._LoadStateFromUrl = function()
{
	var parameterString = GetUrlParameterString();
	if (!parameterString)
	{
		return false;
	}
	
	var combinedQuerySpecified = false;
	
	// Check if we have a combined query - 
	// We need to process the url parameters differently if it is.
	if (parameterString.indexOf("q=") > -1)
	{
		var qIndex = parameterString.indexOf("q=");
		if (qIndex > -1 && (qIndex == 0 || parameterString.charAt(qIndex-1) == "&"))
		{
			combinedQuerySpecified = true;
			parameterString = parameterString.substr(qIndex);
			var nextAmpersand = parameterString.indexOf("&");
			if (nextAmpersand > -1)
			{
				parameterString = parameterString.substring(0, nextAmpersand);
			}
		}
	}
	
	if (combinedQuerySpecified)
	{
		// Load values from the parameters
		this._config.LoadValuesFromString(parameterString);
		
		// Pemove all searches and override any options that would cause the viewport to relocate
		this._config.SetValue(VE_State.ItemStartWhere, "");
		this._config.SetValue(VE_State.ItemAutolocate, false);
		
		return false;
	}
	else
	{
		var cookieZoom = this.GetZoom();

		// Check on special behaviors for URL parameters
		var urlParameters = GetUrlParameters();
		
		var centerPointSpecified = false;
		var zoomSpecified = false;
		
		// Check if a collectionId has been specified as a part of the url
		var collectionIdSpecified = false;
		var collectionId = null;
		
		for (var i = 0; i < urlParameters.length; i += 2)
		{
			switch (urlParameters[i])
			{
				case VE_State.ItemCenterPoint:
					centerPointSpecified = true;
					break;
				case VE_State.ItemZoom:
					zoomSpecified = true;
					break;
				case VE_State.ItemSpecifiedUrlApiVersion:
					// We need the specified URL version first before attempting to parse the URL parameters.
					// Otherwise we won't know how to interpret the parameters.
					// (There isn't a set property for this because we don't want the 
					// users to actually be able to set this value.)
					var urlApiVersionValidator = new VE_RangeValidator(1, 2, true);
					var serializationVersion = urlApiVersionValidator.ParseValue(urlParameters[i+1]);

					// Try to interpret as current version
					if (serializationVersion == null)
					{
						serializationVersion = this.GetCurrentUrlApiVersion();
					}
					
					this._config.serializationVersion = serializationVersion;
					break;
				case VE_State.ItemCollectionId:
					collectionIdSpecified = true;				
					collectionId = urlParameters[i+1];
				default:
					break;
			}
		}
		
		// Clear out all saved information - 
		// The assumption is that URL specifies an exact initial state that
		// the creator wanted to display
		this.SetSearchKeywords([]);
		this.SetScratchpadEntityIds([]);
		this.SetRoutePoints([]);
		
		// Load values from the parameters
		this._config.LoadValuesFromString(parameterString);
				
		// Use default zoom if only a center point has been specified.
		if (!zoomSpecified && centerPointSpecified)
		{
			this.SetZoom(defaultZoom);
		}
		// Otherwise if we only have a zoom specified, restore the old viewport.
		else if (zoomSpecified && !centerPointSpecified)
		{
			this.SetZoom(cookieZoom);
		}
		
		//Here we take the collectionId and give a call to load the items of the collection.
		//which will load the annotations of the collection asynchronously
		if(collectionIdSpecified)
		{
			VE_State.prototype._LoadCollection(collectionId, false);
		}
		
		// Return true for best fit only if we don't have another viewport specified.
		return !centerPointSpecified && this.GetStartWhere() == "" && !this.GetAutoLocate() && this.GetRoutePoints().length == 0;
	}
}

//Loads the specific collection using the collectionId passed into  the url
VE_State.prototype._LoadCollection = function(collectionId, isFromCookie)
{
	if(collectionId)
	{
		var cid = GetCookie(VE_State.CookieNameCollectionId);
		
		if(!isFromCookie)
		{
			if(cid && cid != '' && cid != 'undefined' && cid.length > 0)
			{
				if(!collectionId.match(cid))
					cid = collectionId;
			}
			else
				cid = collectionId;

			SetCookie(VE_State.CookieNameCollectionId,cid,VE_State.CookieExpDateEOT);	//Loading from URL -> Set the cookie for one more load.
		}
		else	
		{
			cid = collectionId;
			SetCookie(VE_State.CookieNameCollectionId,cid,new Date(1000, 12, 31, 23, 59));	//Loading from URL -> Set the cookie for one more load.
			DeleteCookie(VE_State.CookieNameCollectionId);
		}
			
		this._LoadAnnotations(cid,window.locale);		
	}
}

// This method calls the Handler to retrieve all the annotations 
// for a given collectionId related to the user.
VE_State.prototype._LoadAnnotations = function(collectionId,market)
{
	if(collectionId)
	{
		window._collectionIdFromState = collectionId;
		window.setTimeout("VE_CollectionsManager.RetrieveCollectionDetails(window._collectionIdFromState, window.locale);", 2000);
	}
}

// Stores the existing configuration in cookies.
VE_State.prototype.StoreStateInCookies = function()
{
	var searchesString = this._config.ToString(VE_State.CategorySearches, true);
	var viewportString = this._config.ToString(VE_State.CategoryViewport, true);
	var scratchpadString = this._config.ToString(VE_State.CategoryScratchpad, true);
	var optionsOnExitString = this._config.ToString(VE_State.CategoryUserOptionsExit, true);
	var optionsMapNavigationString = this._config.ToString(VE_State.CategoryUserOptionsMap, true);
	var optionsSearchingString = this._config.ToString(VE_State.CategoryUserOptionsSearch, true);
	var optionsLocatingMeString = this._config.ToString(VE_State.CategoryUserOptionsLocate, true);
	var drivingDirectionsString = this._config.ToString(VE_State.CategoryDirections, true);
	var trafficString = this._config.ToString(VE_State.CategoryTraffic, true);
	
	// Generate the cookie strings based on options.
	var sessionCookieString = optionsOnExitString + "&" + optionsMapNavigationString + "&" + optionsSearchingString + "&" + optionsLocatingMeString;
	
	var persistentCookieString = sessionCookieString;
	if (searchesString)
	{
		if (this._config.GetValue(VE_State.ItemSaveSearches))
		{
			persistentCookieString += "&" + searchesString;
		}
		else
		{
			sessionCookieString += "&" + searchesString;
		}
	}
	if (viewportString)
	{
		if (this._config.GetValue(VE_State.ItemSaveLocation))
		{
			persistentCookieString += "&" + viewportString;
		}
		else
		{
			sessionCookieString += "&" + viewportString;
		}
	}
	if (drivingDirectionsString)
	{
		sessionCookieString += "&" + drivingDirectionsString;
	}
	if (scratchpadString)
	{
		if (this._config.GetValue(VE_State.ItemSaveScratchpad))
		{
			var pclen = persistentCookieString.length;
			var sclen = sessionCookieString.length;
			persistentCookieString += "&" + scratchpadString;
			//
			// 3000 is a big fuzz factor.
			// 4000 is about how much can be stored total
			// 1000 is about what we have in overhead (from testing)
			// so 3000 gives pretty reliable results 
			persistentCookieString = IOSec.TruncateUrlSafe (persistentCookieString, 3000-sclen);
			pclen = persistentCookieString.length;
		}
		else
		{
			var sclen = sessionCookieString.length;
			var pclen = persistentCookieString.length;
			//
			// 3000 is a big fuzz factor.
			// 4000 is about how much can be stored total
			// 1000 is about what we have in overhead (from testing)
			// so 3000 gives pretty reliable results 
			sessionCookieString += "&" + scratchpadString;
			sessionCookieString = IOSec.TruncateUrlSafe (sessionCookieString, 3000-pclen);
		}
	}
	
	if (trafficString)
	{
		sessionCookieString += "&" + trafficString;
	}
	
	// Write State Cookies
	SetCookie(VE_State._GetCookieName(VE_State.CookieNameState, false), sessionCookieString);
	SetCookie(VE_State._GetCookieName(VE_State.CookieNameState, true), persistentCookieString, VE_State.CookieExpDateEOT);
	
	// Write Serialization Version Cookie.
	SetCookie(VE_State.CookieNameSerializationVersion, this.GetCurrentUrlApiVersion(), VE_State.CookieExpDateEOT);
}

// Appends a string to a cookie name depending on whether or not it is session or permanent
VE_State._GetCookieName = function(cookieName, persistent)
{
	if (persistent) 
	{
		return cookieName + VE_State.CookieNameSuffixPersistent;
	}
	return cookieName + VE_State.CookieNameSuffixSession;
}

//------------------------------------------------------------------------------
// VE Application Initialization Code
//------------------------------------------------------------------------------

// Restores the VE state from the currently stored state.
VE_State.prototype._RestoreFromState = function()
{	
	// Create message control
	CreateMessageControl();
	
	// Create the scratchpad
	VE_Scratchpad.CreatePanel();
		
	// Setup Map
	var params = new Object();
	params.latitude = this.GetLatitude();
	params.longitude = this.GetLongitude();
	params.zoomlevel = this.GetZoom();
	params.mapstyle = this.GetMapStyle();
	params.obliqueSceneId = this.GetScene();

	StartMap(params, GetWindowWidth(), GetWindowHeight());	
	
	map.SetAnimationEnabled(this.GetAnimatedMovementsEnabled());
	
	// Set-up VE_SearchManager & execute searches
	VE_SearchManager.maxSearches = this.GetMaxSearches();
	VE_SearchManager.displayCount = this.GetClientPageSize();
	VE_SearchManager.requestCount = this.GetServerPageSize();

	var searchKeywords = this.GetSearchKeywords();
	if (searchKeywords.length > 0)
	{
		// Search keywords are stored in escaped form.
		// They need to be unescaped before BulkSearch is called.
		for (var i = 0; i < searchKeywords.length; i++)
		{
			searchKeywords[i] = unescape(searchKeywords[i]);
		}
		
		VE_SearchManager.BulkSearch(searchKeywords);
		VE_Analytics.Log("VE | Search","Local Search");
	}
	
	// Perform a combined search if specified
	var combinedQuery = this.GetCombinedQuery();
	if (combinedQuery)
	{
		VE_SearchManager.CombinedSearch(combinedQuery);
	}

	
	// Display intro panel if necessary
	VE_Help.CreateHelpPanel();	
	
	BuildWelcomeLink();		
	
	// Display Welcome panel if necessary
	if(this.loadActions.displayIntroPanel )
	{
		if ( Msn.VE.Marketization.IsEnabled( Msn.VE.Features.MapAction.Welcome ) )
			VE_Help.CreateIntroPanel();
		else
			toggleTaskArea(false);	//Hide Welcome panel area	
	}
	if ( Msn.VE.Marketization.IsEnabled( Msn.VE.Features.MapAction.Collections ) )
		VE_Scratchpad.Populate(this.GetScratchpadEntityIds(), this.loadActions.doBestFit);
		
	document.wwf.where.value = useCurrentViewText;

	// Constrain what/where & start/end input boxes to the defined limit.
	var what = document.getElementById("what");
	var where = document.getElementById("where");
	what.setAttribute("maxLength", this.GetMaxInputCount());
	where.setAttribute("maxLength", this.GetMaxInputCount());	
	
	var start = document.getElementById("start");
	var end = document.getElementById("end");	
	VE_Directions.maxInputLength = this.GetMaxInputCount();

	
	var initialAction = "";
	
	// Perform initial start-up actions as necessary.
	// Order of precedence is
	// 1) Geocode
	// 2) Route (if both end points are specified)
	// 3) Autolocate
	// The actions are ranked in this manner because the information in route & autolocate is not lost, 
	// while the where1 information is not displayed anywhere on the screen after initial start-up.

	var startWhere = this.GetStartWhere();
	// If we have an address/place specified, search for it.
	if (startWhere != "")
	{
		startWhere = startWhere.replace(/'/g, '\\\'');
		initialAction = "VE_SearchManager.Search('', " + IOSec.EncodeJs(startWhere) + ")";
	}
	
	// If the user has specified route points for directions and Feature Marketization supports Driving Directions, use them to set-up initial values.
	var routePoints = this.GetRoutePoints();
	if (routePoints.length > 0 && Msn.VE.Marketization.IsEnabled( Msn.VE.Features.MapAction.DrivingDirections ) )
	{
	
		// If the start point and end point have not been defined just open empty Directions Panel		
		if ( !routePoints[0] && !routePoints[routePoints.length - 1] )
		{
			VE_Directions.Show();
		}

		var haveStart = false;
		// If the start point has been specified, set it.
		if (routePoints[0])
		{
			var startLocation = VE_Directions.ParseSerializedRoutePoint(routePoints[0]);
			if (startLocation != null)
			{
				VE_Directions.SetStart(startLocation[0] ? startLocation[0] : L_Origin_Text, startLocation[1], startLocation[2], true);
				haveStart = true;
			}
		}
		
		var haveEnd = false;
		// If we have more than point specified and the end point is defined, set it.
		if (routePoints.length > 1 && routePoints[routePoints.length - 1])
		{
			var endLocation = VE_Directions.ParseSerializedRoutePoint(routePoints[routePoints.length - 1]);
			if (endLocation != null)
			{
				VE_Directions.SetEnd(endLocation[0] ? endLocation[0] : L_Destination_Text, endLocation[1], endLocation[2], true);
				haveEnd = true;
			}
		}
		
		// Restore Last Used Direction Unit
		var lastDirectionUnit = this.GetSaveDirectionsUnit();
		if( lastDirectionUnit != "mi" )
		{
			VE_Directions.CurrentUnit = lastDirectionUnit;
		}
		
		if (!initialAction && haveStart && haveEnd)
		{
			initialAction = "VE_Directions.GetDirections(" + this.loadActions.initialLoad + ")";
		}
	}
	
	// Otherwise, execute an auto locate if specified in the URL parameters.
	if (!initialAction && this.GetAutoLocate() && Msn.VE.Marketization.IsEnabled( Msn.VE.Features.MapAction.LocateMe ) )
	{
		initialAction = "AutoLocate()";
	}
	
	if (initialAction)
	{
		window.setTimeout(initialAction, 1000); // kick this off after so start-up requests are done.	
	}
	
	if( this.GetTraffic() == 1 )
	{
		VE_TrafficManager.GetMarkets();
		window.setTimeout("VE_TrafficManager.GetTrafficInfo(true)", 1000);
	}
}
