//------------------------------------------------------------------------------
// UserOptions.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: EShinn
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
// VE_ConfigItem specific methods for User Options
//------------------------------------------------------------------------------

// Creates an HTML form element that best represents the ConfigurationItem
VE_ConfigItem.prototype.ToHtmlElement = function()
{
	if (this._validator.type == "boolean" && this._validator.validatorType != "VE_ArrayValidator")
	{
		var returnElement = document.createElement("div");
		var innerHTML = '<input id="' + VE_ConfigItem.GenerateHtmlControlID(this.name) + '" type="checkbox"';
		if (this.value)
		{
			innerHTML += ' checked="true"';
		}
		innerHTML += '>' + SanitizeHtmlString(this.description);
		returnElement.innerHTML = innerHTML;
		return returnElement;
	}
	else if (this._validator.validatorType == "VE_ValidValuesValidator")
	{
		var returnElement = document.createElement("div");
		var innerHTML = SanitizeHtmlString(this.description) + ' <select id="' + VE_ConfigItem.GenerateHtmlControlID(this.name) + '">';
				
		for (var i = 0; i < this._validator.validValues.length; i++)
		{
			innerHTML += '<option value="' + SanitizeHtmlString(this._validator.validValues[i]) + '"';
			if (this.value == this._validator.validValues[i])
			{
				innerHTML += ' selected="selected"';
			}
			innerHTML += '>' + SanitizeHtmlString(this._validator.validValues[i]) + '</option>';
		}
		
		innerHTML += '</select>';
		returnElement.innerHTML = innerHTML;
		
		return returnElement;
	}
	else
	{
		var returnElement = document.createElement("div");
		var innerHTML = this.description + ' <input id="' + VE_ConfigItem.GenerateHtmlControlID(this.name) + '" type="text" value="' + SanitizeHtmlString(this.value) + '">';
		
		returnElement.innerHTML = innerHTML;
		return returnElement;
	}
	return null;
}

// Sets value retrived from the control created by Default ToHtml() implementation
// Should be overriden if different naming scheme & control output is used.
VE_ConfigItem.prototype.SetValueFromControl = function()
{
	var valueControl = document.getElementById(VE_ConfigItem.GenerateHtmlControlID(this.name));
	if (!valueControl)
	{
		return;
	}
	// Depending on type, attempt to get value.
	if (this._validator.type == "boolean" && this._validatorType != "VE_ArrayValidator")
	{
		this.SetValue(valueControl.checked);
	}
	else if (this._validator.validatorType == "VE_ValidValuesValidator")
	{
		this.SetValue(valueControl.options[valueControl.selectedIndex].value);
	}
	else
	{
		this.SetValue(valueControl.value);
	}	
}

// Generates a unique control id for ToHtmlElement().
VE_ConfigItem.GenerateHtmlControlID = function(name)
{
	return "config_" + SanitizeHtmlString(name);
}

//------------------------------------------------------------------------------
// VE_Config specific methods for User Options
//------------------------------------------------------------------------------

// Has all the VE_ConfigItems in this VE_Config get values from 
// corresponding Html controls (if any)
VE_Config.prototype.GetValuesFromControls = function()
{
	for (var i = 0; i < this._items.length; i++)
	{
		this._items[i].SetValueFromControl();
	}
}

// Creates an HTML element that best represents this VE_Config.
// Relies upon VE_ConfigItem.ToHtmlElement
VE_Config.prototype.ToHtmlElement = function(userCanEditOnly)
{
	var returnElement = document.createElement("div");
	var itemsByCategory = new Array();
	
	// Sort items by category.
	for (var i = 0; i < this._items.length; i++)
	{
		if (userCanEditOnly && !this._items[i].userCanEdit)
		{
			continue;
		}

		var categoryItems = itemsByCategory[this._items[i].category];
		if (!categoryItems)
		{
			categoryItems = new Array();
			itemsByCategory[this._items[i].category] = categoryItems;
		}
		
		var itemElement = this._items[i].ToHtmlElement();
		if (itemElement != null)
		{
			categoryItems.push(itemElement);
		}
	}
	
	// Generate HTML elements by category.
	for (var i = 0; i < this._categories.length; i++)
	{		
		var categoryItems = itemsByCategory[this._categories[i].name];
		if (categoryItems)
		{
			returnElement.appendChild(document.createElement("br"));
			var categoryDiv = document.createElement("div");
			var categoryTitle = document.createElement("b");
			categoryTitle.appendChild(document.createTextNode(this._categories[i].description));
			categoryDiv.appendChild(categoryTitle);
					
			returnElement.appendChild(categoryDiv);
		
			for (var j = 0; j < categoryItems.length; j++)
			{
				categoryDiv.appendChild(categoryItems[j]);
			}
		}
	}

	return returnElement;	
}

//------------------------------------------------------------------------------
// VE_State specific methods for User Options
//------------------------------------------------------------------------------

// Saves the displayed options using the controls' values.
VE_State.prototype.SaveOptionsValues = function()
{
	this._config.GetValuesFromControls();
}

//------------------------------------------------------------------------------
// Global methods for User Options
//------------------------------------------------------------------------------

// Generates the necessary html for the options panel & populates
// the help panel with this this html.
function OpenOptionsPanel()
{
	// Set-up Help Panel.
	VE_Help.EnablePreventLayer();
	VE_Help.helpPanel.Show();	
	VE_Help.helpPanel.SetTitle(L_Settings_Text);
	VE_Help.helpPanel.body.innerHTML = "";
	
	// Creating a containing div with styling information.
	var containerElement = document.createElement("div");
	containerElement.id = "VE_UserOptions";

	// Add options HTML
	var optionsElement = state._config.ToHtmlElement(true);	
	containerElement.appendChild(optionsElement);
	
	// Add a line break after the options.
	containerElement.appendChild(document.createElement("br"));
	
	var buttonsContainer = document.createElement("div");
	
	// Add Save Options button
	var saveOptionsButton = document.createElement("input");
	saveOptionsButton.setAttribute("type", "button");
	saveOptionsButton.setAttribute("id", "saveOptions");
	saveOptionsButton.value = L_UserOptionsOk_Text;
	saveOptionsButton.className = "VE_UserOptions_Button";
	saveOptionsButton.onclick = SaveOptionsValues;
	buttonsContainer.appendChild(saveOptionsButton);
	
	// Add Cancel Save Options Button.
	var cancelSaveOptionsButton = document.createElement("input");
	cancelSaveOptionsButton.setAttribute("type", "button");
	cancelSaveOptionsButton.setAttribute("id", "cancelSaveOptions");
	cancelSaveOptionsButton.value = L_Cancel_Text;
	cancelSaveOptionsButton.className = "VE_UserOptions_Button";
	cancelSaveOptionsButton.onclick = CancelSaveOptionsValues;
	buttonsContainer.appendChild(cancelSaveOptionsButton);

	containerElement.appendChild(buttonsContainer);

	VE_Help.helpPanel.body.appendChild(containerElement);
	VE_Help.Redraw();
	VE_Analytics.Log("VE | Settings Pane");	
}

// Saves values stored in options controls.
function SaveOptionsValues()
{
	state.SaveOptionsValues();

	
	
	// Update the displayCount if necessary
	if (VE_SearchManager.displayCount != state.GetClientPageSize())
	{
		VE_SearchManager.displayCount = state.GetClientPageSize();
		VE_SearchManager.RefreshAll();
	}
	
	// Update animated zoom enabled.
	map.SetAnimationEnabled(state.GetAnimatedMovementsEnabled());
	
	VE_Help.ClosePanel();
}

// Hide the options aka help panel
function CancelSaveOptionsValues()
{
	VE_Help.ClosePanel();
}
