//------------------------------------------------------------------------------
// UserCollections/CollectionsManager.js 
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: shankak
//
//------------------------------------------------------------------------------

// the collection class
VE_Collection = function(name, id, isPublic, keywords, annotations, dateCreated, dateModified, length)
{
	//Name of the collection
	if(name)
	{
		this.Name = name.replace(/%0D%0A/g, '\r\n').replace(/%0A/g, '\n');
		//this.Name = unescape(name);
	}
	else
	{
		this.Name = "";
	}
	
	//Permanent Id of the created collection (Applies to MSN stored collections)
	if(id)
		this.MsnId = id;

	//Visibility of the collection object 
	if(isPublic)
		this.IsPublic = isPublic;
	
	//Date of creation
	if(dateCreated)
		this.DateCreated = dateCreated;
		
	//Date of Modification
	if(dateModified)
		this.DateModified = dateModified;


	//Keywords associated with a collection object
	if(keywords)
	{
		this.Keywords = keywords;
		if(this.Keywords[0] != null && this.Keywords.length > 0 && this.Keywords[0] != null)
		{
			this.Keywords[0] = this.Keywords[0].replace(/%0D%0A/g, '\r\n').replace(/%0A/g, '\n');
		}
	}
	else 
	{
		this.Keywords = new Array();
	}

	//Annotations associated with this collection object
	if(annotations)
		this.Annotations = annotations;
	else this.Annotations = new Array();
	
	//Dirty flag for a collection object
	this.Dirty = false;	

	//Annotation Count
	this.AnnotationCount = function()
	{
		if(this.Annotations)
			return this.Annotations.length;
		else 
			return 0;
	}

	//Clearing of annotations
	this.ClearAnnotations = function()
	{
		while(this.Annotations.length)
		{
			var item = this.Annotations.pop();
			item = null;
		}
	}
		
	//Add an Annotation to the Collection
	this.AddAnnotation = function(annotation, index)
	{
		if(annotation && this.Annotations)
		{ 
			if(index >=0 && index < this.Annotations.length )
			{
				this.Annotations.splice(index,0,annotation);
				return this.Annotations.length;
			}
			else
			{
				return this.Annotations.push(annotation);
			}
		}
	}
		
	this.RetrieveAnnotationById = function(annotationId)
	{
		if(this.Annotations.length > 0 && annotationId)
		{
			var i =0;
			while(i < this.Annotations.length)
			{
				if(annotationId == this.Annotations[i].Id)
					return i;			
				i++;
			}
		}	
		return null;
	}
	
	this.ResetOrder = function()
	{
		if(this.Annotations && this.Annotations != '' && this.Annotations != "undefined" && this.Annotations.length > 0)
		{
			for(var i =0; i < this.Annotations.length; i++)
			{
				if(this.Annotations[i] instanceof VE_Annotation)
				{
					this.Annotations[i].DisplayOrder = i;
					this.Annotations[i].Delta = 0;
				}
			}		
			
			this.Dirty = false;
		}
	}
	
	this.ReOrder = function()
	{
		this.Annotations.sort(this.sortByIndex);
		this.Dirty = false;
	}
	
	this.sortByIndex = function (x, y)
	{
		if(x instanceof VE_Annotation && y instanceof VE_Annotation)
		{
			x.DisplayOrder = x.DisplayOrder + x.Delta;
			y.DisplayOrder = y.DisplayOrder + y.Delta;
			
			x.Delta = 0;
			y.Delta = 0;
			
			return (x.DisplayOrder - y.DisplayOrder);
		}
	}
};



