//------------------------------------------------------------------------------
// UserCollections/CollectionsManager.js 
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: shankak, keithkin, a-pratin
//------------------------------------------------------------------------------

//Constructor for CollectionsManager
function VE_CollectionsManager (){};

//Collections Manager's Current set of Annotations
VE_CollectionsManager.Annotations = new Array();

//Number of queries yet to be done
VE_CollectionsManager.Queue = new Array();

//XML HTTP Object shared by everything
VE_CollectionsManager.xmlHttp = null;

//The collections manager's loading state
VE_CollectionsManager.isLoading = false;

//CollectionsManager's Unsaved collection's collectionIndex.
VE_CollectionsManager.unsavedCollectionIndex = 0;

//CollectionsManager's Unsaved collection's collection Title.
VE_CollectionsManager.unsavedCollectionTitle = L_CollectionManagerUnsavedCollectionTitle_Text;

//Current set of pushPins
VE_CollectionsManager.contextPins = new Array();

//CollectionsManager's Signed in property
VE_CollectionsManager.SignedIn = null;

//CollectionsManager's Signing in property
//This falg is set to true while we are still waiting for PP to return
VE_CollectionsManager.CheckingSignIn = true;

//CollectionsManager's Request timeout.
VE_CollectionsManager.RequestRoundTripExpiryTimer  = -1;

//CollectionsManager's XMLHTTP Request stack.
VE_CollectionsManager.RequestStack = new Array();

//Current Collection Index
VE_CollectionsManager.collectionIndex = null;

//Viewer handle
VE_CollectionsManager.Viewer = null; 

//Collections Manager's unsaved Collection
VE_CollectionsManager.Collection = null;

//Collections Manager's current lis retrieved collections
VE_CollectionsManager.CollectionList = null;

//Z Index for the pin
VE_CollectionsManager.pinZIndex = VE_MapPushpin.baseZIndex;

//Timer for delayed update requests
VE_CollectionsManager.timerId = -1;
										
// The CreateAnnotation method has a set of mandatory and non
// mandatory arguments that have to be specified for the annotation
// to be created at all

VE_CollectionsManager.CreateAnnotation = function ( creationID,
													collectionID,
													title,
													latitude,
													longitude,
													notes,
													keywords,
													infoUrl,
													photoUrl,
													userDate,
													displayOrder,
													businessListingId,
													iconId,
													state,
													market,
													insertIndex)
													
{
	
	var unsavedAnnotation = new VE_Annotation(	creationID,
												title,
												infoUrl,
												photoUrl,
												userDate,
												notes,
												latitude,
												longitude,
												businessListingId,
												displayOrder,
												iconId,
												Date(),
												keywords
												);
												
	if(collectionID == VE_CollectionsManager.unsavedCollectionIndex)
	{
		if(!VE_CollectionsManager.Collection)
		{
			VE_CollectionsManager.Collection = new  VE_Collection(VE_CollectionsManager.unsavedCollectionTitle,						  //Name
																	VE_CollectionsManager.unsavedCollectionIndex,					  //Id
																	'false',														  //Non Public by default
																	null,															  //No keywords
																	null);															  //No Annotations
																	
			//VE_CollectionsManager.Annotations = VE_CollectionsManager.Collection.Annotations;	
			
		}
				
		if(VE_CollectionsManager.Collection.RetrieveAnnotationById(creationID) == null)
		{	
			if(VE_CollectionsManager.Collection.AnnotationCount() >= 10)
			{
				ShowMessage ( VE_Annotations.ScratchPadFullMsg );
				return false;				
			}
			
			if(typeof(insertIndex) == "number")
			{
				VE_CollectionsManager.Collection.AddAnnotation(unsavedAnnotation, insertIndex);
			}
			else
			{
				VE_CollectionsManager.Collection.AddAnnotation(unsavedAnnotation);
			}
		}

	}
	else if (collectionID)
	{
				
		if(VE_CollectionsManager.Annotations)
		{
			if(VE_CollectionsManager.Annotations.length >= VE_CollectionsManagerConstants.MAX_COLLECTION_SIZE)
			{
				ShowMessage ( L_CollectionManagerMaxLimitReached_Text );
				if(latitude && latitude != 'undefined' && latitude != '' && longitude && longitude != 'undefined' && longitude != '')
					map.PanToLatLong(parseFloat(latitude),parseFloat(longitude));
				return false;				
			}
		}
		
		//Query for the server
		var query = VE_CollectionsManagerConstants.Handler + VE_AnnotationActions.Create + "&" ;

		//Scratchpad is supposed to do all the validation on the data.
		//Create the POST request body and send the request to the server
		//get the response and call the appropriate ScratchPad method to 
		//to add the created annotation on the display.

		query += (VE_CollectionsManagerConstants.C_Id + "=" + collectionID + "&") ;
		
		query += VE_CollectionsManager.SerializeAnnotation(title,
														   latitude,
														   longitude,
														   notes,
														   infoUrl,
														   photoUrl,
														   userDate,
														   displayOrder,
														   businessListingId,
														   iconId,
														   keywords,
														   null,	//index is null (single annotation)
														   null,	//state	is null (state applies to update annotation only)
														   0		//delta (for new annotations it is always 0)
														   );
		
	
		query += (VE_CollectionsManagerConstants.Market + market  + "&");
		query += (VE_CollectionsManagerConstants.A_CreationId + "=" + creationID);
		query += ("&" + VE_CollectionsManagerConstants.Random + Math.random());
		
		
		if(VE_CollectionsManager.Annotations == null)
			VE_CollectionsManager.Annotations = new Array();
		
		VE_CollectionsManager.Annotations.push(unsavedAnnotation);
		
		//push any updates for a later time...
		VE_CollectionsManager.UpdateCollection(VE_CollectionsManager.collectionIndex, VE_CollectionsManager.Annotations,window.locale,VE_CollectionsManagerConstants.TimerDelay);
		
		VE_CollectionsManager.SendRequest(query, true);
	}
}


//Creates the query string for creation and updates of collections.
//Right now this one function does the job. but in future it might be
//split into multiple helper functions / might be generalized.
VE_CollectionsManager.SerializeAnnotation = function(title,
													 latitude,
													 longitude,
													 notes,
													 infoUrl,
													 photoUrl,
													 userDate,
													 displayOrder,
													 businessListingId,
													 iconId,
													 keywords,
													 index,
													 state,
													 delta)
{
	var query = "";

	if(title.length > 0)
	{
		query += VE_CollectionsManager.CreateQueryParameter(VE_CollectionsManagerConstants.A_Title,index);
		query += ("=" + IOSec.EncodeUrl(title) + "&");
	}
	else return;

	if(latitude)
	{
		query += VE_CollectionsManager.CreateQueryParameter(VE_CollectionsManagerConstants.A_Latitude,index);
		query += ("=" + latitude + "&");
	}
	else return;
	
	if(longitude)
	{
		query += VE_CollectionsManager.CreateQueryParameter(VE_CollectionsManagerConstants.A_Longitude,index);
		query += ("=" + longitude + "&");	
	}
	else return;
	
	query += VE_CollectionsManager.CreateQueryParameter(VE_CollectionsManagerConstants.A_Notes,index);
	if(notes)		
		query += ("=" + IOSec.EncodeUrl(notes) + "&");	
	else query += ("=" + '' + "&");								//Localize and Make Resource. No Hardcoding


	query += VE_CollectionsManager.CreateQueryParameter(VE_CollectionsManagerConstants.A_InfoUrl,index);
	if(infoUrl)
		query += ("=" + IOSec.EncodeUrl(infoUrl.replace(/\|/g, ' ').replace(/\^/g, ' ').replace(/^\s+/g, "").replace(/\s+$/g, "")) + "&");
	else query += ("=" + '' + "&");								//Localize and Make Resource. No Hardcoding


	query += VE_CollectionsManager.CreateQueryParameter(VE_CollectionsManagerConstants.A_PhotoUrl,index);
	if(photoUrl)
		query += ("=" + IOSec.EncodeUrl(photoUrl.replace(/\|/g, ' ').replace(/\^/g, ' ').replace(/^\s+/g, "").replace(/\s+$/g, "")) + "&");
	else query += ("=" + '' + "&");								//Localize and Make Resource. No Hardcoding

	
	query += VE_CollectionsManager.CreateQueryParameter(VE_CollectionsManagerConstants.A_UserDate,index);
		

	userDate = new Date();
			
	query += ("=" + (userDate.getUTCMonth() + 1) + "/" + userDate.getUTCDate() + "/" + userDate.getUTCFullYear() + "&");		

	
	//Delta is always initialized to 0
	query += VE_CollectionsManager.CreateQueryParameter(VE_CollectionsManagerConstants.A_DisplayOrder,index);
	if(displayOrder != null)
		query += ("=" + (parseInt(displayOrder) + (isNaN(parseInt(delta)) ? 0 : parseInt(delta))) + "&");						
	else query += ("=" + '0' + "&");										//Localize and Make Resource. No Hardcoding
		
	query += VE_CollectionsManager.CreateQueryParameter(VE_CollectionsManagerConstants.A_BusinessId,index);
	if(businessListingId)
		query += ("=" + businessListingId + "&");
	else query += ("=" + '0' + "&");										//Localize and Make Resource. No Hardcoding
	
	query += VE_CollectionsManager.CreateQueryParameter(VE_CollectionsManagerConstants.A_IconId,index);
	if(iconId)
		query += ("=" + iconId + "&");
	else query += ("=" + '0' + "&");										//Localize and Make Resource. No Hardcoding

	query += VE_CollectionsManager.CreateQueryParameter(VE_CollectionsManagerConstants.A_Keywords,index);
	if(keywords)
		query += ("=" + IOSec.EncodeUrl(keywords));
	else query += ("=" + escape(' '));										//Localize and Make Resource. No Hardcoding
	
	if(state)
	{
		query += ("&" + VE_CollectionsManager.CreateQueryParameter(VE_CollectionsManagerConstants.A_State,index));
		query += ("=" + state);
	}
	
	if(index == null)
		query += "&";
	
	return query;
}

//Helper method
VE_CollectionsManager.CreateQueryParameter = function (parameter, index)
{
	if(parameter != null && index != null)
		return (parameter+index);
	else if(parameter != null && index == null)
		return parameter;
	else return null;
}
									
// The CreateCollection  method has a manadatory argument of
// the type Collection. This method takes the Collection objec
// and call the appropriate method on the handler to create the
// collection.
VE_CollectionsManager.CreateCollection = function ( name,
													ispublic,
													keywords,
													annotationCount,
													annotations,
													market)
{

	//First verify if the user has logged in or not. If not, ask them to login and return
	if(!VE_Passport || !VE_Passport.isAuthenticated)
	{
		ShowMessage(L_FavoritesSignInSuggestion_Text);							
		return;
	}
	
	//Else try to save it

	var query = VE_CollectionsManagerConstants.Handler + VE_CollectionActions.Create + "&";
		
	if(name != null && name != "undefined" && name.length > 0 && annotationCount >= 0 && ispublic != null && ispublic != "undefined")
	{
		
		query += (VE_CollectionsManagerConstants.C_Name + "=" + IOSec.EncodeUrl(name) + "&");
		query += (VE_CollectionsManagerConstants.C_IsPublic + "=" + ispublic + "&");
		query += (VE_CollectionsManagerConstants.C_Keywords + "=" + IOSec.EncodeUrl(keywords.join(",")) + "&");
		query += (VE_CollectionsManagerConstants.C_Length + "=" + annotationCount + "&");
		query += (VE_CollectionsManagerConstants.Market + market + "&");
		
		
		if(annotationCount > 0 && annotations != null)
		{
			for(var i =0; i < annotationCount;  i++)
			{	
				
				query +=	VE_CollectionsManager.SerializeAnnotation(	annotations[i].Title,
																		annotations[i].Latitude,
																		annotations[i].Longitude,
																		annotations[i].Notes,
																		annotations[i].Url,
																		annotations[i].PhotoUrl,
																		annotations[i].UserDate,
																		annotations[i].DisplayOrder,
																		annotations[i].BusinessListingId,
																		annotations[i].IconId,
																		annotations[i].Keywords,
																		i,								//index
																		null,							//state - only applies to update
																		annotations[i].Delta);
				
				if(i < (annotations.length - 1))
					query += "&";
			}
		}
		
		query += ("&" + VE_CollectionsManagerConstants.Random + Math.random());	
		VE_CollectionsManager.SendRequest(query, true);
		
	}

	return;
}


//This method calls the Handler to update a given annotation
VE_CollectionsManager.UpdateAnnotation = function ( creationId, 
													annotationId,
													title,
													latitude,
													longitude,
													notes,
													keywords,
													infoUrl,
													photoUrl,
													userDate,
													displayOrder,
													businessListingId,
													iconId,
													state,
													market,
													index 
												)
{

	
	var persistentId = null;
	
	if(annotationId)
		persistentId = annotationId;
	else if(creationId)
		persistentId = creationId;
	
	var isStored = false;
	var annotation = null;	
	
	if(VE_CollectionsManager.collectionIndex && 
		VE_CollectionsManager.collectionIndex != "undefined" && 
		VE_CollectionsManager.collectionIndex != '' &&
		VE_CollectionsManager.collectionIndex != VE_CollectionsManager.unsavedCollectionIndex)
	{
		if(VE_CollectionsManager.Annotations && VE_CollectionsManager.Annotations.length > 0 && VE_CollectionsManager.Annotations != "undefined")
		{
			if(index >= 0 && index < VE_CollectionsManager.Annotations.length )
			{
				annotation = VE_CollectionsManager.Annotations[index];						
			}
		}
		
		isStored = true;		
	}
	else
	{
		if(index >= 0 && 
			VE_CollectionsManager.Collection instanceof VE_Collection && 
			index < VE_CollectionsManager.Collection.AnnotationCount())
		{
			annotation = VE_CollectionsManager.Collection.Annotations[index];						
		}
	}
	
	if(annotation instanceof VE_Annotation)
	{
		annotation.Title = title;
		annotation.Latitude = latitude;
		annotation.Longitude = longitude;
		annotation.Notes = notes;
		annotation.Url = infoUrl;
		annotation.PhotoUrl = photoUrl;
		annotation.UserDate = userDate;
		annotation.State = state;
		//annotation.Keywords = keywords;
		//annotation.BusinessListingId = businessListingId;
		//annotation.IconId = iconId;
	}
	else
	{
		return;
	}
	
	//Don't bother going to server. This is the unsaved Collection.
	if(!isStored)
	{
		return;
	}
	else if(persistentId && persistentId != '' && persistentId != "undefined")
	{
		
		VE_CollectionsManager.ClearTimedRequest();
		
		var query = VE_CollectionsManagerConstants.Handler + VE_AnnotationActions.Update + "&";
		
		query += (VE_CollectionsManagerConstants.A_Id + "=" + persistentId + "&") ;
		
		query += VE_CollectionsManager.SerializeAnnotation(annotation.Title,
													annotation.Latitude,
													annotation.Longitude,
													annotation.Notes,
													annotation.Url,
													annotation.PhotoUrl,
													annotation.UserDate,
													annotation.DisplayOrder,
													annotation.BusinessListingId,
													annotation.IconId,
													annotation.Keywords);
													
		query += (VE_CollectionsManagerConstants.Market + market);
		query += ("&" + VE_CollectionsManagerConstants.Random + Math.random());

		if(VE_CollectionsManager.timerId == -1)
		{
			//There are no pending requests for updating the collection.
			VE_CollectionsManager.SendRequest(query);
		}
		else
		{
			//push any updates for a later time...if the collection is already dirty
			VE_CollectionsManager.UpdateCollection(VE_CollectionsManager.collectionIndex, VE_CollectionsManager.Annotations,window.locale,VE_CollectionsManagerConstants.TimerDelay); 
		}
	}	
}

//Update a collection with all the collection paramaters
VE_CollectionsManager.UpdateCollection = function (	collectionId, annotations, market, delay)
{
	if(collectionId && annotations)
	{
		//Retrieve the collection List item	
		var collectionItem = VE_CollectionsManager.RetrieveByKey(collectionId);
		
		VE_CollectionsManager.ClearTimedRequest();
		
		if(collectionItem)
		{
			//Now create the update query
			if(collectionItem.Dirty)
			{
				var query = VE_CollectionsManagerConstants.Handler + VE_CollectionActions.Update + "&";
				query += (VE_CollectionsManagerConstants.C_Id + "=" + collectionId + "&");
				query += (VE_CollectionsManagerConstants.C_Name + "=" + IOSec.EncodeUrl(collectionItem.Name) + "&");
				query += (VE_CollectionsManagerConstants.C_IsPublic + "=" + collectionItem.IsPublic + "&");
				if(collectionItem.keywords != null && collectionItem.keywords.length > 0)				
					query += (VE_CollectionsManagerConstants.C_Keywords + "=" + IOSec.EncodeUrl(collectionItem.keywords[0]) + "&");
				else
					query += (VE_CollectionsManagerConstants.C_Keywords + "=&");
				query += (VE_CollectionsManagerConstants.C_Length + "=" + annotations.length + "&");
				query += (VE_CollectionsManagerConstants.Market + market + "&");
				
				var persistentId = null;
				
				for(var i =0; i < annotations.length;  i++)
				{	
					var annoItem = annotations[i];
					
					if(annoItem instanceof VE_Annotation)
					{
						persistentId = annotations[i].Id;	
							
						if(annoItem.msnId  && 
							annoItem.msnId != null && 
							annoItem.msnId != '' && 
							annoItem.msnId != 'undefined')
						{
							persistentId = annoItem.msnId;						
						}
		
						query += (VE_CollectionsManagerConstants.A_Id + i + "=" + persistentId + "&") ;
						
						query +=	VE_CollectionsManager.SerializeAnnotation(	annotations[i].Title,
																				annotations[i].Latitude,
																				annotations[i].Longitude,
																				annotations[i].Notes,
																				annotations[i].Url,
																				annotations[i].PhotoUrl,
																				annotations[i].UserDate,
																				annotations[i].DisplayOrder,
																				annotations[i].BusinessListingId,
																				annotations[i].IconId,
																				annotations[i].Keywords,
																				i,							//Index of this annotation in a series of annotations
																				annotations[i].State,
																				annotations[i].Delta);
						
						if(i < (annotations.length - 1))
						{
							query += "&";
						}
						
					}
				}
				
				query += ("&" + VE_CollectionsManagerConstants.Random + Math.random());	
				
				//Pop last queued query if any. we got new one now...
				if(VE_CollectionsManager.Queue.length > 0)
					VE_CollectionsManager.Queue.pop();
					
				//push the query on to the queue
				VE_CollectionsManager.Queue.push(query);
				
				//For immediate query set delay to 0...
				
				if(delay != null && delay != '' && delay != 'undefined' && delay > 0)
				{
					//For immediate query set delay to 0...
					VE_CollectionsManager.timerId = window.setTimeout("VE_CollectionsManager.FireTimedRequest()",delay);
				}
				else
				{
					VE_CollectionsManager.SendRequest(query, true);
				}
				
			}
		}
		
		collectionItem = null;
	}	
}

//This method is the thread start delegate for update requests.
VE_CollectionsManager.FireTimedRequest = function()
{
	if(	VE_CollectionsManager.Queue && 
		VE_CollectionsManager.Queue != 'undefined' && 
		VE_CollectionsManager.Queue != ''&&
		VE_CollectionsManager.Queue.length > 0 )
		
	var query = VE_CollectionsManager.Queue.pop();
	
	if(query && query != 'undefined' && query != '' && query.length > 0)
	{
		VE_CollectionsManager.SendRequest(query, true);
		query = null;
	}
}

// This method calls the Handler to retrieve all the collections 
// related to the user.
VE_CollectionsManager.RetrieveAllCollections = function(market)
{
	//Clear the timer - this is a complete refresh
	VE_CollectionsManager.ClearTimedRequest();
			
	var query = VE_CollectionsManagerConstants.Handler + VE_CollectionActions.RetrieveAll + "&";
	    query += VE_CollectionsManagerConstants.Market + market;
	    query += ("&" + VE_CollectionsManagerConstants.Random + Math.random());
	    
	VE_CollectionsManager.SendRequest(query, true);
}

// This method calls the Handler to retrieve all the annotations 
// for a given collectionId related to the user.
VE_CollectionsManager.RetrieveAllAnnotations = function(collectionId,market)
{	
	//push in any pending updates.
	VE_CollectionsManager.UpdateCollection(VE_CollectionsManager.collectionIndex,VE_CollectionsManager.Annotations,window.locale);
		
	//Setting the new collectionId 
	VE_CollectionsManager.collectionIndex = collectionId;
	
	//Check if the unsaved collection has been selected and 
	//act appropriately
	if(collectionId == VE_CollectionsManager.unsavedCollectionIndex)
	{
		if(!VE_CollectionsManager.Collection)
		{
			VE_CollectionsManager.Collection = new  VE_Collection(VE_CollectionsManager.unsavedCollectionTitle,						  //Name
																	VE_CollectionsManager.unsavedCollectionIndex,					  //Id
																	'false',														  //Non Public by default
																	null,															  //No keywords
																	null);															  //No Annotations
		}	
		
		if(VE_CollectionsManager.Collection instanceof VE_Collection)
		{
			if(VE_CollectionsManager.Collection.Dirty)
			{
				VE_CollectionsManager.Collection.ReOrder();
			}
			
			VE_CollectionsManager.RetrieveAllAnnotationsCallback(VE_CollectionsManager.Collection.Annotations);	
		}
	}
	else
	{
		var query = VE_CollectionsManagerConstants.Handler + VE_AnnotationActions.RetrieveAll + "&";
		query += (VE_CollectionsManagerConstants.C_Id + "=" + collectionId + "&" );
		query += VE_CollectionsManagerConstants.Market + market;
		query += ("&" + VE_CollectionsManagerConstants.Random + Math.random());

		VE_CollectionsManager.SendRequest(query, true);
	}

}

//This method calls the handler to retrieve all information about a specific
//collection
VE_CollectionsManager.RetrieveCollectionDetails = function(collectionId,market)
{
	if(collectionId && collectionId != VE_CollectionsManager.unsavedCollectionIndex)
	{
		var query = VE_CollectionsManagerConstants.Handler + VE_CollectionActions.RetrieveDetails + "&";
			query += (VE_CollectionsManagerConstants.C_Id + "=" + collectionId + "&");
			query += (VE_CollectionsManagerConstants.Market + market);
			query += ("&" + VE_CollectionsManagerConstants.Random + Math.random());
		
		//No Authentication is needed for collection viewer calls. hence IsAuthNeeded is false.
		VE_CollectionsManager.SendRequest(query, false);

		//Update the states and permalinks
		state.UpdateFromScratchpad();
		
		if(VE_CollectionsManager.Viewer)
		{
			VE_CollectionsManager.KillViewer(false);
		}
		
		VE_CollectionsManager.Viewer = new VE_CollectionsViewer(VE_CollectionsManagerConstants.ViewerDefaultTitle, collectionId);
	}
}


//This method calls the handler to delete all annotations related to a collection
VE_CollectionsManager.DeleteAllAnnotations = function(collectionId,market)
{
	if(collectionId == VE_CollectionsManager.unsavedCollectionIndex || collectionId == null)
	{
		//local delete of unsaved collection's annotations
		if(VE_CollectionsManager.Collection)
		{
			//set the current set of annotations to the unsaved collection's
			//annotations and spoof the retrieveallannotationscallback to clear the pushpins
			
			if(VE_CollectionsManager.Annotations == null)
				VE_CollectionsManager.Annotations = VE_CollectionsManager.Collection.Annotations;
							
			VE_CollectionsManager.Collection.ClearAnnotations();
			
			cancelDeleteCollection();
			
			VE_CollectionsManager.RetrieveAllAnnotationsCallback(VE_CollectionsManager.Collection.Annotations);
		}
	}
	else
	{
		//Clear the timer and push any updates for later time 
		VE_CollectionsManager.ClearTimedRequest();
				
		var query = VE_CollectionsManagerConstants.Handler + VE_CollectionActions.DeleteAllAnnotations + "&";
			query += (VE_CollectionsManagerConstants.C_Id + "=" + collectionId + "&");
			query += (VE_CollectionsManagerConstants.Market + market);
			query += ("&" + VE_CollectionsManagerConstants.Random + Math.random());
		
		VE_CollectionsManager.SendRequest(query, true);		
	}
}


//This method calls the Handler to delete all collections
//pertaining to the current user
VE_CollectionsManager.DeleteAllCollections = function(market)
{
	//Clear the timer
	VE_CollectionsManager.ClearTimedRequest();
	
	var query = VE_CollectionsManagerConstants.Handler + VE_CollectionActions.DeleteAll + "&";
		query += VE_CollectionsManagerConstants.Market + market;
		query += ("&" + VE_CollectionsManagerConstants.Random + Math.random());
		
	VE_CollectionsManager.SendRequest(query, true);
}

//This method calls the Handler to delete a specific annotation
//identified by the annotationId
VE_CollectionsManager.DeleteAnnotation = function(annotationId,market)
{
	if(annotationId != 'undefined' && annotationId != null && annotationId.length > 0 )
	{
		if(VE_CollectionsManager.collectionIndex == null || VE_CollectionsManager.collectionIndex == VE_CollectionsManager.unsavedCollectionIndex)
		{
			var spliceIndex = -1;
			
			if(VE_CollectionsManager.Collection && VE_CollectionsManager.Collection.Annotations)
			{
				spliceIndex = VE_CollectionsManager.Collection.RetrieveAnnotationById(annotationId);	
				
				if(spliceIndex >= 0 && spliceIndex < VE_CollectionsManager.Collection.Annotations.length)
				{
					VE_CollectionsManager.Collection.Annotations.splice(spliceIndex,1);
					VE_CollectionsManager.RetrieveAllAnnotationsCallback(VE_CollectionsManager.Collection.Annotations,VE_CollectionsManager.collectionIndex);
				}
			}
		}
		else
		{
			//clear the timer...
			VE_CollectionsManager.ClearTimedRequest();
			
			var query = VE_CollectionsManagerConstants.Handler + VE_AnnotationActions.Delete + "&";
			query += (VE_CollectionsManagerConstants.A_Id + "=" + annotationId + "&");
			query += VE_CollectionsManagerConstants.Market + market;
			query += ("&" + VE_CollectionsManagerConstants.Random + Math.random());
			 
			VE_CollectionsManager.SendRequest(query, true);
		}
		
		try
		{
			map.RemovePushpin(annotationId);
		}
		catch(everything){ /* swallow... */ }
	}
}

VE_CollectionsManager.OnSignIn = function()
{
	VE_CollectionsManager.SignedIn = true;
	buildScratchPadInfoBar();
	
}

VE_CollectionsManager.OnSignOut = function()
{
	VE_CollectionsManager.SignedIn = false;
	buildScratchPadInfoBar();
}

//Management of the XML Http Object
VE_CollectionsManager.CreateRequest = function()
{
	//Build the XML Http.
	if ( !VE_CollectionsManager.xmlHttp || window.XMLHttpRequest )
	{
		VE_CollectionsManager.xmlHttp = null;
		VE_CollectionsManager.xmlHttp = GetXmlHttp();
	}
	
	// Send the request
	var x = VE_CollectionsManager.xmlHttp;
	if(x)
	{
		VE_CollectionsManager.xmlHttp = x;
	}	

}


//This method calls the CreateRequest method to build the 
//xmlHttp and then uses it to send the request to the server
//using the url argument.
VE_CollectionsManager.SendRequest = function(url,IsAuthNeeded)
{			
	var request = GetXmlHttp();
	if(request)
	{
		var split =null;
		if(url)
		{
			split = url.split("?");
		}
		if(split.length >= 1)
			request.open("POST", split.shift(), true);

		//VE_CollectionsManager.LastSentRequest = request;
		
		//The request has been sent and we kick off a timer for 5 seconds.
		//if the response came back before that the timer is cleared. else
		//the server error message is shown and the request is aborted.
		//VE_CollectionsManager.RequestRoundTripExpiryTimer = window.setTimeout(VE_CollectionsManager.ClearLastSentRequest, VE_CollectionsManagerConstants.ExtendedTimerDelay);

			
		request.onreadystatechange = function()
		{			
			if(request.readyState==4) 
			{
				var code = request.responseText;				
				try
				{
					eval(code);
				}
				catch(ex)
				{
					ShowMessage(L_ErrorServerBusy_Text);					
				}
				finally
				{
					request = null;
				}
			}
		}
		request.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
		if(split.length >= 1)
		{
			request.send(split.join("?"));
		}
		split = null;
	}
}

VE_CollectionsManager.ClearLastSentRequest = function()
{	
	ShowMessage(L_ErrorServerBusy_Text);
		
	if(	VE_CollectionsManager.LastSentRequest != null &&
		VE_CollectionsManager.LastSentRequest != '' &&
		VE_CollectionsManager.LastSentRequest != "undefined" )
		{
			try
			{
				VE_CollectionsManager.LastSentRequest.abort();
			}
			catch(ex)
			{
			}
			finally
			{
				VE_CollectionsManager.LastSentRequest = null;
			}
		}
}

//This function is the response handler and does and eval on the 
//returned response code. because some methods in the user collections
//handler return code which on eval calls the call back functions.
VE_CollectionsManager._ResponseHandler = function(request)
{
}

// This the callback method for DeleteAllAnnotations which is called
// on eval of the response recieved from the server.
VE_CollectionsManager.DeleteAllAnnotationsCallback = function(collectionId,status)
{
	//get rid of the deletion UI
	cancelDeleteCollection();
	//more straight forward to do it this way. so if the collections annotations did
	//not get deleted for some reason, the scratchpad will reflect it...
	if(VE_CollectionsManager.collectionIndex == collectionId && eval(status))
	{
		scratchPad.clear();
		VE_CollectionsManager.ClearPushPins();
		while(this.Annotations.length)
		{
			this.Annotations.pop();
		}
		VE_CollectionsManager.RetrieveAllAnnotations(collectionId,window.locale);
	}
}

// This the callback method for CreateCollection which is called
// on eval of the response recieved from the server.
VE_CollectionsManager.CreateCollectionCallback = function (collectionId,status)
{
	if(status == "false")
	{
		ShowMessage(L_ErrorServerBusy_Text);
		return;
	}
	else if(status == "exists" && collectionId != null && collectionId != 'undefined' && collectionId != '')
	{
		ShowMessage(L_CollectionManagerNameTakenMessage_Text);
		return;
	}

	scratchPad.clearTitles();
	VE_CollectionsManager.RetrieveAllCollections(window.locale);
	if(VE_CollectionsManager.Collection)
		VE_CollectionsManager.Collection = null;
	VE_CollectionsManager.collectionIndex = collectionId;
	scratchPad.clear();
}

// This the callback method for CreateAnnotation which is called
// on eval of the response recieved from the server.
VE_CollectionsManager.CreateAnnotationCallback = function (annotationCreationId, annotationId,status)
{
	//Here is the trick to do thing without breaking a lot of stuff with 
	//scratchpad and other items that are related to collections and annotations
	
	//On the fly we are creating a property for the scratchpad item called msnId
	//which will help us in making updates to the annotations on MSN Storage
	
	if(annotationCreationId && annotationId && eval(status))
	{		
		var sItem = scratchPad.getItems().retrieveByKey(annotationCreationId);
		if(sItem && sItem != null && sItem != 'undefined')
			sItem.msnId = annotationId;
		else
			return;
		
		if(	VE_CollectionsManager.Annotations && 
			VE_CollectionsManager.Annotations != 'undefined' && 
			VE_CollectionsManager.Annotations.length > 0)
		{		
			//Updating the local annotations list after we create one and server returned a positive message
			var length = VE_CollectionsManager.Annotations.length - 1;
			while( VE_CollectionsManager.Annotations[length].Id != annotationCreationId && length >= 0 )
			{
				--length;
			}
			
			if(VE_CollectionsManager.Annotations.length  > length && VE_CollectionsManager.Annotations[length] instanceof VE_Annotation )
				VE_CollectionsManager.Annotations[length].msnId = annotationId;
		}
	}
}

// This the callback method for DeleteCollection which is called
// on eval of the response recieved from the server
VE_CollectionsManager.DeleteCollectionCallback = function (collectionID,status)
{
	if(collectionID != null  && collectionID != "undefined"  && collectionID != '' && status)
	{
		scratchPad.removeCurrentTitle();
		
		if(VE_CollectionsManager.CollectionList == null || VE_CollectionsManager.CollectionList.length == 0 || VE_CollectionsManager.CollectionList == "undefined")
		{		
			return;
		}
			
		var i =0;
		var index = 0;
		var ilen = VE_CollectionsManager.CollectionList.length;
		for(var i=0; i<ilen;i++)
		{
			if(!VE_CollectionsManager.CollectionList[i])
				continue;
				
			if(VE_CollectionsManager.CollectionList[i].ID == collectionID )
			{
				VE_CollectionsManager.CollectionList.splice(i,1);
				break;
			}
		}
		
	}
	else
	{
		ShowMessage(L_CollectionManagerCollectionCannotBeDeleted_Text);				
	}
}

// This the callback method for DeleteAllCollections which is called
// on eval of the response recieved from the server
VE_CollectionsManager.UpdateAnnotationCallback = function (collectionID,status)
{
	//Add code here to create the HTML Content for
	//Collection updated from MSN Storage
	/*
	if(!status)
	{
		ShowMessage(L_ErrorServerBusy_Text);							
	}
	*/
}

// This is the callback method for UpdateCollection which is called
// on eval of the response recieved from the server
VE_CollectionsManager.UpdateCollectionCallback = function (collectionId,status)
{
	//Any code for indicating response from the server goes here...
	if(status == "true")
	{
		//checking for queue length == 0 ensures, the thread not getting corrupted. there was a request
		//(timerId != -1) and there are no new requests (Queue.length == 0)
		if(VE_CollectionsManager.Queue.length == 0	&& VE_CollectionsManager.timerId != -1)
		{
			//update the state
			var coll = VE_CollectionsManager.RetrieveByKey(collectionId);
			if(coll != null)
				VE_CollectionsManager.RetrieveByKey(collectionId).Dirty = false;
		}
		else
		{
			//We still have queue
			//Wait until they are complete
		}		
	}
	else
	{
		ShowMessage(L_ErrorServerBusy_Text);		
	}
}

// This the callback method for DeleteCollection which is called
// on eval of the response recieved from the server
VE_CollectionsManager.DeleteAnnotationCallback = function (annotationID,status)
{
	//Update the push pins on the map
	//temporary solutions for now...
	if(annotationID)
	{
		VE_CollectionsManager.ClearPushPins();
		
		if(status == "true")
		{
			for( var i =0; i < this.Annotations.length; i++)
			{
				if(this.Annotations[i].Id == annotationID )
					this.Annotations.splice(i,1);
				else if ( this.Annotations[i].msnId && this.Annotations[i].msnId == annotationID)
					this.Annotations.splice(i,1);	
			}
		}
		
		if(this.Annotations == null || this.Annotations.length == 0)
		{
			scratchPad.clear();
			buildScratchPadEmptyListContent();
		}
		
		if(VE_CollectionsManager.collectionIndex == VE_CollectionsManager.unsavedCollectionIndex && VE_CollectionsManager.Collection.AnnotationCount == 0)
			scratchPad.clear();
		
		VE_CollectionsManager.CreatePushPins();	
		
		if(status == "true")
			VE_CollectionsManager.UpdateCollection(VE_CollectionsManager.collectionIndex, VE_CollectionsManager.Annotations,window.locale,VE_CollectionsManagerConstants.TimerDelay);
	}		
}

// This the callback method for RetrieveAllCollections which is called
// on eval of the response recieved from the server
VE_CollectionsManager.RetrieveAllCollectionsCallback = function(collectionList)
{	
	//Clear the timer - this is a complete refresh
	VE_CollectionsManager.ClearTimedRequest();
	
	//Rebuild info bar
	buildScratchPadInfoBar();
	
	if(collectionList == null || collectionList.length == 0 || collectionList == "undefined")
	{		
		return;
	}
		
	//Set the collection list
	if(VE_CollectionsManager.CollectionList != null)
	{
		var len = VE_CollectionsManager.CollectionList.length;
		for(var i=0;i<len;i++)
		{
			var c = VE_CollectionsManager.CollectionList.pop();
			c = null;
		}
		VE_CollectionsManager.CollectionList = null;		
	}
	
	VE_CollectionsManager.CollectionList = collectionList;
	
	var clength = collectionList.length;
	
	for(var pos =0; pos < clength; pos++)
	{
		if(collectionList[pos] == null)
			continue;			
		scratchPad.addTitle(collectionList[pos].ID, collectionList[pos].Name);

	}
	
	if(	VE_CollectionsManager.collectionIndex != '' && 
		VE_CollectionsManager.collectionIndex != null && 
		VE_CollectionsManager.collectionIndex != 'undefined' && 
		VE_CollectionsManager.collectionIndex != VE_CollectionsManager.unsavedCollectionIndex)
	{	
		//no more collection Id to select the new collection.
		var i =0;
		var index = 0;
		var ilen = collectionList.length;
		for(var i=0; i<ilen;i++)
		{
			if(!collectionList[i])
				continue;
				
			if(collectionList[i].ID == VE_CollectionsManager.collectionIndex )
			{
				scratchPad.selectTitle(i+1);
				break;
			}
		}
	}
	else	
	{
		//Update the states and permalinks
		//Get the unsaved collection selected.
		scratchPad.selectTitle(VE_CollectionsManager.unsavedCollectionIndex);		
		state.UpdateFromScratchpad();
		VE_Analytics.Log('VE | ScratchPad','Passport Signin');				
	}	
	return;	
}


// This the callback method for RetrieveAllCollections which is called
// on eval of the response recieved from the server
VE_CollectionsManager.RetrieveAllAnnotationsCallback = function(annotations, collectionId)
{
	buildScratchPadInfoBar();
	
	VE_CollectionsManager.isLoading = true;
	
	//Clear the scratchPad
	if(collectionId != VE_CollectionsManager.unsavedCollectionIndex)
	{
		scratchPad.clear();
	}
	
	//Manage the secondary toolbar
	if(VE_CollectionsManager.collectionIndex == VE_CollectionsManager.unsavedCollectionIndex)
	{
		VE_Scratchpad._HidePins();
		scratchPad.hideFooter();		
	}
	else 
	{
		scratchPad.showFooter();
		if(VE_CollectionsManager.RetrieveByKey(VE_CollectionsManager.collectionIndex) != null)		
			setCollectionVisibility(VE_CollectionsManager.RetrieveByKey(VE_CollectionsManager.collectionIndex).IsPublic);
	}
	
	//clear pushpins related to the current collection from the map
	VE_CollectionsManager.ClearPushPins();
	
	//Update the states and permalinks
	state.UpdateFromScratchpad();
	
	//no null and empty lists and set appropriate screen content
	if(annotations == null || annotations.length == 0)
	{
		buildScratchPadEmptyListContent();
		if(this.Annotations)
		{
			//clean up
			while(this.Annotations.length)
			{
				var item = this.Annotations.pop();
				item = null;
			}
		}
		VE_CollectionsManager.isLoading = false;
		return; //swallow for now..
	}
	
	//If it is unsaved collectionIndex, copy else the unsavedCollection gets corrupted				
	if(VE_CollectionsManager.collectionIndex == VE_CollectionsManager.unsavedCollectionIndex)
	{
		if(this.Annotations)
		{
			//clean up
			while(this.Annotations.length)
			{
				var item = this.Annotations.pop();
				item = null;
			}
		}
		else 
			this.Annotations = new Array();
		
		//clean up's done.. so populate now...
		this.Annotations = this.Annotations.concat(annotations);
	}	
	else
		this.Annotations = annotations;
			
	var type = null;
	
	if(collectionId != VE_CollectionsManager.unsavedCollectionIndex)
	{
		for(var pos =0; pos < annotations.length; pos++)
		{ 
			if(this.Annotations[pos] instanceof VE_Annotation)
			{
				if(parseInt(this.Annotations[pos].BusinessListingId) == 0 || !this.Annotations[pos].BusinessListingId || this.Annotations[pos].BusinessListingId == 'undefined')
					type = 0;
				else 
					type = 1;
				
				var item = scratchPad.createItem(annotations[pos].Id,type,annotations[pos].Latitude,annotations[pos].Longitude);
				item.setTitle(annotations[pos].Title);
				item.setDescription(annotations[pos].Notes.split("<CR>").join("\n"));
				item.setBusinessListingId(annotations[pos].BusinessListingId);
				item.setInfoUrl(annotations[pos].Url);
				item.setPhotoUrl(annotations[pos].PhotoUrl);
				item.setDisplayOrder(annotations[pos].DisplayOrder);
				scratchPad.addItem(item);
			}
			else
			{
				this.Annotations.splice(pos,1);
			}					
		}
	}
	
	VE_CollectionsManager.CreatePushPins();	
	
	VE_CollectionsManager.isLoading = false;
}


// This is the callback method for RetrieveCollectionDetails which is called
// on eval of the response recieved from the server.
VE_CollectionsManager.RetrieveCollectionDetailsCallback = function (collection, status)
{	
	if(status != "true")
	{
		if(VE_CollectionsManager.Viewer != null && VE_CollectionsManager.Viewer.panel != null)
		{
			var panelcontent = '<div id="VE_Traffic_Legend_Header"><div class="infoBar infoBar-warning"><P>' + L_CollectionViewerNonExistentCollectionError_Text + '</P></div></div>';	
			VE_CollectionsManager.Viewer.panel.SetBody(panelcontent);
		}
		else
		{
			ShowMessage(L_ErrorServerBusy_Text);			
		}
		return;
	}

	if(collection instanceof VE_Collection)
	{
		if(VE_CollectionsManager.Viewer instanceof VE_CollectionsViewer)
		{
			VE_CollectionsManager.Viewer.collection = collection;
			
			collection.Name = collection.Name;
			
			VE_CollectionsManager.FillViewer(collection);			
		}
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///																																		///
///										METHODS BELOW THIS MARKER ARE NOT PART OF THE CRUD												///
///																																		///
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//To clear the pushpins associated with an annotation from the map
VE_CollectionsManager.CreatePushPins = function()
{	
	if(this.Annotations == null || this.Annotations.length < 1)
		return;
		
	var color = null;
	for(var pos =0; pos < this.Annotations.length; pos++)
	{
		try
		{
			//set pin dimensions here
			var pinw = 23; // different dimensions for annotations
			var pinh = 17;
			
			//Set the pin Type here...
			var pinType = Msn.VE.PushPinTypes.Annotation;
			
			//Set the pushpin color here.
			color = "VE_Pushpin VE_Pushpin_";
			if(parseInt(this.Annotations[pos].BusinessListingId) == 0 || !this.Annotations[pos].BusinessListingId)
			{
				color += "aN";
				pinw = 22;
			}	
			else 
			{
				color += "yp";
				pinType = Msn.VE.PushPinTypes.SearchResult;
			}
			
				
			var pin = 
			map.AddPushpin(					this.Annotations[pos].Id,			//Id for the pushpin
											this.Annotations[pos].Latitude,		//Latitude of the pushpin
											this.Annotations[pos].Longitude,	//Longitude of the pushpin
											pinw,									//width
											pinh,									//height
											color,								//Pushpin style
											pos + 1 + '',						// innerHTML
											this.pinZIndex,						//Z Index of push pin
											pinType		// pin type
											);					
			
			this.Annotations[pos].PushPin = pin;
			
			this.Annotations[pos].PushPin.onmouseover = function()
			{
				var item = scratchPad.getItems().retrieveByKey(this.id);
				var content = VE_CollectionsManager.CreateEroContent(item);
				ero.setContent(content);
				ero.dockToElement(this);
				
				//clean up
				item = null;
			}
			
			this.Annotations[pos].PushPin.onmouseout = function()
			{
				ero.hide();
			}
			
			//clean up 
			pin = null;

		}
		catch(everything)
		{
			//swallow...
		}
	}	
}


//To clear the pushpins associated with an annotation from the map
VE_CollectionsManager.ClearPushPins = function()
{
	if(this.Annotations == null || this.Annotations.length < 1)
		return;

	
	for(var pos =0; pos < this.Annotations.length; pos++)
	{
		try
		{
			if(this.Annotations[pos].PushPin)
			{
				map.RemovePushpin(this.Annotations[pos].PushPin.id);
				this.Annotations[pos].PushPin = null;
			}
			else
			{
				map.RemovePushpin(this.Annotations[pos].Id);
				this.Annotations[pos].PushPin = null;
			}						
		}
		catch(everything)
		{
			//swallow...
		}
	}	
} 


//Creation of the ERO popup content
// Creates the html content inside the popup that is shown when hovering over map icons or clicking on search result or scratchpad links
VE_CollectionsManager.CreateEroContent=function(spItem)
{
	if(spItem == null)
	{
		//Empty content!
		return "<p/>";
	}
		
	var id = spItem.getGuid();
	var poiType = spItem.getType();
	var poiName = IOSec.EncodeHtml(spItem.getTitle());
	
	var poiAddress = IOSec.EncodeHtml(spItem.getDescription());
	
	if(poiAddress != null && poiAddress != "undefined")
		poiAddress = poiAddress.replace(/^\s+/g, "").replace(/\s+$/g, ""); // trim it.
	else 
		poiAddress = "";
		 
	
	var ypId = spItem.getBusinessListingId();
	var escapedID =  VE_CollectionsManager.SerializeScratchPadItem ( id, null );//'yp.' + escape(ypId);
	var body = IOSec.EncodeUrl(poiName + "\n" + poiAddress + "\n" + GetUrlPrefix() + "?v=" + state.GetCurrentUrlApiVersion() + "&sp=" + escapedID + "\n\n");
	
	
	//Set the search manager LatLong
	VE_SearchManager.SetLatLong(spItem.getLatitude(), spItem.getLongitude());

	var content =
	'<p><div class = "title">%IMG%' + poiName + '</div>' +
	poiAddress + '<br /></p>'; 
	ero.clearActions();	

	//Fix: Bug 3909
	//For Edit Link, remove the context pin if it's present. This is necessary in case the user
	//has the Annotations panel already open as a result of trying to add a pushpin
	//and then clicking on Edit here which will remove the panel and open a new one.
	var editLink = 'javascript:VE_Analytics.Log(\'VE | ERO\',\'Edit Pushpin\');VE_ContextMenu.RemoveContextPin(); VE_Annotations.OpenPanelEX(\'' + id + '\',\'' + VE_Annotations.PanelColor + '\',\'' + VE_Annotations.EditTitle + '\')';
	var delLink = 'javascript:VE_Analytics.Log(\'VE | ERO\',\'Delete Pushpin\');scratchPad.removeItem(scratchPad.getItems().retrieveByKey(\'' + id  + '\'))';
	var moveLink = 'javascript:VE_Analytics.Log(\'VE | ERO\',\'Move Pushpin\');VE_CollectionsManager.MovePin(\'' + id + '\')';

	
	var editMoveDelLink = '<a href = "' + editLink + '">' + L_AnnotationPopupEdit_Text + '</a>';
	
	if(poiType == 0)
		editMoveDelLink += ', ' + '<a href = "' + moveLink + '">' + L_AnnotationPopupMove_Text + '</a>' +	', ';
	else 
		editMoveDelLink += ' ';
	
	editMoveDelLink += 	L_Or_Text + ' ' +	'<a href = "' + delLink + '">' + L_AnnotationPopupDelete_Text + '</a>';
	  
	ero.addAction(editMoveDelLink);


	// Zoom to [ city | street ] level
	ero.addAction(VE_SearchManager.PopupZoomContent());
	
	// Drive from / to Action
	if ( Msn.VE.Marketization.IsEnabled( Msn.VE.Features.MapAction.DrivingDirections ) )
	{
		if(poiType == 1)
		{		
			var driveAction =
			'<a href = "' + VE_SearchManager._GenerateDriveFrom(poiAddress.replace(/&#39;/g,'\\\'')) + '; VE_Analytics.Log(\'VE | Push Pin Popup\', \'Get Directions\');" class = "VE_Panel_sublink">' + L_SearchPopupDriveFrom_Text + '</a>' +
			' ' + L_Or_Text + ' ' +
			'<a href = "' + VE_SearchManager._GenerateDriveTo(poiAddress.replace(/&#39;/g,'\\\'')) + '; VE_Analytics.Log(\'VE | Push Pin Popup\', \'Get Directions\');" class = "VE_Panel_sublink">' + L_SearchPopupDriveTo_Text + '</a>';
			ero.addAction(driveAction);	
		}
		else
		{
			var driveAction =
			'<a href = "javascript:VE_SearchManager.SetDirectionsStart(\'' + (poiName == ''? 'Origin' : poiName.replace(/&#39;/g,'\\\'')) + '\'); VE_Analytics.Log(\'VE | User PushPin Pane\', \'Get Directions\');" class = "VE_Panel_sublink">' + L_SearchPopupDriveFrom_Text + '</a>' +
			' ' + L_Or_Text + ' ' +
			'<a href = "javascript:VE_SearchManager.SetDirectionsEnd(\'' + (poiName == ''? 'Origin' : poiName.replace(/&#39;/g,'\\\'')) + '\'); VE_Analytics.Log(\'VE | Push Pin Popup\', \'Get Directions\');" class = "VE_Panel_sublink">' + L_SearchPopupDriveTo_Text + '</a>';
			ero.addAction(driveAction);	
		}
	}
	
	if (poiType == 0 && Msn.VE.Marketization.IsEnabled( Msn.VE.Features.MenuOption.SendEmailOnPushpinItem ) )    
	{	
	
		
		var link = ' ';
		//Email for annotation.
		// use IOSec ()
		body = IOSec.EncodeUrl(spItem.getTitle() + "\n" + spItem.getDescription() + "\n"  + GetUrlPrefix() + "?v=" + state.GetCurrentUrlApiVersion() + "&sp=" + IOSec.EncodeUrl ( VE_CollectionsManager.SerializeScratchPadItem ( id, null ) ) + "\n\n");
		link += 'mailto:?subject=' + IOSec.EncodeUrl(L_AtWindowsLiveLocal_Text.replace(/%1/g, spItem.getTitle()).replace(/%2/g,Msn.VE.Features.ProductName)) + '&body=' + body;
		
		ero.addAction('<a href = "' + link + '" class = "VE_Panel_sublink" onclick = "VE_Analytics.Log(\'VE | User PushPin Pane\',\'Email\')">' + L_AnnotationPopupEmailAFriend_Text + '</a>');	
	}	  
	if (escapedID && Msn.VE.Marketization.IsEnabled( Msn.VE.Features.MenuOption.SendEmailOnSearchItem ) && poiType == 1)	
		ero.addAction('<a href = "mailto:?subject=' + escape(L_AtWindowsLiveLocal_Text.replace('%1', poiName).replace(/%2/g,Msn.VE.Features.ProductName)) + '&body=' + body + '" onclick = "VE_Analytics.Log(\'VE | Push Pin Popup\', \'Email\');" class = "VE_Panel_sublink" oncontextmenu = "return false;">' + L_SearchPopupEmailAFriend_Text + '</a>');
	
	//var ypId = spItem.getBusinessListingId();
	var windowW = 775;
	
	//Add the details page link....
	if(ypId && parseInt(ypId) != 0)
	{
		ero.addAction('<a href = "javascript:OpenNewWindow(\'' + GetUrlPrefix() + 'details.aspx?dt=yp.' + spItem.getBusinessListingId() + '&style=' + map.GetMapStyle() + '\', \'Detail' + spItem.getBusinessListingId() + '\', \'all\', ' + windowW + ', 500, 200, 200); VE_Analytics.Log(\'VE | Push Pin Popup\', \'Get Details\');" class = "VE_Panel_sublink" oncontextmenu = "return false;">' + L_SearchPopupDetails_Text + '</a>');
	}
	
	//More info url
	if(spItem.getInfoUrl() && spItem.getInfoUrl() != 'undefined' && spItem.getInfoUrl().length > 0)
	{

		var url = spItem.getInfoUrl();
		
		try
		{
			url = VE_CollectionsManager.GetValidatedUrl(url);
		}
		catch(exp)
		{
			url = "";
		}
				
		if(url != null || url != "undefined")
			url = IOSec.EncodeUrl(url);			

		ero.addAction('<a href = "javascript:OpenNewWindow(\'' + url + '\', \'Information'  + '\', \'all\', ' + windowW + ', 500, 200, 200); VE_Analytics.Log(\'VE | Push Pin Popup\', \'More Information\');" class = "VE_Panel_sublink" oncontextmenu = "return false;">' + L_AnnotationPopupMoreInfo_Text + '</a>');
	}
	
	if(spItem.getPhotoUrl() && spItem.getPhotoUrl() != 'undefined' && spItem.getPhotoUrl().length > 1)
	{

		var purl = spItem.getPhotoUrl();						
		try
		{
			purl = VE_CollectionsManager.GetValidatedUrl(spItem.getPhotoUrl());
		}
		catch(errorx)
		{
			purl = "";
		}
	
		var purl1 = IOSec.EncodeUrl(purl);															
		//start html content 							
		var htmlContent = "<a href=\"javascript:OpenNewWindow('%P1%', '_blank' , 'menubar=0,resizable=1,scrollbars=0,status=0,titlebar=0,toolbar=0', 800, 600, 200, 200);\">";
		htmlContent = htmlContent.replace(/%P1%/g,purl1);
		//Create image content and encode it
		var imgContent = "<IMG border=0 id=eroImg_" + id + " style=\"position: relative; display: block; padding-right: 10px; padding-left: 0px; padding-bottom: 5px; padding-top: 3px; width: 80px; height: 80px; float: left;\" src=\"%P2%\" />";
		imgContent = imgContent.replace(/%P2%/g,IOSec.EncodeHtml(purl));
		//Append the img content to the html content
		htmlContent = htmlContent + imgContent + "</a>";		
		content = content.replace(/%IMG%/g,htmlContent);			

	}
	
	content = content.replace(/%IMG%/g,' ');
		
	ypId = null;
	
	return content;
}

//Change the collection from public to private and vice versa...
VE_CollectionsManager.UpdateCollectionVisibility = function()
{
	var item = VE_CollectionsManager.RetrieveByKey(VE_CollectionsManager.collectionIndex);
	
	if(item != null)
	{	
		if(item.IsPublic)
			item.IsPublic = false;
		else
			item.IsPublic = true;
			
		if(!item.Dirty)
			item.Dirty = true;
		
		item = null;
		
		cancelVisibilityChange();
		
		//push updates for later time.
		VE_CollectionsManager.UpdateCollection(VE_CollectionsManager.collectionIndex, VE_CollectionsManager.Annotations,window.locale,VE_CollectionsManagerConstants.TimerDelay);
	}
}

VE_CollectionsManager.MovePin = function(id)
{
	var ann = null;
		
	if(VE_CollectionsManager.collectionIndex)
	{
		ann = VE_CollectionsManager.RetrieveAnnotationByKey (id, VE_CollectionsManager.Annotations);
	}
	else if(VE_CollectionsManager.Collection instanceof VE_Collection)	
	{
		ann = VE_CollectionsManager.RetrieveAnnotationByKey (id, VE_CollectionsManager.Collection.Annotations);
	}
	
	if(ann instanceof VE_Annotation)
	{
		if(ann.PushPin)
		{
			VE_Annotations.EnterMovePushpinMode(ann.PushPin);
		}
		else
		{
			VE_Annotations._FollowAnnotationPopupLink('move',id);
		}
	}

}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///																																		///
///														HELPER FUNCTIONS																///
///																																		///
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

VE_CollectionsManager.RetrieveByKey = function(collectionId)
{
	if(VE_CollectionsManager.CollectionList == null ||	VE_CollectionsManager.CollectionList.length == 0 ||	collectionId == VE_CollectionsManager.unsavedCollectionIndex || !collectionId)
		return;
		
	if(collectionId)
	{
		var pos = VE_CollectionsManager.CollectionList.length;
		while(--pos >= 0)
		{
			if(VE_CollectionsManager.CollectionList[pos].ID == collectionId)
				return VE_CollectionsManager.CollectionList[pos];
		}
	}		
}

VE_CollectionsManager.RetrieveAnnotationByKey = function(annotationId, annotations)
{
	if(	annotationId && annotationId != '' && annotationId != 'undefined' && annotations  && annotations != '' && annotations != 'undefined' && annotations.length > 0)
	{
		var pos = annotations.length;
		while(--pos >= 0)
		{
			if(annotations[pos] instanceof VE_Annotation && annotations[pos].Id == annotationId)
				return annotations[pos];
		}
	}
}

//The move up and move down function could have been implemented
//as one function, but that will obscure the clarity. 

VE_CollectionsManager.MoveAnnotationUp = function(index, collectionId)
{
	var item = null;
	
	if(index > 0 && collectionId != null)
	{
		if(collectionId == VE_CollectionsManager.unsavedCollectionIndex)
		{
			if(VE_CollectionsManager.Collection && VE_CollectionsManager.Collection.Annotations[index] && VE_CollectionsManager.Collection.Annotations[index - 1])
			{
				item = VE_CollectionsManager.Collection.Annotations[index];
				
				if(VE_CollectionsManager.Collection.Annotations[index - 1])
				{
					//delta = 0 on all annotations in unsaved annotations - else double negative = positive
					VE_CollectionsManager.Collection.Annotations[index].MoveUp();
					VE_CollectionsManager.Collection.Annotations[index - 1].MoveDown();
					
					//shift the annotations physically. because unsaved collections 
					//are not sorted when retrieved and swapping will do as the length is only 10.
					VE_CollectionsManager.Collection.Annotations[index] = VE_CollectionsManager.Collection.Annotations[index - 1];
					VE_CollectionsManager.Collection.Annotations[index - 1]  = item;
					
					//Order of annotations have changed. set the dirty flag to true.
					VE_CollectionsManager.Collection.Dirty = true;
				}
			}
		}
		else if(collectionId == VE_CollectionsManager.collectionIndex)
		{
			if(VE_CollectionsManager.Annotations && VE_CollectionsManager.Annotations[index] && VE_CollectionsManager.Annotations[index - 1])
			{
				item = VE_CollectionsManager.Annotations[index];
					
				//shift the annotations physically.
				VE_CollectionsManager.Annotations[index] = VE_CollectionsManager.Annotations[index - 1];
				VE_CollectionsManager.Annotations[index - 1]  = item;
				
				//update the display order
				VE_CollectionsManager.Annotations[index].DisplayOrder = index;
				VE_CollectionsManager.Annotations[index - 1].DisplayOrder = index - 1;												

				//update the state of the annotation								
				VE_CollectionsManager.Annotations[index].State = VE_AnnotationState.Updated;
				VE_CollectionsManager.Annotations[index - 1].State = VE_AnnotationState.Updated;
				
				//set dirty to true for the the delayed updater to pick up changes
				if(VE_CollectionsManager.RetrieveByKey(collectionId) != null)
					VE_CollectionsManager.RetrieveByKey(collectionId).Dirty = true;
				
				//push updates for later time.
				VE_CollectionsManager.UpdateCollection(VE_CollectionsManager.collectionIndex, VE_CollectionsManager.Annotations,window.locale,VE_CollectionsManagerConstants.TimerDelay);			
			}
		}		
	}
	
	item = null;
}

VE_CollectionsManager.MoveAnnotationDown = function(index, collectionId)
{
	var item = null;
	
	if(index >= 0 && collectionId != null)
	{
		if(collectionId == VE_CollectionsManager.unsavedCollectionIndex && VE_CollectionsManager.Collection && VE_CollectionsManager.Collection.AnnotationCount() >= index + 2)
		{
			if(VE_CollectionsManager.Collection.Annotations[index] && VE_CollectionsManager.Collection.Annotations[index + 1])
			{ 
				item = VE_CollectionsManager.Collection.Annotations[index];
				
				if(VE_CollectionsManager.Collection.Annotations[index + 1])
				{
					//delta = 0 on all annotations in unsaved annotations - else double negative = positive
					VE_CollectionsManager.Collection.Annotations[index].MoveDown();
					VE_CollectionsManager.Collection.Annotations[index + 1].MoveUp();
					
					//shift the annotations physically. because unsaved collections 
					//are not sorted when retrieved and swapping will do as the length is only 10.
					VE_CollectionsManager.Collection.Annotations[index] = VE_CollectionsManager.Collection.Annotations[index + 1];
					VE_CollectionsManager.Collection.Annotations[index + 1]  = item;
					
					//Order of annotations have changed. set the dirty flag to true.
					VE_CollectionsManager.Collection.Dirty = true;
				}
			}
		}
		else if(collectionId == VE_CollectionsManager.collectionIndex && VE_CollectionsManager.Annotations && VE_CollectionsManager.Annotations.length >= index + 2)
		{
			if(VE_CollectionsManager.Annotations[index] && VE_CollectionsManager.Annotations[index + 1])
			{
				item = VE_CollectionsManager.Annotations[index];
					
				//shift the annotations physically.
				VE_CollectionsManager.Annotations[index] = VE_CollectionsManager.Annotations[index + 1];
				VE_CollectionsManager.Annotations[index + 1]  = item;
				
				//update the display order
				VE_CollectionsManager.Annotations[index].DisplayOrder = index;
				VE_CollectionsManager.Annotations[index + 1].DisplayOrder = index + 1;
				
				//update the state of the annotation								
				VE_CollectionsManager.Annotations[index].State = VE_AnnotationState.Updated;
				VE_CollectionsManager.Annotations[index + 1].State = VE_AnnotationState.Updated;
				
				//TODO: Clean this up for perf
				//set dirty to true for the the delayed updater to pick up changes
				if(VE_CollectionsManager.RetrieveByKey(collectionId) != null)
					VE_CollectionsManager.RetrieveByKey(collectionId).Dirty = true;
				
				//push updates for later time.
				VE_CollectionsManager.UpdateCollection(VE_CollectionsManager.collectionIndex, VE_CollectionsManager.Annotations,window.locale,VE_CollectionsManagerConstants.TimerDelay);
			}
		}		
	}
	
	item = null;	
}


//This method clears all pending update requests
VE_CollectionsManager.ClearTimedRequest = function()
{
	if(VE_CollectionsManager.timerId != -1)
	{
		try
		{
			window.clearTimeout(VE_CollectionsManager.timerId);
		}
		catch(ex)
		{
			// swallow, the thread may have completed
		}
		VE_CollectionsManager.timerId = -1;
	}
}

//This method serializes a scratchpadItem to link
VE_CollectionsManager.SerializeScratchPadItem = function(spItemId, Annotation)
{	
	if(spItemId && spItemId != '' && spItemId != 'undefined')
	{
		var ad = null;
		
		if(Annotation == null || Annotation == 'undefined')
		{
			ad = scratchPad.getItems().retrieveByKey( spItemId );
		}
		else if(Annotation instanceof VE_Annotation)
		{
			ad = scratchPad.createItem(Annotation.Id,0,Annotation.Latitude,Annotation.Longitude);
			ad.setTitle(Annotation.Title);
			ad.setBusinessListingId(Annotation.BusinessListingId);
			ad.setDescription(Annotation.Notes.split("<CR>").join("\n"));
			ad.setInfoUrl(Annotation.Url);
			ad.setPhotoUrl(Annotation.PhotoUrl);
		}
					
		
		if(ad && ad != '' && ad != 'undefined')
		{
			var type = ad.getBusinessListingId();
			
			if(parseInt(type) == 0 || !type || type == 'undefined')
				type = 0;
			else 
				type = 1;
				
			switch(parseInt(type))
			{
				case 0:
					var lat = parseFloat(ad.getLatitude());
					var lon = parseFloat(ad.getLongitude());
					
					var primarySeparator = VE_SerializationConstants.primarySeparator[state.GetCurrentUrlApiVersion()];
					var secondarySeparator = VE_SerializationConstants.secondarySeparator[state.GetCurrentUrlApiVersion()];
					
					var serial = "";
					serial += VE_Annotations.PinType;
					serial += ".";
					
					//Encode the LatLong coordinate into a hash for the serialization
					var encoder = new VELatLongEncoding();
					latLongEncoding = encoder.Encode(lat.toFixed(6), lon.toFixed(6));
					serial += latLongEncoding;

					// escape the delimiter and replace any any primary separators with spaces.
					// It is unfortunate that we are stripping out primary separators, but it must be done because browsers
					// will convert primary separators into url encoded version and if we don't decode them, 
					// deserializing more important things (such center point) will not work.
					var removePrimarySeparatorRegExp = new RegExp("(" + primarySeparator + ")|(" + IOSec.EncodeUrl(primarySeparator) + ")", "g");
					serial += secondarySeparator;
					var trunc = "";
					var title = ad.getTitle();
				
					if(title != '' && title != null && title != "undefined" && title.length > 0)
					{
						trunc = IOSec.EncodeUrlDelims ( secondarySeparator, title.replace(removePrimarySeparatorRegExp, " "));
						serial += trunc.substring (0, VE_Annotations.MaxAnnotationTitle);
					}
					serial += secondarySeparator;
					var desc = ad.getDescription();
					if(desc != '' && desc != null && desc != "undefined" && desc.length > 0)
					{
						trunc = IOSec.EncodeUrlDelims ( secondarySeparator, desc.replace(removePrimarySeparatorRegExp, " "));
						serial += trunc.substring (0, VE_Annotations.MaxAnnotationNotes);
					}
					serial += secondarySeparator;
					var iUrl = ad.getInfoUrl();
					if(iUrl != '' && iUrl != null && iUrl != "undefined" && iUrl.length > 0)
					{
						trunc = IOSec.EncodeUrlDelims ( secondarySeparator, iUrl.replace(removePrimarySeparatorRegExp, " "));
						serial += trunc.substring (0, VE_Annotations.MaxAnnotationPhotoUrl);
					}
					serial += secondarySeparator;
					var pUrl = ad.getPhotoUrl();
					if(pUrl != '' && pUrl != null && pUrl != "undefined" && pUrl.length > 0)
					{

						trunc = IOSec.EncodeUrlDelims ( secondarySeparator, pUrl.replace(removePrimarySeparatorRegExp, " ") );

						serial += trunc.substring (0, VE_Annotations.MaxAnnotationMoreInfoUrl);
					}

					return IOSec.EncodeUrl(serial);
					break;
				
				case 1:
					var escapedId = '';
					var ypId = ad.getBusinessListingId();
					if(ypId != null || ypId != '' || ypId != 'undefined')
						escapedId = escape('yp.' + escape(ypId));
					ypId = null;
					return escapedId;
					break;
				default:
					break;
			}
		}
	}
}

VE_CollectionsManager.GetErrorMessage = function(status)
{
	var errorMessage = null;
	
	if(status != null && status != '' & status != "undefined" && status.length > 0)
	{
		switch(status)
		{
			case "missing":
				errorMessage = L_CollectionViewerNonExistentCollectionError_Text;
				break;
			case "invalid":
				errorMessage = L_CollectionViewerNonExistentCollectionError_Text;
				break;
			case "true":
				break;
			default:
				errorMessage = L_ErrorServerBusy_Text;
				break;
		}

		if(errorMessage && errorMessage.length > 0)
			return errorMessage;
	}
	
	return errorMessage;
}


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///																																		///
///												COLLECTION VIEWER RELATED METHODS														///
///																																		///
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//FillViewer fills the viewer with annotations returned by the 
//storage
VE_CollectionsManager.FillViewer = function(collection)
{
	if(collection instanceof VE_Collection && VE_CollectionsManager.Viewer instanceof VE_CollectionsViewer && collection != null)
	{				
		
		//VE_CollectionsManager.Viewer = new VE_CollectionsViewer(modeName, modeId);
		VE_CollectionsManager.Viewer.title = collection.Name;
		VE_CollectionsManager.Viewer.panel.SetTitle(collection.Name);
		VE_CollectionsManager.Viewer.AddItems(collection.Annotations,collection.MsnId);
		//create toolbar
		VE_CollectionsManager.Viewer.CreateToolbar();
		VE_CollectionsManager.Viewer.SetBestMapView();
		VE_CollectionsManager.Viewer.panel.Maximize();
	}
	else
	{
		
		VE_CollectionsManager.KillViewer(false);
	}
}

VE_CollectionsManager.KillViewer = function(onClose)
{
	if(VE_CollectionsManager.Viewer instanceof VE_CollectionsViewer)
	{
		VE_CollectionsManager.Viewer.Destroy(onClose);
		VE_CollectionsManager.Viewer = null;
		
		//Update the states and permalinks
		state.UpdateFromScratchpad();
	}
}


/*shankak:
Global function : saveCollection
Purpose			: To save a collection */		
VE_CollectionsManager.SaveCollection = function ()
{			
	if(VE_CollectionsManager.Collection == null)
	{
		VE_CollectionsManager.Collection = new  VE_Collection(VE_CollectionsManager.unsavedCollectionTitle,						  //Name
												VE_CollectionsManager.unsavedCollectionIndex,					  //Id
												'false',														  //Non Public by default
												null,															  //No keywords
												null);															  //No Annotations				
	}	
		
	if(VE_CollectionsManager.Collection)
	{	 	
		
		var cTitle = document.getElementById('collectionFormTitle');
		var cNotes = document.getElementById('collectionFormNotes');
		var cChecked = document.getElementById('collectionFormAccess');
		
		if(cTitle && cNotes && cChecked)
		{
			var title  = cTitle.value;
			var keywords = cNotes.value;
			var isPublic = cChecked.checked;		

			title = unescape(escape(title).replace(/^(%20)*/g, "").replace(/(%20)*$/g, "").replace(/^(%09)*/g, "").replace(/(%09)*$/g, ""));	
			keywords = unescape(escape(keywords).replace(/^(%20)*/g, "").replace(/(%20)*$/g, "").replace(/^(%09)*/g, "").replace(/(%09)*$/g, ""));	

			if(isPublic)
				VE_CollectionsManager.Collection.IsPublic = true;
				
			if(keywords.length > 0)
			{
				VE_CollectionsManager.Collection.Keywords = keywords.split(",");
			}	

			if(title.length > 0)
			{	
				VE_CollectionsManager.CreateCollection (title, // This will be a part of the query string,
										VE_CollectionsManager.Collection.IsPublic,
										VE_CollectionsManager.Collection.Keywords,
										VE_CollectionsManager.Collection.AnnotationCount(),
										VE_CollectionsManager.Collection.Annotations,
										window.locale
										);
			}
			else
			{
				if(document.getElementById('collectionFormTitle'))
				{
					SelectText(cTitle,0,0);
				}	
			}
		}
	}
}



		
/*shankak
Global function : DeleteCollection
Purpose			: To delete a specified collection. */
//Always deletes the current collection		
VE_CollectionsManager.DeleteCollection = function()
{	
	
	if( VE_CollectionsManager.collectionIndex != null && 
		VE_CollectionsManager.collectionIndex != "undefined" &&
		VE_CollectionsManager.collectionIndex.length > 0 &&
		VE_CollectionsManager.collectionIndex != VE_CollectionsManager.unsavedCollectionIndex)
	{
		//Clear the timer - sorry the collection will itself be gone..
		VE_CollectionsManager.ClearTimedRequest();
	
		var query = VE_CollectionsManagerConstants.Handler + VE_CollectionActions.Delete + "&";
		query += (VE_CollectionsManagerConstants.C_Id + "=" + VE_CollectionsManager.collectionIndex + "&");
		query += VE_CollectionsManagerConstants.Market + window.locale;
		query += ("&" + VE_CollectionsManagerConstants.Random + Math.random());
		
		VE_CollectionsManager.SendRequest(query, true);
	}
	else
	{
		ShowMessage(L_CollectionManagerUnsavedCollectionCannotBeDeleted_Text);	
	}		
	
	return;					
}		
		
VE_CollectionsManager.GetValidatedUrl = function(url)
{	
	
	if(url == null || url == "undefined" || url.length <= 0)
		return "";	
	
	try
	{
		url = unescape(url);
		//Trim spaces
		url = url.replace(/\|/g, ' ').replace(/\^/g, ' ').replace(/^\s+/g, "").replace(/\s+$/g, "");			
		
		var regexp = new RegExp(/(ftp|http|https):\/\/(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?/);
		
		if(url.match(regexp))
		{
			return url;		
		}
		else
		{
			return "";
		}
	}
	catch(errorex)
	{
		//swallow
		return "";
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//																																	  //
//													CollectionsManager Related Events												  //
//																																	  //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


VE_CollectionsManager.CleanUp = function()
{
	//This is the function which is called on page unload for the VE_CollectionsManager.
	//Flush any updates....
	VE_CollectionsManager.UpdateCollection(VE_CollectionsManager.collectionIndex,VE_CollectionsManager.Annotations,window.locale);
	
	//Clean up the unsaved collection
	if(VE_CollectionsManager.Collection instanceof VE_Collection)
	{
		VE_CollectionsManager.Collection.ClearAnnotations();
	}
	
	VE_CollectionsManager.Collection = null;
	
	//Clean up the current set of annotations
	if(	VE_CollectionsManager.Annotations && 
		VE_CollectionsManager.Annotations != 'undefined' &&
		VE_CollectionsManager.Annotations != '' &&
		VE_CollectionsManager.Annotations.length > 0)
	{
		while(VE_CollectionsManager.Annotations.length)
		{	
			var item = VE_CollectionsManager.Annotations.pop();
			item = null;
		}
	}
	
	VE_CollectionsManager.Annotations = null;	
}

