//------------------------------------------------------------------------------
// UserCollections/CollectionsViewer.js 
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: shankak, keithkin, kamonmo
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

// Creates a new search based on a search title
VE_CollectionsViewer = function(Title, Id)
{
	// Account for a bug in IE whereby the 'this' keyword is not properly maintained
	var self = this;
	this.title = null;
	if(Title)
		this.title = Title;
	else this.title = "^";
	this.startIndex = 0;
	this.displayIndex = 0;
	this.pinZIndex = VE_MapPushpin.baseZIndex;
	this.color = 'viewer';
	this.results = null;
	this.hasMore = false;
	this.Annotations = null;
	this.c_id = Id;
	this.collection = "";
	this.message = "";
	this.zIndex = 31;
	var collectionIndex = Id;
	this.panel = new VE_Panel(							'view_' + collectionIndex,     // id
														8,                          // x
														0,	                        // y
														'auto',                     // width
														'auto',                     // height
														'viewer',            // type
														this.zIndex,						// z index
														this.title,											// title
														'<p class = "loading">' + L_SearchLoading_Text + '</p>' ,      // body
														"",													// footer
														true,												// show mouseover for title bar 			
														true,												// bOpaque
														taskArea_data);											// parent
		
			
	this.panel.title.className +=  ' VE_Panel_title_' + this.color;
			
	this.panel.foot.style.overflow = "auto";

	taskArea.addItem(this.panel);		
	
		
	// Make IE obey the pseudo class :hover for any search's TitleBar
	pseudoHover(this.panel.title);

	//now show the panel
	this.panel.Show();
	this.panel.Maximize();
	taskArea.setCurrent(this.panel);
	// sstchur:  Add this panel to the TaskArea; bring it to the front
	toggleTaskArea(true);			
		
	this.panel.SetFooterSize(this.FooterSize);

	this.panel.onTitleClick = function(e)
	{
		var p = e.srcPanel;
		taskArea.setCurrent(p);
	};
		
	this.panel.onCloseClick = function(e)
	{
		VE_CollectionsManager.KillViewer(true);	
	};
	
	this.panel.onMaximize = function(e)
	{
		var p = e.srcPanel;
		if (typeof p.title != 'undefined')
			setActivePanel(p.title.className);
	};
	

	//footer size.
	this.FooterSize = 120;
	
	//Destroy the viewer
	this.Destroy = function(onClose)
	{
		this.ClearPushPins();
		if(this.panel != null)
		{
			taskArea.removeItem(this.panel);
			this.panel.Destroy();
		}
	}

	//Add items to the viewer
	this.AddItems = function(annotations, id)
	{
		if(annotations && id)
		{
			this.Annotations = annotations;
			this.c_id = id;
			
			//create the pushpins for the items there is no paging now. if paging
			//is needed, then specify no of pushpins to be created
			
			this.CreatePushPins(null);
			
			var body = "";
			
			body += this.getBodyHeader();
	
			body += "<div id='" + id + "'>";
			
			var searchResults = '<ul class = "list_searchResults">';
			
			//iterate through the annotations and add them to the list.
			for(var i =0; i < annotations.length; i++)
			{
				if(annotations[i])
				{
				
					var pinId = this.c_id + '_' + annotations[i].Id;
					var liMouseOverEvent = 'pinToTop(\'' + pinId + '\')';
					var liMouseOutEvent = 'ero.hide(); revertPin(\'' + pinId + '\');';
					var h3DivMouseOverEvent = 'VE_CollectionsManager.Viewer.ShowEroToItem(event, \'' + annotations[i].Id + '\', this,' + annotations[i].DisplayOrder + ');';					
					var clickEvent = 'map.PanToLatLong(' + annotations[i].Latitude + ', ' + annotations[i].Longitude + ')';
				
					var title = IOSec.EncodeHtml(annotations[i].Title).replace(/&#10;/g,"<BR/>");
					var notes = '';
						
					if(annotations[i].Notes != null)
						notes = IOSec.EncodeHtml(annotations[i].Notes).replace(/&#10;/g,"<BR/>");
					
					var li =
					'<li onmouseover = "##liMouseOverEvent##" onmouseout = "##liMouseOutEvent##" class = "VE_Pushpin_searchResult VE_Pushpin_' + this.color + '">' +
						'<div class = "poiNumber" onmouseover = "##h3DivMouseOverEvent##">' + (this.startIndex + i + 1) + '</div>' +
						'<h3 onmouseover = "##h3DivMouseOverEvent##" onclick = "##clickEvent##">' + title + '</h3>' +
						'<p>' + ((annotations[i].Notes.length > 0) ? notes : " ") + '</p>' +	
					'</li>';
					
					li = li.replace(/##liMouseOverEvent##/g, liMouseOverEvent);
					li = li.replace(/##liMouseOutEvent##/g, liMouseOutEvent);
					li = li.replace(/##h3DivMouseOverEvent##/g, h3DivMouseOverEvent);
					li = li.replace(/##clickEvent##/g, clickEvent);
					
					searchResults += li;
				}
			}
		}
		
		searchResults += '</ul>';
		body += searchResults + "</div>";	
		this.panel.SetBody(body);
		
		pseudoHoverForChildren(this.panel.body, 'h3');
		pseudoHoverForChildren(this.panel.body, 'li');
	}
	
	this.getBodyHeader = function()
	{
		if(this.collection instanceof VE_Collection)
		{	
			var body = "";
			var notes = "";
			var lastModifiedDate = this.collection.DateModified;		
						
			if(this.collection.Keywords && this.collection.Keywords != "undefined" && this.collection.Keywords != '' && this.collection.Keywords.length > 0)
			{
				notes = IOSec.EncodeHtml(this.collection.Keywords[0]).replace(/&#10;/g,"<BR/>");
			}
				 
			body += '<div id=\"view_' + this.c_id + '_bodyHdr\" class=\"ve_viewerBodyHdr\">';
			
			if(notes && notes != '' && notes.length > 0)
				body += '<p>' + notes + '</p>';
			
			if(lastModifiedDate != null && lastModifiedDate != "undefined" && lastModifiedDate != '')
			{
				if(lastModifiedDate && lastModifiedDate != 'undefined' && lastModifiedDate != '' && lastModifiedDate.length > 0)
					body += '<p class=\"ve_viewerBodyHdrUpdateText\">' + L_CollectionViewerLastUpdated_Text + ' : ' + lastModifiedDate + '</p>';
			}
			
			body += '</div>';	
			
			return body;
		}
		
		return "";
	}

	//Creation of the ERO popup content
	// Creates the html content inside the popup that is shown when hovering over map icons or clicking on search result or scratchpad links
	this.CreatePopupContent=function(Annotation)
	{
		if(Annotation == null)
		{
			//throw ("Argument cannot null. The Annotation item argument was null");
			return;
		}
		
		var poiType = 	null;
		
		if(Annotation.BusinessListingId && Annotation.BusinessListingId != 0)
			poiType = 1;
		else 
			poiType = 0;
			
		var poiName = IOSec.EncodeHtml(Annotation.Title).replace(/&#10;/g,"<BR/>");
		var poiAddress = IOSec.EncodeHtml(Annotation.Notes).replace(/&#10;/g,"<BR/>");
		
		if(poiAddress != null && poiAddress != "undefined")
			poiAddress = poiAddress.replace(/^\s+/g, "").replace(/\s+$/g, ""); // trim it.
		else 
			poiAddress = poiName;
		//var poiPhone = Annotation.phone;
		var ypId = Annotation.BusinessListingId;
		var escapedID = 'yp.' + escape(ypId);	
		var body = escape(poiName + "\n" + poiAddress + "\n" + GetUrlPrefix() + "?v=" + state.GetCurrentUrlApiVersion() + "&sp=" + escapedID + "\n\n");
		
		var content =
		'<p><div class = "title">%IMG%' + poiName + '</div>' +
		poiAddress + '<br /></p>'; 
		
		ero.clearActions();	

		//Set the Latitude/Longitude of the Search Manager
		VE_SearchManager.SetLatLong(Annotation.Latitude, Annotation.Longitude);
		
		// Add To Collection Action
		if (Msn.VE.Marketization.IsEnabled( Msn.VE.Features.MapAction.Collections ) )
			ero.addAction('<a href = "javascript:VE_CollectionsManager.Viewer.AddToCollection(\'' + Annotation.Id  + '\',' + Annotation.DisplayOrder + ');" class = "VE_Panel_sublink" oncontextmenu = "return false;">' + L_SearchPopupAddToScratchPad_Text + '</a>');	

		// Drive from / to Action
		if ( Msn.VE.Marketization.IsEnabled( Msn.VE.Features.MapAction.DrivingDirections ) )
		{
			if(poiType == 1)
			{
				var driveAction =
				'<a href = "' + VE_SearchManager._GenerateDriveFrom(poiAddress) + '; VE_Analytics.Log(\'VE | Push Pin Popup\', \'Get Directions\');" class = "VE_Panel_sublink">' + L_SearchPopupDriveFrom_Text + '</a>' +
				' ' + L_Or_Text + ' ' +
				'<a href = "' + VE_SearchManager._GenerateDriveTo(poiAddress) + '; VE_Analytics.Log(\'VE | Push Pin Popup\', \'Get Directions\');" class = "VE_Panel_sublink">' + L_SearchPopupDriveTo_Text + '</a>';
				ero.addAction(driveAction);	
			}
			else if(poiType == 0)
			{
				var driveAction =
				'<a href = "javascript:VE_SearchManager.SetDirectionsStart(\'' + (poiName == ''? 'Origin' : poiName) + '\'); VE_Analytics.Log(\'VE | User PushPin Pane\', \'Get Directions\');" class = "VE_Panel_sublink">' + L_SearchPopupDriveFrom_Text + '</a>' +
				' ' + L_Or_Text + ' ' +
				'<a href = "javascript:VE_SearchManager.SetDirectionsEnd(\'' + (poiName == ''? 'Origin' : poiName) + '\'); VE_Analytics.Log(\'VE | Push Pin Popup\', \'Get Directions\');" class = "VE_Panel_sublink">' + L_SearchPopupDriveTo_Text + '</a>';
				ero.addAction(driveAction);
			}
		}
			
		if (poiType == 0 && Msn.VE.Marketization.IsEnabled( Msn.VE.Features.MenuOption.SendEmailOnPushpinItem ) )    
		{	
			var link = '';
			//Email for annotation.
			// use IOSec ()
			body = IOSec.EncodeUrl(Annotation.Title + "\n" + Annotation.Notes + "\n"  + GetUrlPrefix() + "?v=" + state.GetCurrentUrlApiVersion() + "&sp=" + IOSec.EncodeUrl ( VE_CollectionsManager.SerializeScratchPadItem ( Annotation.Id , Annotation) ) + "\n\n");
			link += 'mailto:?subject=' + IOSec.EncodeUrl( L_AtWindowsLiveLocal_Text.replace(/%1/g, Annotation.Title).replace(/%2/g,Msn.VE.Features.ProductName) ) + '&body=' + body;
			
			ero.addAction('<a href = "' + link + '" class = "VE_Panel_sublink" onclick = "VE_Analytics.Log(\'VE | User PushPin Pane\',\'Email\')">' + L_AnnotationPopupEmailAFriend_Text + '</a>');	
		}	
		else if (escapedID && Msn.VE.Marketization.IsEnabled( Msn.VE.Features.MenuOption.SendEmailOnSearchItem ) && poiType == 1)	
			ero.addAction('<a href = "mailto:?subject=' + escape(L_AtWindowsLiveLocal_Text.replace('%1', poiName).replace(/%2/g,Msn.VE.Features.ProductName)) + '&body=' + body + '" onclick = "VE_Analytics.Log(\'VE | Push Pin Popup\', \'Email\');" class = "VE_Panel_sublink" oncontextmenu = "return false;">' + L_SearchPopupEmailAFriend_Text + '</a>');	
		
		// Zoom to [ city | street ] level
		ero.addAction(VE_SearchManager.PopupZoomContent());

		//Add the details page link....
		if(ypId && parseInt(ypId) != 0)
		{
			var windowW = 775;
		ero.addAction('<a href = "javascript:OpenNewWindow(\'' + GetUrlPrefix() + 'details.aspx?dt=yp.' + Annotation.BusinessListingId + '&style=' + map.GetMapStyle() + '\', \'Detail' + Annotation.getBusinessListingId + '\', \'all\', ' + windowW + ', 500, 200, 200); VE_Analytics.Log(\'VE | Push Pin Popup\', \'Get Details\');" class = "VE_Panel_sublink" oncontextmenu = "return false;">' + L_SearchPopupDetails_Text + '</a>');
		}
		
		//Additional Info URL
		
		if(Annotation.Url && Annotation.Url.length > 0)
		{
			var iurl = ""
			try
			{
				iurl = VE_CollectionsManager.GetValidatedUrl(Annotation.Url);
			}
			catch(errorx)
			{
				iurl = "";
			}
			
			ero.addAction('<a href = "javascript:OpenNewWindow(\'' + IOSec.EncodeUrl(iurl) + '\', \'Information'  + '\', \'all\', ' + windowW + ', 500, 200, 200); VE_Analytics.Log(\'VE | Push Pin Popup\', \'More Information\');" class = "VE_Panel_sublink" oncontextmenu = "return false;">' + L_AnnotationPopupMoreInfo_Text + '</a>');
		}
			
		if(Annotation.PhotoUrl && Annotation.PhotoUrl != "undefined" && Annotation.PhotoUrl.length > 1)
		{		
			var purl = Annotation.PhotoUrl;	
			try
			{
				purl = VE_CollectionsManager.GetValidatedUrl(Annotation.PhotoUrl);
			}
			catch(errorx)
			{
				purl = "";
			}
				
			var purl1 = IOSec.EncodeUrl(purl);															
			//start html content 							
			var htmlContent = "<a href=\"javascript:OpenNewWindow('%P1%', '_blank' , 'menubar=0,resizable=1,scrollbars=0,status=0,titlebar=0,toolbar=0', 800, 600, 200, 200);\">";
			htmlContent = htmlContent.replace(/%P1%/g,purl1);		
			//Create image content and encode it
			var imgContent = "<IMG border=0 id=eroImg_" + Annotation.Id + " style=\"position: relative; display: block; padding-right: 10px; padding-left: 0px; padding-bottom: 5px; padding-top: 3px; width: 80px; height: 80px; float: left;\" src=\"%P2%\" />";
			imgContent = imgContent.replace(/%P2%/g,IOSec.EncodeHtml(purl));
			//Append the img content to the html content
			htmlContent = htmlContent + imgContent + "</a>";		
			content = content.replace(/%IMG%/g,htmlContent);	
		}
	
		content = content.replace(/%IMG%/g,' ');
		
		ypId = null;
		
		return content;
	}

	//Show the popup
	this.ShowEroToItem = function(e, itemId, item, index)
	{
		var annoItem = null;
		var foundItem = false;

		if(index != null && index != '' && index != 'undefined')
		{
			annoItem = this.RetrieveByIndex(index);	//faster : as easy as array access on index
			if(annoItem && annoItem.Id != itemId)
				annoItem = null;
		}

		if(annoItem instanceof VE_Annotation)
			foundItem = true;
			
		if(!foundItem)
		{
			annoItem = this.RetrieveByKey(itemId); //slower : can be made faster.
		}
			
		if(itemId && annoItem instanceof VE_Annotation)
		{
			var content = this.CreatePopupContent(annoItem);
			ero.setContent(content);
			
			if (item.tagName != 'H3')
				ero.dockToElement(item);		
			else
				ero.dockToText(item,e);
								
			content = null;
		}
	}

	//get the annotation matched by id
	this.RetrieveByKey = function(itemId)
	{
		if(itemId)
		{
			var length = this.Annotations.length - 1;
			while( this.Annotations[length].Id != itemId && length >= 0 )
			{
				--length;
			}
			
			if(this.Annotations[length])
				return this.Annotations[length];
		}
	}
	
	this.RetrieveByIndex = function(index)
	{
		if(index != null && index != '' && index != 'undefined' && index >= 0)
		{
			if(	this.Annotations && 
				this.Annotations != '' &&
				this.Annotations != 'undefined' && 
				this.Annotations.length > index)
			{
				var item = this.Annotations[index];
				if(item instanceof VE_Annotation)
				{
					if(	item.DisplayOrder != null && 
						item.DisplayOrder != '' && 
						item.DisplayOrder != 'undefined'
						&& parseInt(item.DisplayOrder) == index)
					{
						return item;
					}
					
					return null;
				}
				
				return null;
			}
			
			return null;			
		}
	}

	//To clear the pushpins associated with an annotation from the map
	this.CreatePushPins = function(pageLength)
	{
		if(this.Annotations == null || this.Annotations.length < 1)
			return;
		
		var pLength = (parseInt(pageLength) > 0) ? pageLength : this.Annotations.length;
		
		for(var pos =0; pos < pLength ; pos++)
		{
			try
			{
				this.Annotations[pos].PushPin = 
				map.AddPushpin(					this.c_id + '_' + this.Annotations[pos].Id ,				//Id for the pushpin
												this.Annotations[pos].Latitude,			//Latitude of the pushpin
												this.Annotations[pos].Longitude,			//Longitude of the pushpin
												25,									//width
												29,									//height
												"VE_Pushpin VE_Pushpin_" + this.color,		//Pushpin style
												pos + 1 + '',           // innerHTML
												this.pinZIndex,			//Z Index of push pin
												Msn.VE.PushPinTypes.Annotation		// pin type											
												);						
			
				this.Annotations[pos].PushPin.onmouseover = function()
				{
					var idPair = this.id.split("_");
					if(idPair.length == 2)
						VE_CollectionsManager.Viewer.ShowEroToItem(event, idPair[1], this);
				}
				
				this.Annotations[pos].PushPin.onmouseout = function()
				{
					ero.hide();
				}
				

			}
			catch(everything)
			{
				//swallow...
			}
		}
		
		pLength = null;	
	}


	//To clear the pushpins associated with an annotation from the map
	this.ClearPushPins = function(pageLength)
	{
		if(this.Annotations == null || this.Annotations.length < 1)
			return;
		
		var pLength = (parseInt(pageLength) > 0) ? pageLength : this.Annotations.length;
		
		for(var pos =0; pos < pLength; pos++)
		{
			try
			{
				if(this.Annotations[pos].PushPin)
				{
					map.RemovePushpin(this.Annotations[pos].PushPin.id);
					this.Annotations[pos].PushPin = null;
				}								
			}
			catch(everything)
			{
				//swallow...
			}
		}
		
		pLength = null;	
	}
	
	this.SetBestMapView = function()
	{
		if(this.Annotations)
		{
			var mapAllArray = new Array();
			
			for( var i=0; i < this.Annotations.length; i++)
			{
				var latlong = new Msn.VE.LatLong( this.Annotations[i].Latitude, this.Annotations[i].Longitude);
				mapAllArray.push(latlong);
				VE_Scratchpad._SwitchBestMapViewForBirdsEye(latlong);
			}
			
			if(mapAllArray.length > 0)
				map.SetBestMapView(mapAllArray);
		}
		
		while(mapAllArray.length)
		{
			var item = mapAllArray.pop();
			item = null;
		}

		mapAllArray = null;
	}
	
	//Email the shared collection
	this.Email=function()
	{	
		var e = this.Annotations;
		if(e == null)
		{
			return;
		}
			
		var body = "";
		var urlprefix = GetUrlPrefix() + "?v=" + state.GetCurrentUrlApiVersion() + "&";
		
		var ids =  VE_CollectionsManagerConstants.C_Id + "=" + this.c_id;
		
		var primarySeparator = VE_SerializationConstants.primarySeparator[state.GetCurrentUrlApiVersion()];
				
		var allids = escape( L_ScratchPadWindowsLiveLocal_Text.replace(/%1/g,Msn.VE.Features.ProductName) + "\n" + urlprefix + ids + "\n\n");	
		var url = 'mailto:?subject=' + escape(L_ScratchPadWindowsLiveLocal_Text.replace(/%1/g,Msn.VE.Features.ProductName)) + '&body=' + allids + body;
		url = IOSec.TruncateUrlSafe ( url, 2020, VE_Scratchpad._emailTruncationMsg );
		document.location.href = url;
		VE_Analytics.Log("VE | Collections Viewer","Forward Favorite link");
		
		//Clean up
		e = null;
	}


	// Blog the shared collection
	this.Blog=function()
	{
		var e = this.Annotations;
		if(e == null)
		{
			return;
		}

		var blog = "http://spaces.msn.com/BlogIt.aspx?Title=" + L_ScratchPadWindowsLiveLocal_Text.replace(/%1/g,Msn.VE.Features.ProductName);	
		var primarySeparator = VE_SerializationConstants.primarySeparator[state.GetCurrentUrlApiVersion()];
		var ids =  VE_CollectionsManagerConstants.C_Id + "=" + this.c_id;
		var urlprefix = GetUrlPrefix() + "?v=" + state.GetCurrentUrlApiVersion() + "&";
		var description = "<a href=\"" + urlprefix + ids + "\">"+ L_ScratchPadWindowsLiveLocal_Text.replace(/%1/g,Msn.VE.Features.ProductName) +"</a><br /><br />";
		
		for(var i = 0; i < e.length; i++)
		{
			description += e[i].name + ( ( e[i].description ) ? "<br>" + e[i].description : "" );
			if (i < e.length - 1)
			{
				description += "<br /><br />";
			}
		}
		
		blog += "&description=" + escape ( description );
		blog = IOSec.TruncateUrlSafe ( blog, 2020, VE_Scratchpad._blogTruncationMsg );
		window.open(blog);
		VE_Analytics.Log("VE | Collections Viewer","Blog");
		
		//Clean up
		e = null;
	}
	this.GetFavoriteUrl=function()
	{
		var url = GetUrlPrefix() + "?v=" + state.GetCurrentUrlApiVersion() + "&" + VE_CollectionsManagerConstants.C_Id + "=" + this.c_id;
		url = IOSec.TruncateUrlSafe ( url, 2020 );	
		return(url);
	}
	this.CreateToolbar=function()
	{
		var toolbarHTML = this.GetToolbarHTML();
		
		if(toolbarHTML) 
		{
			this.panel.SetToolbar(toolbarHTML);
			var toolbarDiv = document.getElementById("col_viewer_toolbar_"+collectionIndex);
			if(toolbarDiv) 
			{
				pseudoHoverForChildren(toolbarDiv);
			}
			if(passportReturned) 
			{
				if(passportSignedIn)
				{
					this.EnableAddFavoriteMenu();
					this.HideCollectionViewerInfoBar();
				} 
				else 
				{
					this.DisableAddFavoriteMenu();
					this.ShowCollectionViewerInfoBar();
				}
			}	
		}	
	}
	this.GetToolbarHTML=function()
	{
		var menuDiv = "<div>";
		var msg = L_CollectionViewerMustSignInToAddFavorite_Text.replace(/%1/g,window.passportLoginButtonHtml);
		
		var messageDiv = "<div id=\"col_viewer_message_"+this.c_id+"\" class=\"infoBar_general\">"+msg+"</div>";
		
		var toolbarDiv = "<div id=\"col_viewer_toolbar_"+this.c_id+"\" class=\"panel_toolbar\">";
		
		var ul = "<ul class=\"wll_dropDownMenu\">";
		
		var shareMenuOption = (Msn.VE.Marketization.IsEnabled('Msn.VE.Features.MenuOption.ShareScratchPadItems.SendEmail') ? 
									'<li onclick = "VE_CollectionsManager.Viewer.Email()">' + L_MainShare_Text + '</li>' : '');
								
		
		//create Add Favorite menu	
		var favoriteUrl = this.GetFavoriteUrl();
		var favoriteTitle = this.title;
		favoriteTitle = favoriteTitle.replace(/'/g,"\\'");
		var li = "<li id=\"col_viewer_toolbar_add_favorite_" + this.c_id + "\" class=\"hiddenMenu\" onclick=\"VE_Favorites.AddFavorites('" + IOSec.EncodeHtml(favoriteTitle) + "','" + IOSec.TruncateUrlSafe ( favoriteUrl, 2020 ) + "');VE_Analytics.Log('VE | Collection Viewer','Add to Favorites link')\">" + L_CollectionViewerAddFavorite_Text + "</li>";
		ul += li;
		li = "<li onclick=\"VE_CollectionsManager.Viewer.SetBestMapView();\">"+L_MainMapAll_Text+"</li>";
		ul += li;
		li = shareMenuOption;
		ul += li;
		li = '<span class=\"viewer_help\" onclick=\"showViewerHelp();\"></span>';
		ul += li;
		
		ul += "</ul>";
		
		toolbarDiv += ul;
		toolbarDiv += '<div class = "clear" ></div></div>';
		
		menuDiv += messageDiv;
		menuDiv += toolbarDiv;
		
		menuDiv += "</div>";
		return(menuDiv);
	}
	this.EnableAddFavoriteMenu=function()
	{
		var addFavoriteLi = document.getElementById("col_viewer_toolbar_add_favorite_"+this.c_id);
		if(addFavoriteLi) {
			addFavoriteLi.style.display = "block";
		}
	}
	this.DisableAddFavoriteMenu=function()
	{
		var addFavoriteLi = document.getElementById("col_viewer_toolbar_add_favorite_"+this.c_id);
		if(addFavoriteLi) {
			addFavoriteLi.style.display = "none";
		}
	}
	this.ShowCollectionViewerInfoBar=function()
	{
		var messageDiv = document.getElementById("col_viewer_message_"+this.c_id);
		if(messageDiv) {
			messageDiv.style.display = "block";
		} 
	}
	this.HideCollectionViewerInfoBar=function()
	{
		var messageDiv = document.getElementById("col_viewer_message_"+this.c_id);
		if(messageDiv) {
			messageDiv.style.display = "none";
		} 
	}
	
	this.AddToCollection = function(annotationId, index)
	{
		if(annotationId && annotationId != '' && annotationId != 'undefined' && annotationId.length > 0)
		{
			var annoItem = null;
			var foundItem = false;
			
			if(index != null && index != '' && index != 'undefined')
			{
				annoItem = this.RetrieveByIndex(index);	//faster : as easy as array access on index
				if(annoItem && annoItem.Id != annotationId)
					annoItem = null;
			}
			
			if(annoItem instanceof VE_Annotation)
				foundItem = true;
				
			if(!foundItem)
			{
				annoItem = this.RetrieveByKey(annotationId); //slower : can be made faster.
			}
			
			if(annoItem instanceof VE_Annotation)
			{
				var ypId = annoItem.BusinessListingId;
						//Add the details page link....
						
				if(ypId && parseInt(ypId) != 0)
				{						
					var entity = new VE_SearchResult(ypId,escape(annoItem.Title),escape(annoItem.Notes),'',0,'yp',annoItem.Latitude,annoItem.Longitude, annoItem.Url, annoItem.PhotoUrl, annoItem.Keywords);
					if(entity && entity != '' && entity != 'undefined')
						var pinId = VE_Scratchpad.AddResult(entity,ypId);
					else
						return;
						
					if(pinId && pinId != '' && pinId != 'undefined')
					{
						//the item was successfully added to the scratchpad
						closeAllPopups();
					}
				}
				else
				{	
					var spItemId = VE_Scratchpad.AddLocation(	escape(annoItem.Title), 
																annoItem.Latitude, 
																annoItem.Longitude, 
																escape(annoItem.Notes), 
																"aN", 
																escape(annoItem.Url),
																escape(annoItem.PhotoUrl), 
																escape(annoItem.Keywords));
																
					if(spItemId && spItemId != '' && spItemId != 'undefined' && spItemId.length > 0)
					{
						closeAllPopups();
						var spItem = scratchPad.getItems().retrieveByKey(spItemId);
					}
				}
			}
		}
	}
	
	this.Serialize = function()
	{
		var serializedViewer = "";
		
		if(this.c_id != null && this.c_id != '' && this.c_id != 'undefined' && this.c_id.length > 0)
		{
			if(	this.Annotations && 
				this.Annotations != '' &&
				this.Annotations != 'undefined' && 
				this.Annotations.length > 0)		
			{
				serializedViewer = VE_CollectionsManagerConstants.C_Id + "=" + this.c_id;	
			}
		}
		
		return serializedViewer;
	}

}


function showViewerHelp()
{
	VE_Help.Open(L_Help_Text, './help/' + window.locale + '/cAbout.htm');
}

function GetFavoriteUrl(collectionId)
{
	var url = GetUrlPrefix() + "?v=" + state.GetCurrentUrlApiVersion() + "&" + VE_CollectionsManagerConstants.C_Id + "=" + collectionId;
	url = IOSec.TruncateUrlSafe ( url, 2020 );	
	return(url);
}
