//------------------------------------------------------------------------------
// Microsoft.Web.VirtualEarth.Collections.ScratchPad
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: sstchur, keithkin
// Dependencies:
// 1.  Microsoft.Web.ADT.Hashtable (microsoft.web.adt.hashtable.js)
//
// Comments:  ScratchPad presentation layer for VE
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

var Microsoft;
if (!Microsoft) Microsoft = {};
if (!Microsoft.Web) Microsoft.Web = {};
if (!Microsoft.Web.VirtualEarth) Microsoft.Web.VirtualEarth = {};
if (!Microsoft.Web.VirtualEarth.Collections) Microsoft.Web.VirtualEarth.Collections = {};

// ScratchPad Enumerations
Microsoft.Web.VirtualEarth.Collections.ScratchPadClasses = 
{
	Container: 'container',
	TitleBar: 'titleBar',
	DragArea: 'dragArea',	
	CloseBox: 'closeBox',
	ToggleBox: 'toggleBox',
	Expanded: 'expanded',
	Collapsed: 'collapsed',
	InfoBar: 'infoBar',
	HideInfoBar : 'hideInfoBar',
	InfoBarGeneral: 'infoBar-general',
	InfoBarWarning: 'infoBar-warning',
	InfoBarError: 'infoBar-error',
	ActionPanel: 'actionPanel',
	MenuBar: 'menuBar',
	CollectionList: 'collectionList',
	EmptyMessage: 'emptyMessage',
	Footer: 'footer',
	ItemToolbar: 'itemToolbar',
	ListItem: 'scratchPadPOI',
	AnnotationPOI: 'annotationPOI',
	AddressPOI: 'addressPOI',
	BusinessListingPOI: 'businessListingPOI'
};
	
Microsoft.Web.VirtualEarth.Collections.ScratchPadComponents =
{
	Title: 0,						
	Info: 1,
	ActionPanel: 2,
	EmptyMessage: 3,
	Footer: 4
};
	
Microsoft.Web.VirtualEarth.Collections.ScratchPadItemTypes =
{
	Annotation: 0,
	BusinessListing: 1
};

// ScratchPad Class
Microsoft.Web.VirtualEarth.Collections.ScratchPad = function(_id)
{
	// Account for IE bug
	var self = this;
	
	// Shortcut the Microsoft.Web.VirtualEarth.Collections NamespaceChain
	var collections = Microsoft.Web.VirtualEarth.Collections;
	
	// -- LOCAL ENUMERATIONS -- //
	var NodeChangeType =
	{
		Add: 0,
		Remove: 1,
		Insert: 2,
		Move: 3
	};
		
	// -- PRIVATE VARIABLES -- //
	var m_strId = _id;
	var m_htItems = new Microsoft.Web.ADT.Hashtable();
	var m_divContainer = document.createElement('div');
	var m_h3Title = document.createElement('h3');	
	var m_ddlTitles = new Microsoft.Web.UI.DropDownList(_id + '_titles');
	var m_divCloseBox = document.createElement('div');
	var m_divToggleBox = document.createElement('div');
	var m_divInfoBar = document.createElement('div');
	var m_divActionPanel = document.createElement('div');
	var m_pInfo = document.createElement('p');
	var m_ulMenu = document.createElement('ul');
	var m_ulCollectionList = document.createElement('ul');	
	var m_divEmptyMessage = document.createElement('div');
	var m_divFooter = document.createElement('div');
	var m_pFooter = document.createElement('p');
	var m_itemTools = new Array();
	
	// -- EVENT FUNCTION POINTERS -- //
	this.onadditem = null;
	this.onremoveitem = null;
	this.oninsertitem = null;
	this.onmoveitem = null;
	this.onupdateitem = null;
	this.onselecttitle = null;
	this.oncloseboxclick = null;
	this.onshow = null;
	this.onhide = null;
	
	Construct();			
	function Construct()
	{
		// Create all temporary DOM elements
		var titleWrapper = document.createElement('div');
		var menuWrapper = document.createElement('div');
		var collectionListWrapper = document.createElement('div');		
		
		// Set CSS Classes of all temporary DOM elements
		titleWrapper.className = collections.ScratchPadClasses.TitleBar;
		menuWrapper.className = collections.ScratchPadClasses.MenuBar;
		collectionListWrapper.className = collections.ScratchPadClasses.CollectionList;		
		
		// Set CSS Classes of all permenant DOM elements
		m_divContainer.className = collections.ScratchPadClasses.Container + ' ' + collections.ScratchPadClasses.Expanded;
		m_divToggleBox.className = collections.ScratchPadClasses.ToggleBox;
		m_divCloseBox.className = collections.ScratchPadClasses.CloseBox;
		m_divInfoBar.className = collections.ScratchPadClasses.InfoBar + ' ' + collections.ScratchPadClasses.InfoBarGeneral;
		m_divActionPanel.className = collections.ScratchPadClasses.ActionPanel;
		m_divEmptyMessage.className = collections.ScratchPadClasses.EmptyMessage;
		m_divFooter.className = collections.ScratchPadClasses.Footer;
		//TODO:  Remove this hard-coded class reference
		m_ulMenu.className = 'wll_dropDownMenu';
		
		// Set the Id of the ScratchPad's Container <div>
		m_divContainer.id = m_strId;
		
		// Build the ScratchPad by combining DOM elements 
		document.body.appendChild(m_divContainer);					// TODO:  Allow the parent to be something other than document.body (user-defined)		
		
		// TitleBar ( <h3> title text & <div> close box)
		m_divContainer.appendChild(titleWrapper);
		titleWrapper.appendChild(m_h3Title);
		titleWrapper.appendChild(m_divToggleBox);
		titleWrapper.appendChild(m_divCloseBox);		
		
		// Drop Down List of Collection Titles
		m_divContainer.appendChild(m_ddlTitles.getElement());
		m_ddlTitles.onselectoption = function(_ddlosea)
		{
			OnSelectTitle(_ddlosea);
		};
		
		// InfoBar
		m_divContainer.appendChild(m_divInfoBar);
		m_divInfoBar.appendChild(m_pInfo);
		
		// ActionPanel
		m_divContainer.appendChild(m_divActionPanel);
		
		// MenuBar
		m_divContainer.appendChild(menuWrapper);
		menuWrapper.appendChild(m_ulMenu);
		var divClear = document.createElement('div');
		divClear.style.clear = 'both';
		m_ulMenu.appendChild(divClear);
		
		
		// CollectionList
		m_divContainer.appendChild(collectionListWrapper);
		collectionListWrapper.appendChild(m_ulCollectionList);
		collectionListWrapper.appendChild(m_divEmptyMessage);
		
		// Footer
		m_divContainer.appendChild(m_divFooter);
		m_divFooter.appendChild(m_pFooter);
		
		m_divToggleBox.onclick = OnToggleBoxClick;
		m_divCloseBox.onclick = OnCloseBoxClick;
		
		// Remove any temporary DOM element references
		titleWrapper = null;
		menuWraper = null;
		collectionListWraper = null;		
		
		// ScratchPad is empty to start
		m_divEmptyMessage.style.display = 'none';											// Set the starting state for getting m_divEmptyMessage and m_ulCollectionList to have mutually exclusive visibility
		ToggleEmptyMessage(true);
		
		// ActionPanel is invisible to start
		ToggleActionPanel(false);
	}
	
	this.destroy = function()
	{
		// self reference and shortcuts cleanup
		self = null;
		collections = null;
		
		// DOM element cleanup
		m_divContainer = null;
		m_h3Title = null;
		m_divCloseBox = null;
		m_ddlTitles.destroy();
		m_divActionPanel = null;
		m_divInfoBar = null;
		m_pInfo = null;
		m_ulMenu = null;
		m_ulCollectionList = null;
		m_divEmptyMessage = null;
		m_divFooter = null;
		m_pFooter = null;
		

		var i, len;
		len = m_htItems.getCount();
		for (i = 0; i < len; i++)
			m_htItems.retrieveByIndex(i).destroy();
		
		m_htItems.destroy();
		
		len = m_itemTools.length;
		for (i = 0; i < len; i++)
		{
			m_itemTools[i].onclick = null;
			m_itemTools[i].onmouseover = null;
			m_itemTools[i].onmouseout = null;
		}	
	}

	
	// -- PUBLIC FUNCTIONS -- //					
	this.getElement = function() { return m_divContainer; };
	this.getMenu = function() { return m_ulMenu; };
	this.getItems = function() { return m_htItems; };
	
	/* Note:	showInfoBar() and hideInfoBar() would normally have used a more traditional approach of simply setting
				.style.display = 'none' / .style.display = 'block', but for some mysterious reason, IE7 crashes with
				this approach (???), so we opted to swap out a className which sets visibility: hidden & height: 0 instead.
	*/
	this.showInfoBar = function() 
	{
		var regex = new RegExp(' ' + collections.ScratchPadClasses.HideInfoBar, 'g');
		m_divInfoBar.className = m_divInfoBar.className.replace(regex, '');	
	};
	this.hideInfoBar = function() 
	{
		if (m_divInfoBar.className.indexOf(collections.ScratchPadClasses.HideInfoBar) == -1)
			{ m_divInfoBar.className += ' ' + collections.ScratchPadClasses.HideInfoBar; }
	};
	
	
	this.setGeneral = function() { m_divInfoBar.className = collections.ScratchPadClasses.InfoBar + ' ' + collections.ScratchPadClasses.InfoBarGeneral; };
	this.setWarning = function() { m_divInfoBar.className = collections.ScratchPadClasses.InfoBar + ' ' + collections.ScratchPadClasses.InfoBarWarning; };
	this.setError = function() { m_divInfoBar.className = collections.ScratchPadClasses.InfoBar + ' ' + collections.ScratchPadClasses.InfoBarError; };
	this.setExpanded = function() { m_divContainer.className = collections.ScratchPadClasses.Container + ' ' + collections.ScratchPadClasses.Expanded; };
	this.setCollapsed = function() { m_divContainer.className = collections.ScratchPadClasses.Container + ' ' + collections.ScratchPadClasses.Collapsed; };
	this.showFooter = function() { m_divFooter.style.display = 'block'; };
	this.hideFooter = function() { m_divFooter.style.display = 'none'; };
	this.isVisible = function()
	{
		var computedStyle = null;
		if (document.defaultView && document.defaultView.getComputedStyle && typeof document.defaultView.getComputedStyle != 'undefined')
			{ computedStyle = document.defaultView.getComputedStyle(m_divContainer, null); }
		else		
			{ computedStyle = m_divContainer.currentStyle; }
		
		var display = computedStyle['display'];		
		return (display != 'none');
	};
	this.show = function()
	{
		// Only fire the .onshow event if the scratchpad was previously hidden
		if (!self.isVisible())
			{ OnShow(); }
			
		m_divContainer.style.display = 'block';		
	};
	this.hide = function()
	{
		// Only fire the .onhide event if the scratchpad was previously visible
		if (self.isVisible())
			{ OnHide(); }
			
		m_divContainer.style.display = 'none';		
	};
	
	this.scrollTo = function(_spItem)
	{
		if (_spItem instanceof ScratchPadItem)
		{
			var pos = _spItem.getElement().offsetTop;			
			m_ulCollectionList.parentNode.scrollTop = pos;
		}
	};
	
	
	this.addMenuItem = function(_markup, _fn, _multiLevel, _lastChild)
	{
		if (_markup && _markup != '')
		{
			var lastChild = m_ulMenu.lastChild;
			
			var li = document.createElement('li');
			if (_lastChild)
				li.className += ' lastChild';
				
			if (_multiLevel)
				li.className += ' multiLevel';
				
			li.innerHTML = _markup;
			if (_fn)
				li.onclick = _fn;
				
			m_ulMenu.insertBefore(li, lastChild);
		}
	};
	
	this.addItemTool = function(_control)
	{
		m_itemTools.push(_control);			
	};
	
	this.setContent = function(markup, component)
	{
		var append = false;		// setting content; not appending it
		var isText = true;		// content type is plain text
		SetContent(markup, component, append, isText);
	};
	
	this.appendContent = function(markup, component)
	{
		var append = true;		// append to existing content
		var isText = true;		// content type is plain text
		SetContent(markup, component, append, isText);
	};
	
	this.addControl = function(ctrl, component)
	{
		var append = true;		// append to existing content
		var isText = false;		// content type is object/element (NOT text)
		SetContent(ctrl, component, append, isText);
	};
	
	this.clearControls = function(component)
	{
		ClearControls(component);
	};
	
	this.createItem = function(_guid, _itemType, _latitude, _longitude)
	{		
		var floatLat = 0.0;
		var floatLon = 0.0;
		
		if(_latitude != null && _latitude != '' && isNaN(_latitude))
		{
			floatLat = parseFloat(_latitude);
		}
		else
			floatLat = _latitude;
		
		if(_longitude != null && _longitude != '' && isNaN(_longitude))
		{
			floatLon = parseFloat(_longitude);
		}
		else
			floatLon = _longitude;
		
		
		var spItem = new ScratchPadItem(_guid, _itemType, floatLat, floatLon);
		return spItem;
	};
	
	this.setHeaderText = function(_text)
	{
		var txtNode = document.createTextNode(_text);
		var firstChild = m_h3Title.firstChild;
		if (firstChild)
			m_h3Title.replaceChild(firstChild, txtNode);
		else
			m_h3Title.appendChild(txtNode);
			
		txtNode = null;
		firstChild = null;
	};
	
	this.addTitle = function(_id, _title)
	{
		m_ddlTitles.addOption(_title, _id)
	};

	this.removeCurrentTitle = function()
	{
		var index = m_ddlTitles.getSelectedIndex();
		if (index != 0)
			m_ddlTitles.removeOption(index);		
	};
	
	this.selectTitle = function(_index)
	{
		m_ddlTitles.selectOption(_index);
	};
	
	this.clearTitles = function()
	{
		var i, len = m_ddlTitles.getCount();		
		for (i = 1; i < len; i++)		
			m_ddlTitles.removeOption(1, true);		
	};
	
	this.indexOf = function(_spItem)
	{
		return m_htItems.indexOf(_spItem.getGuid())
	};
	
	this.addItem = function(_spItem)
	{
		if (_spItem instanceof ScratchPadItem)						
		{
			if (m_htItems.addItem(_spItem.getGuid(), _spItem))
			{
				_spItem.setBullet(m_htItems.getCount());
				DOM_NodeListChange(NodeChangeType.Add, _spItem.getElement());
				ToggleEmptyMessage(false);																				// We've just added an item, so the ScratchPad can't be empty anymore now can it?
				OnAddItem(_spItem);
			}
		}			
	};
	
	this.removeItem = function(_spItem, isClear)
	{
		if (_spItem instanceof ScratchPadItem)
		{
			var itemIndex = m_htItems.indexOf(_spItem.getGuid());
			if (m_htItems.removeItem(itemIndex))
			{
				DOM_NodeListChange(NodeChangeType.Remove, _spItem.getElement());
				if (m_htItems.getCount() == 0)
					ToggleEmptyMessage(true)																				// No items in the ScratchPad?  Better let the user know.
				OnRemoveItem(_spItem, isClear);			
				self.refreshOrder();				
			}
		}
	};
	
	this.getItems = function()
	{
		return m_htItems;
	}
	
	this.insertItem = function(_spItem, _index)
	{
		if (_spItem instanceof ScratchPadItem)
		{
			if (m_htItems.insertItem(_spItem.getGuid(), _spItem, _index))
			{
				DOM_NodeListChange(NodeChangeType.Insert, _spItem.getElement(), _index);
				ToggleEmptyMessage(false);																				// We've just inserted an item, so the ScratchPad can't be empty anymore now can it?
				OnInsertItem(_spItem);
				self.refreshOrder();
			}
		}
	};
	
	this.moveItem = function(_spItem, _index)
	{
		if (_spItem instanceof ScratchPadItem)
		{
			var key = _spItem.getGuid();
			var startPosition = m_htItems.indexOf(key);
			if (m_htItems.moveItem(key, _index))
			{
				DOM_NodeListChange(NodeChangeType.Move, _spItem.getElement(), _index);
				OnMoveItem(_spItem, _index - startPosition);
				self.refreshOrder();
			}
		}		
	};
	
	this.updateItem = function(_spItem, _title, _desc, _latitude, _longitude, _infoUrl, _photoUrl, _keywords)
	{
		if (_spItem instanceof ScratchPadItem)
		{
			_title = unescape(_title);
			_desc = unescape(_desc);
			_infoUrl = unescape(_infoUrl);
			_photoUrl = unescape(_photoUrl);
			_keywords = unescape(_keywords);
				
			_spItem.setTitle(_title);
			_spItem.setDescription(_desc);
			_spItem.setLatitude(_latitude);
			_spItem.setLongitude(_longitude);
			_spItem.setInfoUrl(_infoUrl);
			_spItem.setPhotoUrl(_photoUrl);
			_spItem.setKeywords(_keywords);
			
			OnUpdateItem(_spItem);
		}
	};
	
	this.clear = function()
	{
		var count = m_htItems.getCount();
		for (i = 0; i < count; i++)
		{
			var spItem = m_htItems.retrieveByIndex(0);
			self.removeItem(spItem, true);
		}
	};
	
	this.refreshOrder = function()
	{
		var i, count = m_htItems.getCount();
		for (i = 0; i < count; i++)						
			m_htItems.retrieveByIndex(i).setBullet(i + 1);						
	};
	
	this.showActionPanel = function()
	{
		ToggleActionPanel(true);
	};
	
	this.hideActionPanel = function()
	{
		ToggleActionPanel(false);
	}
	
	
	// -- PRIVATE FUNCTIONS -- //
	function ToggleActionPanel(_visible)
	{
		if (_visible)
		{			
			if (m_divContainer.className.indexOf('actioning') == -1)
				{ m_divContainer.className += ' actioning'; }
		}
		else
		{
			m_divContainer.className = m_divContainer.className.replace(/\s*actioning/g, '');			
		}
	}
	
	function GetComponent(_component)
	{
		var elem = null;
		switch (_component)
		{
			case collections.ScratchPadComponents.Title:
				elem = m_h3Title;
				break;
				
			case collections.ScratchPadComponents.Info:
				elem = m_pInfo;
				break;
				
			case collections.ScratchPadComponents.ActionPanel:
				elem = m_divActionPanel;
				break;
				
			case collections.ScratchPadComponents.EmptyMessage:
				elem = m_divEmptyMessage
				break;
				
			case collections.ScratchPadComponents.Footer:
				elem = m_pFooter;
				break;
		}
		
		return elem;
	}
	
	function SetContent(_content, _component, _append, _isText)
	{
		var elem = GetComponent(_component);
		
		/* if elem is null, short-circuit the function and return false -- no valid component found to insert content into */
		if (!elem)
			return false;
		
		if (_isText)
		{
			if (_append)
				elem.innerHTML += _content;
			else
				elem.innerHTML = _content;
		}
		else				
			elem.appendChild(_content);
	}
	
	function ClearControls(_component)
	{
		var elem = GetComponent(_component);
		
		if (!elem || !elem.childNodes)
			return;
			
		var len = elem.childNodes.length;
		var i;
		for (i = 0; i < len; i++)
		{
			var child = elem.childNodes[0];			
			elem.removeChild(child);
		}
	}
	
	
	// Makes a change to the DOM of the m_ulCollectionList item
	function DOM_NodeListChange(_type, _elem, _index)
	{
		switch (_type)
		{
			// Yes, we ARE pusposefully not using a 'break' and allowing the case to fall through for Move and Insert
			case NodeChangeType.Move:
				m_ulCollectionList.removeChild(_elem);
			case NodeChangeType.Insert:
				if (_index == m_ulCollectionList.childNodes.length)
					m_ulCollectionList.appendChild(_elem);
				else
				{
					var beforeNode = m_ulCollectionList.childNodes[_index];
					if (beforeNode)
						m_ulCollectionList.insertBefore(_elem, beforeNode);
				}
				break;
			
			case NodeChangeType.Add:
				m_ulCollectionList.appendChild(_elem);
				break;
				
			case NodeChangeType.Remove:
				m_ulCollectionList.removeChild(_elem);
				break;
		}
	}
	
	// Toggles the m_divEmpytList visibility w/ the m_ulCollectionList (the two are mutually exclusive)
	function ToggleEmptyMessage(_visible)
	{
		// Yes, I'm well aware that the following could have been done in about two lines of code using the tertiary operator,
		// but the following will ensure that the .display property of the two DOM elements of interest only get set when
		// there is an explicit request to change their current value.  There is something that makes me uncomfortable about
		// unconditionally setting the display property when it isn't necessary, because I suspect that it causes
		// unnecessary rendering overhead.
	
		if (_visible && m_divEmptyMessage.style.display == 'none')
		{
			m_divEmptyMessage.style.display = 'block';
			m_ulCollectionList.style.display = 'none';
		}
		else if (!_visible && m_divEmptyMessage.style.display != 'none')
		{
			m_divEmptyMessage.style.display = 'none';
			m_ulCollectionList.style.display = 'block';
		}
	}
	
	
	
	// -- ScratchPad EVENT FUNCTIONS -- //
	function OnAddItem(_spItem)
	{
		if (typeof self.onadditem == 'function')
		{
			var spiea = new ScratchPadItemEventArgs(_spItem);
			self.onadditem(spiea);
		}
	}
	
	function OnRemoveItem(_spItem, isClear)
	{
		if (typeof self.onremoveitem == 'function')
		{
			var spiea = new ScratchPadItemEventArgs(_spItem);
			self.onremoveitem(spiea, isClear);
		}
	}
	
	function OnInsertItem(_spItem)
	{
		if (typeof self.oninsertitem == 'function')
		{
			var spiea = new ScratchPadItemEventArgs(_spItem);
			self.oninsertitem(spiea);
		}
	}
	
	function OnMoveItem(_spItem, _moveAmount)
	{
		if (typeof self.onmoveitem == 'function')
		{
			var spiea = new ScratchPadItemEventArgs(_spItem, _moveAmount);
			self.onmoveitem(spiea);
		}
	}
	
	function OnUpdateItem(_spItem)
	{
		if (typeof self.onupdateitem == 'function')
		{
			var spiea = new ScratchPadItemEventArgs(_spItem);
			self.onupdateitem(spiea);
		}
	}
	
	function OnSelectTitle(_ddlosea)
	{
		if (typeof self.onselecttitle == 'function')
		{
			var sptea = new ScratchPadTitleEventArgs(_ddlosea);
			self.onselecttitle(sptea);
		}
	}
	
	function OnCloseBoxClick()
	{
		if (typeof self.oncloseboxclick == 'function')
			self.oncloseboxclick();
	}
	
	function OnToggleBoxClick()
	{
		if (typeof self.ontoggleboxclick == 'function')
			self.ontoggleboxclick();
	}
	
	function OnShow()
	{
		if (typeof self.onshow == 'function')
			self.onshow();
	}
	
	function OnHide()
	{
		if (typeof self.onhide == 'function')
			self.onhide();
	}
	
	// -- PRIVATE (INNER) CLASSES -- //	
	
	/* ScratchPadItem Class */
	function ScratchPadItem(_guid, _type, _latitude, _longitude)
	{
		var me = this;
		var m_type = _type;
		var m_guid = _guid;
		var m_listItem = document.createElement('li');
		var m_divPOI = document.createElement('div');
		var m_h4Title = document.createElement('h4');
		var m_ddlTitles = document.createElement('select');
		var m_pDesc = document.createElement('p');
		var m_divItemToolbar = document.createElement('div');
		var m_latitude = _latitude;
		var m_longitude = _longitude;
		var m_keywords = '';
		var m_infoUrl = '';
		var m_photoUrl = '';
		var m_userDate = '';
		var m_businessListingId = '';
		var m_iconId = '';
		var m_annotationState = '';
		var m_displayOrder = '';
		Construct();
		
		function Construct()
		{
			SetItemType();			
			m_listItem.appendChild(m_divPOI);
			m_listItem.appendChild(m_h4Title);
			m_listItem.appendChild(m_pDesc);
			m_listItem.appendChild(m_divItemToolbar);
			m_divItemToolbar.className = collections.ScratchPadClasses.ItemToolbar;
			
			var i;
			for (i = 0; i < m_itemTools.length; i++)
			{
				var tool = m_itemTools[i];
				var clone = tool.cloneNode(true);
				clone.ScratchPadItemRef = me;
				clone.onclick = tool.onclick;
				clone.onmouseover = tool.onmouseover;
				clone.onmouseout = tool.onmouseout;					
				m_divItemToolbar.appendChild(clone);					
			}			
				
			m_listItem.appendChild(m_divItemToolbar);
		}
		
		this.destroy = function()
		{
			var i;				
			for (i = 0; i < m_divItemToolbar.childNodes.length; i++)
			{
				var tool = m_divItemToolbar.childNodes[i];
				tool.onclick = null;
				tool.onmouseover = null;
				tool.onmouseout = null;
				tool.ScratchPadItemRef = null;
			}
		};
		

		// -- PUBLIC FUNCTIONS -- //
		this.getType = function() { return m_type; };
		this.getGuid = function() { return m_guid; };
		this.getElement = function() { return m_listItem; };
		this.getPoi = function() { return m_divPOI; };
		this.getTitle = function() { return m_h4Title.firstChild.nodeValue; };
		this.getDescription = function() { return m_pDesc.firstChild.nodeValue; };
		this.getBullet = function() { return m_divPOI.firstChild.nodeValue; };
		this.getLatitude = function() { return m_latitude; };
		this.getLongitude = function() { return m_longitude; };
		this.getKeywords = function() { /* Keywords for next release -- not Discovery: return m_keywords; */  return ''; };
		this.getInfoUrl = function() { return m_infoUrl; };
		this.getPhotoUrl = function() { return m_photoUrl; };
		this.getUserDate = function() { return m_userDate; };
		this.getBusinessListingId = function() { return m_businessListingId; };
		this.getIconId = function() { return m_iconId; };
		this.getAnnotationState = function() { return m_annotationState; };	
		this.getDisplayOrder = function() { return m_displayOrder; };

		this.setTitle = function(_text) { DOM_SetContent(m_h4Title, _text); };
		this.setBullet = function(_text) { DOM_SetContent(m_divPOI, _text); };
		this.setDescription = function(_text) { DOM_SetContent(m_pDesc, _text); };
		this.setKeywords = function(_text) { /* Keywords for next release -- Not Discovery: m_keywords = SetSafeContent(_text); */ };
		this.setInfoUrl = function(_text) { m_infoUrl = SetSafeContent(_text); };
		this.setPhotoUrl = function(_text) { m_photoUrl = SetSafeContent(_text); };

		this.setLatitude = function(_latitude) { m_latitude = _latitude; };
		this.setLongitude = function(_longitude) { m_longitude = _longitude; };
		this.setUserDate = function(_userDate) { m_userDate = _userDate; };
		this.setBusinessListingId = function(_businessListingId) { m_businessListingId = _businessListingId; };
		this.setIconId = function(_iconId) { m_iconId = iconId; };
		this.setAnnotationState = function(_annotationState) { m_annotationState = _annotationState; };
		this.setDisplayOrder = function(_displayOrder) { m_displayOrder = _displayOrder; };


		// -- PRIVATE FUNCTIONS -- //		
		function DOM_SetContent(_elem, _text)
		{
			if (false)
				{ DOM_SetContentAndPreserveCR(_elem, _text); }
			else
			{
				var txtNode = document.createTextNode(_text);
				var firstChild = _elem.firstChild;
				if (!firstChild)
					_elem.appendChild(txtNode);
				else
					_elem.replaceChild(txtNode, firstChild);
					
				txtNode = null;
				firstChild = null;
			}
		}
		
		function DOM_SetContentAndPreserveCR(_elem, _text)
		{
			var children = _elem.childNodes;
			while (children.length > 0)
			{
				var child = children[0];
				_elem.removeChild(child);
			}
			
			var i, txtNode;
			var arr = _text.split('\n');			
			for (i = 0; i < arr.length; i++)
			{
				txtNode = document.createTextNode(arr[i]);
				
				_elem.appendChild(txtNode);
				_elem.appendChild(document.createElement('br'));				
			}
		}
		
		function SetSafeContent(_text)
		{
			if (_text == null)
				return;
				
			var safeTextValue = document.createTextNode(_text).nodeValue;
			return safeTextValue;
		}
		

		function SetItemType()
		{
			switch (m_type)
			{
				case collections.ScratchPadItemTypes.Annotation:
					m_divPOI.className = collections.ScratchPadClasses.ListItem + ' ' + collections.ScratchPadClasses.AnnotationPOI;
					break;
					
				case collections.ScratchPadItemTypes.Address:
					m_divPOI.className = collections.ScratchPadClasses.ListItem + ' ' + collections.ScratchPadClasses.AddressPOI;
					break;
					
				case collections.ScratchPadItemTypes.BusinessListing:
					m_divPOI.className = collections.ScratchPadClasses.ListItem + ' ' + collections.ScratchPadClasses.BusinessListingPOI;
					break;
			}						
		}
	}
	
	/* ScratchPadItemEventArgs Class */
	function ScratchPadItemEventArgs(_spItem, _moveAmount)
	{
		// more properties to come
		this.ScratchPadItem = _spItem;
		this.MoveAmount = _moveAmount;
	}
	

	/* ScratchPadTitleEventArgs Class */	
	function ScratchPadTitleEventArgs(_ddlosea)
	{
		this.Index = _ddlosea.Index
		this.Id = _ddlosea.Key
		this.Title = _ddlosea.Title;
	}

}