//------------------------------------------------------------------------------
// Microsoft.Web.VirtualEarth.Collections.ScratchPad
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: sstchur
// Dependencies:
// 1.  pseudoHover / pseudoHoverForChildren (stylefunctions.js)
//
// Comments:  ScratchPad presentation layer for VE
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

var Microsoft;
if (!Microsoft) Microsoft = {};
if (!Microsoft.Web) Microsoft.Web = {};
if (!Microsoft.Web.UI) Microsoft.Web.UI = {};

Microsoft.Web.UI.DropDownListClasses =
{
	Container: 'container',
	List: 'wll_dropDownMenu',
	Glyph: 'glyph_subOptions'
}

Microsoft.Web.UI.DropDownList = function(_id)
{
	var self = this;
	
	//-- PRIVATE VARIABLES --//
	var m_divContainer;
	var m_ulMenu;
	var m_pSelectedItem;
	var m_ulItems;	
	var m_selectedIndex = 0;
	
	//-- EVENT FUNCTION POINTERS --//
	this.onselectoption = null;
	
	Construct();
	function Construct()
	{
		// Create the various components that make up a DropDownList
		m_divContainer = document.createElement('div');
		m_divContainer.id = _id;
		m_ulMenu = document.createElement('ul');
		m_divContainer.appendChild(m_ulMenu);
		
		var divClear = document.createElement('div');
		divClear.style.clear = 'both';
		m_divContainer.appendChild(divClear);
		
		var li = document.createElement('li');
		li.className = 'firstChild';
		m_ulMenu.appendChild(li);		
		pseudoHover(li);
		
		m_pSelectedItem = document.createElement('p');
		var glyph = document.createElement('div');			
		glyph.className = Microsoft.Web.UI.DropDownListClasses.Glyph;			
		m_pSelectedItem.appendChild(glyph);		
		li.appendChild(m_pSelectedItem);
		
		m_ulItems = document.createElement('ul');		
		li.appendChild(m_ulItems);		
		
		// Set component classes
		m_ulMenu.className = Microsoft.Web.UI.DropDownListClasses.List;
		m_divContainer.className = Microsoft.Web.UI.DropDownListClasses.Container;
		
		
		// No loner need this temporary DOM elements
		divClear = null;
		li = null;
		glyph = null;
	}
	
	this.destroy = function()
	{
		m_divContainer = null;
		m_ulMenu = null;
		m_pSelectedItem = null;
		m_ulItems = null;
	}
	
	
	//-- PUBLIC FUNCTIONS --//
	this.getElement = function() { return m_divContainer; };
	this.getSelectedIndex = function() { return m_selectedIndex; };
	this.getCount = function() { return m_ulItems.childNodes.length; };
	
	this.addOption = function(_text, _key)
	{
		var li = document.createElement('li');
		li.appendChild(document.createTextNode(_text));
		li.setAttribute('key', _key);
		li.onclick = SelectOption;
		pseudoHover(li);
		m_ulItems.appendChild(li);
		
		if (m_ulItems.childNodes.length == 1)
			self.selectOption(0);
	};
	
	this.removeOption = function(_index, _supressEvent)
	{
		var li = m_ulItems.childNodes[_index];
		if (!li)
			return;
			
		m_ulItems.removeChild(li);		
		self.selectOption(0, _supressEvent);
	};
	
	this.selectOption = function(_index, _supressEvent)
	{
		var option = m_ulItems.childNodes[_index];
		if (!option)
		{
			option = m_ulItems.childNodes[0];
			_index = 0;
		}
		
		// If option is STILL null, we're out of luck
		if (!option)
			return;
		
		var optionText = option.firstChild.nodeValue;
		var firstChild = m_pSelectedItem.firstChild;		
		if (firstChild.nodeType != 3 /* TexNode */)		
			m_pSelectedItem.insertBefore(document.createTextNode(optionText), firstChild);		
		else
			m_pSelectedItem.replaceChild(document.createTextNode(optionText), firstChild);		
			
		m_selectedIndex = _index;
		
		if (!_supressEvent)
			OnSelectOption(option);
			
		option = null;
		optionText = null;
		firstChild = null;
	};
	
	this.getCurrentOption = function()
	{
		var li = m_ulItems.childNodes[m_selectedIndex];
		if (!li)
			return null;
			
		return li;
	};
	
	
	//-- PRIVATE FUNCTIONS --//
	function SelectOption(e)
	{
		var li = this;
		var children = m_ulItems.childNodes;
		var i;
		for (i = 0; i < children.length; i++)
		{
			var child = children[i];
			if (child == li)
			{
				self.selectOption(i);
				
				/* Force a "mouseout-like" event (for Firefox) so that, upon selecting an option, the dropdown menu disappears */
				li.parentNode.className += ' forceMouseout';
				li.onmouseout = function() { li.parentNode.className = li.parentNode.className.replace(/\s*forceMouseout/g, ''); };
				
				/* IE fails to fire a mouseout event as it should when the style changes above, so we'll fire it manually here for IE only */
				if (li.fireEvent)
					li.fireEvent('onmouseout');
					
				break;
			}
		}
	}
	
	
	//-- EVENT FUNCTIONS --//
	function OnSelectOption(_listItem)
	{
		var _ddlosea = new DropDownListOptionSelectEventArgs(_listItem);
		if (self.onselectoption)
			self.onselectoption(_ddlosea);
	}
	
	
	/* DropDownListOptionSelectEventArgs Class */
	function DropDownListOptionSelectEventArgs(_listItem)
	{
		this.Title = m_ulItems.childNodes[m_selectedIndex].firstChild.nodeValue;
		this.Key = _listItem.getAttribute('key');
		this.Index = m_selectedIndex;		
	}	
}