//------------------------------------------------------------------------------
// Microsoft.Web.UI.Glyph
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: sstchur
// Dependencies: none
//
// Comments:  A IE-Specific Glyph class to handle the various states of glyph icons in VE (unselected, hover, pressed)
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

var Microsoft;
if (!Microsoft) Microsoft = {};
if (!Microsoft.Web) Microsoft.Web = {};
if (!Microsoft.Web.UI) Microsoft.Web.UI = {};

Microsoft.Web.UI.IEGlyphStates = 
{
	Unselected: '-unselected',
	Hover: '-hover',
	Pressed: '-pressed'		
}

Microsoft.Web.UI.IEGlyph = function(_id, _defaultClass)
{
	// Shortcut the Microsoft.Web.UI Namespace Chaind
	var ui = Microsoft.Web.UI;
	
	// -- PRIVATE VARIABLES -- //
	var m_elem;	
	var m_stateHover;
	var m_statePressed;
	var m_defaultClass;
	Construct();
	
	// -- CONSTRUCTOR -- //
	function Construct()
	{
		// Create the container element for the Glyph
		m_elem = document.createElement('div');
		if (_id)
			{ m_elem.id = _id; }
		
		m_defaultClass = _defaultClass || 'ieglyph';		
		m_stateHover = m_defaultClass + ui.IEGlyphStates.Hover
		m_statePressed = m_defaultClass + ui.IEGlyphStates.Pressed;
		
		m_elem.className = m_defaultClass;
		

		// TODO:  Find a better object detection method.  If it weren't for Atlas, if (!document.attachEvent) would be ideal
		if (!document.all)
			return;

		m_elem.attachEvent('onmouseover', Hover);
		m_elem.attachEvent('onmousedown', Press);
		m_elem.attachEvent('onmouseup', Hover);
		m_elem.attachEvent('onmouseout', Unselect);
		window.attachEvent('onunload', Destroy);
	}
	
	// -- PUBLIC FUNCTIONS -- //
	this.getElement = function() { return m_elem; }
	this.setContent = function(_markup) { m_elem.innerHTML = _markup; }
	
	// -- PRIVATE FUNCTIONS -- //
	function Destroy()
	{
		m_elem.detachEvent('onmouseover', Hover);
		m_elem.detachEvent('onmousedown', Press);
		m_elem.detachEvent('onmouseup', Hover);
		m_elem.detachEvent('onmouseout', Unselect);
		
		m_elem = null;
		m_stateHover = null;
		m_statePressed = null;
		m_defaultClass = null;
	}
	
	function Hover()
	{
		Unselect();		
		ApplyState(m_stateHover);
	}
	
	function Press()
	{
		ApplyState(m_statePressed);
	}
	
	function Unselect()
	{
		RemoveState(m_stateHover);
		RemoveState(m_statePressed);
	}
	
	function ApplyState(_glyphState)
	{
		m_elem.className += ' ' + _glyphState;		
	}
	
	function RemoveState(_glyphState)
	{
		var regex = new RegExp(' ' + _glyphState, 'g');
		m_elem.className = m_elem.className.replace(regex, '');		
	}
}