var Microsoft_Web_UI_VirtualEarth =
{
		// Enum		
		CssUnits: { Pixels: 'px', Points: 'pt', Percentage: '%', Ems: 'em' },
		Positioning: { Top: 1, Right: 2, Bottom: 4, Left: 8, Static: 16, Relative: 32, Absolute: 64 },
		
		// Defaults
		CssClasses:
		{
			Panel:
			{
				Container: 'panelContainer',
				TitleBar: 'panelTitleBar',
				TitleBarControls: 'panelTitleBarControls',
				ToolBar: 'panelToolbar',
				Data: 'panelData',
				Footer: 'panelFooter'
			}
		},
		
		Components:
		{
			Panel:
			{
				Container: 0,
				TitleBar: 1,
				TitleBarControls: 2,
				Toolbar: 3,
				Data: 4,
				Footer: 5
			}
		},		
		
		TaskArea: function(id)
		{
			var startX;
			var startWidth;
			var dragging = false;
			
			var m_defaultUnits = Microsoft_Web_UI_VirtualEarth.CssUnits.Pixels;
			
			
			// make life easier
			var ms = Microsoft_Web_UI_VirtualEarth;
		
			// private members
			var m_strId = id;
			var m_panelCollection = new Array();
			
			//  border element
			var m_divBorder = document.createElement('div');
			m_divBorder.id = m_strId + '_border';
			
			// dataArea
			m_divDataArea = document.createElement('div');
			m_divDataArea.id = m_strId + '_data';
			
			// slider
			m_divSlider = document.createElement('div');
			m_divSlider.id = m_strId + '_slider';			
			
			// put it all together
			m_divBorder.appendChild(m_divDataArea);
			m_divBorder.appendChild(m_divSlider);
			
			
			/* [ -- CONSTRUCTOR -- ] */
			Construct();
			function Construct()
			{
				m_divSlider.attachEvent('onmouseover', ShowResizeCursor);
				m_divSlider.attachEvent('onmouseout', ShowNormalCursor);
				
				document.attachEvent('onmousedown', BeginDrag);	
				document.attachEvent('onmouseup', EndDrag);
				document.attachEvent('onmousemove', Drag);
			}
			/* [ -- END CONSTRUCTOR --] */
			
			
			// private event handlers
			function ShowResizeCursor(e)
			{
				document.body.style.cursor = 'col-resize';
			}
			
			function ShowNormalCursor(e)
			{
				document.body.style.cursor = 'default';
			}
			
			function BeginDrag(e)
			{
				window.status = 'begin drag';
				startX = window.event.clientX;
				startWidth = m_divBorder.offsetWidth - 22;				/* HACK -- fix the hard-coded 22 (border padding & width) */
				var target = e.srcElement;
				if (!target)
					target = e.target;
					
				if (target.id == m_strId + '_slider')
				{
					dragging = true;
					m_divSlider.detachEvent('onmouseout', ShowNormalCursor);					
				}
			}
			
			function Drag(e)
			{
				if (dragging)
				{
					ResizeElement(m_divBorder, e.clientX);					
				}
				else
				{
					dragging = false;						
				}				
			}
			
			function ResizeElement(elem, xPos)
			{
				setTimeout(innerFunction, 0);
				
				function innerFunction()
				{
					var deltaX = xPos - startX;
					var newWidth = startWidth + deltaX;
					elem.style.width = newWidth + 'px';
				}
			}
			
			function EndDrag(e)
			{
				dragging = false;
				window.status = 'end drag';
				m_divBorder.style.width = startWidth + (window.event.clientX - startX) + 'px';
				
				var target = e.srcElement;
				if (!target)
					target = e.target;
					
				if (target.id == m_strId + '_slider')
					ShowResizeCursor();
				else
					ShowNormalCursor();
				
				m_divSlider.attachEvent('onmouseout', ShowNormalCursor);				
			}			
			
			
						
			// public methods
			this.getId = function() { return m_strId; }
			this.getElement = function() { return m_divBorder; }
			
			this.addPanel = function(panel)
			{
				m_panelCollection.push(panel);
				panel.owner = this;
				m_divDataArea.appendChild(panel.getElement());
			}
			
			this.minimizePanel = function(index)
			{
				var panel = m_panelCollection[0];				
				panel.minimize();
			}
			
			this.resize = function(width, height, units)
			{
				if (!units)
					units = m_defaultUnits;

				if (width)
					m_divDataArea.style = width + units;
					
				if (height)
					m_divDataArea.style.height = height + units;
			}
			
			this.close = function()
			{
				for (var i = 0; i < m_panelCollection.length; i++)
					m_panelCollection[i].close();
			}
		},
		
		Panel: function(id)
		{
			// make life easiser
			var ms = Microsoft_Web_UI_VirtualEarth;
			var self = this;			
			
			// private members
			var m_bMinimized = false;
			var m_objParent = null;
			
			// container component
			var m_divContainer = document.createElement('div');
			m_divContainer.id = id;
			m_divContainer.className = ms.CssClasses.Panel.Container;
			
			// titlebar component
			var m_divTitleBar = document.createElement('div');
			m_divTitleBar.id = id + '_titleBar';
			m_divTitleBar.className = ms.CssClasses.Panel.TitleBar;
			//m_divTitleBar.attachEvent('onclick', Toggle);
			m_divTitleBar.onclick = Toggle;
			
			// titlebarcontrols component
			var m_divTitleBarControls = document.createElement('div');
			m_divTitleBarControls.id = m_divTitleBar.id + '_titleBarControls';
			m_divTitleBarControls.className = ms.CssClasses.Panel.TitleBarControls;
			m_divTitleBarControls.attachEvent('onclick', Close);
			
			// toolbar component
			var m_divToolbar = document.createElement('div');
			m_divToolbar.id = id + '_toolbar';
			m_divToolbar.className = ms.CssClasses.Panel.Toolbar;
			
			// data component
			var m_divData = document.createElement('div');
			m_divData.id = id + '_data';
			m_divData.className = ms.CssClasses.Panel.Data;
			
			// footer component
			var m_divFooter = document.createElement('div');
			m_divFooter.id = id + '_footer';
			m_divFooter.className = ms.CssClasses.Panel.Footer;
			
			// put it all together
			/* life is more complicated when the TitleBarControls are a child of the TitlrBar ----- m_divTitleBar.appendChild(m_divTitleBarControls); */
			m_divContainer.appendChild(m_divTitleBar);
			m_divContainer.appendChild(m_divData);
			m_divContainer.appendChild(m_divFooter);
			m_divContainer.appendChild(m_divTitleBarControls);
			
			//public methods
			this.getId = function() { return m_divContainer.id; }
			this.getElement = function() { return m_divContainer; }			
			this.close = function() { Close(); }
			this.maximize = function() { Maximize(); }
			this.minimize = function() { Minimize(); }			
			
			this.setContent = function(markup, component)
			{
				var append = false;		// setting content; not appending it
				var isText = true;		// content type is plain text
				SetContent(markup, component, append, isText);
			}
			
			this.appendContent = function(markup, component)
			{
				var append = true;		// append to existing content
				var isText = true;		// content type is plain text
				SetContent(markup, component, append, isText);
			}
			
			this.addControl = function(ctrl, component)
			{
				var append = true;		// append to existing content
				var isText = false;		// content type is object/element (NOT text)
				SetContent(ctrl, component, append, isText);
			}
			
			this.removeControl = function(ctrl, component)
			{
				// TODO:  Implement this
			}
			
			this.close = function()
			{
				Close();
			}

			// private methods
			function Close()
			{
				Destroy();
				var parentElement = m_divContainer.parentNode;
				if (parentElement)				
					parentElement.removeChild(m_divContainer);						
			}
			
			function Destroy()
			{
				m_divTitleBar.detachEvent('onclick', Toggle);
				m_divTitleBarControls.detachEvent('onclick', Close);
			}
			
			function Toggle()
			{
				var args = document.createEventObject('test');
				args.sourceObject = self;
				args.cancelBubble = true;
				self.owner.getElement().fireEvent('onclick', args);
				
				if (m_bMinimized)
					Maximize();
				else
					Minimize();				
			}
			
			function Maximize()
			{
				m_bMinimized = false;
				m_divContainer.style.height = '150px';				
			}
			
			function Minimize()
			{
				m_bMinimized = true;
				m_divContainer.style.height = m_divTitleBar.offsetHeight - 1 + 'px';				
			}
			
			function SetContent(content, component, append, isText)
			{
				var elem = null;
				switch (component)
				{
					case ms.Components.Panel.TitleBar:
						elem = m_divTitleBar;						
						break;
						
					case ms.Components.Panel.Data:
						elem = m_divData;
						break;
						
					case ms.Components.Panel.Footer:
						elem = m_divFooter;
						break;
				}

				/* if elem is null, short-circuit the function and return false -- no valid component found to insert content into */
				if (!elem)
					return false;
				
				if (isText)
				{
					if (append)
						elem.innerHTML += content;
					else
						elem.innerHTML = content;
				}
				else				
					elem.appendChild(content);
			}
		}
}
